/*******************************************************************************
*  FFTW C Interface Functions Verification Program                             *
********************************************************************************
*                                                                              *
*    LICENSED MATERIALS - PROPERTY OF IBM                                      *
*                                                                              *
*    "RESTRICTED MATERIALS OF IBM"                                             *
*                                                                              *
*    5765-F82                                                                  *
*    5765-G17                                                                  *
*    5765-H25                                                                  *
*    5765-L51                                                                  *
*                                                                              *
*    COPYRIGHT IBM CORP. 2008, 2010                                            *
*    ALL RIGHTS RESERVED                                                       *
*                                                                              *
*    US GOVERNMENT USERS RESTRICTED RIGHTS - USE, DUPLICATION OR DISCLOSURE    *
*    RESTRICTED BY GSA ADP SCHEDULE CONTRACT WITH IBM CORP.                    *
*                                                                              *
********************************************************************************
*                                                                              *
*     MODULE NAME = fftwcivp.c                                                 *
*                                                                              *
*     MODULE-DESCRIPTION :                                                     *
*       This program acts as the driver for the FFTW C Interface Functions     *
*       Verification test.                                                     *
*       This program calls all the FFTW C interface functions for testing.     *
*       It calls FFTW interface function and [DS]CFTD ([DS]CRFTD, [DS]RCFTD)   *
*       for same problem and then compares the answers. An error message is    *
*       given if the FFTW interface function does not return the correct       *
*       answer. Return code 8 signifies a function failure.                    *
*                                                                              *
*                                                                              *
*     NOTES =                                                                  *
*        DEPENDENCIES = (1) Requires ESSL library                              *
*                       (2) Requires FFTW3 Wrapper library                     *
*                       (3) Requires ESSL header file (essl.h)                 *
*                       (4) Requires FFTW3 header file (fftw3_essl.h)          *
*                                                                              *
*        REVISIONS =                                                           *
*          05/2010 - Added real-complex and complex-real functions             *
********************************************************************************
********************************************************************************
*                                                                              *
*     SUBPROGRAMS CALLED BY MAIN :                                             *
*                                                                              *
*      FFTW    =  CALL TO ALL FFTW INTERFACE FUNCTIONS FOR COMPLEX-TO-COMPLEX  *
*                 FFT WITH LONG PRECISION                                      *
*      FFTWRC  =  CALL TO ALL FFTW INTERFACE FUNCTIONS FOR REAL-TO-COMPLEX     *
*                 FFT WITH LONG PRECISION                                      *
*      FFTWCR  =  CALL TO ALL FFTW INTERFACE FUNCTIONS FOR COMPLEX-TO-REAL     *
*                 FFT WITH LONG PRECISION                                      *
*      FFTWF   =  CALL TO ALL FFTW INTERFACE FUNCTIONS FOR COMPLEX-TO-COMPLEX  *
*                 FFT WITH SHORT PRECISION                                     *
*      FFTWFRC =  CALL TO ALL FFTW INTERFACE FUNCTIONS FOR REAL-TO-COMPLEX     *
*                 FFT WITH SHORT PRECISION                                     *
*      FFTWFCR =  CALL TO ALL FFTW INTERFACE FUNCTIONS FOR COMPLEX-TO-REAL     *
*                 FFT WITH SHORT PRECISION                                     *
*                                                                              *
********************************************************************************
********************************************************************************
*                       MAIN PROGRAM                                           *
********************************************************************************/

#include <stdio.h>
#include <complex.h>
#include <math.h>
#include <essl.h>
#include "fftw3_essl.h"

void fftw(), fftwf(), fftwrc(), fftwfrc(), fftwcr(), fftwfcr();
void zvecck(), cvecck(), dvecck(), svecck();

main()
{
  int pass, rc;

  pass = 0;

  fftw(&pass);
  fftwf(&pass);

  fftwrc(&pass);
  fftwfrc(&pass);

  fftwcr(&pass);
  fftwfcr(&pass);

  if(pass == 0) 
    {
      printf(" FFTW3 C Wrapper functions run successfully\n");
      rc = 0;
    }
  else 
    {
      printf(" NUMBER OF FAILURES = %4d\n", pass);
      rc = 8;
    }
  
  return(rc);
}

/**********************************************************/

void fftw(int *pass)
{  
  fftw_plan p;
  fftw_complex  *in, *out1, *out2, *out3, *out4, *out5, *out6;
  fftw_iodim dims[1] = {8,1,1}, howmany_dims[1]={4,8,8};

  double complex y1[8],y2[48],y3[64],y4[8],y5[32],y6[32];
  double aux1[1000],aux2[2];
  int n1[1]={8},n2[2]={6,8},n3[3]={2,4,8};
  int incx1[1]={1},incx2[2]={8,1},incx3[3]={32,8,1};
  int incy1[1]={1},incy2[2]={8,1},incy3[3]={32,8,1};
  int isign[3]={1,1,1};

  int j;

  /**************************************************
   *  INPUT DATA :                                  *
   **************************************************/

  double complex x1[8] = 
    { ( 1.0000+I*0.0000), ( 0.7071+I*0.7071), ( 0.0000+I*1.0000),
      (-0.7071+I*0.7071), (-1.0000+I*0.0000), (-0.7071-I*0.7071),
      ( 0.0000-I*1.0000), ( 0.7071-I*0.7071) };

  double complex x2[48]; /* initialized in program */

  double complex x3[64]; /* initialized in program */

  double complex x4[32] = 
    { ( 1.0000+I*0.0000), ( 1.0000+I*0.0000), ( 1.0000+I*0.0000), 
      ( 1.0000+I*0.0000), ( 1.0000+I*0.0000), ( 1.0000+I*0.0000), 
      ( 1.0000+I*0.0000), ( 1.0000+I*0.0000), ( 1.0000+I*0.0000),
      ( 0.7071+I*0.7071), ( 0.0000+I*1.0000), (-0.7071+I*0.7071),
      (-1.0000+I*0.0000), (-0.7071-I*0.7071), ( 0.0000-I*1.0000), 
      ( 0.7071-I*0.7071), ( 1.0000+I*0.0000), ( 0.0000+I*1.0000), 
      (-1.0000+I*0.0000), ( 0.0000-I*1.0000), ( 1.0000+I*0.0000),  
      ( 0.0000+I*1.0000), (-1.0000+I*0.0000), ( 0.0000-I*1.0000),
      ( 1.0000+I*0.0000), (-0.7071+I*0.7071), ( 0.0000-I*1.0000), 
      ( 0.7071+I*0.7071), (-1.0000+I*0.0000), ( 0.7071-I*0.7071), 
      ( 0.0000+I*1.0000), (-0.7071-I*0.7071) };

/********************************************************/

  out1 = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*8);
  out2 = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*48);
  out3 = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*64);
  out4 = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*8);
  out5 = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*32);
  out6 = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*32);

  /********************************************************/

  in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*8);
  for (j=0; j<8; j++) in[j] = x1[j];
  p = fftw_plan_dft_1d(8,in,out1,FFTW_FORWARD,FFTW_ESTIMATE);
  fftw_execute(p);

  dcftd(1,1,x1,incx1,8,y1,incy1,8,n1,1,isign,1.0,aux1,1000,aux2,0);
  dcftd(0,1,x1,incx1,8,y1,incy1,8,n1,1,isign,1.0,aux1,1000,aux2,0);

  zvecck(8,1,out1,y1,"fftw_plan_dft_1d",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  for (j=0; j<48; j++) x2[j] = 0.0;
  for (j=0; j< 6; j++) x2[8*j] = 1.0;

  in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*48);
  for (j=0; j<48; j++) in[j] = x2[j];
  p = fftw_plan_dft_2d(6,8,in,out2,FFTW_FORWARD,FFTW_ESTIMATE);
  fftw_execute_dft(p,in,out2);

  dcftd(1,2,x2,incx2,48,y2,incy2,48,n2,1,isign,1.0,aux1,1000,aux2,0);
  dcftd(0,2,x2,incx2,48,y2,incy2,48,n2,1,isign,1.0,aux1,1000,aux2,0);

  zvecck(48,1,out2,y2,"fftw_plan_dft_2d",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  for (j=0; j<64; j++) x3[j] = 0.0;
  x3[0] = -1.0;
  for (j=1; j< 8; j++) x3[j] = -x3[j-1];

  in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*64);
  for (j=0; j<64; j++) in[j] = x3[j];
  p = fftw_plan_dft_3d(2,4,8,in,out3,FFTW_FORWARD,FFTW_ESTIMATE);
  fftw_execute(p);

  dcftd(1,3,x3,incx3,64,y3,incy3,64,n3,1,isign,1.0,aux1,1000,aux2,0);
  dcftd(0,3,x3,incx3,64,y3,incy3,64,n3,1,isign,1.0,aux1,1000,aux2,0);

  zvecck(64,1,out3,y3,"fftw_plan_dft_3d",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*8);
  for (j=0; j<8; j++) in[j] = x1[j];
  p = fftw_plan_dft(1,n1,in,out4,FFTW_FORWARD,FFTW_ESTIMATE);
  fftw_execute_dft(p,in,out4);

  dcftd(1,1,x1,incx1,8,y4,incy1,8,n1,1,isign,1.0,aux1,1000,aux2,0);
  dcftd(0,1,x1,incx1,8,y4,incy1,8,n1,1,isign,1.0,aux1,1000,aux2,0);

  zvecck(8,1,out4,y4,"fftw_plan_dft",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*32);
  for (j=0; j<32; j++) in[j] = x4[j];
  p = fftw_plan_many_dft(1,n1,4,in,n1,1,8,
			 out5,n1,1,8,FFTW_FORWARD,FFTW_ESTIMATE);
  fftw_execute(p);

  dcftd(1,1,x4,incx1,8,y5,incy1,8,n1,4,isign,1.0,aux1,1000,aux2,0);
  dcftd(0,1,x4,incx1,8,y5,incy1,8,n1,4,isign,1.0,aux1,1000,aux2,0);

  zvecck(32,1,out5,y5,"fftw_plan_many_dft",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*32);
  for (j=0; j<32; j++) in[j] = x4[j];
  p = fftw_plan_guru_dft(1,dims,1,howmany_dims,in,out6,
			 FFTW_FORWARD,FFTW_ESTIMATE);
  fftw_execute_dft(p,in,out6);

  dcftd(1,1,x4,incx1,8,y6,incy1,8,n1,4,isign,1.0,aux1,1000,aux2,0);
  dcftd(0,1,x4,incx1,8,y6,incy1,8,n1,4,isign,1.0,aux1,1000,aux2,0);

  zvecck(32,1,out6,y6,"fftw_plan_guru_dft",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  fftw_free(out1);
  fftw_free(out2);
  fftw_free(out3);
  fftw_free(out4);
  fftw_free(out5);
  fftw_free(out6);

  /********************************************************/

  return;
}

/**********************************************************/

void fftwrc(int *pass)
{  
  fftw_plan p;
  double *in;
  fftw_complex *out1, *out2, *out3, *out4, *out5, *out6;
  fftw_iodim dims[1] = {16,1,1}, howmany_dims[1]={4,16,9};

  double complex y1[9],y2[30],y3[40],y4[9],y5[36],y6[36];
  double aux1[1000],aux2[2];
  int n1[1]={16},n2[2]={8,6},n3[3]={8,4,2};
  int incx1[1]={1},incx2[2]={1,8},incx3[3]={1,8,32};
  int incy1[1]={1},incy2[2]={1,5},incy3[3]={1,5,20};
  int isign[3]={1,1,1};

  int j;

  /**************************************************
   *  INPUT DATA :                                  *
   **************************************************/

  double x1[16] = 
    { 1.0000,  0.9239,  0.7071,  0.3827,  0.0000, -0.3827,  
     -0.7071, -0.9239, -1.0000, -0.9239, -0.7071, -0.3827,
      0.0000,  0.3827,  0.7071,  0.9239 };

  double x2[48]; /* initialized in program */

  double x3[64]; /* initialized in program */

  double x4[64] = 
    { 1.0000,  1.0000,  1.0000,  1.0000,  1.0000,  1.0000,
      1.0000,  1.0000,  1.0000,  1.0000,  1.0000,  1.0000,
      1.0000,  1.0000,  1.0000,  1.0000,  
      1.0000,  0.9239,  0.7071,  0.3827,  0.0000, -0.3827,  
     -0.7071, -0.9239, -1.0000, -0.9239, -0.7071, -0.3827,
      0.0000,  0.3827,  0.7071,  0.9239,   
      1.0000,  0.7071,  0.0000, -0.7071, -1.0000, -0.7071,
      0.0000,  0.7071,  1.0000,  0.7071,  0.0000, -0.7071,
     -1.0000, -0.7071,  0.0000,  0.7071,
      1.0000,  0.3827, -0.7071, -0.9239,  0.0000,  0.9239,
      0.7071, -0.3827, -1.0000, -0.3827,  0.7071,  0.9239,
      0.0000, -0.9239, -0.7071,  0.3827 };

/********************************************************/

  out1 = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*9);
  out2 = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*30);
  out3 = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*40);
  out4 = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*9);
  out5 = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*36);
  out6 = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*36);

  /********************************************************/

  in = (double*) fftw_malloc(sizeof(double)*16);
  for (j=0; j<16; j++) in[j] = x1[j];
  p = fftw_plan_dft_r2c_1d(16,in,out1,FFTW_ESTIMATE);
  fftw_execute(p);

  drcftd(1,1,x1,incx1,16,y1,incy1,9,n1,1,isign,1.0,aux1,1000,aux2,0);
  drcftd(0,1,x1,incx1,16,y1,incy1,9,n1,1,isign,1.0,aux1,1000,aux2,0);

  zvecck(9,1,out1,y1,"fftw_plan_dft_r2c_1d",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  for (j=0; j<48; j++) x2[j] = 0.0;
  for (j=0; j< 6; j++) x2[8*j] = 1.0;

  in = (double*) fftw_malloc(sizeof(double)*48);
  for (j=0; j<48; j++) in[j] = x2[j];
  p = fftw_plan_dft_r2c_2d(6,8,in,out2,FFTW_ESTIMATE);
  fftw_execute_dft_r2c(p,in,out2);

  drcftd(1,2,x2,incx2,48,y2,incy2,30,n2,1,isign,1.0,aux1,1000,aux2,0);
  drcftd(0,2,x2,incx2,48,y2,incy2,30,n2,1,isign,1.0,aux1,1000,aux2,0);

  zvecck(30,1,out2,y2,"fftw_plan_dft_r2c_2d",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  for (j=0; j<64; j++) x3[j] = 0.0;
  x3[0] = -1.0;
  for (j=1; j< 8; j++) x3[j] = -x3[j-1];

  in = (double*) fftw_malloc(sizeof(double)*64);
  for (j=0; j<64; j++) in[j] = x3[j];
  p = fftw_plan_dft_r2c_3d(2,4,8,in,out3,FFTW_ESTIMATE);
  fftw_execute(p);

  drcftd(1,3,x3,incx3,64,y3,incy3,40,n3,1,isign,1.0,aux1,1000,aux2,0);
  drcftd(0,3,x3,incx3,64,y3,incy3,40,n3,1,isign,1.0,aux1,1000,aux2,0);

  zvecck(40,1,out3,y3,"fftw_plan_dft_r2c_3d",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  in = (double*) fftw_malloc(sizeof(double)*16);
  for (j=0; j<16; j++) in[j] = x1[j];
  p = fftw_plan_dft_r2c(1,n1,in,out4,FFTW_ESTIMATE);
  fftw_execute_dft_r2c(p,in,out4);

  drcftd(1,1,x1,incx1,16,y4,incy1,9,n1,1,isign,1.0,aux1,1000,aux2,0);
  drcftd(0,1,x1,incx1,16,y4,incy1,9,n1,1,isign,1.0,aux1,1000,aux2,0);

  zvecck(9,1,out4,y4,"fftw_plan_dft_r2c",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  in = (double*) fftw_malloc(sizeof(double)*64);
  for (j=0; j<64; j++) in[j] = x4[j];
  p = fftw_plan_many_dft_r2c(1,n1,4,in,n1,1,16,
			     out5,n1,1,9,FFTW_ESTIMATE);
  fftw_execute(p);

  drcftd(1,1,x4,incx1,16,y5,incy1,9,n1,4,isign,1.0,aux1,1000,aux2,0);
  drcftd(0,1,x4,incx1,16,y5,incy1,9,n1,4,isign,1.0,aux1,1000,aux2,0);

  zvecck(36,1,out5,y5,"fftw_plan_many_dft_r2c",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  in = (double*) fftw_malloc(sizeof(double)*64);
  for (j=0; j<64; j++) in[j] = x4[j];
  p = fftw_plan_guru_dft_r2c(1,dims,1,howmany_dims,in,out6,
			     FFTW_ESTIMATE);
  fftw_execute_dft_r2c(p,in,out6);

  drcftd(1,1,x4,incx1,16,y6,incy1,9,n1,4,isign,1.0,aux1,1000,aux2,0);
  drcftd(0,1,x4,incx1,16,y6,incy1,9,n1,4,isign,1.0,aux1,1000,aux2,0);

  zvecck(36,1,out6,y6,"fftw_plan_guru_dft_r2c",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  fftw_free(out1);
  fftw_free(out2);
  fftw_free(out3);
  fftw_free(out4);
  fftw_free(out5);
  fftw_free(out6);

  /********************************************************/

  return;
}

/**********************************************************/

void fftwcr(int *pass)
{  
  fftw_plan p;
  fftw_complex  *in;
  double  *out1, *out2, *out3, *out4, *out5, *out6;
  fftw_iodim dims[1] = {16,1,1}, howmany_dims[1]={2,9,16};

  double y1[16],y2[60],y3[80],y4[16],y5[32],y6[32];
  double aux1[1000],aux2[2];
  int n1[1]={16},n2[2]={8,6},n3[3]={8,4,2};
  int incx1[1]={1},incx2[2]={1,5},incx3[3]={1,5,20};
  int incy1[1]={1},incy2[2]={1,10},incy3[3]={1,10,40};
  int isign[3]={-1,-1,-1};

  int i, j;

  /**************************************************
   *  INPUT DATA :                                  *
   **************************************************/

  double complex x1[9] = 
    { ( 1.0000+I*0.0000), ( 0.0000+I*1.0000), (-1.0000+I*0.0000), 
      ( 0.0000-I*1.0000), ( 1.0000+I*0.0000), ( 0.0000+I*1.0000), 
      (-1.0000+I*0.0000), ( 0.0000-I*1.0000), ( 1.0000+I*0.0000) };

  double complex x2[30]; /* initialized in program */

  double complex x3[40]; /* initialized in program */

  double complex x4[18] = 
    { ( 1.0000+I*0.0000), ( 0.0000+I*1.0000), (-1.0000+I*0.0000), 
      ( 0.0000-I*1.0000), ( 1.0000+I*0.0000), ( 0.0000+I*1.0000), 
      (-1.0000+I*0.0000), ( 0.0000-I*1.0000), ( 1.0000+I*0.0000),
      ( 1.0000+I*0.0000), ( 0.0000-I*1.0000), (-1.0000+I*0.0000), 
      ( 0.0000+I*1.0000), ( 1.0000+I*0.0000), ( 0.0000-I*1.0000), 
      (-1.0000+I*0.0000), ( 0.0000+I*1.0000), ( 1.0000+I*0.0000) };

/********************************************************/

  out1 = (double*) fftw_malloc(sizeof(double)*16);
  out2 = (double*) fftw_malloc(sizeof(double)*60);
  out3 = (double*) fftw_malloc(sizeof(double)*80);
  out4 = (double*) fftw_malloc(sizeof(double)*16);
  out5 = (double*) fftw_malloc(sizeof(double)*32);
  out6 = (double*) fftw_malloc(sizeof(double)*32);

  /********************************************************/

  in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*9);
  for (j=0; j<9; j++) in[j] = x1[j];
  p = fftw_plan_dft_c2r_1d(16,in,out1,FFTW_ESTIMATE);
  fftw_execute(p);

  dcrftd(1,1,x1,incx1,9,y1,incy1,16,n1,1,isign,1.0,aux1,1000,aux2,0);
  dcrftd(0,1,x1,incx1,9,y1,incy1,16,n1,1,isign,1.0,aux1,1000,aux2,0);

  dvecck(16,1,out1,y1,"fftw_plan_dft_c2r_1d",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  for (j=0; j<30; j++) x2[j] = 0.0;
  for (j=0; j< 6; j++) x2[5*j] = 1.0;

  in = (fftw_complex*)out2;
  for (j=0; j<30; j++) in[j] = x2[j];
  p = fftw_plan_dft_c2r_2d(6,8,in,out2,FFTW_ESTIMATE);
  fftw_execute_dft_c2r(p,in,out2);

  dcrftd(1,2,x2,incx2,30,y2,incy2,60,n2,1,isign,1.0,aux1,1000,aux2,0);
  dcrftd(0,2,x2,incx2,30,y2,incy2,60,n2,1,isign,1.0,aux1,1000,aux2,0);

  for (i=0; i<6; i++)
    {
      dvecck(8,1,&out2[i*10],&y2[i*10],"fftw_plan_dft_c2r_2d",pass);
    }

  fftw_destroy_plan(p);

  /********************************************************/

  for (j=0; j<40; j++) x3[j] = 0.0;
  x3[0] = -1.0;
  for (j=1; j< 5; j++) x3[j] = -x3[j-1];

  in = (fftw_complex*)out3;
  for (j=0; j<40; j++) in[j] = x3[j];
  p = fftw_plan_dft_c2r_3d(2,4,8,in,out3,FFTW_ESTIMATE);
  fftw_execute(p);

  dcrftd(1,3,x3,incx3,40,y3,incy3,80,n3,1,isign,1.0,aux1,1000,aux2,0);
  dcrftd(0,3,x3,incx3,40,y3,incy3,80,n3,1,isign,1.0,aux1,1000,aux2,0);

  for (j=0; j<2; j++)
    {
      for (i=0; i<4; i++)
	{
	  dvecck(8,1,&out3[i*10+j*40],&y3[i*10+j*40],
		 "fftw_plan_dft_c2r_3d",pass);
	}
    }

  fftw_destroy_plan(p);

  /********************************************************/

  in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*9);
  for (j=0; j<9; j++) in[j] = x1[j];
  p = fftw_plan_dft_c2r(1,n1,in,out4,FFTW_ESTIMATE);
  fftw_execute_dft_c2r(p,in,out4);

  dcrftd(1,1,x1,incx1,9,y4,incy1,16,n1,1,isign,1.0,aux1,1000,aux2,0);
  dcrftd(0,1,x1,incx1,9,y4,incy1,16,n1,1,isign,1.0,aux1,1000,aux2,0);

  dvecck(16,1,out4,y4,"fftw_plan_dft_c2r",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*18);
  for (j=0; j<18; j++) in[j] = x4[j];
  p = fftw_plan_many_dft_c2r(1,n1,2,in,n1,1,9,
			 out5,n1,1,16,FFTW_ESTIMATE);
  fftw_execute(p);

  dcrftd(1,1,x4,incx1,9,y5,incy1,16,n1,2,isign,1.0,aux1,1000,aux2,0);
  dcrftd(0,1,x4,incx1,9,y5,incy1,16,n1,2,isign,1.0,aux1,1000,aux2,0);

  dvecck(32,1,out5,y5,"fftw_plan_many_dft_c2r",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex)*18);
  for (j=0; j<18; j++) in[j] = x4[j];
  p = fftw_plan_guru_dft_c2r(1,dims,1,howmany_dims,in,out6,
			     FFTW_ESTIMATE);
  fftw_execute_dft_c2r(p,in,out6);

  dcrftd(1,1,x4,incx1,9,y6,incy1,16,n1,2,isign,1.0,aux1,1000,aux2,0);
  dcrftd(0,1,x4,incx1,9,y6,incy1,16,n1,2,isign,1.0,aux1,1000,aux2,0);

  dvecck(32,1,out6,y6,"fftw_plan_guru_dft_c2r",pass);

  fftw_destroy_plan(p);
  fftw_free(in);

  /********************************************************/

  fftw_free(out1);
  fftw_free(out2);
  fftw_free(out3);
  fftw_free(out4);
  fftw_free(out5);
  fftw_free(out6);

  /********************************************************/

  return;
}

/**********************************************************/


void fftwf(int *pass)
{  
  fftwf_plan p;
  fftwf_complex  *in, *out1, *out2, *out3, *out4, *out5, *out6;
  fftwf_iodim dims[1] = {8,1,1}, howmany_dims[1]={4,8,8};

  float complex y1[8],y2[48],y3[64],y4[8],y5[32],y6[32];
  double aux1[1000],aux2[2];
  int n1[1]={8},n2[2]={6,8},n3[3]={2,4,8};
  int incx1[1]={1},incx2[2]={8,1},incx3[3]={32,8,1};
  int incy1[1]={1},incy2[2]={8,1},incy3[3]={32,8,1};
  int isign[3]={1,1,1};

  int j;

  /**************************************************
   *  INPUT DATA :                                  *
   **************************************************/

  float complex x1[8] = 
    { ( 1.0000+I*0.0000), ( 0.7071+I*0.7071), ( 0.0000+I*1.0000),
      (-0.7071+I*0.7071), (-1.0000+I*0.0000), (-0.7071-I*0.7071),
      ( 0.0000-I*1.0000), ( 0.7071-I*0.7071) };

  float complex x2[48]; /* initialized in program */

  float complex x3[64]; /* initialized in program */

  float complex x4[32] = 
    { ( 1.0000+I*0.0000), ( 1.0000+I*0.0000), ( 1.0000+I*0.0000), 
      ( 1.0000+I*0.0000), ( 1.0000+I*0.0000), ( 1.0000+I*0.0000), 
      ( 1.0000+I*0.0000), ( 1.0000+I*0.0000), ( 1.0000+I*0.0000),
      ( 0.7071+I*0.7071), ( 0.0000+I*1.0000), (-0.7071+I*0.7071),
      (-1.0000+I*0.0000), (-0.7071-I*0.7071), ( 0.0000-I*1.0000), 
      ( 0.7071-I*0.7071), ( 1.0000+I*0.0000), ( 0.0000+I*1.0000), 
      (-1.0000+I*0.0000), ( 0.0000-I*1.0000), ( 1.0000+I*0.0000),  
      ( 0.0000+I*1.0000), (-1.0000+I*0.0000), ( 0.0000-I*1.0000),
      ( 1.0000+I*0.0000), (-0.7071+I*0.7071), ( 0.0000-I*1.0000), 
      ( 0.7071+I*0.7071), (-1.0000+I*0.0000), ( 0.7071-I*0.7071), 
      ( 0.0000+I*1.0000), (-0.7071-I*0.7071) };

/********************************************************/

  out1 = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*8);
  out2 = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*48);
  out3 = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*64);
  out4 = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*8);
  out5 = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*32);
  out6 = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*32);

  /********************************************************/

  in = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*8);
  for (j=0; j<8; j++) in[j] = x1[j];
  p = fftwf_plan_dft_1d(8,in,out1,FFTW_FORWARD,FFTW_ESTIMATE);
  fftwf_execute(p);

  scftd(1,1,x1,incx1,8,y1,incy1,8,n1,1,isign,1.0,aux1,1000,aux2,0);
  scftd(0,1,x1,incx1,8,y1,incy1,8,n1,1,isign,1.0,aux1,1000,aux2,0);

  cvecck(8,1,out1,y1,"fftwf_plan_dft_1d",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  for (j=0; j<48; j++) x2[j] = 0.0;
  for (j=0; j< 6; j++) x2[8*j] = 1.0;

  in = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*48);
  for (j=0; j<48; j++) in[j] = x2[j];
  p = fftwf_plan_dft_2d(6,8,in,out2,FFTW_FORWARD,FFTW_ESTIMATE);
  fftwf_execute_dft(p,in,out2);

  scftd(1,2,x2,incx2,48,y2,incy2,48,n2,1,isign,1.0,aux1,1000,aux2,0);
  scftd(0,2,x2,incx2,48,y2,incy2,48,n2,1,isign,1.0,aux1,1000,aux2,0);

  cvecck(48,1,out2,y2,"fftwf_plan_dft_2d",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  for (j=0; j<64; j++) x3[j] = 0.0;
  x3[0] = -1.0;
  for (j=1; j< 8; j++) x3[j] = -x3[j-1];

  in = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*64);
  for (j=0; j<64; j++) in[j] = x3[j];
  p = fftwf_plan_dft_3d(2,4,8,in,out3,FFTW_FORWARD,FFTW_ESTIMATE);
  fftwf_execute(p);

  scftd(1,3,x3,incx3,64,y3,incy3,64,n3,1,isign,1.0,aux1,1000,aux2,0);
  scftd(0,3,x3,incx3,64,y3,incy3,64,n3,1,isign,1.0,aux1,1000,aux2,0);

  cvecck(64,1,out3,y3,"fftwf_plan_dft_3d",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  in = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*8);
  for (j=0; j<8; j++) in[j] = x1[j];
  p = fftwf_plan_dft(1,n1,in,out4,FFTW_FORWARD,FFTW_ESTIMATE);
  fftwf_execute_dft(p,in,out4);

  scftd(1,1,x1,incx1,8,y4,incy1,8,n1,1,isign,1.0,aux1,1000,aux2,0);
  scftd(0,1,x1,incx1,8,y4,incy1,8,n1,1,isign,1.0,aux1,1000,aux2,0);

  cvecck(8,1,out4,y4,"fftwf_plan_dft",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  in = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*32);
  for (j=0; j<32; j++) in[j] = x4[j];
  p = fftwf_plan_many_dft(1,n1,4,in,n1,1,8,
			 out5,n1,1,8,FFTW_FORWARD,FFTW_ESTIMATE);
  fftwf_execute(p);

  scftd(1,1,x4,incx1,8,y5,incy1,8,n1,4,isign,1.0,aux1,1000,aux2,0);
  scftd(0,1,x4,incx1,8,y5,incy1,8,n1,4,isign,1.0,aux1,1000,aux2,0);

  cvecck(32,1,out5,y5,"fftwf_plan_many_dft",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  in = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*32);
  for (j=0; j<32; j++) in[j] = x4[j];
  p = fftwf_plan_guru_dft(1,dims,1,howmany_dims,in,out6,
			 FFTW_FORWARD,FFTW_ESTIMATE);
  fftwf_execute_dft(p,in,out6);

  scftd(1,1,x4,incx1,8,y6,incy1,8,n1,4,isign,1.0,aux1,1000,aux2,0);
  scftd(0,1,x4,incx1,8,y6,incy1,8,n1,4,isign,1.0,aux1,1000,aux2,0);

  cvecck(32,1,out6,y6,"fftwf_plan_guru_dft",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  fftwf_free(out1);
  fftwf_free(out2);
  fftwf_free(out3);
  fftwf_free(out4);
  fftwf_free(out5);
  fftwf_free(out6);

  /********************************************************/

  return;
}

/**********************************************************/

void fftwfrc(int *pass)
{  
  fftwf_plan p;
  float *in;
  fftwf_complex *out1, *out2, *out3, *out4, *out5, *out6;
  fftwf_iodim dims[1] = {16,1,1}, howmany_dims[1]={4,16,9};

  float complex y1[9],y2[30],y3[40],y4[9],y5[36],y6[36];
  double aux1[1000],aux2[2];
  int n1[1]={16},n2[2]={8,6},n3[3]={8,4,2};
  int incx1[1]={1},incx2[2]={1,8},incx3[3]={1,8,32};
  int incy1[1]={1},incy2[2]={1,5},incy3[3]={1,5,20};
  int isign[3]={1,1,1};

  int j;

  /**************************************************
   *  INPUT DATA :                                  *
   **************************************************/

  float x1[16] = 
    { 1.0000,  0.9239,  0.7071,  0.3827,  0.0000, -0.3827,  
     -0.7071, -0.9239, -1.0000, -0.9239, -0.7071, -0.3827,
      0.0000,  0.3827,  0.7071,  0.9239 };

  float x2[48]; /* initialized in program */

  float x3[64]; /* initialized in program */

  float x4[64] = 
    { 1.0000,  1.0000,  1.0000,  1.0000,  1.0000,  1.0000,
      1.0000,  1.0000,  1.0000,  1.0000,  1.0000,  1.0000,
      1.0000,  1.0000,  1.0000,  1.0000,  
      1.0000,  0.9239,  0.7071,  0.3827,  0.0000, -0.3827,  
     -0.7071, -0.9239, -1.0000, -0.9239, -0.7071, -0.3827,
      0.0000,  0.3827,  0.7071,  0.9239,   
      1.0000,  0.7071,  0.0000, -0.7071, -1.0000, -0.7071,
      0.0000,  0.7071,  1.0000,  0.7071,  0.0000, -0.7071,
     -1.0000, -0.7071,  0.0000,  0.7071,
      1.0000,  0.3827, -0.7071, -0.9239,  0.0000,  0.9239,
      0.7071, -0.3827, -1.0000, -0.3827,  0.7071,  0.9239,
      0.0000, -0.9239, -0.7071,  0.3827 };

/********************************************************/

  out1 = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*9);
  out2 = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*30);
  out3 = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*40);
  out4 = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*9);
  out5 = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*36);
  out6 = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*36);

  /********************************************************/

  in = (float*) fftwf_malloc(sizeof(float)*16);
  for (j=0; j<16; j++) in[j] = x1[j];
  p = fftwf_plan_dft_r2c_1d(16,in,out1,FFTW_ESTIMATE);
  fftwf_execute(p);

  srcftd(1,1,x1,incx1,16,y1,incy1,9,n1,1,isign,1.0,aux1,1000,aux2,0);
  srcftd(0,1,x1,incx1,16,y1,incy1,9,n1,1,isign,1.0,aux1,1000,aux2,0);

  cvecck(9,1,out1,y1,"fftwf_plan_dft_r2c_1d",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  for (j=0; j<48; j++) x2[j] = 0.0;
  for (j=0; j< 6; j++) x2[8*j] = 1.0;

  in = (float*) fftw_malloc(sizeof(float)*48);
  for (j=0; j<48; j++) in[j] = x2[j];
  p = fftwf_plan_dft_r2c_2d(6,8,in,out2,FFTW_ESTIMATE);
  fftwf_execute_dft_r2c(p,in,out2);

  srcftd(1,2,x2,incx2,48,y2,incy2,30,n2,1,isign,1.0,aux1,1000,aux2,0);
  srcftd(0,2,x2,incx2,48,y2,incy2,30,n2,1,isign,1.0,aux1,1000,aux2,0);

  cvecck(30,1,out2,y2,"fftwf_plan_dft_r2c_2d",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  for (j=0; j<64; j++) x3[j] = 0.0;
  x3[0] = -1.0;
  for (j=1; j< 8; j++) x3[j] = -x3[j-1];

  in = (float*) fftw_malloc(sizeof(float)*64);
  for (j=0; j<64; j++) in[j] = x3[j];
  p = fftwf_plan_dft_r2c_3d(2,4,8,in,out3,FFTW_ESTIMATE);
  fftwf_execute(p);

  srcftd(1,3,x3,incx3,64,y3,incy3,40,n3,1,isign,1.0,aux1,1000,aux2,0);
  srcftd(0,3,x3,incx3,64,y3,incy3,40,n3,1,isign,1.0,aux1,1000,aux2,0);

  cvecck(40,1,out3,y3,"fftwf_plan_dft_r2c_3d",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  in = (float*) fftw_malloc(sizeof(float)*16);
  for (j=0; j<16; j++) in[j] = x1[j];
  p = fftwf_plan_dft_r2c(1,n1,in,out4,FFTW_ESTIMATE);
  fftwf_execute_dft_r2c(p,in,out4);

  srcftd(1,1,x1,incx1,16,y4,incy1,9,n1,1,isign,1.0,aux1,1000,aux2,0);
  srcftd(0,1,x1,incx1,16,y4,incy1,9,n1,1,isign,1.0,aux1,1000,aux2,0);

  cvecck(9,1,out4,y4,"fftwf_plan_dft_r2c",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  in = (float*) fftw_malloc(sizeof(float)*64);
  for (j=0; j<64; j++) in[j] = x4[j];
  p = fftwf_plan_many_dft_r2c(1,n1,4,in,n1,1,16,
			     out5,n1,1,9,FFTW_ESTIMATE);
  fftwf_execute(p);

  srcftd(1,1,x4,incx1,16,y5,incy1,9,n1,4,isign,1.0,aux1,1000,aux2,0);
  srcftd(0,1,x4,incx1,16,y5,incy1,9,n1,4,isign,1.0,aux1,1000,aux2,0);

  cvecck(36,1,out5,y5,"fftwf_plan_many_dft_r2c",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  in = (float*) fftw_malloc(sizeof(float)*64);
  for (j=0; j<64; j++) in[j] = x4[j];
  p = fftwf_plan_guru_dft_r2c(1,dims,1,howmany_dims,in,out6,
			     FFTW_ESTIMATE);
  fftwf_execute_dft_r2c(p,in,out6);

  srcftd(1,1,x4,incx1,16,y6,incy1,9,n1,4,isign,1.0,aux1,1000,aux2,0);
  srcftd(0,1,x4,incx1,16,y6,incy1,9,n1,4,isign,1.0,aux1,1000,aux2,0);

  cvecck(36,1,out6,y6,"fftwf_plan_guru_dft_r2c",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  fftwf_free(out1);
  fftwf_free(out2);
  fftwf_free(out3);
  fftwf_free(out4);
  fftwf_free(out5);
  fftwf_free(out6);

  /********************************************************/

  return;
}

/**********************************************************/

void fftwfcr(int *pass)
{  
  fftwf_plan p;
  fftwf_complex  *in;
  float  *out1, *out2, *out3, *out4, *out5, *out6;
  fftwf_iodim dims[1] = {16,1,1}, howmany_dims[1]={2,9,16};

  float y1[16],y2[60],y3[80],y4[16],y5[32],y6[32];
  double aux1[1000],aux2[2];
  int n1[1]={16},n2[2]={8,6},n3[3]={8,4,2};
  int incx1[1]={1},incx2[2]={1,5},incx3[3]={1,5,20};
  int incy1[1]={1},incy2[2]={1,10},incy3[3]={1,10,40};
  int isign[3]={-1,-1,-1};

  int i, j;

  /**************************************************
   *  INPUT DATA :                                  *
   **************************************************/

  float complex x1[9] = 
    { ( 1.0000+I*0.0000), ( 0.0000+I*1.0000), (-1.0000+I*0.0000), 
      ( 0.0000-I*1.0000), ( 1.0000+I*0.0000), ( 0.0000+I*1.0000), 
      (-1.0000+I*0.0000), ( 0.0000-I*1.0000), ( 1.0000+I*0.0000) };

  float complex x2[30]; /* initialized in program */

  float complex x3[40]; /* initialized in program */

  float complex x4[18] = 
    { ( 1.0000+I*0.0000), ( 0.0000+I*1.0000), (-1.0000+I*0.0000), 
      ( 0.0000-I*1.0000), ( 1.0000+I*0.0000), ( 0.0000+I*1.0000), 
      (-1.0000+I*0.0000), ( 0.0000-I*1.0000), ( 1.0000+I*0.0000),
      ( 1.0000+I*0.0000), ( 0.0000-I*1.0000), (-1.0000+I*0.0000), 
      ( 0.0000+I*1.0000), ( 1.0000+I*0.0000), ( 0.0000-I*1.0000), 
      (-1.0000+I*0.0000), ( 0.0000+I*1.0000), ( 1.0000+I*0.0000) };

/********************************************************/

  out1 = (float*) fftwf_malloc(sizeof(float)*16);
  out2 = (float*) fftwf_malloc(sizeof(float)*60);
  out3 = (float*) fftwf_malloc(sizeof(float)*80);
  out4 = (float*) fftwf_malloc(sizeof(float)*16);
  out5 = (float*) fftwf_malloc(sizeof(float)*32);
  out6 = (float*) fftwf_malloc(sizeof(float)*32);

  /********************************************************/

  in = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*9);
  for (j=0; j<9; j++) in[j] = x1[j];
  p = fftwf_plan_dft_c2r_1d(16,in,out1,FFTW_ESTIMATE);
  fftwf_execute(p);

  scrftd(1,1,x1,incx1,9,y1,incy1,16,n1,1,isign,1.0,aux1,1000,aux2,0);
  scrftd(0,1,x1,incx1,9,y1,incy1,16,n1,1,isign,1.0,aux1,1000,aux2,0);

  svecck(16,1,out1,y1,"fftwf_plan_dft_c2r_1d",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  for (j=0; j<30; j++) x2[j] = 0.0;
  for (j=0; j< 6; j++) x2[5*j] = 1.0;

  in = (fftwf_complex*)out2;
  for (j=0; j<30; j++) in[j] = x2[j];
  p = fftwf_plan_dft_c2r_2d(6,8,in,out2,FFTW_ESTIMATE);
  fftwf_execute_dft_c2r(p,in,out2);

  scrftd(1,2,x2,incx2,30,y2,incy2,60,n2,1,isign,1.0,aux1,1000,aux2,0);
  scrftd(0,2,x2,incx2,30,y2,incy2,60,n2,1,isign,1.0,aux1,1000,aux2,0);

  for (i=0; i<6; i++)
    {
      svecck(8,1,&out2[i*10],&y2[i*10],"fftw_plan_dft_c2r_2d",pass);
    }

  fftwf_destroy_plan(p);

  /********************************************************/

  for (j=0; j<40; j++) x3[j] = 0.0;
  x3[0] = -1.0;
  for (j=1; j< 5; j++) x3[j] = -x3[j-1];

  in = (fftwf_complex*)out3;
  for (j=0; j<40; j++) in[j] = x3[j];
  p = fftwf_plan_dft_c2r_3d(2,4,8,in,out3,FFTW_ESTIMATE);
  fftwf_execute(p);

  scrftd(1,3,x3,incx3,40,y3,incy3,80,n3,1,isign,1.0,aux1,1000,aux2,0);
  scrftd(0,3,x3,incx3,40,y3,incy3,80,n3,1,isign,1.0,aux1,1000,aux2,0);

  for (j=0; j<2; j++)
    {
      for (i=0; i<4; i++)
	{
	  svecck(8,1,&out3[i*10+j*40],&y3[i*10+j*40],
		 "fftw_plan_dft_c2r_3d",pass);
	}
    }

  fftwf_destroy_plan(p);

  /********************************************************/

  in = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*9);
  for (j=0; j<9; j++) in[j] = x1[j];
  p = fftwf_plan_dft_c2r(1,n1,in,out4,FFTW_ESTIMATE);
  fftwf_execute_dft_c2r(p,in,out4);

  scrftd(1,1,x1,incx1,9,y4,incy1,16,n1,1,isign,1.0,aux1,1000,aux2,0);
  scrftd(0,1,x1,incx1,9,y4,incy1,16,n1,1,isign,1.0,aux1,1000,aux2,0);

  svecck(16,1,out4,y4,"fftwf_plan_dft_c2r",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  in = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*18);
  for (j=0; j<18; j++) in[j] = x4[j];
  p = fftwf_plan_many_dft_c2r(1,n1,2,in,n1,1,9,
			 out5,n1,1,16,FFTW_ESTIMATE);
  fftwf_execute(p);

  scrftd(1,1,x4,incx1,9,y5,incy1,16,n1,2,isign,1.0,aux1,1000,aux2,0);
  scrftd(0,1,x4,incx1,9,y5,incy1,16,n1,2,isign,1.0,aux1,1000,aux2,0);

  svecck(32,1,out5,y5,"fftwf_plan_many_dft_c2r",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  in = (fftwf_complex*) fftwf_malloc(sizeof(fftwf_complex)*18);
  for (j=0; j<18; j++) in[j] = x4[j];
  p = fftwf_plan_guru_dft_c2r(1,dims,1,howmany_dims,in,out6,
			     FFTW_ESTIMATE);
  fftwf_execute_dft_c2r(p,in,out6);

  scrftd(1,1,x4,incx1,9,y6,incy1,16,n1,2,isign,1.0,aux1,1000,aux2,0);
  scrftd(0,1,x4,incx1,9,y6,incy1,16,n1,2,isign,1.0,aux1,1000,aux2,0);

  svecck(32,1,out6,y6,"fftwf_plan_guru_dft_c2r",pass);

  fftwf_destroy_plan(p);
  fftwf_free(in);

  /********************************************************/

  fftwf_free(out1);
  fftwf_free(out2);
  fftwf_free(out3);
  fftwf_free(out4);
  fftwf_free(out5);
  fftwf_free(out6);

  /********************************************************/

  return;
}

/**********************************************************
*     CHECK FUNCTIONS 
**********************************************************/

/***************************
*  LONG PRECISION COMPLEX  *
***************************/

void zvecck(n, incx, rans, rst, cname, pass)
int n, incx, *pass;
double complex *rans, *rst;
char *cname;
{
    int i;
    double tols;
    double complex c;
 
    tols = 10.e-3;
    for(i = 0; i < n * incx; i += incx) 
      {
        c = rst[i] - rans[i];
        if(cabs(c) > tols) 
	  {
            printf("%s  FAILED\n", cname);
            printf("RESULTS  %d  (%g  %g)  (%g  %g)\n", i, creal(rst[i]),
		   cimag(rst[i]), creal(rans[i]), cimag(rans[i]));
            ++*pass;
            return;
	  }
      }
}

/***************************
*  LONG PRECISION REAL     *
***************************/

void dvecck(n, incx, rans, rst, cname, pass)
int n, incx, *pass;
double *rans, *rst;
char *cname;
{
    int i;
    double tols;
    double c;
 
    tols = 10.e-3;
    for(i = 0; i < n * incx; i += incx) 
      {
        c = rst[i] - rans[i];
        if(fabs(c) > tols) 
	  {
            printf("%s  FAILED\n", cname);
            printf("RESULTS  %d  %g   %g  \n", i, rst[i], rans[i]);
            ++*pass;
            return;
	  }
      }
}

/**************************
* SHORT PRECISION COMPLEX *
**************************/

void cvecck(n, incx, rans, rst, cname, pass)
int n, incx, *pass;
float complex *rans, *rst;
char *cname;
{
    int i;
    float tols;
    float complex c;
 
    tols = 10.e-3;
    for(i = 0; i < n * incx; i += incx) 
      {
        c = rst[i] - rans[i];
        if(cabsf(c) > tols) 
	  {
            printf("%s  FAILED\n", cname);
            printf("RESULTS  %d  (%g  %g)  (%g  %g)\n", i, crealf(rst[i]),
		   cimagf(rst[i]), crealf(rans[i]), cimagf(rans[i]));
            ++*pass;
            return;
	  }
      }
}

/**************************
* SHORT PRECISION REAL    *
**************************/

void svecck(n, incx, rans, rst, cname, pass)
int n, incx, *pass;
float *rans, *rst;
char *cname;
{
    int i;
    float tols;
    float c;
 
    tols = 10.e-3;
    for(i = 0; i < n * incx; i += incx) 
      {
        c = rst[i] - rans[i];
        if(fabs(c) > tols) 
	  {
            printf("%s  FAILED\n", cname);
            printf("RESULTS  %d  %g   %g  \n", i, rst[i], rans[i]);
            ++*pass;
            return;
	  }
      }
}
