#!/bin/bash
##########################################################################
# Shell script to run the FFTW3 C Wrapper Installation Verification      #
# Program                                                                #
##########################################################################
# LICENSED MATERIALS - PROPERTY OF IBM                                   #
# "RESTRICTED MATERIALS OF IBM"                                          #
#                                                                        #
# 5765-G17                                                               #
# 5765-L51                                                               #
#                                                                        #
# COPYRIGHT IBM CORP. 2009, 2014.                                        #
#                                                                        #
# U.S. GOVERNMENT USERS RESTRICTED RIGHTS - USE, DUPLICATION             #
# OR DISCLOSURE RESTRICTED BY GSA ADP SCHEDULE CONTRACT WITH             #
# IBM CORP.                                                              #
##########################################################################
# Usage:                                                                 #
# fftw3ivp.c.sh                                                          #
#                                                                        #
# This script assumes the following:                                     #
# 1. fftw3_essl.h is in /usr/local/include                               #
# 2. The 32-bit pointer FFTW3 Wrapper library is in /usr/local/lib       #
# 3. The 64-bit pointer FFTW3 Wrapper library is in /usr/local/lib64     #
# 4. XL Fortran for Linux, V15.1 is installed in the default location    #
#    - or -                                                              #
#    XL Fortran for Linux, V14.1 is installed in the default location    #
#    - or -                                                              #
#    XL Fortran for Linux, V13.1 is installed in the default location    #
##########################################################################
export PATH=$PATH:/opt/ibm/xlC/13.1.0/bin
export PATH=$PATH:/opt/ibmcmp/vac/12.1/bin:/opt/ibmcmp/vacpp/12.1/bin
export PATH=$PATH:/opt/ibmcmp/vac/11.1/bin:/opt/ibmcmp/vacpp/11.1/bin
LIBDIR32=${LIBDIR32:=/usr/local/lib}
LIBDIR64=${LIBDIR64:=/usr/local/lib64}
INCLUDEDIR=${INCLUDEDIR:=/usr/local/include}
#
ESSLLIB=${ESSLLIB:=essl}
ESSLLIBDIR32=${ESSLLIBDIR32:=/usr/lib}
ESSLLIBDIR64=${ESSLLIBDIR64:=/usr/lib64}
#
XLF15LIBDIR32=${XLF15LIBDIR32:=/opt/ibm/xlf/15.1.0/lib}
XLF15LIBDIR64=${XLF15LIBDIR64:=/opt/ibm/xlf/15.1.0/lib64}
#
XLF14LIBDIR32=${XLF14LIBDIR32:=/opt/ibmcmp/xlf/14.1/lib}
XLF14LIBDIR64=${XLF14LIBDIR64:=/opt/ibmcmp/xlf/14.1/lib64}
#
XLF13LIBDIR32=${XLF13LIBDIR32:=/opt/ibmcmp/xlf/13.1/lib}
XLF13LIBDIR64=${XLF13LIBDIR64:=/opt/ibmcmp/xlf/13.1/lib64}
#
XLCMPLIB32=${XLCMPLIB32:=/opt/ibmcmp/lib}
XLCMPLIB64=${XLCMPLIB64:=/opt/ibmcmp/lib64}
#
libs="-lxlf90_r -lxlfmath -lxlomp_ser"
#
# Check for 32-bit pointer FFTW3 Wrapper library
#
if [ -s ${LIBDIR32}/libfftw3_essl.a ]
then
   cc_r -O -qmaxmem=-1 fftw3civp.c -o fftw3ivp.exe -I${INCLUDEDIR} \
        ${LIBDIR32}/libfftw3_essl.a -l${ESSLLIB} -L${ESSLLIBDIR32} \
        -R${ESSLLIBDIR32} $libs -L${XLF15LIBDIR32} \
        -L${XLF14LIBDIR32} -L${XLF13LIBDIR32} \
        -R${XLCMPLIB32}
   echo "Testing the 32-bit pointer FFTW3 Wrapper library"
   ./fftw3ivp.exe
   rm -f ./fftw3ivp.exe
fi
#
# Check for 64-bit pointer FFTW3 Wrapper library
#
if [ -s ${LIBDIR64}/libfftw3_essl.a ]
then
   cc_r -q64 -qmaxmem=-1 -O fftw3civp.c -o fftw3ivp.exe -I${INCLUDEDIR} \
        ${LIBDIR64}/libfftw3_essl.a -l${ESSLLIB} -L${ESSLLIBDIR64} \
        -R${ESSLLIBDIR64} $libs -L${XLF15LIBDIR64} \
        -L${XLF14LIBDIR64} -L${XLF13LIBDIR64} \
        -R${XLCMPLIB64}
   echo "Testing the 64-bit pointer FFTW3 Wrapper library"
   ./fftw3ivp.exe
   rm -f ./fftw3ivp.exe
fi
