********************************************************************************
*  FFTW3 Fortran Interface Subprograms Verification Program                    *
********************************************************************************
*                                                                              *
*    LICENSED MATERIALS - PROPERTY OF IBM                                      *
*                                                                              *
*    "RESTRICTED MATERIALS OF IBM"                                             *
*                                                                              *
*    5765-F82                                                                  *
*    5765-G17                                                                  *
*    5765-H25                                                                  *
*    5765-L51                                                                  *
*                                                                              *
*    COPYRIGHT IBM CORP. 2008, 2010                                            *
*    ALL RIGHTS RESERVED                                                       *
*                                                                              *
*    US GOVERNMENT USERS RESTRICTED RIGHTS - USE, DUPLICATION OR DISCLOSURE    *
*    RESTRICTED BY GSA ADP SCHEDULE CONTRACT WITH IBM CORP.                    *
*                                                                              *
********************************************************************************
*                                                                              *
*  MODULE NAME = fftw3ivp.f                                                    *
*                                                                              *
*  MODULE-DESCRIPTION :                                                        *
*    This program acts as the driver for the FFTW3 Fortran Interface           *
*    Subprograms Verification test.                                            *
*    This program calls all the FFTW3 Fortran interface subprograms for        *
*    testing.                                                                  *
*    It calls the FFTW interface subprogram and [DS]CFTD ([DS]CRFTD, [DS]RCFTD)*
*    for same problem and then compares the answers. An error message is given *
*    if the FFTW interface subprogram does not return the correct answer.      *
*    Return code 1 signifies a subprogram failure.                             *
*                                                                              *
*                                                                              *
*     NOTES =                                                                  *
*        DEPENDENCIES = (1) Requires ESSL library                              *
*                       (2) Requires FFTW3 Wrapper library                     *
*                       (3) Requires ESSL header file (essl.h)                 *
*                       (4) Requires FFTW3 Fortran header file (fftw3.f)       *
*                                                                              *
*        REVISIONS =                                                           *
*          05/2010 - Added real-complex and complex-real functions             *
********************************************************************************
********************************************************************************
*                                                                              *
*     SUBPROGRAMS CALLED BY MAIN :                                             *
*                                                                              *
*      DFFTW   =  CALL TO ALL FFTW INTERFACE FUNCTIONS FOR COMPLEX-TO-COMPLEX  *
*                 FFT WITH LONG PRECISION                                      *
*      DFFTWRC =  CALL TO ALL FFTW INTERFACE FUNCTIONS FOR REAL-TO-COMPLEX     *
*                 FFT WITH LONG PRECISION                                      *
*      DFFTWCR =  CALL TO ALL FFTW INTERFACE FUNCTIONS FOR COMPLEX-TO-REAL     *
*                 FFT WITH LONG PRECISION                                      *
*      SFFTW   =  CALL TO ALL FFTW INTERFACE FUNCTIONS FOR COMPLEX-TO-COMPLEX  *
*                 FFT WITH SHORT PRECISION                                     *
*      SFFTWRC =  CALL TO ALL FFTW INTERFACE FUNCTIONS FOR REAL-TO-COMPLEX     *
*                 FFT WITH SHORT PRECISION                                     *
*      SFFTWCR =  CALL TO ALL FFTW INTERFACE FUNCTIONS FOR COMPLEX-TO-REAL     *
*                 FFT WITH SHORT PRECISION                                     *
*                                                                              *
********************************************************************************
********************************************************************************
*                       MAIN PROGRAM                                           *
********************************************************************************

      PROGRAM MAIN
      IMPLICIT NONE
      INTEGER PASS,LUNIT
 
      PASS = 0
      LUNIT = 6

      CALL DFFTW(PASS)
      CALL SFFTW(PASS)

      CALL DFFTWRC(PASS)
      CALL SFFTWRC(PASS)

      CALL DFFTWCR(PASS)
      CALL SFFTWCR(PASS)

      IF (PASS.NE.0) THEN
         WRITE(LUNIT,100) PASS
 100     FORMAT(2X,'NUMBER OF FAILURES = ',I4)
         STOP 1
      ELSE
         WRITE (LUNIT,*) 'FFTW3 Fortran Wrapper ', 
     &                   'subprograms run successfully'
      ENDIF

      END

**********************************************************************

      SUBROUTINE DFFTW(PASS)
      IMPLICIT NONE
      INCLUDE 'fftw3.f'
      INTEGER PASS
      INTEGER I

      INTEGER*8 P
      COMPLEX*16 OUT1(8), OUT2(48), OUT3(64)
      COMPLEX*16 OUT4(8), OUT5(32), OUT6(32)
      INTEGER D_N(1), D_IS(1), D_OS(1)
      INTEGER HD_N(1), HD_IS(1), HD_OS(1)
      DATA D_N / 8 /
      DATA D_IS / 1 /
      DATA D_OS / 1 /
      DATA HD_N / 4 /
      DATA HD_IS / 8 /
      DATA HD_OS / 8 /

      COMPLEX*16 X1(8), X2(48), X3(64), X4(32)
      COMPLEX*16 Y1(8), Y2(48), Y3(64), Y4(8), Y5(32), Y6(32) 
      REAL*8  AUX1(1000), AUX2(2)
      INTEGER N1(1), N2(2), N3(3)
      INTEGER INCX1(1), INCX2(2), INCX3(3)
      INTEGER INCY1(1), INCY2(2), INCY3(3)
      INTEGER ISIGN(3)
      DATA N1 / 8 /
      DATA N2 / 8, 6 /
      DATA N3 / 8, 4, 2/
      DATA INCX1 / 1 /
      DATA INCX2 / 1, 8 /
      DATA INCX3 / 1, 8, 32 /
      DATA INCY1 / 1 /
      DATA INCY2 / 1, 8 /
      DATA INCY3 / 1, 8, 32 /
      DATA ISIGN / 1, 1, 1 /

**********************************************************************
*  INPUT DATA :                                  
**********************************************************************
 
      DATA X1 / 
     &  ( 1.0000, 0.0000), ( 0.7071, 0.7071), ( 0.0000, 1.0000),
     &  (-0.7071, 0.7071), (-1.0000, 0.0000), (-0.7071,-0.7071),
     &  ( 0.0000,-1.0000), ( 0.7071,-0.7071) /

      DATA X4 / 
     &  ( 1.0000, 0.0000), ( 1.0000, 0.0000), ( 1.0000, 0.0000), 
     &  ( 1.0000, 0.0000), ( 1.0000, 0.0000), ( 1.0000, 0.0000), 
     &  ( 1.0000, 0.0000), ( 1.0000, 0.0000), ( 1.0000, 0.0000),
     &  ( 0.7071, 0.7071), ( 0.0000, 1.0000), (-0.7071, 0.7071),
     &  (-1.0000, 0.0000), (-0.7071,-0.7071), ( 0.0000,-1.0000), 
     &  ( 0.7071,-0.7071), ( 1.0000, 0.0000), ( 0.0000, 1.0000), 
     &  (-1.0000, 0.0000), ( 0.0000,-1.0000), ( 1.0000, 0.0000),  
     &  ( 0.0000, 1.0000), (-1.0000, 0.0000), ( 0.0000,-1.0000),
     &  ( 1.0000, 0.0000), (-0.7071, 0.7071), ( 0.0000,-1.0000), 
     &  ( 0.7071, 0.7071), (-1.0000, 0.0000), ( 0.7071,-0.7071), 
     &  ( 0.0000, 1.0000), (-0.7071,-0.7071) /

**********************************************************************

      CALL DFFTW_PLAN_DFT_1D(P,8,X1,OUT1,FFTW_FORWARD,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE(P)

      CALL DCFTD(1,1,X1,INCX1,8,Y1,INCY1,8,
     &           N1,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DCFTD(0,1,X1,INCX1,8,Y1,INCY1,8,
     &           N1,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL ZVECCK(8,1,OUT1,Y1,'DFFTW_PLAN_DFT_1D       ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      DO I = 1, 48
         X2(I) = ( 0.0, 0.0)
      ENDDO
      DO I = 0, 5
         X2(1+I*8) = ( 1.0, 0.0)
      ENDDO

      CALL DFFTW_PLAN_DFT_2D(P,8,6,X2,OUT2,FFTW_FORWARD,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE_DFT(P,X2,OUT2)

      CALL DCFTD(1,2,X2,INCX2,48,Y2,INCY2,48,
     &           N2,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DCFTD(0,2,X2,INCX2,48,Y2,INCY2,48,
     &           N2,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL ZVECCK(48,1,OUT2,Y2,'DFFTW_PLAN_DFT_2D       ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      DO I = 1, 64
         X3(I) = ( 0.0, 0.0)
      ENDDO
      X3(1) = (-1.0, 0.0)
      DO I = 2, 8
         X3(I) = -X3(I-1)
      ENDDO

      CALL DFFTW_PLAN_DFT_3D(P,8,4,2,X3,OUT3,FFTW_FORWARD,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE(P)

      CALL DCFTD(1,3,X3,INCX3,64,Y3,INCY3,64,
     &           N3,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DCFTD(0,3,X3,INCX3,64,Y3,INCY3,64,
     &           N3,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL ZVECCK(64,1,OUT3,Y3,'DFFTW_PLAN_DFT_3D       ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL DFFTW_PLAN_DFT(P,1,N1,X1,OUT4,FFTW_FORWARD,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE_DFT(P,X1,OUT4)

      CALL DCFTD(1,1,X1,INCX1,8,Y4,INCY1,8,
     &           N1,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DCFTD(0,1,X1,INCX1,8,Y4,INCY1,8,
     &           N1,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL ZVECCK(8,1,OUT4,Y4,'DFFTW_PLAN_DFT          ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL DFFTW_PLAN_MANY_DFT(P,1,N1,4,X4,N1,1,8,
     &                         OUT5,N1,1,8,FFTW_FORWARD,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE(P)

      CALL DCFTD(1,1,X4,INCX1,8,Y5,INCY1,8,
     &           N1,4,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DCFTD(0,1,X4,INCX1,8,Y5,INCY1,8,
     &           N1,4,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL ZVECCK(32,1,OUT5,Y5,'DFFTW_PLAN_MANY_DFT     ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL DFFTW_PLAN_GURU_DFT(P,1,D_N,D_IS,D_OS,1,HD_N,HD_IS,HD_OS,
     &                         X4,OUT6,FFTW_FORWARD,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE_DFT(P,X4,OUT6)

      CALL DCFTD(1,1,X4,INCX1,8,Y6,INCY1,8,
     &           N1,4,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DCFTD(0,1,X4,INCX1,8,Y6,INCY1,8,
     &           N1,4,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL ZVECCK(32,1,OUT6,Y6,'DFFTW_PLAN_GURU_DFT     ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      END

**********************************************************************

      SUBROUTINE DFFTWRC(PASS)
      IMPLICIT NONE
      INCLUDE 'fftw3.f'
      INTEGER PASS
      INTEGER I

      INTEGER*8 P
      COMPLEX*16 OUT1(9), OUT2(30), OUT3(40)
      COMPLEX*16 OUT4(9), OUT5(36), OUT6(36)
      INTEGER D_N(1), D_IS(1), D_OS(1)
      INTEGER HD_N(1), HD_IS(1), HD_OS(1)
      DATA D_N / 16 /
      DATA D_IS / 1 /
      DATA D_OS / 1 /
      DATA HD_N / 4 /
      DATA HD_IS / 16 /
      DATA HD_OS / 9 /

      REAL*8 X1(16), X2(48), X3(64), X4(64)
      COMPLEX*16 Y1(9), Y2(30), Y3(40), Y4(9), Y5(36), Y6(36) 
      REAL*8  AUX1(1000), AUX2(2)
      INTEGER N1(1), N2(2), N3(3)
      INTEGER INCX1(1), INCX2(2), INCX3(3)
      INTEGER INCY1(1), INCY2(2), INCY3(3)
      INTEGER ISIGN(3)
      DATA N1 / 16 /
      DATA N2 / 8, 6 /
      DATA N3 / 8, 4, 2/
      DATA INCX1 / 1 /
      DATA INCX2 / 1, 8 /
      DATA INCX3 / 1, 8, 32 /
      DATA INCY1 / 1 /
      DATA INCY2 / 1, 5 /
      DATA INCY3 / 1, 5, 20 /
      DATA ISIGN / 1, 1, 1 /

**********************************************************************
*  INPUT DATA :                                  
**********************************************************************
 
      DATA X1 /  
     &  1.0000,  0.9239,  0.7071,  0.3827,  0.0000, -0.3827,  
     & -0.7071, -0.9239, -1.0000, -0.9239, -0.7071, -0.3827,
     &  0.0000,  0.3827,  0.7071,  0.9239  /   

      DATA X4 /  
     &  1.0000,  1.0000,  1.0000,  1.0000,  1.0000,  1.0000,
     &  1.0000,  1.0000,  1.0000,  1.0000,  1.0000,  1.0000,
     &  1.0000,  1.0000,  1.0000,  1.0000,  
     &  1.0000,  0.9239,  0.7071,  0.3827,  0.0000, -0.3827,  
     & -0.7071, -0.9239, -1.0000, -0.9239, -0.7071, -0.3827,
     &  0.0000,  0.3827,  0.7071,  0.9239,   
     &  1.0000,  0.7071,  0.0000, -0.7071, -1.0000, -0.7071,
     &  0.0000,  0.7071,  1.0000,  0.7071,  0.0000, -0.7071,
     & -1.0000, -0.7071,  0.0000,  0.7071,
     &  1.0000,  0.3827, -0.7071, -0.9239,  0.0000,  0.9239,
     &  0.7071, -0.3827, -1.0000, -0.3827,  0.7071,  0.9239,
     &  0.0000, -0.9239, -0.7071,  0.3827  /

**********************************************************************

      CALL DFFTW_PLAN_DFT_R2C_1D(P,16,X1,OUT1,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE(P)

      CALL DRCFTD(1,1,X1,INCX1,16,Y1,INCY1,9,
     &           N1,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DRCFTD(0,1,X1,INCX1,16,Y1,INCY1,9,
     &           N1,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL ZVECCK(9,1,OUT1,Y1,'DFFTW_PLAN_DFT_R2C_1D   ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      DO I = 1, 48
         X2(I) = 0.0
      ENDDO
      DO I = 0, 5
         X2(1+I*8) = 1.0
      ENDDO

      CALL DFFTW_PLAN_DFT_R2C_2D(P,8,6,X2,OUT2,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE_DFT_R2C(P,X2,OUT2)

      CALL DRCFTD(1,2,X2,INCX2,48,Y2,INCY2,30,
     &           N2,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DRCFTD(0,2,X2,INCX2,48,Y2,INCY2,30,
     &           N2,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL ZVECCK(30,1,OUT2,Y2,'DFFTW_PLAN_DFT_R2C_2D   ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      DO I = 1, 64
         X3(I) = 0.0
      ENDDO
      X3(1) = -1.0
      DO I = 2, 8
         X3(I) = -X3(I-1)
      ENDDO

      CALL DFFTW_PLAN_DFT_R2C_3D(P,8,4,2,X3,OUT3,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE(P)

      CALL DRCFTD(1,3,X3,INCX3,64,Y3,INCY3,40,
     &           N3,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DRCFTD(0,3,X3,INCX3,64,Y3,INCY3,40,
     &           N3,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL ZVECCK(40,1,OUT3,Y3,'DFFTW_PLAN_DFT_R2C_3D   ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL DFFTW_PLAN_DFT_R2C(P,1,N1,X1,OUT4,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE_DFT_R2C(P,X1,OUT4)

      CALL DRCFTD(1,1,X1,INCX1,16,Y4,INCY1,9,
     &           N1,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DRCFTD(0,1,X1,INCX1,16,Y4,INCY1,9,
     &           N1,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL ZVECCK(9,1,OUT4,Y4,'DFFTW_PLAN_DFT_R2C      ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL DFFTW_PLAN_MANY_DFT_R2C(P,1,N1,4,X4,N1,1,16,
     &                         OUT5,N1,1,9,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE(P)

      CALL DRCFTD(1,1,X4,INCX1,16,Y5,INCY1,9,
     &           N1,4,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DRCFTD(0,1,X4,INCX1,16,Y5,INCY1,9,
     &           N1,4,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL ZVECCK(36,1,OUT5,Y5,'DFFTW_PLAN_MANY_DFT_R2C ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL DFFTW_PLAN_GURU_DFT_R2C(P,1,D_N,D_IS,D_OS,1,HD_N,HD_IS,HD_OS,
     &                         X4,OUT6,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE_DFT_R2C(P,X4,OUT6)

      CALL DRCFTD(1,1,X4,INCX1,16,Y6,INCY1,9,
     &           N1,4,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DRCFTD(0,1,X4,INCX1,16,Y6,INCY1,9,
     &           N1,4,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL ZVECCK(36,1,OUT6,Y6,'DFFTW_PLAN_GURU_DFT_R2C ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      END

**********************************************************************

      SUBROUTINE DFFTWCR(PASS)
      IMPLICIT NONE
      INCLUDE 'fftw3.f'
      INTEGER PASS
      INTEGER I, J

      INTEGER*8 P
      REAL*8 OUT1(16), OUT2(60), OUT3(80)
      REAL*8 OUT4(16), OUT5(32), OUT6(32)
      INTEGER D_N(1), D_IS(1), D_OS(1)
      INTEGER HD_N(1), HD_IS(1), HD_OS(1)
      DATA D_N / 16 /
      DATA D_IS / 1 /
      DATA D_OS / 1 /
      DATA HD_N / 2 /
      DATA HD_IS / 9 /
      DATA HD_OS / 16 /

      COMPLEX*16 X1(9), X2(30), X3(40), X4(18), X22(30), X33(40)
      REAL*8 Y1(16), Y2(60), Y3(80), Y4(16), Y5(32), Y6(32) 
      REAL*8  AUX1(1000), AUX2(2)
      INTEGER N1(1), N2(2), N3(3)
      INTEGER INCX1(1), INCX2(2), INCX3(3)
      INTEGER INCY1(1), INCY2(2), INCY3(3)
      INTEGER ISIGN(3)
      DATA N1 / 16 /
      DATA N2 / 8, 6 /
      DATA N3 / 8, 4, 2/
      DATA INCX1 / 1 /
      DATA INCX2 / 1, 5 /
      DATA INCX3 / 1, 5, 20 /
      DATA INCY1 / 1 /
      DATA INCY2 / 1, 10 /
      DATA INCY3 / 1, 10, 40 /
      DATA ISIGN / -1, -1, -1 /
      EQUIVALENCE (X2, OUT2)
      EQUIVALENCE (X3, OUT3)

**********************************************************************
*  INPUT DATA :                                  
**********************************************************************
 
      DATA X1 / 
     &  ( 1.0000, 0.0000), ( 0.0000, 1.0000), (-1.0000, 0.0000), 
     &  ( 0.0000,-1.0000), ( 1.0000, 0.0000), ( 0.0000, 1.0000), 
     &  (-1.0000, 0.0000), ( 0.0000,-1.0000), ( 1.0000, 0.0000) /

      DATA X4 / 
     &  ( 1.0000, 0.0000), ( 0.0000, 1.0000), (-1.0000, 0.0000), 
     &  ( 0.0000,-1.0000), ( 1.0000, 0.0000), ( 0.0000, 1.0000), 
     &  (-1.0000, 0.0000), ( 0.0000,-1.0000), ( 1.0000, 0.0000),
     &  ( 1.0000, 0.0000), ( 0.0000,-1.0000), (-1.0000, 0.0000), 
     &  ( 0.0000, 1.0000), ( 1.0000, 0.0000), ( 0.0000,-1.0000), 
     &  (-1.0000, 0.0000), ( 0.0000, 1.0000), ( 1.0000, 0.0000) /


**********************************************************************

      CALL DFFTW_PLAN_DFT_C2R_1D(P,16,X1,OUT1,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE(P)

      CALL DCRFTD(1,1,X1,INCX1,9,Y1,INCY1,16,
     &           N1,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DCRFTD(0,1,X1,INCX1,9,Y1,INCY1,16,
     &           N1,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL DVECCK(16,1,OUT1,Y1,'DFFTW_PLAN_DFT_C2R_1D   ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      DO I = 1, 30
         X2(I) = ( 0.0, 0.0)
      ENDDO
      DO I = 0, 5
         X2(1+I*5) = ( 1.0, 0.0)
      ENDDO

      CALL DFFTW_PLAN_DFT_C2R_2D(P,8,6,X2,OUT2,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE_DFT_C2R(P,X2,OUT2)

      DO I = 1, 30
         X22(I) = ( 0.0, 0.0)
      ENDDO
      DO I = 0, 5
         X22(1+I*5) = ( 1.0, 0.0)
      ENDDO

      CALL DCRFTD(1,2,X22,INCX2,30,Y2,INCY2,60,
     &           N2,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DCRFTD(0,2,X22,INCX2,30,Y2,INCY2,60,
     &           N2,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      DO I=0,5
         CALL DVECCK(8,1,OUT2(1+I*10),Y2(1+I*10),
     &               'DFFTW_PLAN_DFT_C2R_2D   ',PASS)
      ENDDO

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      DO I = 1, 40
         X3(I) = ( 0.0, 0.0)
      ENDDO
      X3(1) = (-1.0, 0.0)
      DO I = 2, 5
         X3(I) = -X3(I-1)
      ENDDO

      CALL DFFTW_PLAN_DFT_C2R_3D(P,8,4,2,X3,OUT3,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE(P)

      DO I = 1, 40
         X33(I) = ( 0.0, 0.0)
      ENDDO
      X33(1) = (-1.0, 0.0)
      DO I = 2, 5
         X33(I) = -X33(I-1)
      ENDDO

      CALL DCRFTD(1,3,X33,INCX3,40,Y3,INCY3,80,
     &           N3,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DCRFTD(0,3,X33,INCX3,40,Y3,INCY3,80,
     &           N3,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      DO J=0,1
      DO I=0,3
         CALL DVECCK(8,1,OUT3(1+I*10+J*40),Y3(1+I*10+J*40),
     &              'DFFTW_PLAN_DFT_C2R_3D   ',PASS)
      ENDDO
      ENDDO

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL DFFTW_PLAN_DFT_C2R(P,1,N1,X1,OUT4,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE_DFT_C2R(P,X1,OUT4)

      CALL DCRFTD(1,1,X1,INCX1,9,Y4,INCY1,16,
     &           N1,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DCRFTD(0,1,X1,INCX1,9,Y4,INCY1,16,
     &           N1,1,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL DVECCK(16,1,OUT4,Y4,'DFFTW_PLAN_DFT_C2R      ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL DFFTW_PLAN_MANY_DFT_C2R(P,1,N1,2,X4,N1,1,9,
     &                         OUT5,N1,1,16,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE(P)

      CALL DCRFTD(1,1,X4,INCX1,9,Y5,INCY1,16,
     &           N1,2,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DCRFTD(0,1,X4,INCX1,9,Y5,INCY1,16,
     &           N1,2,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL DVECCK(32,1,OUT5,Y5,'DFFTW_PLAN_MANY_DFT_C2R ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL DFFTW_PLAN_GURU_DFT_C2R(P,1,D_N,D_IS,D_OS,1,HD_N,HD_IS,HD_OS,
     &                         X4,OUT6,FFTW_ESTIMATE)
      CALL DFFTW_EXECUTE_DFT_C2R(P,X4,OUT6)

      CALL DCRFTD(1,1,X4,INCX1,9,Y6,INCY1,16,
     &           N1,2,ISIGN,1.0D0,AUX1,1000,AUX2,0)
      CALL DCRFTD(0,1,X4,INCX1,9,Y6,INCY1,16,
     &           N1,2,ISIGN,1.0D0,AUX1,1000,AUX2,0)

      CALL DVECCK(32,1,OUT6,Y6,'DFFTW_PLAN_GURU_DFT_C2R ',PASS)

      CALL DFFTW_DESTROY_PLAN(P)

**********************************************************************

      END

**********************************************************************

      SUBROUTINE SFFTW(PASS)
      IMPLICIT NONE
      INCLUDE 'fftw3.f'
      INTEGER PASS
      INTEGER I

      INTEGER*8 P
      COMPLEX*8 OUT1(8), OUT2(48), OUT3(64)
      COMPLEX*8 OUT4(8), OUT5(32), OUT6(32)
      INTEGER D_N(1), D_IS(1), D_OS(1)
      INTEGER HD_N(1), HD_IS(1), HD_OS(1)
      DATA D_N / 8 /
      DATA D_IS / 1 /
      DATA D_OS / 1 /
      DATA HD_N / 4 /
      DATA HD_IS / 8 /
      DATA HD_OS / 8 /

      COMPLEX*8 X1(8), X2(48), X3(64), X4(32)
      COMPLEX*8 Y1(8), Y2(48), Y3(64), Y4(8), Y5(32), Y6(32) 
      REAL*8  AUX1(1000), AUX2(2)
      INTEGER N1(1), N2(2), N3(3)
      INTEGER INCX1(1), INCX2(2), INCX3(3)
      INTEGER INCY1(1), INCY2(2), INCY3(3)
      INTEGER ISIGN(3)
      DATA N1 / 8 /
      DATA N2 / 8, 6 /
      DATA N3 / 8, 4, 2/
      DATA INCX1 / 1 /
      DATA INCX2 / 1, 8 /
      DATA INCX3 / 1, 8, 32 /
      DATA INCY1 / 1 /
      DATA INCY2 / 1, 8 /
      DATA INCY3 / 1, 8, 32 /
      DATA ISIGN / 1, 1, 1 /

**********************************************************************
*  INPUT DATA :                                  
**********************************************************************
 
      DATA X1 / 
     &  ( 1.0000, 0.0000), ( 0.7071, 0.7071), ( 0.0000, 1.0000),
     &  (-0.7071, 0.7071), (-1.0000, 0.0000), (-0.7071,-0.7071),
     &  ( 0.0000,-1.0000), ( 0.7071,-0.7071) /

      DATA X4 / 
     &  ( 1.0000, 0.0000), ( 1.0000, 0.0000), ( 1.0000, 0.0000), 
     &  ( 1.0000, 0.0000), ( 1.0000, 0.0000), ( 1.0000, 0.0000), 
     &  ( 1.0000, 0.0000), ( 1.0000, 0.0000), ( 1.0000, 0.0000),
     &  ( 0.7071, 0.7071), ( 0.0000, 1.0000), (-0.7071, 0.7071),
     &  (-1.0000, 0.0000), (-0.7071,-0.7071), ( 0.0000,-1.0000), 
     &  ( 0.7071,-0.7071), ( 1.0000, 0.0000), ( 0.0000, 1.0000), 
     &  (-1.0000, 0.0000), ( 0.0000,-1.0000), ( 1.0000, 0.0000),  
     &  ( 0.0000, 1.0000), (-1.0000, 0.0000), ( 0.0000,-1.0000),
     &  ( 1.0000, 0.0000), (-0.7071, 0.7071), ( 0.0000,-1.0000), 
     &  ( 0.7071, 0.7071), (-1.0000, 0.0000), ( 0.7071,-0.7071), 
     &  ( 0.0000, 1.0000), (-0.7071,-0.7071) /

**********************************************************************

      CALL SFFTW_PLAN_DFT_1D(P,8,X1,OUT1,FFTW_FORWARD,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE(P)

      CALL SCFTD(1,1,X1,INCX1,8,Y1,INCY1,8,
     &           N1,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SCFTD(0,1,X1,INCX1,8,Y1,INCY1,8,
     &           N1,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL CVECCK(8,1,OUT1,Y1,'SFFTW_PLAN_DFT_1D       ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      DO I = 1, 48
         X2(I) = ( 0.0, 0.0)
      ENDDO
      DO I = 0, 5
         X2(1+I*8) = ( 1.0, 0.0)
      ENDDO

      CALL SFFTW_PLAN_DFT_2D(P,8,6,X2,OUT2,FFTW_FORWARD,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE_DFT(P,X2,OUT2)

      CALL SCFTD(1,2,X2,INCX2,48,Y2,INCY2,48,
     &           N2,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SCFTD(0,2,X2,INCX2,48,Y2,INCY2,48,
     &           N2,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL CVECCK(48,1,OUT2,Y2,'SFFTW_PLAN_DFT_2D       ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      DO I = 1, 64
         X3(I) = ( 0.0, 0.0)
      ENDDO
      X3(1) = (-1.0, 0.0)
      DO I = 2, 8
         X3(I) = -X3(I-1)
      ENDDO

      CALL SFFTW_PLAN_DFT_3D(P,8,4,2,X3,OUT3,FFTW_FORWARD,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE(P)

      CALL SCFTD(1,3,X3,INCX3,64,Y3,INCY3,64,
     &           N3,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SCFTD(0,3,X3,INCX3,64,Y3,INCY3,64,
     &           N3,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL CVECCK(64,1,OUT3,Y3,'SFFTW_PLAN_DFT_3D       ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL SFFTW_PLAN_DFT(P,1,N1,X1,OUT4,FFTW_FORWARD,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE_DFT(P,X1,OUT4)

      CALL SCFTD(1,1,X1,INCX1,8,Y4,INCY1,8,
     &           N1,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SCFTD(0,1,X1,INCX1,8,Y4,INCY1,8,
     &           N1,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL CVECCK(8,1,OUT4,Y4,'SFFTW_PLAN_DFT          ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL SFFTW_PLAN_MANY_DFT(P,1,N1,4,X4,N1,1,8,
     &                         OUT5,N1,1,8,FFTW_FORWARD,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE(P)

      CALL SCFTD(1,1,X4,INCX1,8,Y5,INCY1,8,
     &           N1,4,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SCFTD(0,1,X4,INCX1,8,Y5,INCY1,8,
     &           N1,4,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL CVECCK(32,1,OUT5,Y5,'SFFTW_PLAN_MANY_DFT     ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL SFFTW_PLAN_GURU_DFT(P,1,D_N,D_IS,D_OS,1,HD_N,HD_IS,HD_OS,
     &                         X4,OUT6,FFTW_FORWARD,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE_DFT(P,X4,OUT6)

      CALL SCFTD(1,1,X4,INCX1,8,Y6,INCY1,8,
     &           N1,4,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SCFTD(0,1,X4,INCX1,8,Y6,INCY1,8,
     &           N1,4,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL CVECCK(32,1,OUT6,Y6,'SFFTW_PLAN_GURU_DFT     ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      END

**********************************************************************

      SUBROUTINE SFFTWRC(PASS)
      IMPLICIT NONE
      INCLUDE 'fftw3.f'
      INTEGER PASS
      INTEGER I

      INTEGER*8 P
      COMPLEX*8 OUT1(9), OUT2(30), OUT3(40)
      COMPLEX*8 OUT4(9), OUT5(36), OUT6(36)
      INTEGER D_N(1), D_IS(1), D_OS(1)
      INTEGER HD_N(1), HD_IS(1), HD_OS(1)
      DATA D_N / 16 /
      DATA D_IS / 1 /
      DATA D_OS / 1 /
      DATA HD_N / 4 /
      DATA HD_IS / 16 /
      DATA HD_OS / 9 /

      REAL*4 X1(16), X2(48), X3(64), X4(64)
      COMPLEX*8  Y1(9), Y2(30), Y3(40), Y4(9), Y5(36), Y6(36) 
      REAL*8  AUX1(1000), AUX2(2)
      INTEGER N1(1), N2(2), N3(3)
      INTEGER INCX1(1), INCX2(2), INCX3(3)
      INTEGER INCY1(1), INCY2(2), INCY3(3)
      INTEGER ISIGN(3)
      DATA N1 / 16 /
      DATA N2 / 8, 6 /
      DATA N3 / 8, 4, 2/
      DATA INCX1 / 1 /
      DATA INCX2 / 1, 8 /
      DATA INCX3 / 1, 8, 32 /
      DATA INCY1 / 1 /
      DATA INCY2 / 1, 5 /
      DATA INCY3 / 1, 5, 20 /
      DATA ISIGN / 1, 1, 1 /

**********************************************************************
*  INPUT DATA :                                  
**********************************************************************
 
      DATA X1 /  
     &  1.0000,  0.9239,  0.7071,  0.3827,  0.0000, -0.3827,  
     & -0.7071, -0.9239, -1.0000, -0.9239, -0.7071, -0.3827,
     &  0.0000,  0.3827,  0.7071,  0.9239  /   

      DATA X4 /  
     &  1.0000,  1.0000,  1.0000,  1.0000,  1.0000,  1.0000,
     &  1.0000,  1.0000,  1.0000,  1.0000,  1.0000,  1.0000,
     &  1.0000,  1.0000,  1.0000,  1.0000,  
     &  1.0000,  0.9239,  0.7071,  0.3827,  0.0000, -0.3827,  
     & -0.7071, -0.9239, -1.0000, -0.9239, -0.7071, -0.3827,
     &  0.0000,  0.3827,  0.7071,  0.9239,   
     &  1.0000,  0.7071,  0.0000, -0.7071, -1.0000, -0.7071,
     &  0.0000,  0.7071,  1.0000,  0.7071,  0.0000, -0.7071,
     & -1.0000, -0.7071,  0.0000,  0.7071,
     &  1.0000,  0.3827, -0.7071, -0.9239,  0.0000,  0.9239,
     &  0.7071, -0.3827, -1.0000, -0.3827,  0.7071,  0.9239,
     &  0.0000, -0.9239, -0.7071,  0.3827  /

**********************************************************************

      CALL SFFTW_PLAN_DFT_R2C_1D(P,16,X1,OUT1,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE(P)

      CALL SRCFTD(1,1,X1,INCX1,16,Y1,INCY1,9,
     &           N1,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SRCFTD(0,1,X1,INCX1,16,Y1,INCY1,9,
     &           N1,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL CVECCK(9,1,OUT1,Y1,'SFFTW_PLAN_DFT_R2C_1D   ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      DO I = 1, 48
         X2(I) = 0.0
      ENDDO
      DO I = 0, 5
         X2(1+I*8) = 1.0
      ENDDO

      CALL SFFTW_PLAN_DFT_R2C_2D(P,8,6,X2,OUT2,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE_DFT_R2C(P,X2,OUT2)

      CALL SRCFTD(1,2,X2,INCX2,48,Y2,INCY2,30,
     &           N2,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SRCFTD(0,2,X2,INCX2,48,Y2,INCY2,30,
     &           N2,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL CVECCK(30,1,OUT2,Y2,'SFFTW_PLAN_DFT_R2C_2D   ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      DO I = 1, 64
         X3(I) = 0.0
      ENDDO
      X3(1) = -1.0
      DO I = 2, 8
         X3(I) = -X3(I-1)
      ENDDO

      CALL SFFTW_PLAN_DFT_R2C_3D(P,8,4,2,X3,OUT3,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE(P)

      CALL SRCFTD(1,3,X3,INCX3,64,Y3,INCY3,40,
     &           N3,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SRCFTD(0,3,X3,INCX3,64,Y3,INCY3,40,
     &           N3,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL CVECCK(40,1,OUT3,Y3,'SFFTW_PLAN_DFT_R2C_3D   ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL SFFTW_PLAN_DFT_R2C(P,1,N1,X1,OUT4,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE_DFT_R2C(P,X1,OUT4)

      CALL SRCFTD(1,1,X1,INCX1,16,Y4,INCY1,9,
     &           N1,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SRCFTD(0,1,X1,INCX1,16,Y4,INCY1,9,
     &           N1,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL CVECCK(9,1,OUT4,Y4,'SFFTW_PLAN_DFT_R2C      ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL SFFTW_PLAN_MANY_DFT_R2C(P,1,N1,4,X4,N1,1,16,
     &                         OUT5,N1,1,9,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE(P)

      CALL SRCFTD(1,1,X4,INCX1,16,Y5,INCY1,9,
     &           N1,4,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SRCFTD(0,1,X4,INCX1,16,Y5,INCY1,9,
     &           N1,4,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL CVECCK(36,1,OUT5,Y5,'SFFTW_PLAN_MANY_DFT_R2C ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL SFFTW_PLAN_GURU_DFT_R2C(P,1,D_N,D_IS,D_OS,1,HD_N,HD_IS,HD_OS,
     &                         X4,OUT6,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE_DFT_R2C(P,X4,OUT6)

      CALL SRCFTD(1,1,X4,INCX1,16,Y6,INCY1,9,
     &           N1,4,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SRCFTD(0,1,X4,INCX1,16,Y6,INCY1,9,
     &           N1,4,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL CVECCK(36,1,OUT6,Y6,'SFFTW_PLAN_GURU_DFT_R2C ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      END

**********************************************************************

      SUBROUTINE SFFTWCR(PASS)
      IMPLICIT NONE
      INCLUDE 'fftw3.f'
      INTEGER PASS
      INTEGER I, J

      INTEGER*8 P
      REAL*4 OUT1(16), OUT2(60), OUT3(80)
      REAL*4 OUT4(16), OUT5(32), OUT6(32)
      INTEGER D_N(1), D_IS(1), D_OS(1)
      INTEGER HD_N(1), HD_IS(1), HD_OS(1)
      DATA D_N / 16 /
      DATA D_IS / 1 /
      DATA D_OS / 1 /
      DATA HD_N / 2 /
      DATA HD_IS / 9 /
      DATA HD_OS / 16 /

      COMPLEX*8 X1(9), X2(30), X3(40), X4(18), X22(30), X33(40)
      REAL*4 Y1(16), Y2(60), Y3(80), Y4(16), Y5(32), Y6(32) 
      REAL*8  AUX1(1000), AUX2(2)
      INTEGER N1(1), N2(2), N3(3)
      INTEGER INCX1(1), INCX2(2), INCX3(3)
      INTEGER INCY1(1), INCY2(2), INCY3(3)
      INTEGER ISIGN(3)
      DATA N1 / 16 /
      DATA N2 / 8, 6 /
      DATA N3 / 8, 4, 2/
      DATA INCX1 / 1 /
      DATA INCX2 / 1, 5 /
      DATA INCX3 / 1, 5, 20 /
      DATA INCY1 / 1 /
      DATA INCY2 / 1, 10 /
      DATA INCY3 / 1, 10, 40 /
      DATA ISIGN / -1, -1, -1 /
      EQUIVALENCE (X2, OUT2)
      EQUIVALENCE (X3, OUT3)

**********************************************************************
*  INPUT DATA :                                  
**********************************************************************
 
      DATA X1 / 
     &  ( 1.0000, 0.0000), ( 0.0000, 1.0000), (-1.0000, 0.0000), 
     &  ( 0.0000,-1.0000), ( 1.0000, 0.0000), ( 0.0000, 1.0000), 
     &  (-1.0000, 0.0000), ( 0.0000,-1.0000), ( 1.0000, 0.0000) /

      DATA X4 / 
     &  ( 1.0000, 0.0000), ( 0.0000, 1.0000), (-1.0000, 0.0000), 
     &  ( 0.0000,-1.0000), ( 1.0000, 0.0000), ( 0.0000, 1.0000), 
     &  (-1.0000, 0.0000), ( 0.0000,-1.0000), ( 1.0000, 0.0000),
     &  ( 1.0000, 0.0000), ( 0.0000,-1.0000), (-1.0000, 0.0000), 
     &  ( 0.0000, 1.0000), ( 1.0000, 0.0000), ( 0.0000,-1.0000), 
     &  (-1.0000, 0.0000), ( 0.0000, 1.0000), ( 1.0000, 0.0000) /


**********************************************************************

      CALL SFFTW_PLAN_DFT_C2R_1D(P,16,X1,OUT1,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE(P)

      CALL SCRFTD(1,1,X1,INCX1,9,Y1,INCY1,16,
     &           N1,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SCRFTD(0,1,X1,INCX1,9,Y1,INCY1,16,
     &           N1,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL SVECCK(16,1,OUT1,Y1,'SFFTW_PLAN_DFT_C2R_1D   ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      DO I = 1, 30
         X2(I) = ( 0.0, 0.0)
      ENDDO
      DO I = 0, 5
         X2(1+I*5) = ( 1.0, 0.0)
      ENDDO

      CALL SFFTW_PLAN_DFT_C2R_2D(P,8,6,X2,OUT2,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE_DFT_C2R(P,X2,OUT2)

      DO I = 1, 30
         X22(I) = ( 0.0, 0.0)
      ENDDO
      DO I = 0, 5
         X22(1+I*5) = ( 1.0, 0.0)
      ENDDO

      CALL SCRFTD(1,2,X22,INCX2,30,Y2,INCY2,60,
     &           N2,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SCRFTD(0,2,X22,INCX2,30,Y2,INCY2,60,
     &           N2,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      DO I=0,5
         CALL SVECCK(8,1,OUT2(1+I*10),Y2(1+I*10),
     &               'SFFTW_PLAN_DFT_C2R_2D   ',PASS)
      ENDDO

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      DO I = 1, 40
         X3(I) = ( 0.0, 0.0)
      ENDDO
      X3(1) = (-1.0, 0.0)
      DO I = 2, 5
         X3(I) = -X3(I-1)
      ENDDO

      CALL SFFTW_PLAN_DFT_C2R_3D(P,8,4,2,X3,OUT3,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE(P)

      DO I = 1, 40
         X33(I) = ( 0.0, 0.0)
      ENDDO
      X33(1) = (-1.0, 0.0)
      DO I = 2, 5
         X33(I) = -X33(I-1)
      ENDDO

      CALL SCRFTD(1,3,X33,INCX3,40,Y3,INCY3,80,
     &           N3,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SCRFTD(0,3,X33,INCX3,40,Y3,INCY3,80,
     &           N3,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      DO J=0,1
      DO I=0,3
         CALL SVECCK(8,1,OUT3(1+I*10+J*40),Y3(1+I*10+J*40),
     &              'SFFTW_PLAN_DFT_C2R_3D   ',PASS)
      ENDDO
      ENDDO

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL SFFTW_PLAN_DFT_C2R(P,1,N1,X1,OUT4,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE_DFT_C2R(P,X1,OUT4)

      CALL SCRFTD(1,1,X1,INCX1,9,Y4,INCY1,16,
     &           N1,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SCRFTD(0,1,X1,INCX1,9,Y4,INCY1,16,
     &           N1,1,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL SVECCK(16,1,OUT4,Y4,'SFFTW_PLAN_DFT_C2R      ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL SFFTW_PLAN_MANY_DFT_C2R(P,1,N1,2,X4,N1,1,9,
     &                         OUT5,N1,1,16,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE(P)

      CALL SCRFTD(1,1,X4,INCX1,9,Y5,INCY1,16,
     &           N1,2,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SCRFTD(0,1,X4,INCX1,9,Y5,INCY1,16,
     &           N1,2,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL SVECCK(32,1,OUT5,Y5,'SFFTW_PLAN_MANY_DFT_C2R ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      CALL SFFTW_PLAN_GURU_DFT_C2R(P,1,D_N,D_IS,D_OS,1,HD_N,HD_IS,HD_OS,
     &                         X4,OUT6,FFTW_ESTIMATE)
      CALL SFFTW_EXECUTE_DFT_C2R(P,X4,OUT6)

      CALL SCRFTD(1,1,X4,INCX1,9,Y6,INCY1,16,
     &           N1,2,ISIGN,1.0E0,AUX1,1000,AUX2,0)
      CALL SCRFTD(0,1,X4,INCX1,9,Y6,INCY1,16,
     &           N1,2,ISIGN,1.0E0,AUX1,1000,AUX2,0)

      CALL SVECCK(32,1,OUT6,Y6,'SFFTW_PLAN_GURU_DFT_C2R ',PASS)

      CALL SFFTW_DESTROY_PLAN(P)

**********************************************************************

      END


**********************************************************************
*     CHECK SUBROUTINES 
**********************************************************************

****************************
*  LONG PRECISION COMPLEX  *
****************************
 
      SUBROUTINE ZVECCK(N,INCX,RANS,RST,CNAME,PASS)
      INTEGER PASS
      INTEGER   N,I,INCX
      COMPLEX*16 RANS(*),RST(*)
      REAL*8 TOLS,CDABS
      CHARACTER*24 CNAME
      TOLS = 10D-3
      DO I = 1,N*INCX,INCX
         IF (CDABS(RST(I)-RANS(I)).GT.TOLS) THEN
             WRITE(6,100) CNAME
             WRITE(6,*) 'RESULTS ',I,RST(I),RANS(I)
             PASS = PASS + 1
             RETURN
         ENDIF
      ENDDO
 100  FORMAT(1X,A24,2X,'FAILED')
      RETURN
      END

****************************
*  LONG PRECISION REAL     *
****************************
 
      SUBROUTINE DVECCK(N,INCX,RANS,RST,CNAME,PASS)
      INTEGER PASS
      INTEGER   N,I,INCX
      REAL*8 RANS(*),RST(*)
      REAL*8 TOLS,DABS
      CHARACTER*24 CNAME
      TOLS = 10D-3
      DO I = 1,N*INCX,INCX
         IF (DABS(RST(I)-RANS(I)).GT.TOLS) THEN
             WRITE(6,100) CNAME
             WRITE(6,*) 'RESULTS ',I,RST(I),RANS(I)
             PASS = PASS + 1
             RETURN
         ENDIF
      ENDDO
 100  FORMAT(1X,A24,2X,'FAILED')
      RETURN
      END


*****************************
*  SHORT PRECISION COMPLEX  *
*****************************
 
      SUBROUTINE CVECCK(N,INCX,RANS,RST,CNAME,PASS)
      INTEGER PASS
      INTEGER   N,I,INCX
      COMPLEX*8 RANS(*),RST(*)
      REAL*4 TOLS,CABS
      CHARACTER*24 CNAME
      TOLS = 10E-3
      DO I = 1,N*INCX,INCX
         IF (CABS(RST(I)-RANS(I)).GT.TOLS) THEN
             WRITE(6,100) CNAME
             WRITE(6,*) 'RESULTS ',I,RST(I),RANS(I)
             PASS = PASS + 1
             RETURN
         ENDIF
      ENDDO
 100  FORMAT(1X,A24,2X,'FAILED')
      RETURN
      END

*****************************
*  SHORT PRECISION REAL     *
*****************************
 
      SUBROUTINE SVECCK(N,INCX,RANS,RST,CNAME,PASS)
      INTEGER PASS
      INTEGER   N,I,INCX
      REAL*4 RANS(*),RST(*)
      REAL*4 TOLS,ABS
      CHARACTER*24 CNAME
      TOLS = 10E-3
      DO I = 1,N*INCX,INCX
         IF (ABS(RST(I)-RANS(I)).GT.TOLS) THEN
             WRITE(6,100) CNAME
             WRITE(6,*) 'RESULTS ',I,RST(I),RANS(I)
             PASS = PASS + 1
             RETURN
         ENDIF
      ENDDO
 100  FORMAT(1X,A24,2X,'FAILED')
      RETURN
      END
