#!/bin/bash
##########################################################################
# Shell script to run the FFTW3 Fortran Wrapper Installation Verification#
# Program                                                                #
##########################################################################
# LICENSED MATERIALS - PROPERTY OF IBM                                   #
# "RESTRICTED MATERIALS OF IBM"                                          #
#                                                                        #
# 5765-G17                                                               #
# 5765-L51                                                               #
#                                                                        #
# COPYRIGHT IBM CORP. 2009, 2014.                                        #
#                                                                        #
# U.S. GOVERNMENT USERS RESTRICTED RIGHTS - USE, DUPLICATION             #
# OR DISCLOSURE RESTRICTED BY GSA ADP SCHEDULE CONTRACT WITH             #
# IBM CORP.                                                              #
##########################################################################
# Usage:                                                                 #
# fftw3ivp.f.sh                                                          #
#                                                                        #
# This script assumes the following:                                     #
# 1. fftw3.f is in /usr/local/include                                    #
# 2. The 32-bit pointer FFTW3 Wrapper library is in /usr/local/lib       #
# 3. The 64-bit pointer FFTW3 Wrapper library is in /usr/local/lib64     #
##########################################################################
export PATH=$PATH:/opt/ibm/xlf/15.1.0/bin
export PATH=$PATH:/opt/ibmcmp/xlf/14.1/bin:/opt/ibmcmp/xlf/13.1/bin
LIBDIR32=${LIBDIR32:=/usr/local/lib}
LIBDIR64=${LIBDIR64:=/usr/local/lib64}
INCLUDEDIR=${INCLUDEDIR:=/usr/local/include}
#
ESSLLIB=${ESSLLIB:=essl}
ESSLLIBDIR32=${ESSLLIBDIR32:=/usr/lib}
ESSLLIBDIR64=${ESSLLIBDIR64:=/usr/lib64}
#
# Check for 32-bit pointer FFTW3 Wrapper Library
#
if [ -s ${LIBDIR32}/libfftw3_essl.a ]
then
   xlf_r -O fftw3ivp.f -o fftw3ivp.exe -qnosave -I${INCLUDEDIR} \
         ${LIBDIR32}/libfftw3_essl.a -l${ESSLLIB} -L${ESSLLIBDIR32} \
         -R${ESSLLIBDIR32}
   echo "Testing the 32-bit pointer FFTW3 Wrapper library"
   ./fftw3ivp.exe
   rm -f ./fftw3ivp.exe
fi
#
# Check for 64-bit pointer FFTW3 Wrapper Library
#
if [ -s ${LIBDIR64}/libfftw3_essl.a ]
then
   xlf_r -q64 -O fftw3ivp.f -o fftw3ivp.exe -qnosave -I${INCLUDEDIR} \
         ${LIBDIR64}/libfftw3_essl.a -l${ESSLLIB} -L${ESSLLIBDIR64} \
         -R${ESSLLIBDIR64}
   echo "Testing the 64-bit pointer FFTW3 Wrapper library"
   ./fftw3ivp.exe
   rm -f ./fftw3ivp.exe
fi
