/*******************************************************************************
*  ESSL         C Installation Verification Program                            *
********************************************************************************
*                                                                              *
*    LICENSED MATERIALS - PROPERTY OF IBM                                      *
*                                                                              *
*    "RESTRICTED MATERIALS OF IBM"                                             *
*                                                                              *
*    5765-042                                                                  *
*    5765-C42                                                                  *
*    5765-F82                                                                  *
*    5765-G17                                                                  *
*    5765-H25                                                                  *
*    5765-L51                                                                  *
*                                                                              *
*    (C) COPYRIGHT IBM CORP. 1991, 2013.                                       *
*                                                                              *
*    US Government Users Restricted Rights - Use, duplication or disclosure    *
*    restricted by GSA ADP Schedule Contract with IBM Corp.                    *
*                                                                              *
********************************************************************************
*                                                                              *
*     MODULE NAME = esvcivp.c                                                  *
*                                                                              *
*     MODULE-DESCRIPTION :                                                     *
*       This program acts as the driver for the Installation                   *
*       Vertification test.                                                    *
*       This program calls all the user-callable ESSL subprograms for          *
*       testing.  One of the examples for each subprogram from the             *
*       ESSL Guide and Reference is used.  An error message                    *
*       is given if the subprogram does not return the correct answer.         *
*       Return code 8 signifies a subprogram failure.                          *
*       (Some subprograms have been modified so that less memory               *
*        is used.)                                                             *
*                                                                              *
*                                                                              *
*     NOTES =                                                                  *
*        DEPENDENCIES = (1) REQUIRES ESSL LIBRARY                              *
*                                                                              *
*        REVISIONS =                                                           *
*                    ESSL Version 3.1.0.0                       12/97          *
*                    ESSL Version 3.1.1.0                       10/98          *
*                    ESSL Version 3.1.2.0                       10/99          *
*                    ESSL Version 3.2.0.0                       07/00          *
*                    ESSL Version 3.3.0.0                       12/01          *
*                    ESSL Version 4.1.0.0                       07/03          *
*                    ESSL Version 4.2.0.0                       10/04          *
*                    ESSL Version 4.3.0.0                       08/07          *
*                    ESSL Version 4.4.0.0                       11/08          *
*                    ESSL Version 5.1.0.0                       06/10          *
*                    ESSL Version 5.2.0.0                       02/13          *
********************************************************************************
********************************************************************************
*                                                                              *
*     SUBPROGRAMS CALLED BY MAIN :                                             *
*                                                                              *
*       LIALVS  =  CALL TO ALL ESSL LINEAR ALGEBRA SUBROUTINES                 *
*       LIALMV  =  CALL TO ALL ESSL LINEAR ALGEBRA SUBROUTINES                 *
*       MAOPS   =  CALL TO ALL ESSL MATRIX OPERATION SUBROUTINES               *
*       LIEQS   =  CALL TO ALL ESSL LINEAR ALGEBRAIC EQUATIONS                 *
*       EIGNS   =  CALL TO ALL ESSL EIGENSYSTEM ANALYSIS SUBROUTINES           *
*       SIGNS   =  CALL TO ALL ESSL SIGNAL PROCESSING SUBROUTINES              *
*       SORTS   =  CALL TO ALL ESSL SORTING AND SEACHING SUBROUTINES           *
*       INPOS   =  CALL TO ALL ESSL INTERPOLATION SUBROUTINES                  *
*       QUADS   =  CALL TO ALL ESSL NUMERICAL QUADRATURE SUBROUTINES           *
*       RANDS   =  CALL TO ALL ESSL RANDOM NUMBER GENERATION                   *
*       UTILS   =  CALL TO ALL ESSL UTILITY SUBROUTINES                        *
*       PARS    =  CALL TO ALL ESSL PARALLEL SUBROUTINES                       *
*                                                                              *
********************************************************************************
********************************************************************************
*                       MAIN PROGRAM                                           *
********************************************************************************/
#include <stdio.h>
#include <math.h>
#include "essl.h"
 
#define CSET(x, a, b)   (RE(x) = a, IM(x) = b)
 
void lialvs(), lialmv(), maops(), lieqs(), eigns(), signs(), sorts(), inpos();
void quads(), rands(), utils(), pars();
void intchk(), svecck(), sverck(), cvecck(), dvecck(), dverck(), zvecck();
void imtchk(), smtchk(), dmtchk(), cmtchk(), zmtchk(),ieschk();
void smtckd(), dmtckd();
 
/* The following temporary variables are used to create fortran linkage */
 
_ESVINT  __esvi1,__esvi2,__esvi3,__esvi4;
 
void  rspwzr(_ESVINT *,  _ESVINT *,  _ESVINT *, _ESVINT *, void *, double *,
             void *, double *, double *);
 
#define rspwzr(i1,i2,i3,i4,x1,x2,x3,x4,x5)                                  \
              (__esvi1=i1,__esvi2 =i2,__esvi3=i3, __esvi4=i4,               \
               rspwzr (&__esvi1,&__esvi2,&__esvi3,&__esvi4,x1,x2,x3,x4,x5))
main()
{
    _ESVINT pass;
    int rc;
    pass = 0;
    lialvs(&pass);
    lialmv(&pass);
    maops(&pass);
    lieqs(&pass);
    eigns(&pass);
    signs(&pass);
    sorts(&pass);
    inpos(&pass);
    rands(&pass);
    quads(&pass);
    utils(&pass);
    pars(&pass);
    if(pass == 0) {
        printf(" ESSL installed successfully\n");
        rc = 0;
    }
    else {
        printf(" NUMBER OF FAILURES = %4d\n", pass);
        rc = 8;
    }
 return(rc);
 
}
 
/*********************************************************************
*                    SUBROUTINE:  LIALSVS                            *
*********************************************************************/
 
void lialvs(pass)
_ESVINT *pass;
{
/**********************************************
*  VARIABLES OF LINEAR ALGEBRA SUBROUTINES :  *
***********************************************
*                          *
****************************
*  (VECTOR-SCALAR LINEAR)  *
***************************/
 
    _ESVINT    max[1], min[1];
    float  ssum[1], csum[1], sy7[9];
    double dsum[1], zsum[1], dy7[9];
    cmplx  cy7[7];
    dcmplx zy7[7];
    float  st[1];
    double dt[1];
    cmplx  ct[1];
    dcmplx zt[1];
    float  ss10[3];
    double ds10[3];
    cmplx  cs13[1];
    float  cc13[1];
    dcmplx zs13[1];
    double zc13[1];
    float  sy17[9];
    double dy17[9];
    cmplx  cy17[3];
    dcmplx zy17[3];
    float  sz18[5];
    double dz18[5];
    cmplx  cz18[3];
    dcmplx zz18[3];
    float  sa1n[5];
    double da1n[5];
    cmplx  ca1n[3];
    dcmplx za1n[3];
 
    float  ss2n[5];
    double ds2n[5];
    cmplx  cs2n[3];
    dcmplx zs2n[3];
 
    float  sm3n[5];
    double dm3n[5];
    cmplx  cm3n[3];
    dcmplx zm3n[3];
    cmplx  cgx5n[3];
    dcmplx zgx5n[3];
    cmplx  cgzx6n[3];
    dcmplx zgzx6n[3];
    cmplx  c1n;
    dcmplx dc1n;
    cmplx  cdotc7n[1];
    dcmplx zdotc7n[1];
    cmplx  cdotu8n[1];
    dcmplx zdotu8n[1];
 
/*********************************
*  (SPARSE VECTOR-SCALAR LINEAR) *
*********************************/
 
    float  sx20[5];
    double dx20[5];
 
/*****************************************************
*  EXPECTED VALUES FOR LINEAR ALGEBRA SUBPROGRAMS :  *
*****************************************************/
 
    _ESVINT    ires1[1]  = { 6 };
    _ESVINT    ires2[1]  = { 3 };
    _ESVINT    ires3[1]  = { 4 };
    _ESVINT    ires4[1]  = { 3 };
    _ESVINT    ires11[1] = { 2 };
    float  sres5[1] = { 28. };
    double dres5[1] = { 28. };
    float  cres5[1] = { 55. };
    double zres5[1] = { 55. };
    float  sres6[9] = { 3., 0., 5., 0., 7., 0., 9., 0., 11. };
    double dres6[9] = { 3., 0., 5., 0., 7., 0., 9., 0., 11. };
    float  sres7[9] = { 1., 0., 2., 0., 3., 0., 4., 0.,  5. };
    double dres7[9] = { 1., 0., 2., 0., 3., 0., 4., 0.,  5. };
    cmplx  cres6[5] = { { -3., 8. }, { 0., 0. }, { 4., 8. }, { 0., 0. },
                        { -4., 23. } };
    dcmplx zres6[5] = { { -3., 8. }, { 0., 0. }, { 4., 8. }, { 0., 0. },
                        { -4., 23. } };
    cmplx  cres7[7] = { { 1., 1. }, { 0., 0. }, { 2., 2. }, { 0., 0. },
                        { 3., 3. }, { 0., 0. }, { 4., 4. } };
    dcmplx zres7[7] = { { 1., 1. }, { 0., 0. }, { 2., 2. }, { 0., 0. },
                        { 3., 3. }, { 0., 0. }, { 4., 4. } };
    float  sres8[1] = { 5. };
    double dres8[1] = { 5. };
    cmplx  cr8u[1] = { { -22., 75. } };
    dcmplx zr8u[1] = { { -22., 75. } };
    cmplx  cr8c[1] = { { -14., -37. } };
    dcmplx zr8c[1] = { { -14., -37. } };
    float  sres9[3][5] = { { 7., 9., 11., 13., 0. }, { 9., 8., 7., 6., 0. },
                           { 15., 20., 10., 5., 0. } };
    double dres9[3][5] = { { 7., 9., 11., 13., 0. }, { 9., 8., 7., 6., 0. },
                           { 15., 20., 10., 5., 0. } };
    float  sres10[3] = { 20., 36., 48. };
    double dres10[3] = { 20., 36., 48. };
    double dres11[1] = { 0.8507E+39 };  /* { 0x6127ffee61d83800 };  */
    double zres11[1] = { 0.8507E+39 };  /* { 0x6127ffee61d83800 };  */
    float  sres12[1] = { 10. };
    double dres12[1] = { 10. };
    float  cres12[1] = { 10. };
    double zres12[1] = { 10. };
    float  sres13[4] = { 0., 0., 1., 0. };
    double dres13[4] = { 0., 0., 1., 0. };
    cmplx  care13[1] = { { 1., 0. } };
    dcmplx zare13[1] = { { 1., 0. } };
    float  ccre13[1] = { 0. };
    double zcre13[1] = { 0. };
    cmplx  csre13[1] = { { 1., 0. } };
    dcmplx zsre13[1] = { { 1., 0. } };
    float  srex14[5] = { -0.366025404, -0.732050808, -1.09807621,
                         -1.46410162,  -1.83012702 };
    float  srey14[9] = { -1.3660254,  0., -2.73205081, 0., -4.09807621,	0.,
                         -5.46410162, 0., -6.83012702 };
    double drex14[5] = { -0.366025404, -0.732050808, -1.09807621,
                         -1.46410162,  -1.83012702 };
    double drey14[9] = { -1.3660254,  0., -2.73205081, 0., -4.09807621,	0.,
                        -5.46410162, 0., -6.83012702 };
    cmplx  crex14[3] = { { -2.75, 4.25 }, { -2.50, 3.50 }, { -2.25, 2.75 } };
    cmplx  crey14[5] = { { -2.25, 1.50 }, { 0., 0. }, { -4.00, 0.75 },
                         { 0., 0. }, { -5.75, 0.00 } };
    dcmplx zrex14[3] = { { -2.75, 4.25 }, { -2.50, 3.50 }, { -2.25, 2.75 } };
    dcmplx zrey14[5] = { { -2.25, 1.50 }, { 0., 0. }, { -4.00, 0.75 },
                         { 0., 0. }, { -5.75, 0.00 } };
    cmplx  csrx14[3] = { { -0.366, 5.330 }, { -0.732, 4.964 },
                         { -1.098, 4.598 } };
    cmplx  csry14[5] = { { -1.366, 0.768 }, { 0., 0. }, { -2.732, -0.598 },
                         { 0., 0. }, { -4.098, -1.964 } };
    dcmplx zdrx14[3] = { { -0.366, 5.330 }, { -0.732, 4.964 },
                         { -1.098, 4.598 } };
    dcmplx zdry14[5] = { { -1.366, 0.768 }, { 0., 0. }, { -2.732, -0.598 },
                         { 0., 0. }, { -4.098, -1.964 } };
    float  sres15[5] = { 2., 4., 6., 8., 10. };
    double dres15[5] = { 2., 4., 6., 8., 10. };
    cmplx  cres15[3] = { { -4., 7.}, { 4., 6.}, { -9., 19.} };
    dcmplx zres15[3] = { { -4., 7.}, { 4., 6.}, { -9., 19.} };
    cmplx  csre15[3] = { { 2., 4.}, { 4., 0. }, { 6., 10. } };
    dcmplx zdre15[3] = { { 2., 4.}, { 4., 0. }, { 6., 10. } };
    float  srex16[5] = { -1., -2., -3., -4., -5. };
    float  srey16[9] = { 1., 0., 2., 0., 3., 0., 4., 0., 5. };
    double drex16[5] = { -1., -2., -3., -4., -5. };
    double drey16[9] = { 1., 0., 2., 0., 3., 0., 4., 0., 5. };
    cmplx  crex16[4] = { { -1., -1. }, { -2., -2. }, { -3., -3.},
                         { -4., -4. } };
    cmplx  crey16[7] = { { 1., 6. }, { 0., 0. }, { 2., 7. }, { 0., 0.},
                         { 3., 8. }, { 0., 0. }, { 4., 9.} };
    dcmplx zrex16[4] = { { -1., -1. }, { -2., -2. }, { -3., -3.},
                         { -4., -4. } };
    dcmplx zrey16[7] = { { 1., 6. }, { 0., 0. }, { 2., 7. }, { 0., 0.},
                         { 3., 8. }, { 0., 0. }, { 4., 9.} };
    float  sres17[9] = { 2., 0., 4., 0., 6., 0., 8., 0., 10. };
    double dres17[9] = { 2., 0., 4., 0., 6., 0., 8., 0., 10. };
    cmplx  cres17[3] = { { -4., 7. }, { 4., 6. }, { -9., 19. } };
    dcmplx zres17[3] = { { -4., 7. }, { 4., 6. }, { -9., 19. } };
    cmplx  csre17[3] = { { 2., 4. }, { 4., 0. }, { 6., 10. } };
    dcmplx zdre17[3] = { { 2., 4. }, { 4., 0. }, { 6., 10. } };
    float  sres18[5] = { 3., 5., 7., 9., 11.};
    double dres18[5] = { 3., 5., 7., 9., 11.};
    cmplx  cres18[3] = { { -3., 8. }, { 4., 8. }, { -4., 23.} };
    dcmplx zres18[3] = { { -3., 8. }, { 4., 8. }, { -4., 23.} };
    float  sres19[10] = { 1., 2., 2., 3., 6., 10., 4., 8., 9., 5. };
    double dres19[10] = { 1., 2., 2., 3., 6., 10., 4., 8., 9., 5. };
    float  sres20[5] = { 6., 4., 7., -2., 9. };
    double dres20[5] = { 6., 4., 7., -2., 9. };
    float  sres21[15] = { 6., 4., 7., -2., 9., 0., 2., 0., 0.,
                          6., 10., 0., 8., 0., 0. };
    double dres21[15] = { 6., 4., 7., -2., 9., 0., 2., 0., 0.,
                          6., 10., 0., 8., 0., 0. };
    float  sres22[10] = { 3., 5., 8., 9., 6., 10., 6., 8., 9., 10. };
    double dres22[10] = { 3., 5., 8., 9., 6., 10., 6., 8., 9., 10. };
    float  sres23[1] = { 10. };
    double dres23[1] = { 10. };
    float  sres1n[5] = {2.,3.,4.,5.,6.};
    double dres1n[5] = {2.,3.,4.,5.,6.};
    cmplx  cres1n[3] = {{8.,10.}, {12.,14.}, {16.,18.}};
    dcmplx zres1n[3] = {{8.,10.}, {12.,14.}, {16.,18.}};
 
    float  sres2n[5] = {0.,1.,2.,3.,4.};
    double dres2n[5] = {0.,1.,2.,3.,4.};
    cmplx  cres2n[3] = {{-6.,-6.}, {-6.,-6.}, {-6.,-6.}};
    dcmplx zres2n[3] = {{-6.,-6.}, {-6.,-6.}, {-6.,-6.}};
 
    float  sres3n[5] = {1.,2.,3.,4.,5.};
    double dres3n[5] = {1.,2.,3.,4.,5.};
    cmplx  cres3n[3] = {{-9.,22.}, {-13.,66.}, {-17.,126.}};
    dcmplx zres3n[3] = {{-9.,22.}, {-13.,66.}, {-17.,126.}};
    cmplx  csres4n[4] = {{1.,2.}, {-2.,3.}, {3.,4.}, {5.,6.}};
    dcmplx zsres4n[4] = {{1.,2.}, {-2.,3.}, {3.,4.}, {5.,6.}};
 
    cmplx  cgres5n[3]  = {{6.,5.}, {-2.,3.}, {9.,0.}};
    dcmplx zgres5n[3]  = {{6.,5.}, {-2.,3.}, {9.,0.}};
 
    cmplx  cgxres6n[3] = {{6.,5.}, {-2.,3.}, {9.,0.}};
    cmplx  cgyres6n[4] = {{0.,0.},{ 0.,0.}, {15.,4.}, {0.,0.}};
 
    dcmplx zgxres6n[3] = {{6.,5.}, {-2.,3.}, {9.,0.}};
    dcmplx zgyres6n[4] = {{0.,0.}, {0.,0.}, {15.,4.}, {0.,0.}};
 
    cmplx  cires7n[4]  = {{2.,12.}, {-2.,3.}, {9.,21.}, {1.,27.}};
    dcmplx zires7n[4]  = {{2.,12.}, {-2.,3.}, {9.,21.}, {1.,27.}};
 
    cmplx  cdotcres7n[1]= {{146.,-97.}};
    dcmplx zdotcres7n[1]= {{146.,-97.}};
    cmplx  cdotures8n[1] = {{70.,143.}};
    dcmplx zdotures8n[1] = {{70.,143.}};
/************************************************
*  INPUT DATA FOR LINEAR ALGEBRA SUBPROGRAMS :  *
************************************************/
 
    float  sx1[9] = { 1., 2., 7., -8., -5., -10., -9., 10., 6. };
    double dx1[9] = { 1., 2., 7., -8., -5., -10., -9., 10., 6. };
    cmplx  cx1[5] = { { 9., 2. }, { 7., -8. }, { -5., -10 }, {-4., 10. },
                      { 6., 3. } };
    dcmplx zx1[5] = { { 9., 2. }, { 7., -8. }, { -5., -10 }, {-4., 10. },
                      { 6., 3. } };
    float  sx2[6] = { 3., 4., 1., 8., 1., 3. };
    double dx2[6] = { 3., 4., 1., 8., 1., 3. };
    float  sx3[6] = { 3., 4., 1., 8., 1., 8. };
    double dx3[6] = { 3., 4., 1., 8., 1., 8. };
    float  sx4[6] = { 3., 4., 1., 8., 1., 3. };
    double dx4[6] = { 3., 4., 1., 8., 1., 3. };
    float  sx5[7] = { 1., -3., -6., 7., 5., 2., -4. };
    double dx5[7] = { 1., -3., -6., 7., 5., 2., -4. };
    cmplx  cx5[5] = { { 1., 2. }, { -3., 4. }, { 5., -6. },
                      { -7., -8. }, { 9., 10. } };
    dcmplx zx5[5] = { { 1., 2. }, { -3., 4. }, { 5., -6. },
                      { -7., -8. }, { 9., 10. } };
    float  sx6[5] = { 1., 2., 3., 4., 5. };
    double dx6[5] = { 1., 2., 3., 4., 5. };
    float  sy6[9] = { 1., 0., 1., 0., 1., 0., 1., 0., 1. };
    double dy6[9] = { 1., 0., 1., 0., 1., 0., 1., 0., 1. };
    cmplx  cx6[3] = { { 1., 2. }, { 2., 0. }, { 3., 5. } };
    dcmplx zx6[3] = { { 1., 2. }, { 2., 0. }, { 3., 5. } };
    cmplx  cy6[5] = { { 1., 1. }, { 0., 0. }, { 0., 2. },
                      { 0., 0. }, { 5., 4. } };
    dcmplx zy6[5] = { { 1., 1. }, { 0., 0. }, { 0., 2. },
                      { 0., 0. }, { 5., 4. } };
    float  sx7[5] = { 1., 2., 3., 4., 5. };
    double dx7[5] = { 1., 2., 3., 4., 5. };
    cmplx  cx7[4] = { { 1., 1. }, { 2., 2. }, { 3., 3. }, { 4., 4. } };
    dcmplx zx7[4] = { { 1., 1. }, { 2., 2. }, { 3., 3. }, { 4., 4. } };
    float  sx8[5] = { 1., 2., -3., 4., 5. };
    double dx8[5] = { 1., 2., -3., 4., 5. };
    float  sy8[5] = { 9., 8., 7., -6., 5. };
    double dy8[5] = { 9., 8., 7., -6., 5. };
    cmplx  cx8[3] = { { 1., 2. }, { 3., -4. }, { -5., 6. } };
    dcmplx zx8[3] = { { 1., 2. }, { 3., -4. }, { -5., 6. } };
    cmplx  cy8[5] = { { 10., 9. }, { 0., 0. }, { -6., 5. },
                      { 0., 0. }, { 2., 1. } };
    cmplx  cy9[3] = { { 10., 9. }, { -6., 5. }, { 2., 1. } };
    dcmplx zy8[5] = { { 10., 9. }, { 0., 0. }, { -6., 5. },
                      { 0., 0. }, { 2., 1. } };
    float  sx9[3][10] = { 1., 2., 3., 4., 0., 0., 0., 0., 0., 0.,
                          4., 3., 2., 1., 0., 0., 0., 0., 0., 0.,
                          3., 4., 2., 1., 0., 0., 0., 0., 0., 0. };
    double dx9[3][10] = { 1., 2., 3., 4., 0., 0., 0., 0., 0., 0.,
                          4., 3., 2., 1., 0., 0., 0., 0., 0., 0.,
                          3., 4., 2., 1., 0., 0., 0., 0., 0., 0. };
    float  sy9[3][5] = { 4., 3., 2., 1., 0., 1., 2., 3., 4., 0.,
                         3., 4., 2., 1., 0. };
    double dy9[3][5] = { 4., 3., 2., 1., 0., 1., 2., 3., 4., 0.,
                         3., 4., 2., 1., 0. };
    float  sa9[3] = { 3., 2., 4. };
    double da9[3] = { 3., 2., 4. };
    float  sx10[3][4] = { 1., 2., 3., 4., 2., 3., 4., 5., 3., 4., 5., 6. };
    double dx10[3][4] = { 1., 2., 3., 4., 2., 3., 4., 5., 3., 4., 5., 6. };
    float  sy10[3][4] = { 4., 3., 2., 1., 3., 2., 1., 4., 2., 1., 4., 3. };
    double dy10[3][4] = { 4., 3., 2., 1., 3., 2., 1., 4., 2., 1., 4., 3. };
    double dx11[6] = { 0.68056e+39, 0.25521e+39, 0.34028e+39,
                       0.85071e+38, 0.25521e+39, 0.85071e+38 };
    dcmplx zx11[3] = { { 0.68056e+39, 0.25521e+39 },
                       { 0.34028e+39, 0.85071e+38 },
                       { 0.25521e+39, 0.85071e+38 } };
    float  sx12[6] = { 3., 4., 1., 8., 1., 3. };
    double dx12[6] = { 3., 4., 1., 8., 1., 3. };
    cmplx  cx12[3] = { { 3., 4. }, { 1., 8. }, { -1., 3. } };
    dcmplx zx12[3] = { { 3., 4. }, { 1., 8. }, { -1., 3. } };
    float  sa13[4] = { 0., 0., 0., 0. };
    double da13[4] = { 0., 0., 0., 0. };
    cmplx  ca13[1] = { { 0., 0. } };
    cmplx  cb13[1] = { { 1., 0. } };
    dcmplx za13[1] = { { 0., 0. } };
    dcmplx zb13[1] = { { 1., 0. } };
    float  sx14[5] = { 1., 2., 3., 4., 5. };
    float  sy14[9] = { -1., 0., -2., 0., -3., 0., -4., 0., -5. };
    double dx14[5] = { 1., 2., 3., 4., 5. };
    double dy14[9] = { -1., 0., -2., 0., -3., 0., -4., 0., -5. };
    cmplx  cx14[3] = { { 1., 2. }, { 2., 3. }, { 3., 4. } };
    cmplx  cy14[5] = { { -1., 5. }, { 0., 0. }, { -2., 4. },
                       { 0., 0. }, { -3., 3. } };
    dcmplx zx14[3] = { { 1., 2. }, { 2., 3. }, { 3., 4. } };
    dcmplx zy14[5] = { { -1., 5. }, { 0., 0. }, { -2., 4. },
                       { 0., 0. }, { -3., 3. } };
    cmplx  csx14[3] = { { 1., 2. }, { 2., 3. }, { 3., 4. } };
    cmplx  csy14[5] = { { -1., 5. }, { 0., 0. }, { -2., 4. },
                        { 0., 0. }, { -3., 3. } };
    dcmplx zdx14[3] = { { 1., 2. }, { 2., 3. }, { 3., 4. } };
    dcmplx zdy14[5] = { { -1., 5. }, { 0., 0. }, { -2., 4. },
                        { 0., 0. }, { -3., 3. } };
    float  sx15[5] = { 1., 2., 3., 4., 5. };
    double dx15[5] = { 1., 2., 3., 4., 5. };
    cmplx  cx15[3] = { { 1., 2. }, { 2., 0. }, { 3., 5. } };
    dcmplx zx15[3] = { { 1., 2. }, { 2., 0. }, { 3., 5. } };
    cmplx  csx15[3] = { { 1., 2. }, { 2., 0. }, { 3., 5. } };
    dcmplx zdx15[3] = { { 1., 2. }, { 2., 0. }, { 3., 5. } };
    float  sx16[5] = { 1., 2., 3., 4., 5. };
    float  sy16[9] = { -1., 0., -2., 0., -3., 0., -4., 0., -5. };
    double dx16[5] = { 1., 2., 3., 4., 5. };
    double dy16[9] = { -1., 0., -2., 0., -3., 0., -4., 0., -5. };
    cmplx  cx16[4] = { { 1., 6. }, { 2., 7. }, { 3., 8. }, { 4., 9. } };
    cmplx  cy16[7] = { { -1., -1. }, { 0., 0. }, { -2., -2. }, { 0., 0. },
                       { -3., -3. }, { 0., 0. }, { -4., -4. } };
    dcmplx zx16[4] = { { 1., 6. }, { 2., 7. }, { 3., 8. }, { 4., 9. } };
    dcmplx zy16[7] = { { -1., -1. }, { 0., 0. }, { -2., -2. }, { 0., 0. },
                       { -3., -3. }, { 0., 0. }, { -4., -4. } };
    float  sx17[5] = { 1., 2., 3., 4., 5. };
    double dx17[5] = { 1., 2., 3., 4., 5. };
    cmplx  cx17[3] = { { 1., 2. }, { 2., 0. }, { 3., 5. } };
    dcmplx zx17[3] = { { 1., 2. }, { 2., 0. }, { 3., 5. } };
    float  sx18[5] = { 1., 2., 3., 4., 5. };
    double dx18[5] = { 1., 2., 3., 4., 5. };
    float  sy18[9] = { 1., 0., 1., 0., 1., 0., 1., 0., 1. };
    double dy18[9] = { 1., 0., 1., 0., 1., 0., 1., 0., 1. };
    cmplx  cx18[3] = { { 1., 2. }, { 2., 0. }, { 3., 5. } };
    cmplx  cy18[5] = { { 1., 1. }, { 0., 0. }, { 0., 2. },
                       { 0., 0. }, { 5., 4. } };
    dcmplx zx18[3] = { { 1., 2. }, { 2., 0. }, { 3., 5. } };
    dcmplx zy18[5] = { { 1., 1. }, { 0., 0. }, { 0., 2. },
                       { 0., 0. }, { 5., 4. } };
    _ESVINT    indxa[5] = { 1, 3, 4, 7, 10 };
    _ESVINT    indxb[5] = { 1, 3, 4, 7, 9 };
    float  sx19[5] = { 1., 2., 3., 4., 5. };
    float  sy19[10] = { 6., 2., 4., 7., 6., 10., -2., 8., 9., 0. };
    double dx19[5] = { 1., 2., 3., 4., 5. };
    double dy19[10] = { 6., 2., 4., 7., 6., 10., -2., 8., 9., 0. };
    float  sy20[10] = { 6., 2., 4., 7., 6., 10., -2., 8., 9., 0. };
    double dy20[10] = { 6., 2., 4., 7., 6., 10., -2., 8., 9., 0. };
    float  sx21[15] = { 0., 0., 0., 0., 0., 6., 2., 4., 7., 6.,
                       10., -2., 8., 9., 0. };
    double dx21[15] = { 0., 0., 0., 0., 0., 6., 2., 4., 7., 6.,
                       10., -2., 8., 9., 0. };
    float  sx22[5] = { 1., 2., 3., 4., 5. };
    double dx22[5] = { 1., 2., 3., 4., 5. };
    float  sy22[10] = { 1., 5., 4., 3., 6., 10., -2., 8., 9., 0. };
    double dy22[10] = { 1., 5., 4., 3., 6., 10., -2., 8., 9., 0. };
    float  sx23[5] = { 1., 2., 3., 4., 5. };
    float  sy23[10] = { 1., 5., 4., 3., 6., 10., -2., 8., 9., 0. };
    double dx23[5] = { 1., 2., 3., 4., 5. };
    double dy23[10] = { 1., 5., 4., 3., 6., 10., -2., 8., 9., 0. };
    float  sx1n[5] = {1.,2.,3.,4.,5.};
    double dx1n[5] = {1.,2.,3.,4.,5.};
    cmplx  cx1n[3] = {{1.,2.}, {3.,4.}, {5.,6.}};
    dcmplx zx1n[3] = {{1.,2.}, {3.,4.}, {5.,6.}};
    float  sy1n[9] = {1.,0.,1.,0.,1.,0.,1.,0.,1.};
    double dy1n[9] = {1.,0.,1.,0.,1.,0.,1.,0.,1.};
    cmplx  cy1n[5] = {{7.,8.}, {0.,0.},{ 9.,10.},{ 0.,0.},{ 11.,12.}};
    dcmplx zy1n[5] = {{7.,8.}, {0.,0.}, {9.,10.},{ 0.,0.}, {11.,12.}};
 
    float  sx2n[5] = {1.,2.,3.,4.,5.};
    double dx2n[5] = {1.,2.,3.,4.,5.};
    cmplx  cx2n[3] = {{1.,2.}, {3.,4.}, {5.,6.}};
    dcmplx zx2n[3] = {{1.,2.},{ 3.,4.}, {5.,6.}};
    float  sy2n[9] = {1.,0.,1.,0.,1.,0.,1.,0.,1.};
    double dy2n[9] = {1.,0.,1.,0.,1.,0.,1.,0.,1.};
    cmplx  cy2n[5] = {{7.,8.}, {0.,0.}, {9.,10.}, {0.,0.}, {11.,12.}};
    dcmplx zy2n[5] = {{7.,8.}, {0.,0.}, {9.,10.}, {0.,0.}, {11.,12.}};
 
    float  sx3n[5] = {1.,2.,3.,4.,5.};
    double dx3n[5] = {1.,2.,3.,4.,5.};
    cmplx  cx3n[3] = {{1.,2.}, {3.,4.}, {5.,6.}};
    dcmplx zx3n[3] = {{1.,2.}, {3.,4.}, {5.,6.}};
    float  sy3n[9] = {1.,0.,1.,0.,1.,0.,1.,0.,1.};
    double dy3n[9] = {1.,0.,1.,0.,1.,0.,1.,0.,1.};
    cmplx  cy3n[5] = {{7.,8.}, {0.,0.}, {9.,10.}, {0.,0.}, {11.,12.}};
    dcmplx zy3n[5] = {{7.,8.}, {0.,0.}, {9.,10.}, {0.,0.}, {11.,12.}};
    cmplx  csx4n[3] = {{1.,2.}, {3.,4.}, {5.,6.}};
    _ESVINT    inxcs[3] = {1,3,4};
    cmplx  csy4n[4] = {{6.,5.}, {-2.,3.}, {15.,4.}, {9.,0.}};
    dcmplx zsx4n[4] = {{1.,2.},{ 3.,4.}, {5.,6.}};
    _ESVINT    inxzs[3] = {1,3,4};
    dcmplx zsy4n[4] = {{6.,5.}, {-2.,3.}, {15.,4.}, {9.,0.}};
 
    cmplx  cgy5n[4] = {{6.,5.}, {-2.,3.}, {15.,4.}, {9.,0.}};
    _ESVINT    inxcg[3] = {1,2,4};
    dcmplx zgy5n[4] = {{6.,5.}, {-2.,3.}, {15.,4.}, {9.,0.}};
    _ESVINT    inxzg[3] = {1,2,4};
 
    cmplx  cgzy6n[4] = {{6.,5.}, {-2.,3.}, {15.,4.}, {9., 0.}};
    dcmplx zgzy6n[4] = {{6.,5.}, {-2.,3.}, {15.,4.}, {9., 0.}};
 
    cmplx  cix7n[3] = {{1.,2.}, {3.,4.}, {5.,6.}};
    cmplx  ciy7n[4] = {{6.,5.}, {-2.,3.}, {15.,4.}, {9.,0.}};
    dcmplx zix7n[3] = {{1.,2.}, {3.,4.}, {5.,6.}};
    dcmplx ziy7n[4] = {{6.,5.}, {-2.,3.}, {15.,4.}, {9.,0.}};
 
    cmplx  cdotcx7n[5] = {{1.,2.}, {3.,4.}, {5.,6.}};
    cmplx  cdotcy7n[5] = {{6.,5.}, {-2.,3.}, {15.,4.}, {9.,0.}};
    _ESVINT    dotci[3] = {4,1,3};
    dcmplx zdotcx7n[5] = {{1.,2.}, {3.,4.}, {5.,6.}};
    dcmplx zdotcy7n[5] = {{6.,5.}, {-2.,3.}, {15.,4.}, {9.,0.}};
 
    cmplx  cdotux8n[3] = {{1.,2.}, {3.,4.}, {5.,6.}};
    cmplx  cdotuy8n[5] = {{6.,5.},{-2.,3.},{ 15.,4.}, {9.,0.}};
    _ESVINT    dotui[3] = {1,3,4};
    dcmplx zdotux8n[5] = {{1.,2.}, {3.,4.}, {5.,6.}};
    dcmplx zdotuy8n[5] = {{6.,5.}, {-2.,3.}, {15.,4.}, {9.,0.}};
 
/****************************************************
*  CALL STATEMENTS FOR LINEAR ALGEBRA SUBROUTINES :  *
****************************************************/
 
/***************************
*  VECTOR-SCALAR LINEAR :  *
***************************/
 
    cmplx  c4;
    dcmplx c8;
 
    max[0] = isamax(9, sx1, 1);
    intchk(1, 1, max, ires1, "isamax", pass);
    max[0] = idamax (9, dx1, 1);
    intchk(1, 1, max, ires1, "idamax", pass);
    max[0] = icamax (5, cx1, 1);
    intchk(1, 1, max, ires11, "icamax", pass);
    max[0] = izamax (5, zx1, 1);
    intchk(1, 1, max, ires11, "izamax", pass);
    min[0] = isamin (6, sx2, 1);
    intchk(1, 1, min, ires2, "isamin", pass);
    min[0] = idamin (6, dx2, 1);
    intchk(1, 1, min, ires2, "idamin", pass);
    max[0] = ismax (6, sx3, 1);
    intchk(1, 1, max, ires3, "ismax", pass);
    max[0] = idmax (6, dx3, 1);
    intchk(1, 1, max, ires3, "idmax", pass);
    min[0] = ismin (6, sx4, 1);
    intchk(1, 1, min, ires4, "ismin", pass);
    min[0] = idmin (6, dx4, 1);
    intchk(1, 1, min, ires4, "idmin", pass);
    ssum[0] = sasum (7, sx5, 1);
    svecck(1, 1, ssum, sres5, "sasum", pass);
    dsum[0] = dasum (7, dx5, 1);
    dvecck(1, 1, dsum, dres5, "dasum", pass);
    csum[0] = scasum (5, cx5, 1);
    svecck(1, 1, csum, cres5, "scasum", pass);
    zsum[0] = dzasum (5, zx5, 1);
    dvecck(1, 1, zsum, zres5, "dzasum", pass);
    saxpy (5, 2., sx6, 1, sy6, 2);
    svecck(5, 2, sy6, sres6, "saxpy", pass);
    daxpy (5, 2., dx6, 1, dy6, 2);
    dvecck(5, 2, dy6, dres6, "daxpy", pass);
    CSET(c4, 2., 3.);
    caxpy (3, c4, cx6, 1, cy6, 2);
    cvecck(3, 2, cy6, cres6, "caxpy", pass);
    CSET(c8, 2., 3.);
    zaxpy (3, c8, zx6, 1, zy6, 2);
    zvecck(3, 2, zy6, zres6, "zaxpy", pass);
    scopy (5, sx7, 1, sy7, 2);
    svecck(5, 2, sy7, sres7, "scopy", pass);
    dcopy (5, dx7, 1, dy7, 2);
    dvecck(5, 2, dy7, dres7, "dcopy", pass);
    ccopy (4, cx7, 1, cy7, 2);
    cvecck(4, 2, cy7, cres7, "ccopy", pass);
    zcopy (4, zx7, 1, zy7, 2);
    zvecck(4, 2, zy7, zres7, "zcopy", pass);
    st[0] = sdot (5, sx8, 1, sy8, 1);
    svecck(1, 1, st, sres8, "sdot", pass);
    dt[0] = ddot (5, dx8, 1, dy8, 1);
    dvecck(1, 1, dt, dres8, "ddot", pass);
    ct[0] = cdotu (3, cx8, 1, cy8, 2);
    cvecck(1, 1, ct, cr8u, "cdotu", pass);
    ct[0] = cdotu (3, cx8, 1, cy9, 1);
    cvecck(1, 1, ct, cr8u, "cdotu", pass);
    zt[0] = zdotu (3, zx8, 1, zy8, 2);
    zvecck(1, 1, zt, zr8u, "zdotu", pass);
    ct[0] = cdotc (3, cx8, 1, cy8, 2);
    cvecck(1, 1, ct, cr8c, "cdotc", pass);
    ct[0] = cdotc (3, cx8, 1, cy9, 1);
    cvecck(1, 1, ct, cr8c, "cdotc", pass);
    zt[0] = zdotc (3, zx8, 1, zy8, 2);
    zvecck(1, 1, zt, zr8c, "zdotc", pass);
    snaxpy (3, 4, sa9, 1, sx9, 1, 10, sy9, 1, 5);
    smtchk(4, 3, sy9, sres9, 5, "snaxpy", pass);
    dnaxpy (3, 4, da9, 1, dx9, 1, 10, dy9, 1, 5);
    dmtchk(4, 3, dy9, dres9, 5, "dnaxpy", pass);
    sndot (3, 4, ss10, 1, 1, sx10, 1, 4, sy10, 1, 4);
    svecck(3, 1, ss10, sres10, "sndot", pass);
    dndot (3, 4, ds10, 1, 1, dx10, 1, 4, dy10, 1, 4);
    dvecck(3, 1, ds10, dres10, "dndot", pass);
    st[0] = snrm2 (6, sx12, 1);
    sverck(1, 1, st, sres12, "snrm2", pass);
    dt[0] = dnrm2 (6, dx11, 1);
    dverck(1, 1, dt, dres11, "dnrm2", pass);
    st[0] = scnrm2 (3, cx12, 1);
    sverck(1, 1, st, cres12, "scnrm2", pass);
    dt[0] = dznrm2 (3, zx11, 1);
    dverck(1, 1, dt, zres11, "dznrm2", pass);
    st[0] = snorm2 (6, sx12, 1);
    svecck(1, 1, st, sres12, "snorm2", pass);
    dt[0] = dnorm2 (6, dx12, 1);
    dvecck(1, 1, dt, dres12, "dnorm2", pass);
    st[0] = cnorm2 (3, cx12, 1);
    svecck(1, 1, st, cres12, "cnorm2", pass);
    dt[0] = znorm2 (3, zx12, 1);
    dvecck(1,1,dt,zres12,"znorm2",pass);
    srotg (&sa13[0], &sa13[1], &sa13[2], &sa13[3]);
    svecck(4, 1, sa13, sres13, "srotg", pass);
    drotg (&da13[0], &da13[1], &da13[2], &da13[3]);
    dvecck(4, 1, da13, dres13, "drotg", pass);
    crotg (ca13, cb13, cc13, cs13);
    cvecck(1, 1, ca13, care13, "crotg", pass);
    svecck(1, 1, cc13, ccre13, "crotg", pass);
    cvecck(1, 1, cs13, csre13, "crotg", pass);
    zrotg (za13, zb13, zc13, zs13);
    zvecck(1, 1, za13, zare13, "zrotg", pass);
    dvecck(1, 1, zc13, zcre13, "zrotg", pass);
    zvecck(1, 1, zs13, zsre13, "zrotg", pass);
    srot (5, sx14, 1, sy14, 2, 0.5, (sqrt(3.) / 2.));
    svecck(5, 1, sx14, srex14, "srot", pass);
    svecck(5, 2, sy14, srey14, "srot", pass);
    drot (5, dx14, 1, dy14, 2, 0.5, (sqrt(3.) / 2.));
    dvecck(5, 1, dx14, drex14, "drot", pass);
    dvecck(5, 2, dy14, drey14, "drot", pass);
    CSET(c4, 0.75, 0.50);
    crot (3, cx14, 1, cy14, 2, 0.5, c4);
    cvecck(3, 1, cx14, crex14, "crot", pass);
    cvecck(3, 2, cy14, crey14, "crot", pass);
    CSET(c8, 0.75, 0.50);
    zrot (3, zx14, 1, zy14, 2, 0.5, c8);
    zvecck(3, 1, zx14, zrex14, "zrot", pass);
    zvecck(3, 2, zy14, zrey14, "zrot", pass);
    csrot (3, csx14, 1, csy14, 2, 0.5, (sqrt(3.) / 2.));
    cvecck(3, 1, csx14, csrx14, "csrot", pass);
    cvecck(3, 2, csy14, csry14, "csrot", pass);
    zdrot (3, zdx14, 1, zdy14, 2, 0.5, (sqrt(3.) / 2.));
    zvecck(3, 1, zdx14, zdrx14, "zdrot", pass);
    zvecck(3, 2, zdy14, zdry14, "zdrot", pass);
    sscal (5, 2., sx15, 1);
    svecck(5, 1, sx15, sres15, "sscal", pass);
    dscal (5, 2., dx15, 1);
    dvecck(5, 1, dx15, dres15, "dscal", pass);
    CSET(c4, 2., 3.);
    cscal (3, c4, cx15, 1);
    cvecck(3, 1, cx15, cres15, "cscal", pass);
    CSET(c8, 2., 3.);
    zscal (3, c8, zx15, 1);
    zvecck(3, 1, zx15, zres15, "zscal", pass);
    csscal (3, 2., csx15, 1);
    cvecck(3, 1, csx15, csre15, "csscal", pass);
    zdscal (3, 2., zdx15, 1);
    zvecck(3, 1, zdx15, zdre15, "zdscal", pass);
    sswap (5, sx16, 1, sy16, 2);
    svecck(5, 1, sx16, srex16, "sswap", pass);
    svecck(5, 2, sy16, srey16, "sswap", pass);
    dswap (5, dx16, 1, dy16, 2);
    dvecck(5, 1, dx16, drex16, "dswap", pass);
    dvecck(5, 2, dy16, drey16, "dswap", pass);
    cswap (4, cx16, 1, cy16, 2);
    cvecck(4, 1, cx16, crex16, "cswap", pass);
    cvecck(4, 2, cy16, crey16, "cswap", pass);
    zswap (4, zx16, 1, zy16, 2);
    zvecck(4, 1, zx16, zrex16, "zswap", pass);
    zvecck(4, 2, zy16, zrey16, "zswap", pass);
    syax (5, 2., sx17, 1, sy17, 2);
    svecck(5, 2, sy17, sres17, "syax", pass);
    dyax (5, 2., dx17, 1, dy17, 2);
    dvecck(5, 2, dy17, dres17, "dyax", pass);
    CSET(c4, 2., 3.);
    cyax (3, c4, cx17, 1, cy17, 1);
    cvecck(3, 1, cy17, cres17, "cyax", pass);
    CSET(c8, 2., 3.);
    zyax (3, c8, zx17, 1, zy17, 1);
    zvecck(3, 1, zy17, zres17, "zyax", pass);
    csyax (3, 2., cx17, 1, cy17, 1);
    cvecck(3, 1, cy17, csre17, "csyax", pass);
    zdyax (3, 2., zx17, 1, zy17, 1);
    zvecck(3, 1, zy17, zdre17, "zdyax", pass);
    szaxpy (5, 2., sx18, 1, sy18, 2, sz18, 1);
    svecck(5, 1, sz18, sres18, "szaxpy", pass);
    dzaxpy (5, 2., dx18, 1, dy18, 2, dz18, 1);
    dvecck(5, 1, dz18, dres18, "dzaxpy", pass);
    CSET(c4, 2., 3.);
    czaxpy (3, c4, cx18, 1, cy18, 2, cz18, 1);
    cvecck(3, 1, cz18, cres18, "czaxpy", pass);
    CSET(c8, 2., 3.);
    zzaxpy (3, c8, zx18, 1, zy18, 2, zz18, 1);
    zvecck(3, 1, zz18, zres18, "zzaxpy", pass);
    svea(5,sx1n,1,sy1n,2,sa1n,1);
    svecck(5,1,sa1n,sres1n,"svea",pass);
    dvea (5,dx1n,1,dy1n,2,da1n,1);
    dvecck(5,1,da1n,dres1n,"dvea",pass);
    cvea (3,cx1n,1,cy1n,2,ca1n,1);
    cvecck(3,1,ca1n,cres1n,"cvea",pass);
    zvea (3,zx1n,1,zy1n,2,za1n,1);
    zvecck(3,1,za1n,zres1n,"zvea",pass);
    sves (5,sx2n,1,sy2n,2,ss2n,1);
    svecck(5,1,ss2n,sres2n,"sves",pass);
    dves (5,dx2n,1,dy2n,2,ds2n,1);
    dvecck(5,1,ds2n,dres2n,"dves",pass);
    cves (3,cx2n,1,cy2n,2,cs2n,1);
    cvecck(3,1,cs2n,cres2n,"cves",pass);
    zves (3,zx2n,1,zy2n,2,zs2n,1);
    zvecck(3,1,zs2n,zres2n,"zves",pass);
    svem (5,sx3n,1,sy3n,2,sm3n,1);
    svecck(5,1,sm3n,sres3n,"svem",pass);
    dvem (5,dx3n,1,dy3n,2,dm3n,1);
    dvecck(5,1,dm3n,dres3n,"dvem",pass);
    cvem (3,cx3n,1,cy3n,2,cm3n,1);
    cvecck(3,1,cm3n,cres3n,"cvem",pass);
    zvem (3,zx3n,1,zy3n,2,zm3n,1);
    zvecck(3,1,zm3n,zres3n,"zvem",pass);
 
/**********************************
*  SPARSE VECTOR-SCALAR LINEAR :  *
**********************************/
 
    ssctr (5, sx19, indxa, sy19);
    svecck(10, 1, sy19, sres19, "ssctr", pass);
    dsctr (5, dx19, indxa, dy19);
    dvecck(10, 1, dy19, dres19, "dsctr", pass);
    csctr(3,csx4n,inxcs,csy4n);
    cvecck(4,1,csy4n,csres4n,"csctr",pass);
    zsctr(3,zsx4n,inxzs,zsy4n);
    zvecck(4,1,zsy4n,zsres4n,"zsctr",pass);
    sgthr (5, sy20, sx20, indxb);
    svecck(5, 1, sx20, sres20, "sgthr", pass);
    dgthr (5, dy20, dx20, indxb);
    dvecck(5, 1, dx20, dres20, "dgthr", pass);
    cgthr(3,cgy5n,cgx5n,inxcg);
    cvecck(3,1,cgx5n,cgres5n,"cgthr",pass);
    zgthr(3,zgy5n,zgx5n,inxzg);
    zvecck(3,1,zgx5n,zgres5n,"zgthr",pass);
    sgthrz (5, &sx21[5], &sx21[0], indxb);
    svecck(15, 1, sx21, sres21, "sgthrz", pass);
    dgthrz (5, &dx21[5], &dx21[0], indxb);
    dvecck(15, 1, dx21, dres21, "dgthrz", pass);
    cgthrz(3,cgzy6n,cgzx6n,inxcg);
    cvecck(3,1,cgzy6n,cgyres6n,"cgthrz",pass);
    cvecck(3,1,cgzx6n,cgxres6n,"cgthrz",pass);
    zgthrz(3,zgzy6n,zgzx6n,inxzg);
    zvecck(3,1,zgzy6n,zgyres6n,"zgthrz",pass);
    zvecck(3,1,zgzx6n,zgxres6n,"zgthrz",pass);
    saxpyi (5, 2., sx22, indxa, sy22);
 
    svecck(10, 1, sy22, sres22, "saxpyi", pass);
    daxpyi (5, 2., dx22, indxa, dy22);
    dvecck(10, 1, dy22, dres22, "daxpyi", pass);
    CSET(c1n, 2., 3.);
    caxpyi(3,c1n,cix7n,inxcs,ciy7n);
    cvecck(4,1,ciy7n,cires7n,"caxpyi",pass);
    CSET(dc1n, 2., 3.);
    zaxpyi(3,dc1n,zix7n,inxzs,ziy7n);
    zvecck(4,1,ziy7n,zires7n,"zaxpyi",pass);
 
    st[0] = sdoti (5, sx23, indxa, sy23);
    svecck(1, 1, st, sres23, "sdoti", pass);
    dt[0] = ddoti (5, dx23, indxa, dy23);
    dvecck(1, 1, dt, dres23, "ddoti", pass);
    cdotc7n[0] = cdotci(3,cdotcx7n,dotci,cdotcy7n);
    cvecck(1,1,cdotc7n,cdotcres7n,"cdotci",pass);
    zdotc7n[0] = zdotci(3,zdotcx7n,dotci,zdotcy7n);
    zvecck(1,1,zdotc7n,zdotcres7n,"zdotci",pass);
 
    cdotu8n[0] = cdotui(3,cdotux8n,dotui,cdotuy8n);
    cvecck(1,1,cdotu8n,cdotures8n,"cdotui",pass);
    zdotu8n[0] = zdotui(3,zdotux8n,dotui,zdotuy8n);
    zvecck(1,1,zdotu8n,zdotures8n,"zdotui",pass);
 
}
 
/*********************************************************************
*                    SUBROUTINE:  LIALMV                             *
*********************************************************************/
 
void lialmv(pass)
_ESVINT *pass;
{
/**********************************************
*  VARIABLES OF LINEAR ALGEBRA SUBROUTINES :  *
**********************************************/
 
/********************************
*  (DENSE MATRIX-VECTOR LINEAR) *
********************************/
    cmplx  c1n,c2n;
    dcmplx dc1n,dc2n;
 
/*********************************
*  (SPARSE MATRIX-VECTOR LINEAR) *
*********************************/
 
    _ESVINT    ikt31a[4][6];
    float  saux[100];
    double dy31[6];
    double dat31a[4][6];
    double dy32[6];
 
/*****************************************************
*  EXPECTED VALUES FOR LINEAR ALGEBRA SUBPROGRAMS :  *
*****************************************************/
 
    float  sres24[7] = { 14., 0., 19., 0., 17., 0., 20. };
    double dres24[7] = { 14., 0., 19., 0., 17., 0., 20. };
    cmplx  cres24[5] = { { 1.2e1, 2.8e1 }, { 2.4e1, 5.5e1 },
                         { 1.0e1, 3.9e1 }, { 2.3e1, 5.0e1 },
                         { 2.2e1, 4.4e1 } };
    dcmplx zres24[5] = { { 1.2e1, 2.8e1 }, { 2.4e1, 5.5e1 },
                         { 1.0e1, 3.9e1 }, { 2.3e1, 5.0e1 },
                         { 2.2e1, 4.4e1 } };
    float  sres25[7] = { 14., 0., 19., 0., 17., 0., 20. };
    double dres25[7] = { 14., 0., 19., 0., 17., 0., 20. };
    float  sres26[5] = { 27., 0., 22., 0., 26. };
    double dres26[5] = { 27., 0., 22., 0., 26. };
    float  sres27[3][10] = { 4.,  4., 4.,  8., 0., 0., 0., 0., 0., 0.,
                             8.,  6., 4., 10., 0., 0., 0., 0., 0., 0.,
                            12., 10., 5., 13., 0., 0., 0., 0., 0., 0. };
    double dres27[3][10] = { 4.,  4., 4.,  8., 0., 0., 0., 0., 0., 0.,
                             8.,  6., 4., 10., 0., 0., 0., 0., 0., 0.,
                            12., 10., 5., 13., 0., 0., 0., 0., 0., 0. };
    cmplx  cure27[3][5] = { { { -2.0, 6.0 }, { 6.0, 1.1e1 }, { 6.0, 7.0 },
                              { 3.0, 1.2e1 }, { 1.1e1, 5.0 } },
                            { { 7.0, 1.3e1 }, { 2.3e1, 9.0 }, { 5.0, 8.0 },
                              { 1.4e1, 2.1e1 }, { 1.1e1, 6.0 } },
                            { { 5.0, 1.0 }, { 8.0, 4.0 }, { 8.0, 0.0 },
                              { 1.5e1, 1.0 }, { 3.0, -2.0 } } };
    dcmplx zure27[3][5] = { { { -2.0, 6.0 }, { 6.0, 1.1e1 }, { 6.0, 7.0 },
                              { 3.0, 1.2e1 }, { 1.1e1, 5.0 } },
                            { { 7.0, 1.3e1 }, { 2.3e1, 9.0 }, { 5.0, 8.0 },
                              { 1.4e1, 2.1e1 }, { 1.1e1, 6.0 } },
                            { { 5.0, 1.0 }, { 8.0, 4.0 }, { 8.0, 0.0 },
                              { 1.5e1, 1.0 }, { 3.0, -2.0 } } };
    cmplx  ccre27[3][5] = { { { 6.0, 2.0 }, { 6.0, -5.0 }, { 1.0e1, 3.0 },
                              { 1.9e1, 0.0 }, { 1.1e1, -3.0 } },
                          { { 7.0, 1.3e1 }, { 2.3e1, 9.0 }, { 5.0, 8.0 },
                            { 1.4e1, 2.1e1 }, { 1.1e1, 6.0 } },
                          { { 1.0, 3.0 }, { 8.0, 1.2e1 }, { 6.0, 2.0 },
                            { 7.0, 7.0 }, { 3.0, 2.0 } } };
    dcmplx zcre27[3][5] = { { { 6.0, 2.0 }, { 6.0, -5.0 }, { 1.0e1, 3.0 },
                              { 1.9e1, 0.0 }, { 1.1e1, -3.0 } },
                            { { 7.0, 1.3e1 }, { 2.3e1, 9.0 }, { 5.0, 8.0 },
                              { 1.4e1, 2.1e1 }, { 1.1e1, 6.0 } },
                            { { 1.0, 3.0 }, { 8.0, 1.2e1 }, { 6.0, 2.0 },
                              { 7.0, 7.0 }, { 3.0, 2.0 } } };
    float  sres28[5] = { 39., 0., 34., 0., 25. };
    double dres28[5] = { 39., 0., 34., 0., 25. };
    float  sres29[6] = { 17., 10., 5., 10., 9., 4. };
    double dres29[6] = { 17., 10., 5., 10., 9., 4. };
    float  sres30[6] = { 38., 23., 13., 18., 14., 7. };
    double dres30[6] = { 38., 23., 13., 18., 14., 7. };
    double dres31[6] = { 25., 19., 36., 43., 33., 42. };
    double dre31a[4][6] = { 11., 42., 33., 34., 0., 0.,
                            21.,  0., 23., 44., 0., 0.,
                            51.,  0., 53.,  0., 0., 0.,
                             0.,  0.,  0.,  0., 0., 0., };
    _ESVINT    ktr31a[4][6] = { 1, 4, 3, 3, 1, 0, 2, 1, 2, 4, 1, 0,
                            5, 1, 5, 1, 1, 0, 1, 1, 1, 1, 1, 0 };
    _ESVINT    nr31a[1] = { 4 };
    _ESVINT    ntr31a[1] = { 3 };
    double dres32[6] = { 25., 19., 36., 43., 33., 42. };
    float  sreb33[4] = {1.,3.,11.,24.};
    double dreb33[4] = {1.,3.,11.,24.};
    cmplx  creb33[4] = {{1.,1.},{2.,4.},{3.,17.},{4.,44.}};
    dcmplx zreb33[4] = {{1.,1.},{2.,4.},{3.,17.},{4.,44.}};
 
    float  spmvres10n[5] = {39., 0., 34., 0., 25.};
    double dpmvres10n[5] = {39., 0., 34., 0., 25.};
    cmplx  chmvres10n[5] = {{32.,21.}, {0.,0.}, {87.,-8.},
                            {0.,0.}, {32.,64.}};
    dcmplx zhmvres10n[5] = {{32.,21.}, {0.,0.}, {87.,-8.},
                            {0.,0.}, {32.,64.}};
 
    float  symvres11n[5] = {39., 0., 34., 0., 25.};
    double dymvres11n[5] = {39., 0., 34., 0., 25.};
    cmplx  cemvres11n[5] = {{32.,21.}, {0.,0.}, {87.,-8.},
                            {0.,0.}, {32.,64.}};
    dcmplx zemvres11n[5] = {{32.,21.}, {0.,0.}, {87.,-8.},
                            {0.,0.}, {32.,64.}};
 
    float  ssprres12n[6] = {17.,10.,5.,10.,9.,4.};
    double dsprres12n[6] = {17.,10.,5.,10.,9.,4.};
    cmplx  chprres12n[6] = {{6.,0.}, {7.,-13.}, {13., 1.},
                            {23.,0.}, {16.,24.}, {31.,0.}};
    dcmplx zhprres12n[6] = {{6.,0.}, {7.,-13.}, {13., 1.},
                            {23.,0.}, {16.,24.}, {31.,0.}};
 
    float  ssyrres13n[3][3] = {17., 10., 5.,
                                0., 10., 9.,
                                0., 0., 4.};
    double dsyrres13n[3][3] = {17., 10., 5.,
                                0., 10., 9.,
                                0., 0., 4.};
    cmplx  cherres13n[3][3] = {{{6.,0.}, {7.,-13.}, {13.,1.}},
                               {{0.,0.}, {23.,0.},  {16.,24.}},
                               {{0.,0.}, {0.,0.}, {31.,0.}}};
    dcmplx zherres13n[3][3] = {{{6.,0.}, {7.,-13.}, {13.,1.}},
                               {{0.,0.}, {23.,0.},  {16.,24.}},
                               {{0.,0.}, {0.,0.}, {31.,0.}}};
 
    float  ssp2res14n[6] = {38.,23.,13.,18.,14.,7.};
    double dsp2res14n[6] = {38.,23.,13.,18.,14.,7.};
    cmplx  chp2res14n[6] = {{3.,0.}, {7.,-10}, {9.,4.},
                            {23.,0.}, {14.,23.}, {26.,0.}};
    dcmplx zhp2res14n[6] = {{3.,0.}, {7.,-10}, {9.,4.},
                            {23.,0.}, {14.,23.}, {26.,0.}};
 
    float  ssy2res15n[3][3] = {38., 23., 13.,
                                0., 18., 14.,
                                0., 0.,   7.};
    double dsy2res15n[3][3] = {38., 23., 13.,
                                0., 18., 14.,
                                0., 0.,   7.};
    cmplx  che2res15n[3][3] = {{{3.,0.}, {7.,-10.}, {9.,4.}},
                               {{0.,0.}, {23.,0.}, {14.,23.}},
                               {{0.,0.}, {0.,0.},  {26.,0.}}};
    dcmplx zhe2res15n[3][3] = {{{3.,0.}, {7.,-10.}, {9.,4.}},
                               {{0.,0.}, {23.,0.}, {14.,23.}},
                               {{0.,0.}, {0.,0.},  {26.,0.}}};
    float  sgbres16n[28] = {  66.,   0., 164.,   0., 300.,   0., 480.,
                               0., 710.,   0., 996.,   0.,1162.,   0.,
                            1312.,   0.,1440.,   0.,1540.,   0.,1606.,
                               0.,1632.,   0.,1612.,   0.,1540.,   0.};
    double dgbres16n[28] = {  66.,   0., 164.,   0., 300.,   0., 480.,
                               0., 710.,   0., 996.,   0.,1162.,   0.,
                            1312.,   0.,1440.,   0.,1540.,   0.,1606.,
                               0.,1632.,   0.,1612.,   0.,1540.,   0.};
    cmplx  cgbres16n[10] = {{-8.,32.}, {0.,0.}, {-36.,70.},{ 0.,0.},
                            {-54.,100.}, {0.,0.}, {-72.,130.}, {0.,0.},
                            {-70.,150.}, {0.,0.}};
    dcmplx zgbres16n[10] = {{-8.,32.}, {0.,0.}, {-36.,70.}, {0.,0.},
                            {-54.,100.}, {0.,0.}, {-72.,130.}, {0.,0.},
                            {-70.,150.}, {0.,0.}};
 
    float  ssbres17n[14] = { 30.,  0., 78.,  0.,148.,  0.,244.,
                              0.,288.,  0.,316.,  0.,322.,  0.};
    double dsbres17n[14] = { 30.,  0., 78.,  0.,148.,  0.,244.,
                              0.,288.,  0.,316.,  0.,322.,  0.};
    cmplx  chbres17n[14] = {{12.,48.}, {0.,0.}, {32.,124.}, {0.,0.},
                            {68.,228.}, {0.,0.}, {128.,360.}, {0.,0.},
                            {216.,360.}, {0.,0.}, {332.,300.}, {0.,0.},
                            {476.,168.}, {0.,0.}};
    dcmplx zhbres17n[14] = {{12.,48.}, {0.,0.}, {32.,124.}, {0.,0.},
                            {68.,228.}, {0.,0.}, {128.,360.}, {0.,0.},
                            {216.,360.}, {0.,0.}, {332.,300.}, {0.,0.},
                            {476.,168.}, {0.,0.}};
 
    float  stbres18n[9] = {2.,7.,1.,8.,2.,8.,1.,8.,3.};
    double dtbres18n[9] = {2.,7.,1.,8.,2.,8.,1.,8.,3.};
    cmplx  ctbres18n[4] = {{-1.,23.}, {1.,25.}, {-9.,17.}, {4.,12.}};
    dcmplx ztbres18n[4] = {{-1.,23.}, {1.,25.}, {-9.,17.}, {4.,12.}};
 
    float  stpres20n[4] = {1.,3.,11.,24.};
    double dtpres20n[4] = {1.,3.,11.,24.};
    cmplx  ctpres20n[4] = {{1.,1.}, {2.,4.}, {3.,17.}, {4.,44.}};
    dcmplx ztpres20n[4] = {{1.,1.}, {2.,4.}, {3.,17.}, {4.,44.}};
/************************************************
*  INPUT DATA FOR LINEAR ALGEBRA SUBPROGRAMS :  *
************************************************/
 
    float  sx24[3] = { 3., 2., 1. };
    double dx24[3] = { 3., 2., 1. };
    float  sy24[7] = { 4., 0., 5., 0., 2., 0., 3. };
    double dy24[7] = { 4., 0., 5., 0., 2., 0., 3. };
    float  sa24[3][10] = { 1., 2., 3., 4., 0., 0., 0., 0., 0., 0.,
                           2., 2., 2., 2., 0., 0., 0., 0., 0., 0.,
                           3., 4., 2., 1., 0., 0., 0., 0., 0., 0. };
    double da24[3][10] = { 1., 2., 3., 4., 0., 0., 0., 0., 0., 0.,
                           2., 2., 2., 2., 0., 0., 0., 0., 0., 0.,
                           3., 4., 2., 1., 0., 0., 0., 0., 0., 0. };
    cmplx  cx24[3] = { { 1.0, 2.0 }, { 4.0, 0.0 }, { 1.0, 1.0 } };
    dcmplx zx24[3] = { { 1.0, 2.0 }, { 4.0, 0.0 }, { 1.0, 1.0 } };
    cmplx  cy24[5] = { { 1.0, 2.0 }, { 4.0, 0.0 }, { 1.0, -1.0 },
                       { 3.0, 4.0 }, { 2.0, 0.0 } };
    dcmplx zy24[5] = { { 1.0, 2.0 }, { 4.0, 0.0 }, { 1.0, -1.0 },
                       { 3.0, 4.0 }, { 2.0, 0.0 } };
    cmplx  ca24[3][5] = { { { 1.0, 2.0 }, { 2.0, 3.0 }, { 7.0, 4.0 },
                            { 8.0, 2.0 }, {9.0, 1.0 } },
                          { { 3.0, 5.0 }, { 7.0, 9.0 }, { 1.0, 4.0 },
                            { 2.0, 5.0 }, { 3.0, 6.0 } },
                          { { 2.0, 0.0 }, { 4.0, 8.0 }, { 6.0, 0.0 },
                            { 8.0, 0.0 }, { 1.0, 0.0 } } };
    dcmplx za24[3][5] = { { { 1.0, 2.0 }, { 2.0, 3.0 }, { 7.0, 4.0 },
                            { 8.0, 2.0 }, { 9.0, 1.0 } },
                          { { 3.0, 5.0 }, { 7.0, 9.0 }, { 1.0, 4.0 },
                            { 2.0, 5.0 }, { 3.0, 6.0 } },
                          { { 2.0, 0.0 }, { 4.0, 8.0 }, { 6.0, 0.0 },
                            { 8.0, 0.0 }, { 1.0, 0.0 } } };
    float  sx25[3] = { 3., 2., 1. };
    double dx25[3] = { 3., 2., 1. };
    float  sy25[7] = { 4., 0., 5., 0., 2., 0., 3. };
    double dy25[7] = { 4., 0., 5., 0., 2., 0., 3. };
    float  sa25[3][10] = { 1., 2., 3., 4., 0., 0., 0., 0., 0., 0.,
                           2., 2., 2., 2., 0., 0., 0., 0., 0., 0.,
                           3., 4., 2., 1., 0., 0., 0., 0., 0., 0. };
    double da25[3][10] = { 1., 2., 3., 4., 0., 0., 0., 0., 0., 0.,
                           2., 2., 2., 2., 0., 0., 0., 0., 0., 0.,
                           3., 4., 2., 1., 0., 0., 0., 0., 0., 0. };
    float  sx26[4] = { 3., 2., 1., 4. };
    double dx26[4] = { 3., 2., 1., 4. };
    float  sy26[5] = { 1., 0., 2., 0., 3. };
    double dy26[5] = { 1., 0., 2., 0., 3. };
    float  sa26[3][10] = { 1., 2., 3., 4., 0., 0., 0., 0., 0., 0.,
                           2., 2., 2., 2., 0., 0., 0., 0., 0., 0.,
                           3., 4., 2., 1., 0., 0., 0., 0., 0., 0. };
    double da26[3][10] = { 1., 2., 3., 4., 0., 0., 0., 0., 0., 0.,
                           2., 2., 2., 2., 0., 0., 0., 0., 0., 0.,
                           3., 4., 2., 1., 0., 0., 0., 0., 0., 0. };
    float  sa27[3][10] = { 1., 2., 3., 4., 0., 0., 0., 0., 0., 0.,
                           2., 2., 2., 2., 0., 0., 0., 0., 0., 0.,
                           3., 4., 2., 1., 0., 0., 0., 0., 0., 0. };
    double da27[3][10] = { 1., 2., 3., 4., 0., 0., 0., 0., 0., 0.,
                           2., 2., 2., 2., 0., 0., 0., 0., 0., 0.,
                           3., 4., 2., 1., 0., 0., 0., 0., 0., 0. };
    float  sx27[4] = { 3., 2., 1., 4. };
    double dx27[4] = { 3., 2., 1., 4. };
    float  sy27[5] = { 3., 0., 2., 0., 1. };
    double dy27[5] = { 3., 0., 2., 0., 1. };
    cmplx  cua27[3][5] = { { { 1.0, 2.0 }, { 2.0, 3.0 }, { 7.0, 4.0 },
                             { 8.0, 2.0 }, { 9.0, 1.0 } },
                           { { 3.0, 5.0 }, { 7.0, 9.0 }, { 1.0, 4.0 },
                             { 2.0, 5.0 }, { 3.0, 6.0 } },
                           { { 2.0, 0.0 }, { 4.0, 8.0 }, { 6.0, 0.0 },
                             { 8.0, 0.0 }, { 1.0, 0.0 } } };
    dcmplx zua27[3][5] = { { { 1.0, 2.0 }, { 2.0, 3.0 }, { 7.0, 4.0 },
                             { 8.0, 2.0 }, { 9.0, 1.0 } },
                           { { 3.0, 5.0 }, { 7.0, 9.0 }, { 1.0, 4.0 },
                             { 2.0, 5.0 }, { 3.0, 6.0 } },
                           { { 2.0, 0.0 }, { 4.0, 8.0 }, { 6.0, 0.0 },
                             { 8.0, 0.0 }, { 1.0, 0.0 } } };
    cmplx  cca27[3][5] = { { { 1.0, 2.0 }, { 2.0, 3.0 }, { 7.0, 4.0 },
                             { 8.0, 2.0 }, { 9.0, 1.0 } },
                           { { 3.0, 5.0 }, { 7.0, 9.0 }, { 1.0, 4.0 },
                             { 2.0, 5.0 }, { 3.0, 6.0 } },
                           { { 2.0, 0.0 }, { 4.0, 8.0 }, { 6.0, 0.0 },
                             { 8.0, 0.0 }, { 1.0, 0.0 } } };
    dcmplx zca27[3][5] = { { { 1.0, 2.0 }, { 2.0, 3.0 }, { 7.0, 4.0 },
                             { 8.0, 2.0 }, { 9.0, 1.0 } },
                           { { 3.0, 5.0 }, { 7.0, 9.0 }, { 1.0, 4.0 },
                             { 2.0, 5.0 }, { 3.0, 6.0 } },
                           { { 2.0, 0.0 }, { 4.0, 8.0 }, { 6.0, 0.0 },
                             { 8.0, 0.0 }, { 1.0, 0.0 } } };
    cmplx  cx27[5] = { { 1.0, 2.0 }, { 4.0, 0.0 }, { 1.0, 1.0 },
                       { 3.0, 4.0 }, { 2.0, 0.0 } };
    dcmplx zx27[5] = { { 1.0, 2.0 }, { 4.0, 0.0 }, { 1.0, 1.0 },
                       { 3.0, 4.0 }, { 2.0, 0.0 } };
    cmplx  cy27[3] = { { 1.0, 2.0 }, { 4.0, 0.0 }, { 1.0, -1.0 } };
    dcmplx zy27[3] = { { 1.0, 2.0 }, { 4.0, 0.0 }, { 1.0, -1.0 } };
    float  sap28[6] = { 8., 4., 2., 6., 7., 3. };
    double dap28[6] = { 8., 4., 2., 6., 7., 3. };
    float  sx28[3] = { 3., 2., 1. };
    double dx28[3] = { 3., 2., 1. };
    float  sy28[5] = { 5., 0., 3., 0., 2. };
    double dy28[5] = { 5., 0., 3., 0., 2. };
    float  sap29[6] = { 8., 4., 2., 6., 7., 3. };
    double dap29[6] = { 8., 4., 2., 6., 7., 3. };
    float  sx29[3] = { 3., 2., 1. };
    double dx29[3] = { 3., 2., 1. };
    float  sx30[3] = { 3., 2., 1. };
    double dx30[3] = { 3., 2., 1. };
    float  sy30[5] = { 5., 0., 3., 0., 2. };
    double dy30[5] = { 5., 0., 3., 0., 2. };
    float  sap30[6] = { 8., 4., 2., 6., 7., 3. };
    double dap30[6] = { 8., 4., 2., 6., 7., 3. };
    double dac31[4][6] = { 4., 4., 4., 4., 4., 4.,
                           7., 3., 2., 7., 1., 1.,
                           0., 2., 4., 1., 3., 1.,
                           0., 0., 0., 0., 0., 3. };
    _ESVINT    ika31[4][6] = { 1, 2, 3, 4, 5, 6, 3, 1, 2, 3, 1, 1,
                           1, 4, 5, 6, 4, 2, 1, 1, 1, 1, 1, 5 };
    double dx31[6] = { 1., 2., 3., 4., 5., 6. };
    double dac31a[2][5] = { 11., 21., 33., 42., 51.,
                             0., 23., 34., 44., 53. };
    _ESVINT    ika31a[2][5] = { 1, 1, 3, 2, 1, 1, 3, 4, 4, 3 };
    _ESVINT    n31a[1] = { 5 };
    _ESVINT    nt31a[1] = { 4 };
    double dad32[5][6] = { 4., 4., 4., 4., 4., 4., 0., 0., 0., 0.,
                           0., 1., 0., 0., 0., 0., 1., 1., 0., 3.,
                           2., 7., 3., 3., 7., 2., 4., 1., 0., 0. };
    _ESVINT    ila32[5] = { 0, -5, -4, -1, 2 };
    double dx32[6] = { 1., 2., 3., 4., 5., 6. };
    float  sab33[4][4] = {1.,1.,2.,3.,0.,1.,3.,4.,
                         0.,0.,1.,3.,0.,0.,0.,1.};
    float  sxb33[4] = {1.,2.,3.,4.};
    double dab33[4][4] = {1.,1.,2.,3.,0.,1.,3.,4.,
                         0.,0.,1.,3.,0.,0.,0.,1.};
    double dxb33[4] = {1.,2.,3.,4.};
    cmplx  cab33[4][4] = {{{0.,0.},{1.,1.},{1.,1.},{3.,3.}},
                          {{0.,0.},{0.,0.},{3.,3.},{4.,4.}},
                          {{0.,0.},{0.,0.},{0.,0.},{3.,3.}},
                          {{0.,0.},{0.,0.},{0.,0.},{0.,0.}}};
    cmplx  cxb33[4] = {{1.,1.},{2.,2.},{3.,3.},{4.,4.}};
    dcmplx zab33[4][4] = {{{0.,0.},{1.,1.},{1.,1.},{3.,3.}},
                          {{0.,0.},{0.,0.},{3.,3.},{4.,4.}},
                          {{0.,0.},{0.,0.},{0.,0.},{3.,3.}},
                          {{0.,0.},{0.,0.},{0.,0.},{0.,0.}}};
    dcmplx zxb33[4] = {{1.,1.},{2.,2.},{3.,3.},{4.,4.}};
    float  spmva10n[6] = {8.,4.,2.,6.,7.,3.};
    float  spmvx10n[3] = {3.,2.,1.};
    float  spmvy10n[5] = {5.,0.,3.,0.,2.};
    double dpmva10n[6] = {8.,4.,2.,6.,7.,3.};
    double dpmvx10n[3] = {3.,2.,1.};
    double dpmvy10n[5] = {5.,0.,3.,0.,2.};
    cmplx  chmva10n[6] = {{1.,0.}, {3.,-5.}, {2.,3.},
                          {7.,0.}, {4.,8.}, {6.,0.}};
    cmplx  chmvx10n[3] = {{1.,2.}, {4.,0.}, {3.,4.}};
    cmplx  chmvy10n[5] = {{1.,0.}, {0.,0.}, {2.,-1.},
                          {0.,0.}, {2.,1.}};
    dcmplx zhmva10n[6] = {{1.,0.}, {3.,-5.}, {2.,3.},
                          {7.,0.}, {4.,8.}, {6.,0.}};
    dcmplx zhmvx10n[3] = {{1.,2.}, {4.,0.}, {3.,4.}};
    dcmplx zhmvy10n[5] = {{1.,0.}, {0.,0.}, {2.,-1.},
                          {0.,0.}, {2.,1.}};
 
    float  symva11n[3][3] = {8.,4.,2.,
                             0.,6.,7.,
                             0.,0.,3.};
    float  symvx11n[3] = {3.,2.,1.};
    float  symvy11n[5] = {5.,0.,3.,0.,2.};
    double dymva11n[3][3] = {8.,4.,2.,
                             0.,6.,7.,
                             0.,0.,3.};
    double dymvx11n[3] = {3.,2.,1.};
    double dymvy11n[5] = {5.,0.,3.,0.,2.};
    cmplx  cemva11n[3][3] = {{{1.,0.}, {3.,-5.}, {2.,3.}},
                             {{0.,0.}, {7.,0.}, {4.,8.}},
                             {{0.,0.}, {0.,0.}, {6.,0.}}};
    cmplx  cemvx11n[3] = {{1.,2.}, {4.,0.}, {3.,4.}};
    cmplx  cemvy11n[5] = {{1.,0.}, {0.,0.}, {2.,-1.}, { 0.,0.}, {2.,1.}};
    dcmplx zemva11n[3][3] = {{{1.,0.}, {3.,-5.}, {2.,3.}},
                             {{0.,0.}, {7.,0.}, {4.,8.}},
                             {{0.,0.}, {0.,0.}, {6.,0.}}};
    dcmplx zemvx11n[3] = {{1.,2.}, {4.,0.}, {3.,4.}};
    dcmplx zemvy11n[5] = {{1.,0.},{ 0.,0.}, {2.,-1.}, {0.,0.}, {2.,1.}};
 
    float  sspra12n[6] = {8.,4.,2.,6.,7.,3.};
    float  ssprx12n[3] = {3.,2.,1.};
    double dspra12n[6] = {8.,4.,2.,6.,7.,3.};
    double dsprx12n[3] = {3.,2.,1.};
    cmplx  chpra12n[6] = {{1.,0.}, {3.,-5.}, {2.,3.},
                          {7.,0.}, {4.,8.},  {6.,0.}};
    cmplx  chprx12n[3] = {{1.,2.}, {4.,0.}, {3.,4.}};
    dcmplx zhpra12n[6] = {{1.,0.}, {3.,-5.}, {2.,3.},
                          {7.,0.}, {4.,8.},  {6.,0.}};
    dcmplx zhprx12n[3] = {{1.,2.}, {4.,0.}, {3.,4.}};
 
    float  ssyrx13n[3] = {3.,2.,1.};
    float  ssyra13n[3][3] = {8.,4.,2.,
                             0.,6.,7.,
                             0.,0.,3.};
    double dsyrx13n[3] = {3.,2.,1.};
    double dsyra13n[3][3] = {8.,4.,2.,
                             0.,6.,7.,
                             0.,0.,3.};
    cmplx  cherx13n[3] = {{1.,2.}, {4.,0.}, {3.,4.}};
    cmplx  chera13n[3][3] = {{{1.,0.}, {3.,-5.}, {2.,3.}},
                             {{0.,0.}, {7.,0.}, {4.,8.}},
                             {{0.,0.}, {0.,0.}, {6.,0.}}};
    dcmplx  zherx13n[3] = {{1.,2.}, {4.,0.}, {3.,4.}};
    dcmplx  zhera13n[3][3] = {{{ 1.,0.}, {3.,-5.}, {2.,3.}},
                              {{0.,0.}, {7.,0.}, {4.,8.}},
                              {{0.,0.}, {0.,0.}, {6.,0.}}};
 
    float  ssp2x14n[3] = {3.,2.,1.};
    float  ssp2y14n[5] = {5.,0.,3.,0.,2.};
    float  ssp2a14n[6] = {8.,4.,2.,6.,7.,3.};
    double dsp2x14n[3] = {3.,2.,1.};
    double dsp2y14n[5] = {5.,0.,3.,0.,2.};
    double dsp2a14n[6] = {8.,4.,2.,6.,7.,3.};
    cmplx  chp2x14n[3] = {{1.,2.}, {4.,0.}, {3.,4.}};
    cmplx  chp2y14n[5] = {{1.,0.}, {0.,0.}, {2.,-1.}, {0.,0.}, {2.,1.}};
    cmplx  chp2a14n[6] = {{1.,0.}, {3.,-5.}, {2.,3.},
                          {7.,0.}, {4.,8.}, {6.,0.}};
    dcmplx zhp2x14n[3] = {{1.,2.}, {4.,0.}, {3.,4.}};
    dcmplx zhp2y14n[5] = {{1.,0.}, {0.,0.}, {2.,-1.}, {0.,0.},{ 2.,1.}};
    dcmplx zhp2a14n[6] = {{1.,0.}, {3.,-5.}, {2.,3.},
                          {7.,0.}, {4.,8.}, {6.,0}};
 
    float  ssy2x15n[3] = {3.,2.,1.};
    float  ssy2y15n[5] = {5.,0.,3.,0.,2.};
    float  ssy2a15n[3][3] = {8.,4.,2.,
                             0.,6.,7.,
                             0.,0.,3.};
    double dsy2x15n[3] = {3.,2.,1.};
    double dsy2y15n[5] = {5.,0.,3.,0.,2.};
    double dsy2a15n[3][3] = {8.,4.,2.,
                             0.,6.,7.,
                             0.,0.,3.};
    cmplx  che2x15n[3] = {{1.,2.}, {4.,0.}, {3.,4.}};
    cmplx  che2y15n[5] = {{1.,0.}, {0.,0.}, {2.,-1.}, {0.,0.}, {2.,1.}};
    cmplx  che2a15n[3][3] = {{{1.,0.}, {3.,-5.},{ 2.,3.}},
                             {{ 0.,0.}, {7.,0.}, {4.,8.}},
                             {{ 0.,0.}, {0.,0.}, {6.,0.}}};
    dcmplx zhe2x15n[3] = {{1.,2.}, {4.,0.}, {3.,4.}};
    dcmplx zhe2y15n[5] = {{1.,0.}, {0.,0.}, {2.,-1.}, {0.,0.},{ 2.,1.}};
    dcmplx zhe2a15n[3][3] = {{{1.,0.}, {3.,-5.}, {2.,3.}},
                             {{ 0.,0.}, {7.,0.}, {4.,8.}},
                             {{ 0.,0.},{ 0.,0.}, {6.,0.}}};
    float  sgba16n[12][15] = {0.,0.,0.,0.,0.,0.,1.,2.,3.,4.,5.,6.,7.,0.,0.,
                              0.,0.,0.,0.,0.,1.,2.,3.,4.,5.,6.,7.,8.,0.,0.,
                              0.,0.,0.,0.,1.,2.,3.,4.,5.,6.,7.,8.,9.,0.,0.,
                              0.,0.,0.,1.,2.,3.,4.,5.,6.,7.,8.,9.,10.,0.,0.,
                              0.,0.,1.,2.,3.,4.,5.,6.,7.,8.,9.,10.,11.,0.,0.,
                              0.,1.,2.,3.,4.,5.,6.,7.,8.,9.,10.,11.,12.,0.,0.,
                              1.,2.,3.,4.,5.,6.,7.,8.,9.,10.,11.,12.,13.,0.,0.,
                              2.,3.,4.,5.,6.,7.,8.,9.,10.,11.,12.,13.,14.,0.,
                              0.,
                              3.,4.,5.,6.,7.,8.,9.,10.,11.,12.,13.,14.,0.,0.,
                              0.,
                              4.,5.,6.,7.,8.,9.,10.,11.,12.,13.,14.,0.,0.,0.,
                              0.,
                              5.,6.,7.,8.,9.,10.,11.,12.,13.,14.,0.,0.,0.,0.,
                              0.,
                              6.,7.,8.,9.,10.,11.,12.,13.,14.,0.,0.,0.,0.,0.,
                              0.};
    float  sgbx16n[12] = {1.,2.,3.,4.,5.,6.,7.,8.,9.,10.,11.,12.};
    float  sgby16n[28] = {1.,0.,2.,0.,3.,0.,4.,0.,5.,0.,6.,0.,7.,0.,
                          8.,0.,9.,0.,10.,0.,11.,0.,12.,0.,13.,0.,14.,0.};
    double dgba16n[12][15] = {0.,0.,0.,0.,0.,0.,1.,2.,3.,4.,5.,6.,7.,0.,0.,
                              0.,0.,0.,0.,0.,1.,2.,3.,4.,5.,6.,7.,8.,0.,0.,
                              0.,0.,0.,0.,1.,2.,3.,4.,5.,6.,7.,8.,9.,0.,0.,
                              0.,0.,0.,1.,2.,3.,4.,5.,6.,7.,8.,9.,10.,0.,0.,
                              0.,0.,1.,2.,3.,4.,5.,6.,7.,8.,9.,10.,11.,0.,0.,
                              0.,1.,2.,3.,4.,5.,6.,7.,8.,9.,10.,11.,12.,0.,0.,
                              1.,2.,3.,4.,5.,6.,7.,8.,9.,10.,11.,12.,13.,0.,0.,
                              2.,3.,4.,5.,6.,7.,8.,9.,10.,11.,12.,13.,14.,0.,
                              0.,
                              3.,4.,5.,6.,7.,8.,9.,10.,11.,12.,13.,14.,0.,0.,
                              0.,
                              4.,5.,6.,7.,8.,9.,10.,11.,12.,13.,14.,0.,0.,0.,
                              0.,
                              5.,6.,7.,8.,9.,10.,11.,12.,13.,14.,0.,0.,0.,0.,
                              0.,
                              6.,7.,8.,9.,10.,11.,12.,13.,14.,0.,0.,0.,0.,0.,
                              0.};
    double dgbx16n[12] = {1.,2.,3.,4.,5.,6.,7.,8.,9.,10.,11.,12.};
    double dgby16n[28] = {1.,0.,2.,0.,3.,0.,4.,0.,5.,0.,6.,0.,7.,0.,
                          8.,0.,9.,0.,10.,0.,11.,0.,12.,0.,13.,0.,14.,0.};
 
    cmplx  cgba16n[4][8] = {{{0.,0.}, {0.,0.}, {1.,1.}, {2.,2.},
                             {3.,3.}, {4.,4.}, {0.,0.}, {0.,0.}},
                            {{0.,0.}, {1.,1.}, {2.,2.}, {3.,3.},
                             {4.,4.}, {5.,5.}, {0.,0.}, {0.,0.}},
                            {{1.,1.}, {2.,2.}, {3.,3.}, {4.,4.},
                             {5.,5.}, {0.,0.}, {0.,0.}, {0.,0.}},
                            {{2.,2.}, {3.,3.}, {4.,4.}, {5.,5.},
                             {0.,0.}, {0.,0.}, {0.,0.}, {0.,0.}}};
    cmplx  cgbx16n[5] = {{1.,2.}, {2.,3.}, {3.,4.}, {4.,5.}, {5.,6.}};
    cmplx  cgby16n[10] = {{1.,2.}, {0.,0.}, {2.,3.}, {0.,0.}, {3.,4.},
                          {0.,0.}, {4.,5.}, {0.,0.}, {5.,6.}, {0.,0.}};
    dcmplx zgba16n[4][8] = {{{0.,0.}, {0.,0.}, {1.,1.}, {2.,2.},
                             {3.,3.}, {4.,4.}, {0.,0.}, {0.,0.}},
                            {{0.,0.}, {1.,1.}, {2.,2.}, {3.,3.},
                             {4.,4.}, {5.,5.}, {0.,0.}, {0.,0.}},
                            {{1.,1.}, {2.,2.}, {3.,3.}, {4.,4.},
                             {5.,5.}, {0.,0.}, {0.,0.}, {0.,0.}},
                            {{2.,2.}, {3.,3.}, {4.,4.}, {5.,5.},
                             {0.,0.}, {0.,0.}, {0.,0.}, {0.,0.}}};
    dcmplx zgbx16n[5] = {{1.,2.}, {2.,3.}, {3.,4.}, {4.,5.}, {5.,6.}};
    dcmplx zgby16n[10] = {{1.,2.}, {0.,0.}, {2.,3.}, {0.,0.}, {3.,4.},
                          {0.,0.}, {4.,5.}, {0.,0.}, {5.,6.}, {0.,0.}};
 
    float  ssba17n[7][5]= {0.,0.,0.,1.,0.,
                           0.,0.,1.,2.,0.,
                           0.,1.,2.,3.,0.,
                           1.,2.,3.,4.,0.,
                           2.,3.,4.,5.,0.,
                           3.,4.,5.,6.,0.,
                           4.,5.,6.,7.,0.};
    float  ssbx17n[7] = {1.,2.,3.,4.,5.,6.,7.};
    float  ssby17n[14] = {1.,0.,2.,0.,3.,0.,4.,0.,5.,0.,6.,0.,7.,0.};
    double dsba17n[7][5] = {0.,0.,0.,1.,0.,
                            0.,0.,1.,2.,0.,
                            0.,1.,2.,3.,0.,
                            1.,2.,3.,4.,0.,
                            2.,3.,4.,5.,0.,
                            3.,4.,5.,6.,0.,
                            4.,5.,6.,7.,0.};
    double dsbx17n[7] = {1.,2.,3.,4.,5.,6.,7.};
    double dsby17n[14] = {1.,0.,2.,0.,3.,0.,4.,0.,5.,0.,6.,0.,7.,0.};
    cmplx  chba17n[7][5] = {{{0.,0.}, {0.,0.}, {0.,0.}, {1.,0.}, {0.,0.}},
                            {{0.,0.}, {0.,0.}, {1.,1.}, {2.,0.}, {0.,0.}},
                            {{0.,0.}, {1.,1.}, {2.,2.}, {3.,0.}, {0.,0.}},
                            {{1.,1.}, {2.,2.}, {3.,3.}, {4.,0.}, {0.,0.}},
                            {{2.,2.}, {3.,3.}, {4.,4.}, {5.,0.}, {0.,0.}},
                            {{3.,3.}, {4.,4.}, {5.,5.}, {6.,0.}, {0.,0.}},
                            {{4.,4.}, {5.,5.}, {6.,6.}, {7.,0.}, {0.,0.}}};
    cmplx  chbx17n[7] = {{1.,1.}, {2.,2.}, {3.,3.}, {4.,4.},{ 5.,5.},
                         {6.,6.}, {7.,7.}};
    cmplx  chby17n[14] = {{1.,1.}, {0.,0.}, {2.,2.}, {0.,0.}, {3.,3.},
                          {0.,0.}, {4.,4.}, {0.,0.}, {5.,5.}, {0.,0.},
                          {6.,6.}, {0.,0.}, {7.,7.}, {0.,0.}};
    dcmplx zhba17n[7][5] = {{{0.,0.}, {0.,0.}, {0.,0.}, {1.,0.}, {0.,0.}},
                            {{0.,0.}, {0.,0.}, {1.,1.}, {2.,0.}, {0.,0.}},
                            {{0.,0.}, {1.,1.}, {2.,2.}, {3.,0.}, {0.,0.}},
                            {{1.,1.}, {2.,2.}, {3.,3.}, {4.,0.}, {0.,0.}},
                            {{2.,2.}, {3.,3.}, {4.,4.}, {5.,0.}, {0.,0.}},
                            {{3.,3.}, {4.,4.}, {5.,5.}, {6.,0.}, {0.,0.}},
                            {{4.,4.}, {5.,5.}, {6.,6.}, {7.,0.}, {0.,0.}}};
    dcmplx  zhbx17n[7] = {{1.,1.}, {2.,2.}, {3.,3.}, {4.,4.}, {5.,5.},
                          {6.,6.}, {7.,7.}};
    dcmplx  zhby17n[14] = {{1.,1.}, {0.,0.}, {2.,2.}, {0.,0.}, {3.,3.},
                           {0.,0.}, {4.,4.}, {0.,0.}, {5.,5.}, {0.,0.},
                           {6.,6.}, {0.,0.}, {7.,7.}, {0.,0.}};
 
    float  stba18n[9][3] = {0.,0.,1., 0.,1.,4., 1.,2.,4.,
                            3.,1.,4., 1.,2.,3., 2.,1.,3.,
                            1.,2.,3., 2.,1.,2., 0.,2.,1.};
    float  stbx18n[9] = {1.,1.,0.,1.,0.,2.,0.,1.,3.};
    double dtba18n[9][3] = {0.,0.,1., 0.,1.,4., 1.,2.,4.,
                            3.,1.,4., 1.,2.,3., 2.,1.,3.,
                            1.,2.,3., 2.,1.,2., 0.,2.,1.};
    double dtbx18n[9] = {1.,1.,0.,1.,0.,2.,0.,1.,3.};
    cmplx  ctba18n[4][5] = {{{0.,0.}, {0.,0.}, {0.,0.}, {0.,0.}, {1.,1.}},
                            {{0.,0.}, {0.,0.}, {0.,0.}, {2.,0.}, {1.,2.}},
                            {{0.,0.}, {0.,0.}, {2.,1.}, {1.,0.}, {0.,3.}},
                            {{0.,0.}, {0.,2.}, {2.,2.}, {1.,1.}, {2.,1.}}};
    cmplx  ctbx18n[4] = {{1.,1.}, {2.,2.}, {3.,3.}, {4.,4.}};
    dcmplx ztba18n[4][5] ={{{0.,0.}, {0.,0.}, {0.,0.}, {0.,0.}, {1.,1.}},
                           {{0.,0.}, {0.,0.}, {0.,0.}, {2.,0.}, {1.,2.}},
                           {{0.,0.}, {0.,0.}, {2.,1.}, {1.,0.}, {0.,3.}},
                           {{0.,0.}, {0.,2.}, {2.,2.}, {1.,1.}, {2.,1.}}};
    dcmplx ztbx18n[4] = {{1.,1.}, {2.,2.}, {3.,3.}, {4.,4.}};
 
    float  stpa20n[10] = {1.,1.,2.,3.,1.,3.,4.,1.,3.,1.};
    float  stpx20n[4] = {1.,2.,3.,4.};
    double dtpa20n[10] = {1.,1.,2.,3.,1.,3.,4.,1.,3.,1.};
    double dtpx20n[4] = {1.,2.,3.,4.};
    cmplx  ctpa20n[10] = {{1.,0.}, {1.,1.}, {1.,1.}, {3.,3.}, {1.,0.},
                          {3.,3.}, {4.,4.}, {1.,0.}, {3.,3.}, {1.,0.}};
    cmplx  ctpx20n[4] = {{1.,1.}, {2.,2.}, {3.,3.}, {4.,4.}};
    dcmplx ztpa20n[10] = {{1.,0.}, {1.,1.}, {1.,1.}, {3.,3.}, {1.,0.},
                          {3.,3.}, {4.,4.}, {1.,0.}, {3.,3.}, {1.,0.}};
    dcmplx ztpx20n[4] = {{1.,1.}, {2.,2.}, {3.,3.}, {4.,4.}};
 
/****************************************************
*  CALL STATEMENT FOR LINEAR ALGEBRA SUBROUTINES :  *
****************************************************/
 
/***************************
*  MATRIX-VECTOR LINEAR :  *
***************************/
 
    cmplx  c4;
    dcmplx c8;
 
    sgemv ("N", 4, 3, 1., sa24, 10, sx24, 1, 1., sy24, 2);
    svecck(4, 2, sy24, sres24, "sgemv", pass);
    dgemv ("N", 4, 3, 1., da24, 10, dx24, 1, 1., dy24, 2);
    dvecck(4, 2, dy24, dres24, "dgemv", pass);
    CSET(c4, 1., 0.);
    cgemv ("N", 5, 3, c4, ca24, 5, cx24, 1, c4, cy24, 1);
    cvecck(5, 1, cy24, cres24, "cgemv", pass);
    CSET(c8, 1., 0.);
    zgemv ("N", 5, 3, c8, za24, 5, zx24, 1, c8, zy24, 1);
    zvecck(5, 1, zy24, zres24, "zgemv", pass);
    sgemx (4, 3, 1., sa25, 10, sx25, 1, sy25, 2);
    svecck(4, 2, sy25, sres25, "sgemx", pass);
    dgemx (4, 3, 1., da25, 10, dx25, 1, dy25, 2);
    dvecck(4, 2, dy25, dres25, "dgemx", pass);
    sgemtx (4, 3, 1., sa26, 10, sx26, 1, sy26, 2);
    svecck(3, 2, sy26, sres26, "sgemtx", pass);
    dgemtx (4, 3, 1., da26, 10, dx26, 1, dy26, 2);
    dvecck(3, 2, dy26, dres26, "dgemtx", pass);
    sger1 (4, 3, 1., sx27, 1, sy27, -2, sa27, 10);
    smtchk(4, 3, sa27, sres27, 10, "sger1", pass);
    dger1 (4, 3, 1., dx27, 1, dy27, -2, da27, 10);
    dmtchk(4, 3, da27, dres27, 10, "dger1", pass);
    CSET(c4, 1., 0.);
    cgeru (5, 3, c4, cx27, 1, cy27, 1, cua27, 5);
    cmtchk(5, 3, cua27, cure27, 5, "cgeru", pass);
    CSET(c8, 1., 0.);
    zgeru (5, 3, c8, zx27, 1, zy27, 1, zua27, 5);
    zmtchk(5, 3, zua27, zure27, 5, "zgeru", pass);
    CSET(c4, 1., 0.);
    cgerc (5, 3, c4, cx27, 1, cy27, 1, cca27, 5);
    cmtchk(5, 3, cca27, ccre27, 5, "cgerc", pass);
    CSET(c8, 1., 0.);
    zgerc (5, 3, c8, zx27, 1, zy27, 1, zca27, 5);
    zmtchk(5, 3, zca27, zcre27, 5, "zgerc", pass);
    sslmx (3, 1., sap28, sx28, 1, sy28, 2);
    svecck(3, 2, sy28, sres28, "sslmx", pass);
    dslmx (3, 1., dap28, dx28, 1, dy28, 2);
    dvecck(3, 2, dy28, dres28, "dslmx", pass);
    sslr1 (3, 1., sx29, 1, sap29);
    svecck(6, 1, sap29, sres29, "sslr1", pass);
    dslr1 (3, 1., dx29, 1, dap29);
    dvecck(6, 1, dap29, dres29, "dslr1", pass);
    sslr2 (3, 1., sx30, 1, sy30, 2, sap30);
    svecck(6, 1, sap30, sres30, "sslr2", pass);
    dslr2 (3, 1., dx30, 1, dy30, 2, dap30);
    dvecck(6, 1, dap30, dres30, "dslr2", pass);
    strmv("L","N","U",4,sab33,4,sxb33,1);
    svecck(4,1,sxb33,sreb33,"strmv",pass);
    dtrmv("L","N","U",4,dab33,4,dxb33,1);
    dvecck(4,1,dxb33,dreb33,"dtrmv",pass);
    ctrmv("L","N","U",4,cab33,4,cxb33,1);
    cvecck(4,1,cxb33,creb33,"ctrmv",pass);
    ztrmv("L","N","U",4,zab33,4,zxb33,1);
    zvecck(4,1,zxb33,zreb33,"ztrmv",pass);
 
    sspmv("L",3,1.,spmva10n,spmvx10n,1,1.,spmvy10n,2);
    svecck(5,1,spmvy10n,spmvres10n,"sspmv",pass);
    dspmv("L",3,1.,dpmva10n,dpmvx10n,1,1.,dpmvy10n,2);
    dvecck(5,1,dpmvy10n,dpmvres10n,"dspmv",pass);
    CSET(c1n, 1., 0.);
    chpmv("L",3,c1n,chmva10n,chmvx10n,1,c1n,chmvy10n,2);
    cvecck(5,1,chmvy10n,chmvres10n,"chpmv",pass);
    CSET(dc1n, 1., 0.);
    zhpmv("L",3,dc1n,zhmva10n,zhmvx10n,1,dc1n,zhmvy10n,2);
    zvecck(5,1,zhmvy10n,zhmvres10n,"zhpmv",pass);
 
    ssymv("L",3,1.,symva11n,3,symvx11n,1,1.,symvy11n,2);
    svecck(5,1,symvy11n,symvres11n,"ssymv",pass);
    dsymv("L",3,1.,dymva11n,3,dymvx11n,1,1.,dymvy11n,2);
    dvecck(5,1,dymvy11n,dymvres11n,"dsymv",pass);
    chemv("L",3,c1n,cemva11n,3,cemvx11n,1,c1n,cemvy11n,2);
    cvecck(5,1,cemvy11n,cemvres11n,"chemv",pass);
    zhemv("L",3,dc1n,zemva11n,3,zemvx11n,1,dc1n,zemvy11n,2);
    zvecck(5,1,zemvy11n,zemvres11n,"zhemv",pass);
 
    sspr("L",3,1.,ssprx12n,1,sspra12n);
    svecck(6,1,sspra12n,ssprres12n,"sspr",pass);
    dspr("L",3,1.,dsprx12n,1,dspra12n);
    dvecck(6,1,dspra12n,dsprres12n,"dspr",pass);
    chpr("L",3,1.,chprx12n,1,chpra12n);
    cvecck(6,1,chpra12n,chprres12n,"chpr",pass);
    zhpr("L",3,1.,zhprx12n,1,zhpra12n);
    zvecck(6,1,zhpra12n,zhprres12n,"zhpr",pass);
 
    ssyr("L",3,1.,ssyrx13n,1,ssyra13n,3);
    smtchk(3,3,ssyra13n,ssyrres13n,3,"ssyr",pass);
    dsyr("L",3,1.,dsyrx13n,1,dsyra13n,3);
    dmtchk(3,3,dsyra13n,dsyrres13n,3,"dsyr",pass);
    cher("L",3,1.,cherx13n,1,chera13n,3);
    cmtchk(3,3,chera13n,cherres13n,3,"cher",pass);
    zher("L",3,1.,zherx13n,1,zhera13n,3);
    zmtchk(3,3,zhera13n,zherres13n,3,"zher",pass);
 
    sspr2("L",3,1.,ssp2x14n,1,ssp2y14n,2,ssp2a14n);
    svecck(6,1,ssp2a14n,ssp2res14n,"sspr2",pass);
    dspr2("L",3,1.,dsp2x14n,1,dsp2y14n,2,dsp2a14n);
    dvecck(6,1,dsp2a14n,dsp2res14n,"dspr2",pass);
    chpr2("L",3,c1n,chp2x14n,1,chp2y14n,2,chp2a14n);
    cvecck(6,1,chp2a14n,chp2res14n,"chpr2",pass);
    zhpr2("L",3,dc1n,zhp2x14n,1,zhp2y14n,2,zhp2a14n);
    zvecck(6,1,zhp2a14n,zhp2res14n,"zhpr2",pass);
 
    ssyr2("L",3,1.,ssy2x15n,1,ssy2y15n,2,ssy2a15n,3);
    smtchk(3,3,ssy2a15n,ssy2res15n,3,"ssyr2",pass);
    dsyr2("L",3,1.,dsy2x15n,1,dsy2y15n,2,dsy2a15n,3);
    dmtchk(3,3,dsy2a15n,dsy2res15n,3,"dsyr2",pass);
    cher2("L",3,c1n,che2x15n,1,che2y15n,2,che2a15n,3);
    cmtchk(3,3,che2a15n,che2res15n,3,"cher2",pass);
    zher2("L",3,dc1n,zhe2x15n,1,zhe2y15n,2,zhe2a15n,3);
    zmtchk(3,3,zhe2a15n,zhe2res15n,3,"zher2",pass);
    sgbmv("N",14,12,6,6,2.,sgba16n,15,sgbx16n,1,10.,sgby16n,2);
    svecck(28,1,sgby16n,sgbres16n,"sgbmv",pass);
    dgbmv("N",14,12,6,6,2.,dgba16n,15,dgbx16n,1,10.,dgby16n,2);
    dvecck(28,1,dgby16n,dgbres16n,"dgbmv",pass);
    CSET(c1n, 1.,1.);
    CSET(c2n, 10.,0.);
    cgbmv("N",5,4,3,2,c1n,cgba16n,8,cgbx16n,1,c2n,cgby16n,2);
    cvecck(10,1,cgby16n,cgbres16n,"cgbmv",pass);
    CSET(dc1n, 1.,1.);
    CSET(dc2n, 10.,0.);
    zgbmv("N",5,4,3,2,dc1n,zgba16n,8,zgbx16n,1,dc2n,zgby16n,2);
    zvecck(10,1,zgby16n,zgbres16n,"zgbmv",pass);
 
    ssbmv("U",7,3,2.,ssba17n,5,ssbx17n,1,10.,ssby17n,2);
    svecck(14,1,ssby17n,ssbres17n,"ssbmv",pass);
    dsbmv("U",7,3,2.,dsba17n,5,dsbx17n,1,10.,dsby17n,2);
    dvecck(14,1,dsby17n,dsbres17n,"dsbmv",pass);
    CSET(c1n, 2., 0.);
    CSET(c2n, 10.,0.);
    chbmv("U",7,3,c1n,chba17n,5,chbx17n,1,c2n,chby17n,2);
    cvecck(14,1,chby17n,chbres17n,"chbmv",pass);
    CSET(dc1n, 2., 0.);
    CSET(dc2n, 10.,0.);
    zhbmv("U",7,3,dc1n,zhba17n,5,zhbx17n,1,dc2n,zhby17n,2);
    zvecck(14,1,zhby17n,zhbres17n,"zhbmv",pass);
 
    stbmv("U","N","N",9,2,stba18n,3,stbx18n,1);
    svecck(9,1,stbx18n,stbres18n,"stbmv",pass);
    dtbmv("U","N","N",9,2,dtba18n,3,dtbx18n,1);
    dvecck(9,1,dtbx18n,dtbres18n,"dtbmv",pass);
    ctbmv("U","N","N",4,4,ctba18n,5,ctbx18n,1);
    cvecck(4,1,ctbx18n,ctbres18n,"ctbmv",pass);
    ztbmv("U","N","N",4,4,ztba18n,5,ztbx18n,1);
    zvecck(4,1,ztbx18n,ztbres18n,"ztbmv",pass);
 
    stpmv("L","N","U",4,stpa20n,stpx20n,1);
    svecck(4,1,stpx20n,stpres20n,"stpmv",pass);
    dtpmv("L","N","U",4,dtpa20n,dtpx20n,1);
    dvecck(4,1,dtpx20n,dtpres20n,"dtpmv",pass);
    ctpmv("L","N","U",4,ctpa20n,ctpx20n,1);
    cvecck(4,1,ctpx20n,ctpres20n,"ctpmv",pass);
    ztpmv("L","N","U",4,ztpa20n,ztpx20n,1);
    zvecck(4,1,ztpx20n,ztpres20n,"ztpmv",pass);
 
/**********************************
*  SPARSE MATRIX-VECTOR LINEAR :  *
**********************************/
 
    dsmmx (6, 4, dac31, ika31, 6, dx31, dy31);
    dvecck(6, 1, dy31, dres31, "dsmmx", pass);
    dsmtm (5, 2, dac31a, ika31a, 5, &n31a[0], &nt31a[0], dat31a, ikt31a,
           6, saux, 0);
    intchk(1, 1, n31a, nr31a, "dsmtm", pass);
    intchk(1, 1, nt31a, ntr31a, "dsmtm", pass);
    dmtchk(5, 4, dat31a, dre31a, 6, "dsmtm", pass);
    imtchk(5, 4, ikt31a, ktr31a, 6, "dsmtm", pass);
    dsdmx (0, 6, 5, dad32, 6, "N", ila32, dx32, dy32);
    dvecck(6, 1, dy32, dres32, "dsdmx", pass);
 
}
 
/*********************************************************************
*                      SUBROUTINE:  MAOPS                            *
*********************************************************************/
 
void maops(pass)
_ESVINT *pass;
{
/*************************************************
*  VARIABLES OF MATRIX OPERATIONS SUBROUTINES :  *
*************************************************/
 
    float  saux[10];
    double daux[10];
    float  sc33[3][5];
    double dc33[3][5];
    cmplx  cc33[3][5];
    dcmplx zc33[3][5];
    float  sc34[3][5];
    double dc34[3][5];
    cmplx  cc34[3][5];
    dcmplx zc34[3][5];
    float  sc35[4][7];
    double dc35[4][7];
    cmplx  cc35[3][3];
    dcmplx zc35[3][3];
    cmplx  cc36[3][4];
    dcmplx zc36[3][4];
    cmplx  c1n,c2n;
    dcmplx dc1n,dc2n;
 
/*******************************************
* ADDED FOR Z/CGECMO, Z/CGECMI       10/12 *
*******************************************/
 
    dcmplx zb60[7][6];
    cmplx  cb60[7][6];
 
/******************************************************
*  EXPECTED VALUE FOR MATRIX OPERATION SUBROUTINES :  *
******************************************************/
 
    float  sres33[3][5] = { 110011., 210021., 310031., 410041., 000000.,
                            120012., 220022., 320032., 420042., 000000.,
                            130013., 230023., 330033., 430043., 000000. };
    double dres33[3][5] = { 110011., 210021., 310031., 410041., 000000.,
                            120012., 220022., 320032., 420042., 000000.,
                            130013., 230023., 330033., 430043., 000000. };
    cmplx  cres33[3][5] = { { { 2., 13. }, { 6., 8. }, { 9., 5. },
                              { 13., 8. }, { 0., 0. } },
                            { { 11., 9. }, { 14., 11. }, { 11., 10. },
                              { 9., 10. }, { 0., 0. } },
                            { { 4., 11. }, { 7., 11. }, { 5., 12. },
                              { 2., 10. }, { 0., 0. } } };
    dcmplx zres33[3][5] = { { { 2., 13. }, { 6., 8. }, { 9., 5. },
                              { 13., 8. }, { 0., 0. } },
                            { { 11., 9. }, { 14., 11. }, { 11., 10. },
                              { 9., 10. }, { 0., 0. } },
                            { { 4., 11. }, { 7., 11. }, { 5., 12. },
                              { 2., 10. }, { 0., 0. } } };
    float  sres34[3][5] = { 110011., 210021., 310031., 410041., 000000.,
                            120012., 220022., 320032., 420042., 000000.,
                            130013., 230023., 330033., 430043., 000000. };
    double dres34[3][5] = { 110011., 210021., 310031., 410041., 000000.,
                            120012., 220022., 320032., 420042., 000000.,
                            130013., 230023., 330033., 430043., 000000. };
    cmplx  cres34[3][5] = { { { 0., -3. }, { -2., 0. }, { -3., 1. },
                              { -1., 4. }, { 0., 0. } },
                            { { 7., -5. }, { 2., -5. }, { 3., 0. },
                              { -3., 2. }, { 0., 0. } },
                            { { -2., 7. }, { -5., 5. }, { -3., 2. },
                              { 0., -2. }, { 0., 0. } } };
    dcmplx zres34[3][5] = { { { 0., -3. }, { -2., 0. }, { -3., 1. },
                              { -1., 4. }, { 0., 0. } },
                            { { 7., -5. }, { 2., -5. }, { 3., 0. },
                              { -3., 2. }, { 0., 0. } },
                            { { -2., 7. }, { -5., 5. }, { -3., 2. },
                              { 0., -2. }, { 0., 0. } } };
    float  sres35[4][7] = { 23., -4., 3., -3., -5., 15., 0.,
                            12., -5., 0., 5., -7., 6., 0.,
                            -6., 1., 1., -2., 4., -5., 0.,
                             2., 3., 4., -10., 4., 6., 0. };
    double dres35[4][7] = { 23., -4., 3., -3., -5., 15., 0.,
                            12., -5., 0., 5., -7., 6., 0.,
                            -6., 1., 1., -2., 4., -5., 0.,
                             2., 3., 4., -10., 4., 6., 0. };
    cmplx  cres35[3][3] = { { { -45., 85. }, { -50., 90. }, { 0., 0. } },
                            { { 20., 93. }, { 12., 79. }, { 0., 0. } },
                            { { -13., 110. }, { 3., 94. }, { 0., 0. } } };
    dcmplx zres35[3][3] = { { { -45., 85. }, { -50., 90. }, { 0., 0. } },
                            { { 20., 93. }, { 12., 79. }, { 0., 0. } },
                            { { -13., 110. }, { 3., 94. }, { 0., 0. } } };
    cmplx  cres36[3][4] = { { { 20., -1. }, { 18., -23. }, { 26., -23. },
                              { 0., 0. } },
                            { { 12., 25. }, { 80., -2. }, { 56., 37. },
                              { 0., 0. } },
                            { { 24., 26. }, { 49., -37. }, { 76., 2. },
                              { 0., 0. } } };
    dcmplx zres36[3][4] = { { { 20., -1. }, { 18., -23. }, { 26., -23. },
                              { 0., 0. } },
                            { { 12., 25. }, { 80., -2. }, { 56., 37. },
                              { 0., 0. } },
                            { { 24., 26. }, { 49., -37. }, { 76., 2. },
                              { 0., 0. } } };
    float  sre36a[4][7] = { 24., -3., 4., -2., -4., 16., 0.,
                            13., -4., 1., 6., -6., 7., 0.,
                            -5., 2., 2., -1., 5., -4., 0.,
                            3., 4., 5., -9., 5., 7., 0. };
    double dre36a[4][7] = { 24., -3., 4., -2., -4., 16., 0.,
                            13., -4., 1., 6., -6., 7., 0.,
                            -5., 2., 2., -1., 5., -4., 0.,
                             3., 4., 5., -9., 5., 7., 0. };
    cmplx  cre36a[2][8] = { { { -22., 113. }, { -19., 114. }, { -20., 119. },
                              { -27., 110. }, { 8., 103. }, { -55., 116. },
                              { 0., 0. }, { 0., 0. } },
                            { { -35., 142. }, { -35., 141. }, { -43., 146. },
                              { -58., 131. }, { 0., 112. }, { -75., 135. },
                              { 0., 0. }, { 0., 0. } } };
    dcmplx zre36a[2][8] = { { { -22., 113. }, { -19., 114. }, { -20., 119. },
                              { -27., 110. }, { 8., 103. }, { -55., 116. },
                              { 0., 0. }, { 0., 0. } },
                            { { -35., 142. }, { -35., 141. }, { -43., 146. },
                              { -58., 131. }, { 0., 112. }, { -75., 135. },
                              { 0., 0. }, { 0., 0. } } };
    float  sres97[7][10] = { 0., 0.,  0.,  0.,  0.,  0., 0., 0., 0., 0.,
                             0., 0.,  0.,  0.,  0.,  0., 0., 0., 0., 0.,
                             0., 1.,  6., 11., 16., 21., 0., 0., 0., 0.,
                             0., 2.,  7., 12., 17., 22., 0., 0., 0., 0.,
                             0., 3.,  8., 13., 18., 23., 0., 0., 0., 0.,
                             0., 4.,  9., 14., 19., 24., 0., 0., 0., 0.,
                             0., 5., 10., 15., 20., 25., 0., 0., 0., 0. };
    double dres97[7][10] = { 0., 0.,  0.,  0.,  0.,  0., 0., 0., 0., 0.,
                             0., 0.,  0.,  0.,  0.,  0., 0., 0., 0., 0.,
                             0., 1.,  6., 11., 16., 21., 0., 0., 0., 0.,
                             0., 2.,  7., 12., 17., 22., 0., 0., 0., 0.,
                             0., 3.,  8., 13., 18., 23., 0., 0., 0., 0.,
                             0., 4.,  9., 14., 19., 24., 0., 0., 0., 0.,
                             0., 5., 10., 15., 20., 25., 0., 0., 0., 0. };
    cmplx  cres97[7][10] = {
        { {  0., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. },
          {  0., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. },
          {  0., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 1., 0. }, {  6., 0. }, { 11., 0. }, { 16., 0. },
          { 21., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 2., 0. }, {  7., 0. }, { 12., 0. }, { 17., 0. },
          { 22., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 3., 0. }, {  8., 0. }, { 13., 0. }, { 18., 0. },
          { 23., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 4., 0. }, {  9., 0. }, { 14., 0. }, { 19., 0. },
          { 24., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 5., 0. }, { 10., 0. }, { 15., 0. }, { 20., 0. },
          { 25., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } } };
    dcmplx zres97[7][10] = {
        { {  0., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. },
          {  0., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. },
          {  0., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 1., 0. }, {  6., 0. }, { 11., 0. }, { 16., 0. },
          { 21., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 2., 0. }, {  7., 0. }, { 12., 0. }, { 17., 0. },
          { 22., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 3., 0. }, {  8., 0. }, { 13., 0. }, { 18., 0. },
          { 23., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 4., 0. }, {  9., 0. }, { 14., 0. }, { 19., 0. },
          { 24., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 5., 0. }, { 10., 0. }, { 15., 0. }, { 20., 0. },
          { 25., 0. }, { 0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } } };
    float  sres98[7][6] = { 0., 0.,  0.,  0.,  0., 0.,
                            0., 1.,  6., 11., 16., 0.,
                            0., 2.,  7., 12., 17., 0.,
                            0., 3,   8., 13., 18., 0.,
                            0., 4.,  9., 14., 19., 0.,
                            0., 5., 10., 15., 20., 0.,
                            0., 0.,  0.,  0.,  0., 0. };
    double dres98[7][6] = { 0., 0.,  0.,  0.,  0., 0.,
                            0., 1.,  6., 11., 16., 0.,
                            0., 2.,  7., 12., 17., 0.,
                            0., 3,   8., 13., 18., 0.,
                            0., 4.,  9., 14., 19., 0.,
                            0., 5., 10., 15., 20., 0.,
                            0., 0.,  0.,  0.,  0., 0. };
    cmplx  cres98[7][6] = { { {  0., 0. }, {  0., 0. }, {  0., 0. },
                              {  0., 0. }, {  0., 0. }, {  0., 0. } },
                            { {  0., 0. }, {  1., 0. }, {  6., 0. },
                              { 11., 0. }, { 16., 0. }, {  0., 0. } },
                            { {  0., 0. }, {  2., 0. }, {  7., 0. },
                              { 12., 0. }, { 17., 0. }, {  0., 0. } },
                            { {  0., 0. }, {  3., 0. }, {  8., 0. },
                              { 13., 0. }, { 18., 0. }, {  0., 0. } },
                            { {  0., 0. }, {  4., 0. }, {  9., 0. },
                              { 14., 0. }, { 19., 0. }, {  0., 0. } },
                            { {  0., 0. }, {  5., 0. }, { 10., 0. },
                              { 15., 0. }, { 20., 0. }, {  0., 0. } },
                            { {  0., 0. }, {  0., 0. }, {  0., 0. },
                              {  0., 0. }, {  0., 0. }, {  0., 0. } } };
    dcmplx zres98[7][6] = { { {  0., 0. }, {  0., 0. }, {  0., 0. },
                              {  0., 0. }, {  0., 0. }, {  0., 0. } },
                            { {  0., 0. }, {  1., 0. }, {  6., 0. },
                              { 11., 0. }, { 16., 0. }, {  0., 0. } },
                            { {  0., 0. }, {  2., 0. }, {  7., 0. },
                              { 12., 0. }, { 17., 0. }, {  0., 0. } },
                            { {  0., 0. }, {  3., 0. }, {  8., 0. },
                              { 13., 0. }, { 18., 0. }, {  0., 0. } },
                            { {  0., 0. }, {  4., 0. }, {  9., 0. },
                              { 14., 0. }, { 19., 0. }, {  0., 0. } },
                            { {  0., 0. }, {  5., 0. }, { 10., 0. },
                              { 15., 0. }, { 20., 0. }, {  0., 0. } },
                            { {  0., 0. }, {  0., 0. }, {  0., 0. },
                              {  0., 0. }, {  0., 0. }, {  0., 0. } } };
    float  sreb35[5][4] = {10.,10.,-8.,0.,
                            4.,14., 2.,0.,
                            0.,-4.,-5.,0.,
                            0., 6., 4.,0.,
                            1.,-3.,-2.,0.};
    double dreb35[3][6] = { 6.,-16.,-2., 14.,-1.,0.,
                           10., -1., 1.,  0., 2.,0.,
                           -2.,  6.,-4.,-14., 1.,0.};
    cmplx  creb35[3][6] = {{{20.,-26.},{-7.,-10.},{-7.,12.},
                            {8.,5.},{-3.,1.},{0.,0.}},
                           {{13.,-22.},{-7.,-1.},{9.,-27.},
                            {-10.,30.},{4.,0.},{0.,0.}},
                           {{11.,-28.},{0.,1.},{5.,-17.},
                            {-19.,1.},{2.,0.}, {0.,0.}}};
    dcmplx zreb35[3][6] = {{{20.,-26.},{-7.,-10.},{-7.,12.},
                            {8.,5.},{-3.,1.},{0.,0.}},
                           {{13.,-22.},{-7.,-1.},{9.,-27.},
                            {-10.,30.},{4.,0.},{0.,0.}},
                           {{11.,-28.},{0.,1.},{5.,-17.},
                            {-19.,1.},{2.,0.},{0.,0.}}};
    float  sreb36[6][5] = {24., 36., -8.,0.,0.,
                           16., 44.,-24.,0.,0.,
                            4., 20., 12.,0.,0.,
                            4., 20., 12.,0.,0.,
                            0., 12., -4.,0.,0.,
                           -4.,-20., 12.,0.,0.};
    double dreb36[3][5] = { 39.,-33.,0.,0.,0.,
                           -54., 33.,0.,0.,0.,
                            30.,-12.,0.,0.,0.};
    double dreb37[8][9] = {2400.,2436.,2472.,2508.,2544.,2580.,2616.,2652.,0.,
                           0.,2468.,2494.,2520.,2546.,2572.,2598.,2624.,0.,
                           0.,0.,2515.,2531.,2547.,2563.,2579.,2595.,0.,
                           0.,0.,0.,2541.,2547.,2553.,2559.,2565.,0.,
                           0.,0.,0.,0.,2546.,2542.,2538.,2534.,0.,
                           0.,0.,0.,0.,0.,2530.,2516.,2502.,0.,
                           0.,0.,0.,0.,0.,0.,2493.,2469.,0.,
                           0.,0.,0.,0.,0.,0.,0.,2435.,0.};
    float  sreb37[8][9] = {2400.,2436.,2472.,2508.,2544.,2580.,2616.,2652.,0.,
                           0.,2468.,2494.,2520.,2546.,2572.,2598.,2624.,0.,
                           0.,0.,2515.,2531.,2547.,2563.,2579.,2595.,0.,
                           0.,0.,0.,2541.,2547.,2553.,2559.,2565.,0.,
                           0.,0.,0.,0.,2546.,2542.,2538.,2534.,0.,
                           0.,0.,0.,0.,0.,2530.,2516.,2502.,0.,
                           0.,0.,0.,0.,0.,0.,2493.,2469.,0.,
                           0.,0.,0.,0.,0.,0.,0.,2435.,0.};
 
/*******************************************
* ADDED FOR Z/CGECMO, Z/CGECMI       10/12 *
*******************************************/
 
    dcmplx zbres60[7][6] = { { { 99., 99. }, { 99., 99. }, { 99., 99. },
                               { 99., 99. }, { 99., 99. }, { 99., 99. } },
                               { { 99., 99. }, { 1., -1. }, { 6., -6. },
                             { 11., -11. }, { 16., -16. }, { 99., 99. } },
                               { { 99., 99. }, { 2., -2. }, { 7., -7. },
                             { 12., -12. }, { 17., -17. }, { 99., 99. } },
                               { { 99., 99. }, { 3., -3. }, { 8., -8. },
                             { 13., -13. }, { 18., -18. }, { 99., 99. } },
                               { { 99., 99. }, { 4., -4. }, { 9., -9. },
                             { 14., -14. }, { 19., -19. }, { 99., 99. } },
                             { { 99., 99. }, { 5., -5. }, { 10., -10. },
                             { 15., -15. }, { 20., -20. }, { 99., 99. } },
                             { { 99., 99. }, { 99., 99. }, { 99., 99. },
                               { 99., 99. }, { 99., 99. }, { 99., 99. } } };
 
    cmplx cbres60[7][6] = { { { 99., 99. }, { 99., 99. }, { 99., 99. },
                              { 99., 99. }, { 99., 99. }, { 99., 99. } },
                              { { 99., 99. }, { 1., -1. }, { 6., -6. },
                            { 11., -11. }, { 16., -16. }, { 99., 99. } },
                              { { 99., 99. }, { 2., -2. }, { 7., -7. },
                            { 12., -12. }, { 17., -17. }, { 99., 99. } },
                              { { 99., 99. }, { 3., -3. }, { 8., -8. },
                            { 13., -13. }, { 18., -18. }, { 99., 99. } },
                              { { 99., 99. }, { 4., -4. }, { 9., -9. },
                            { 14., -14. }, { 19., -19. }, { 99., 99. } },
                            { { 99., 99. }, { 5., -5. }, { 10., -10. },
                            { 15., -15. }, { 20., -20. }, { 99., 99. } },
                            { { 99., 99. }, { 99., 99. }, { 99., 99. },
                              { 99., 99. }, { 99., 99. }, { 99., 99. } } };
 
    dcmplx zares61[7][10] = { { { 99., 99. }, { 99., 99. }, { 99., 99. },
                  { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                                { 99., 99. }, { 99., 99. }, { 99., 99. } },
                              { { 99., 99. }, { 99., 99. }, { 99., 99. },
                  { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                                { 99., 99. }, { 99., 99. }, { 99., 99. } },
                                { { 99., 99. }, { 1., -1. }, { 6., -6. },
                             { 11., -11. }, { 16., -16. }, { 21., -21. },
                  { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                                { { 99., 99. }, { 2., -2. }, { 7., -7. },
                             { 12., -12. }, { 17., -17. }, { 22., -22. },
                  { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                                { { 99., 99. }, { 3., -3. }, { 8., -8. },
                             { 13., -13. }, { 18., -18. }, { 23., -23. },
                  { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                                { { 99., 99. }, { 4., -4. }, { 9., -9. },
                             { 14., -14. }, { 19., -19. }, { 24., -24. },
                  { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                              { { 99., 99. }, { 5., -5. }, { 10., -10. },
                             { 15., -15. }, { 20., -20. }, { 25., -25. },
                  { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } } };
 
    cmplx cares61[7][10] = { { { 99., 99. }, { 99., 99. }, { 99., 99. },
                 { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                               { 99., 99. }, { 99., 99. }, { 99., 99. } },
                             { { 99., 99. }, { 99., 99. }, { 99., 99. },
                 { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                               { 99., 99. }, { 99., 99. }, { 99., 99. } },
                               { { 99., 99. }, { 1., -1. }, { 6., -6. },
                            { 11., -11. }, { 16., -16. }, { 21., -21. },
                 { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                               { { 99., 99. }, { 2., -2. }, { 7., -7. },
                            { 12., -12. }, { 17., -17. }, { 22., -22. },
                 { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                               { { 99., 99. }, { 3., -3. }, { 8., -8. },
                            { 13., -13. }, { 18., -18. }, { 23., -23. },
                 { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                               { { 99., 99. }, { 4., -4. }, { 9., -9. },
                            { 14., -14. }, { 19., -19. }, { 24., -24. },
                 { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                             { { 99., 99. }, { 5., -5. }, { 10., -10. },
                            { 15., -15. }, { 20., -20. }, { 25., -25. },
                 { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } } };
 
/***************************************************
*  INPUT DATA FOR MATRIX OPERATIONS SUBROUTINES :  *
***************************************************/
 
    float  sa33[3][6] = {110000., 210000., 310000., 410000., 000000., 000000.,
                         120000., 220000., 320000., 420000., 000000., 000000.,
                         130000., 230000., 330000., 430000., 000000., 000000. };
    double da33[3][6] = {110000., 210000., 310000., 410000., 000000., 000000.,
                         120000., 220000., 320000., 420000., 000000., 000000.,
                         130000., 230000., 330000., 430000., 000000., 000000. };
    float  sb33[3][4] = { 11., 21., 31., 41.,
                          12., 22., 32., 42.,
                          13., 23., 33., 43. };
    double db33[3][4] = { 11., 21., 31., 41.,
                          12., 22., 32., 42.,
                          13., 23., 33., 43. };
    cmplx  ca33[3][6] = { { { 1., 5. }, { 2., 4. }, { 3., 3. },
                            { 6., 6. }, { 0., 0. }, { 0., 0. } },
                          { { 9., 2. }, { 8., 3. }, { 7., 5. },
                            { 3., 6. }, { 0., 0. }, { 0., 0. } },
                          { { 1., 9. }, { 1., 8. }, { 1., 7. },
                            { 1., 4. }, { 0., 0. }, { 0., 0. } } };
    cmplx  cb33[3][5] = { { { 1., 8. }, { 4., 4. }, { 6., 2. },
                            { 7., 2. }, { 0., 0. } },
                          { { 2., 7. }, { 6., 8. }, { 4., 5. },
                            { 6., 4. }, { 0., 0. } },
                          { { 3., 2. }, { 6., 3. }, { 4., 5. },
                            { 1., 6. }, { 0., 0. } } };
    dcmplx za33[3][6] = { { { 1., 5. }, { 2., 4. }, { 3., 3. },
                            { 6., 6. }, { 0., 0. }, { 0., 0. } },
                          { { 9., 2. }, { 8., 3. }, { 7., 5. },
                            { 3., 6. }, { 0., 0. }, { 0., 0. } },
                          { { 1., 9. }, { 1., 8. }, { 1., 7. },
                            { 1., 4. }, { 0., 0. }, { 0., 0. } } };
    dcmplx zb33[3][5] = { { { 1., 8. }, { 4., 4. }, { 6., 2. },
                            { 7., 2. }, { 0., 0. } },
                          { { 2., 7. }, { 6., 8. }, { 4., 5. },
                            { 6., 4. }, { 0., 0. } },
                          { { 3., 2. }, { 6., 3. }, { 4., 5. },
                            { 1., 6. }, { 0., 0. } } };
    float  sa34[3][6] = {110000., 210000., 310000., 410000., 000000., 000000.,
                         120000., 220000., 320000., 420000., 000000., 000000.,
                         130000., 230000., 330000., 430000., 000000., 000000. };
    double da34[3][6] = {110000., 210000., 310000., 410000., 000000., 000000.,
                         120000., 220000., 320000., 420000., 000000., 000000.,
                         130000., 230000., 330000., 430000., 000000., 000000. };
    float  sb34[3][4] = { -11., -21., -31., -41.,
                          -12., -22., -32., -42.,
                          -13., -23., -33., -43. };
    double db34[3][4] = { -11., -21., -31., -41.,
                          -12., -22., -32., -42.,
                          -13., -23., -33., -43. };
    float  sa35[5][8] = { 1., 2., 1., -3., 4., -1., 0., 0.,
                          2., 0., -1., 2., 0., -1., 0., 0.,
                         -1., 1., -1., 2., -2., 1., 0., 0.,
                         -1., 1., 1., 2., 1., -3., 0., 0.,
                          4., -1., 2., 0., -1., 2., 0., 0. };
    double da35[5][8] = { 1., 2., 1., -3., 4., -1., 0., 0.,
                          2., 0., -1., 2., 0., -1., 0., 0.,
                         -1., 1., -1., 2., -2., 1., 0., 0.,
                         -1., 1., 1., 2., 1., -3., 0., 0.,
                          4., -1., 2., 0., -1., 2., 0., 0. };
    float  sb35[4][6] = { 1., 2., 1., -3., 4., 1.,
                         -1., 2., 0., -1., 2., 1.,
                          0., -1., -1., 1., -1., 1.,
                          2., -2., 1., -1., 1., 1. };
    double db35[4][6] = { 1., 2., 1., -3., 4., 1.,
                         -1., 2., 0., -1., 2., 1.,
                          0., -1., -1., 1., -1., 1.,
                          2., -2., 1., -1., 1., 1. };
    cmplx  ca35[2][6] = { { { 1., 2. }, { 4., 6. }, { 6., 3. },
                            { 0., 0. }, { 0., 0. }, { 0., 0. } },
                          { { 3., 4. }, { 7., 1. }, { 2., 5. },
                            { 0., 0. }, { 0., 0. }, { 0., 0. } } };
    cmplx  cb35[3][7] = { { { 1., 9. }, { 2., 5. }, { 2., 6. }, { 0., 0. },
                            { 0., 0. }, { 0., 0. }, { 0., 0. } },
                          { { 2., 6. }, { 6., 2. }, { 5., 4. }, { 0., 0. },
                            { 0., 0. }, { 0., 0. }, { 0., 0. } },
                          { { 5., 6. }, { 6., 4. }, { 2., 6. }, { 0., 0. },
                            { 0., 0. }, { 0., 0. }, { 0., 0. } } };
    dcmplx za35[2][6] = { { { 1., 2. }, { 4., 6. }, { 6., 3. },
                            { 0., 0. }, { 0., 0. }, { 0., 0. } },
                          { { 3., 4. }, { 7., 1. }, { 2., 5. },
                            { 0., 0. }, { 0., 0. }, { 0., 0. } } };
    dcmplx zb35[3][7] = { { { 1., 9. }, { 2., 5. }, { 2., 6. }, { 0., 0. },
                            { 0., 0. }, { 0., 0. }, { 0., 0. } },
                          { { 2., 6. }, { 6., 2. }, { 5., 4. }, { 0., 0. },
                            { 0., 0. }, { 0., 0. }, { 0., 0. } },
                          { { 5., 6. }, { 6., 4. }, { 2., 6. }, { 0., 0. },
                            { 0., 0. }, { 0., 0. }, { 0., 0. } } };
    cmplx  ca36[2][4] = { { {  1., 2. }, { 2., 6. }, {  1., 2. },
                            { 0., 0. } },
                          { { -3., 2. }, { 4., 5. }, { -1., 8. },
                            { 0., 0. } } };
    cmplx  cb36[2][3] = { { { 1., 3. }, { 2., 5. }, { 1., 1. } },
                          { { -3., 2. }, { 4., 6. }, { -1., 9. } } };
    dcmplx za36[2][4] = { { {  1., 2. }, { 2., 6. }, {  1., 2. },
                            { 0., 0. } },
                          { { -3., 2. }, { 4., 5. }, { -1., 8. },
                            { 0., 0. } } };
    dcmplx zb36[2][3] = { { { 1., 3. }, { 2., 5. }, { 1., 1. } },
                          { { -3., 2. }, { 4., 6. }, { -1., 9. } } };
    float  sc36a[4][7] = { 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.,
                           0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.,
                           0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.,
                           0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0. };
    double dc36a[4][7] = { 0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.,
                           0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.,
                           0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0.,
                           0.5, 0.5, 0.5, 0.5, 0.5, 0.5, 0. };
    cmplx  ca36a[3][8] = { { { 1., 5. }, { 2., 4. }, { 3., 3. }, { 4., 2. },
                             { 5., 1. }, { 6., 6. }, { 0., 0. }, { 0., 0. } },
                           { { 9., 2. }, { 8., 3. }, { 7., 5. }, { 4., 7. },
                             { 5., 1. }, { 3., 6. }, { 0., 0. }, { 0., 0. } },
                           { { 1., 9. }, { 1., 8. }, { 1., 7. }, { 1., 5. },
                             { 1., 6. }, { 1., 4. }, { 0., 0. }, { 0., 0. } } };
    cmplx  cb36a[2][4] = { { { 1., 8. }, { 4., 4. }, { 6., 2. }, { 0., 0. } },
                           { { 2., 7. }, { 6., 8. }, { 4., 5. }, { 0., 0. } } };
    dcmplx za36a[3][8] = { { { 1., 5. }, { 2., 4. }, { 3., 3. }, { 4., 2. },
                             { 5., 1. }, { 6., 6. }, { 0., 0. }, { 0., 0. } },
                           { { 9., 2. }, { 8., 3. }, { 7., 5. }, { 4., 7. },
                             { 5., 1. }, { 3., 6. }, { 0., 0. }, { 0., 0. } },
                           { { 1., 9. }, { 1., 8. }, { 1., 7. }, { 1., 5. },
                             { 1., 6. }, { 1., 4. }, { 0., 0. }, { 0., 0. } } };
    dcmplx zb36a[2][4] = { { { 1., 8. }, { 4., 4. }, { 6., 2. }, { 0., 0. } },
                           { { 2., 7. }, { 6., 8. }, { 4., 5. }, { 0., 0. } } };
    cmplx  cc36a[2][8] = { { { 0.5, 0. }, { 0.5, 0. }, { 0.5, 0. },
                             { 0.5, 0. }, { 0.5, 0. }, { 0.5, 0. },
                             { 0.,  0. }, { 0.,  0. } },
                           { { 0.5, 0. }, { 0.5, 0. }, { 0.5, 0. },
                             { 0.5, 0. }, { 0.5, 0. }, { 0.5, 0. },
                             { 0.,  0. }, { 0.,  0. } } };
    dcmplx zc36a[2][8] = { { { 0.5, 0. }, { 0.5, 0. }, { 0.5, 0. },
                             { 0.5, 0. }, { 0.5, 0. }, { 0.5, 0. },
                             { 0.,  0. }, { 0.,  0. } },
                           { { 0.5, 0. }, { 0.5, 0. }, { 0.5, 0. },
                             { 0.5, 0. }, { 0.5, 0. }, { 0.5, 0. },
                             { 0.,  0. }, { 0.,  0. } } };
    float  sx97[7][10] = { 0.,  0.,  0.,  0.,  0.,  0., 0., 0., 0., 0.,
                           0.,  0.,  0.,  0.,  0.,  0., 0., 0., 0., 0.,
                           0.,  1.,  2.,  3.,  4.,  5., 0., 0., 0., 0.,
                           0.,  6.,  7.,  8.,  9., 10., 0., 0., 0., 0.,
                           0., 11., 12., 13., 14., 15., 0., 0., 0., 0.,
                           0., 16., 17., 18., 19., 20., 0., 0., 0., 0.,
                           0., 21., 22., 23., 24., 25., 0., 0., 0., 0., };
    double dx97[7][10] = { 0.,  0.,  0.,  0.,  0.,  0., 0., 0., 0., 0.,
                           0.,  0.,  0.,  0.,  0.,  0., 0., 0., 0., 0.,
                           0.,  1.,  2.,  3.,  4.,  5., 0., 0., 0., 0.,
                           0.,  6.,  7.,  8.,  9., 10., 0., 0., 0., 0.,
                           0., 11., 12., 13., 14., 15., 0., 0., 0., 0.,
                           0., 16., 17., 18., 19., 20., 0., 0., 0., 0.,
                           0., 21., 22., 23., 24., 25., 0., 0., 0., 0., };
    cmplx  cx97[7][10] = { { {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } },
                           { {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } },
                           { {  0., 0. }, {  1., 0. }, {  2., 0. },
                             {  3., 0. }, {  4., 0. }, {  5., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } },
                           { {  0., 0. }, {  6., 0. }, {  7., 0. },
                             {  8., 0. }, {  9., 0. }, { 10., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } },
                           { {  0., 0. }, { 11., 0. }, { 12., 0. },
                             { 13., 0. }, { 14., 0. }, { 15., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } },
                           { {  0., 0. }, { 16., 0. }, { 17., 0. },
                             { 18., 0. }, { 19., 0. }, { 20., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } },
                           { {  0., 0. }, { 21., 0. }, { 22., 0. },
                             { 23., 0. }, { 24., 0. }, { 25., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } } };
    dcmplx zx97[7][10] = { { {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } },
                           { {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } },
                           { {  0., 0. }, {  1., 0. }, {  2., 0. },
                             {  3., 0. }, {  4., 0. }, {  5., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } },
                           { {  0., 0. }, {  6., 0. }, {  7., 0. },
                             {  8., 0. }, {  9., 0. }, { 10., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } },
                           { {  0., 0. }, { 11., 0. }, { 12., 0. },
                             { 13., 0. }, { 14., 0. }, { 15., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } },
                           { {  0., 0. }, { 16., 0. }, { 17., 0. },
                             { 18., 0. }, { 19., 0. }, { 20., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } },
                           { {  0., 0. }, { 21., 0. }, { 22., 0. },
                             { 23., 0. }, { 24., 0. }, { 25., 0. },
                             {  0., 0. }, {  0., 0. }, {  0., 0. },
                             {  0., 0. } } };
    float  sxa98[7][10] = { 0.,  0.,  0.,  0.,  0.,  0., 0., 0., 0., 0.,
                            0.,  0.,  0.,  0.,  0.,  0., 0., 0., 0., 0.,
                            0.,  1.,  2.,  3.,  4.,  5., 0., 0., 0., 0.,
                            0.,  6.,  7.,  8.,  9., 10., 0., 0., 0., 0.,
                            0., 11., 12., 13., 14., 15., 0., 0., 0., 0.,
                            0., 16., 17., 18., 19., 20., 0., 0., 0., 0.,
                            0.,  0.,  0.,  0.,  0.,  0., 0., 0., 0., 0. };
    double dxa98[7][10] = { 0.,  0.,  0.,  0.,  0.,  0., 0., 0., 0., 0.,
                            0.,  0.,  0.,  0.,  0.,  0., 0., 0., 0., 0.,
                            0.,  1.,  2.,  3.,  4.,  5., 0., 0., 0., 0.,
                            0.,  6.,  7.,  8.,  9., 10., 0., 0., 0., 0.,
                            0., 11., 12., 13., 14., 15., 0., 0., 0., 0.,
                            0., 16., 17., 18., 19., 20., 0., 0., 0., 0.,
                            0.,  0.,  0.,  0.,  0.,  0., 0., 0., 0., 0. };
    cmplx  cxa98[7][10] = {
        { {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. },
          {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. },
          {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, {  1., 0. }, {  2., 0. }, {  3., 0. }, {  4., 0. },
          {  5., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, {  6., 0. }, {  7., 0. }, {  8., 0. }, {  9., 0. },
          { 10., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 11., 0. }, { 12., 0. }, { 13., 0. }, { 14., 0. },
          { 15., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 16., 0. }, { 17., 0. }, { 18., 0. }, { 19., 0. },
          { 20., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. },
          {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } } };
    dcmplx zxa98[7][10] = {
        { {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. },
          {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. },
          {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, {  1., 0. }, {  2., 0. }, {  3., 0. }, {  4., 0. },
          {  5., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, {  6., 0. }, {  7., 0. }, {  8., 0. }, {  9., 0. },
          { 10., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 11., 0. }, { 12., 0. }, { 13., 0. }, { 14., 0. },
          { 15., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, { 16., 0. }, { 17., 0. }, { 18., 0. }, { 19., 0. },
          { 20., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } },
        { {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. },
          {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. }, {  0., 0. } } };
    float  sxb98[7][6] = { 0.,0.,0.,0.,0.,0.,0.,0., 0., 0., 0., 0., 0., 0.,
                           0.,0.,0.,0.,0.,0.,0.,0., 0., 0., 0., 0., 0., 0.,
                           0.,0.,0.,0.,0.,0.,0.,0., 0., 0., 0., 0., 0., 0. };
    double dxb98[7][6] = { 0.,0.,0.,0.,0.,0.,0.,0., 0., 0., 0., 0., 0., 0.,
                           0.,0.,0.,0.,0.,0.,0.,0., 0., 0., 0., 0., 0., 0.,
                           0.,0.,0.,0.,0.,0.,0.,0., 0., 0., 0., 0., 0., 0. };
    cmplx  cxb98[7][6] = {
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } },
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } },
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } },
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } },
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } },
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } },
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } }};
    dcmplx zxb98[7][6] = {
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } },
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } },
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } },
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } },
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } },
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } },
        { { 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. },{ 0., 0. } }};
 
     float  scr99[8][9] = {
         1920.0,2001.0,2082.0,2163.0,2244.0,2325.0,2406.0,2487.0,   0.0,
            0.0,2093.0,2179.0,2265.0,2351.0,2437.0,2523.0,2609.0,   0.0,
            0.0,   0.0,2275.0,2366.0,2457.0,2548.0,2639.0,2730.0,   0.0,
            0.0,   0.0,   0.0,2466.0,2562.0,2658.0,2754.0,2850.0,   0.0,
            0.0,   0.0,   0.0,   0.0,2666.0,2767.0,2868.0,2969.0,   0.0,
            0.0,   0.0,   0.0,   0.0,   0.0,2875.0,2981.0,3087.0,   0.0,
            0.0,   0.0,   0.0,   0.0,   0.0,   0.0,3093.0,3204.0,   0.0,
            0.0,   0.0,   0.0,   0.0,   0.0,   0.0,   0.0,3320.0,   0.0};
     double dcr99[8][8] = {
            5.0,  15.0,  25.0,  35.0,  45.0,  55.0,  65.0,  75.0,
            1.0,  58.0,  95.0, 132.0, 169.0, 206.0, 243.0, 280.0,
            1.0,   1.0, 164.0, 228.0, 292.0, 356.0, 420.0, 484.0,
            1.0,   1.0,   1.0, 323.0, 414.0, 505.0, 596.0, 687.0,
            1.0,   1.0,   1.0,   1.0, 535.0, 653.0, 771.0, 889.0,
            1.0,   1.0,   1.0,   1.0,   1.0, 800.0, 945.0, 1090.0,
            1.0,   1.0,   1.0,   1.0,   1.0,   1.0,1118.0, 1290.0,
            1.0,   1.0,   1.0,   1.0,   1.0,   1.0,   1.0, 1489.0 };
 
     cmplx  cskres21n[3][4] = {{{-57.,13.}, {-63.,79.}, {-24.,70.}, {0.,0.}},
                               {{0.,0.}, {-28.,90.}, {-55.,103.}, {0.,0.}},
                               {{0.,0.}, {0.,0.}, {13.,75.}, {0.,0.}}};
     dcmplx zskres21n[3][4] = {{{-57.,13.},{ -63.,79.}, {-24.,70.}, {0.,0.}},
                               {{0.,0.}, {-28.,90.}, {-55.,103.}, {0.,0.}},
                               {{0.,0.}, {0.,0.}, {13.,75.}, {0.,0.}}};
 
     cmplx  chkres22n[3][4] = {{{86.,0.}, {74.,7.}, {55.,-15.}, {0.,0.}},
                               {{0.,0.}, {139.,0.}, {66.,-19.}, {0.,0.}},
                               {{0.,0.}, {0.,0.}, {80.,0.}, {0.,0.}}};
    dcmplx zhkres22n[3][4] = {{{86.,0.}, {74.,7.}, {55.,-15.}, {0.,0.}},
                              {{0.,0.}, {139.,0.}, {66.,-19.},{ 0.,0.}},
                              {{0.,0.}, {0.,0.}, {80.,0.}, {0.,0.}}};
 
    cmplx  cs2kres23n[3][4] = {{{-181.,61.},{-212.,182.},
                                {-123.,151.}, {0.,0.}},
                               {{0.,0.},    {-274.,248.},
                                {-163.,205.}, {0.,0.}},
                               {{0.,0.},    {0.,0.},
                                {-151.,115.}, {0.,0.}}};
    dcmplx zs2kres23n[3][4] = {{{-181.,61.},{-212.,182.},
                                {-123.,151.}, {0.,0.}},
                               {{0.,0.},    {-274.,248.},
                                {-163.,205.}, {0.,0.}},
                               {{0.,0.},    {0.,0.},
                                {-151.,115.}, {0.,0.}}};
 
    cmplx  ch2kres23n[3][4] ={{{ 42.,0.},{ 128.,20.}, {66.,3.},{ 0.,0.}},
                          {{0.,0.}, {261.,0.}, {188.,-59.}, {0.,0.}},
                          {{0.,0.}, {0.,0.}, {80.,0.}, {0.,0.}}};
    dcmplx zh2kres23n[3][4] = {{{42.,0.}, {128.,20.}, {66.,3.},{ 0.,0.}},
                          {{0.,0.}, {261.,0.}, {188.,-59.},{ 0.,0.}},
                          {{0.,0.}, {0.,0.}, {80.,0.}, {0.,0.}}};
 
 
    cmplx csyres25n[4][5] = {{{-123.,194.}, {-65.,-59.}, {-147.,15.},
       {-55.,-67.}, {-164.,75.}},{{-133.,-25.},{ -90.,-115.},
       {-220.,-136.},{-167.,-90.}, {-175.,-150.}},
       {{-96.,-128.}, {-77.,1.}, {-37.,-92.},
       {-23.,-41.}, {-63.,-44.}},  {{-268.,-63.}, {-104.,-24.},
       {-342.,-217.},{-159.,-140.}, {-148.,7.}}};
 
    dcmplx zsyres25n[4][5] = {{{-123.,194.}, {-65.,-59.}, {-147.,15.},
       {-55.,-67.}, {-164.,75.}},{{-133.,-25.},{ -90.,-115.},
       {-220.,-136.},{-167.,-90.}, {-175.,-150.}},
       {{-96.,-128.}, {-77.,1.}, {-37.,-92.},
       {-23.,-41.}, {-63.,-44.}}, {{-268.,-63.}, {-104.,-24.},
       {-342.,-217.},{-159.,-140.}, {-148.,7.}}};
 
   cmplx  cheres26n[4][5] = {{{-78.,229.}, {-11.,-17.}, {-30.,80.},
       {17.,-11.}, {-36.,98.}}, {{-128.,15.},{-84.,-67.},{ -128.,28.},
    {7.,132.}, {-9.,+86.}},  {{-66.,-83.},{-41.,55.}, {8.,21.},{ 1.,47.},
       {-52.,57.}},            {{-218.,-53.}, {-44.,-12.}, {-207.,-125.},
       {-45.,70.}, {59.,181.}}};
   dcmplx zheres26n[4][5] = {{{-78.,229.}, {-11.,-17.}, {-30.,80.},
       {17.,-11.}, {-36.,98.}},  {{-128.,15.},{-84.,-67.},{ -128.,28.},
   {7.,132.}, {-9.,+86.}},   {{-66.,-83.},{-41.,55.}, {8.,21.},{ 1.,47.},
       {-52.,57.}},            {{-218.,-53.}, {-44.,-12.}, {-207.,-125.},
       {-45.,70.}, {59.,181.}}};
   cmplx  cska21n[5][3] = {{{2.,0.},{3.,3.}, {1.,3.}},{{ 3.,2.}, {8.,0.},
  {2.,1.}}, {{4.,1.},{2.,5.},{6.,0.}},{{1.,7.},{2.,4.},{3.,2.}},{{0.,0.},
        {1.,2.}, {2.,2.}}};
   cmplx  cskx21n[3][4] = {{{2.,1.}, {1.,9.}, {4.,5.}, {0.,0.}},
    {{0.,0.},{3.,1.},{6.,7.},{0.,0.}},{{0.,0.},{0.,0.},{8.,1.},{0.,0.}}};
   dcmplx zska21n[5][3] = {{{2.,0.},{3.,3.}, {1.,3.}},{{ 3.,2.}, {8.,0.},
  {2.,1.}}, {{4.,1.},{2.,5.},{6.,0.}},{{1.,7.},{2.,4.},{3.,2.}},{{0.,0.},
    {1.,2.}, {2.,2.}}};
   dcmplx zskx21n[3][4] = {{{2.,1.}, {1.,9.}, {4.,5.}, {0.,0.}},
    {{0.,0.},{3.,1.},{6.,7.},{0.,0.}},{{0.,0.},{0.,0.},{8.,1.},{0.,0.}}};
 
   cmplx  chka22n[5][3] = {{{2.,0.},{3.,3.}, {1.,3.}},{{ 3.,2.}, {8.,0.},
   {2.,1.}},{{4.,1.},{2.,5.},{6.,0.}},{{1.,7.},{2.,4.},{3.,2.}},{{0.,0.},
   {1.,2.}, {2.,2.}}};
   cmplx  chkx22n[3][4] = {{{2.,0.}, {1.,9.}, {4.,5.}, {0.,0.}},
   {{0.,0.},{3.,0.},{6.,7.},{0.,0.}},{{0.,0.},{0.,0.},{8.,0.},{0.,0.}}};
   dcmplx zhka22n[5][3] = {{{2.,0.},{3.,3.}, {1.,3.}},{{ 3.,2.}, {8.,0.},
  {2.,1.}},{{4.,1.},{2.,5.},{6.,0.}},{{1.,7.},{2.,4.},{3.,2.}}, {{0.,0.},
   {1.,2.}, {2.,2.}}};
   dcmplx zhkx22n[3][4] = {{{2.,0.}, {1.,9.}, {4.,5.}, {0.,0.}},
   {{0.,0.},{3.,0.},{6.,7.},{0.,0.}},{{0.,0.},{0.,0.},{8.,0.},{0.,0.}}};
 
   cmplx  cs2ka23n[5][3] = {{{ 2.,5.}, {3.,3.}, {1.,3.}},
          {{3.,2.}, {8.,5.}, {2.,1.}},   {{4.,1.}, {2.,5.}, {6.,5.}},
          {{1.,7.}, {2.,4.}, {3.,2.}},   {{0.,0.}, {1.,2.}, {2.,2.}}};
   cmplx  cs2kb23n[5][4] = {{{1.,5.}, {2.,4.}, {3.,5.}, {0.,0.}},
 {{6.,2.}, {7.,5.},{8.,1.},{0.,0.}},{{3.,1.}, {2.,5.},{ 1.,5.},{ 0.,0.}},
 {{2.,5.}, {2.,4.},{1.,0.},{0.,0.}},{{1.,0.},{ 0.,0.},{ 1.,1.},{0.,0.}}};
   cmplx  cs2kc23n[3][4] = {{{2.,3.}, {1.,9.}, {4.,5.}, {0.,0.}},
   {{0.,0.},{ 3.,3.},{6.,7.},{0.,0.}},{{0.,0.},{0.,0.},{8.,3.},{0.,0.}}};
   dcmplx  zs2ka23n[5][3] = {{{ 2.,5.}, {3.,3.}, {1.,3.}},
           {{3.,2.}, {8.,5.}, {2.,1.}},   {{4.,1.}, {2.,5.}, {6.,5.}},
           {{1.,7.}, {2.,4.}, {3.,2.}},   {{0.,0.}, {1.,2.}, {2.,2.}}};
   dcmplx  zs2kb23n[5][4] = {{{1.,5.}, {2.,4.}, {3.,5.}, {0.,0.}},
   {{6.,2.},{7.,5.},{8.,1.},{0.,0.}},{{3.,1.},{2.,5.},{ 1.,5.},{ 0.,0.}},
  {{2.,5.},{2.,4.},{1.,0.},{0.,0.}},{{1.,0.},{ 0.,0.},{ 1.,1.},{0.,0.}}};
   dcmplx  zs2kc23n[3][4] = {{{2.,3.}, {1.,9.}, {4.,5.}, {0.,0.}},
 {{0.,0.},{ 3.,3.}, {6.,7.}, {0.,0.}},{{0.,0.},{0.,0.},{8.,3.},{0.,0.}}};
 
   cmplx  ch2ka23n[5][3] = {{{ 2.,0.}, {3.,3.}, {1.,3.}},
          {{3.,2.}, {8.,0.}, {2.,1.}},   {{4.,1.}, {2.,5.}, {6.,0.}},
          {{1.,7.}, {2.,4.}, {3.,2.}},   {{0.,0.}, {1.,2.}, {2.,2.}}};
   cmplx  ch2kb23n[5][4] = {{{1.,0.}, {2.,4.}, {3.,5.}, {0.,0.}},
   {{6.,2.},{7.,0.},{8.,1.},{0.,0.}},{{3.,1.},{2.,5.},{ 1.,0.},{ 0.,0.}},
  {{2.,0.},{2.,4.},{1.,0.},{0.,0.}},{{1.,0.},{ 0.,0.},{ 1.,1.},{0.,0.}}};
   cmplx  ch2kc23n[3][4] = {{{2.,0.}, {1.,9.}, {4.,5.}, {0.,0.}},
   {{0.,0.},{ 3.,0.},{6.,7.},{0.,0.}},{{0.,0.},{0.,0.},{8.,0.},{0.,0.}}};
   dcmplx  zh2ka23n[5][3] = {{{ 2.,0.}, {3.,3.}, {1.,3.}},
           {{3.,2.}, {8.,0.}, {2.,1.}},   {{4.,1.}, {2.,5.}, {6.,0.}},
           {{1.,7.}, {2.,4.}, {3.,2.}},   {{0.,0.}, {1.,2.}, {2.,2.}}};
   dcmplx  zh2kb23n[5][4] = {{{1.,0.}, {2.,4.}, {3.,5.}, {0.,0.}},
 {{6.,2.},{7.,0.},{8.,1.},{0.,0.}}, {{3.,1.}, {2.,5.},{ 1.,0.},{ 0.,0.}},
 {{2.,0.},{2.,4.},{1.,0.},{0.,0.}}, {{1.,0.},{ 0.,0.},{ 1.,1.},{0.,0.}}};
   dcmplx  zh2kc23n[3][4] = {{{2.,0.}, {1.,9.}, {4.,5.}, {0.,0.}},
   {{0.,0.},{ 3.,0.},{6.,7.},{0.,0.}},{{0.,0.},{0.,0.},{8.,0.},{0.,0.}}};
 
   cmplx  csya25n[5][8] = {{{1.,5.}, {0.,0.}, {0.,0.}, {0.,0.}, {0.,0.},
   {0.,0.},{ 0.,0.}, {0.,0.}},{{2.,3.},{0.,0.},{0.,0.},{0.,0.}, {0.,0.},
   {0.,0.},{ 0.,0.},{0.,0.}},{{-1.,4.},{1.,1.},{-1.,5.},{0.,0.}, {0.,0.},
   {0.,0.},{ 0.,0.},{0.,0.}},{{-1.,1.}, {1.,3.},{2.,3.},{2.,4.}, {0.,0.},
   {0.,0.},{ 0.,0.},{0.,0.}},{{4.,2.},{-1.,0.},{2.,5.},{0.,0.},{-1.,3.},
   {0.,0.},{ 0.,0.}, {0., 0.}}};
 
   dcmplx zsya25n[5][8] = {{{1.,5.}, {0.,0.}, {0.,0.}, {0.,0.}, {0.,0.},
    {0.,0.},{ 0.,0.},{0.,0.}},{{2.,3.},{0.,0.},{0.,0.},{0.,0.}, {0.,0.},
    {0.,0.},{ 0.,0.},{0.,0.}},{{-1.,4.},{1.,1.},{-1.,5.},{0.,0.},{0.,0.},
    {0.,0.},{ 0.,0.},{0.,0.}},{{-1.,1.},{1.,3.},{2.,3.},{2.,4.}, {0.,0.},
    {0.,0.},{ 0.,0.},{0.,0.}},{{4.,2.},{-1.,0.},{2.,5.},{0.,0.},{-1.,3.},
    {0.,0.},{ 0.,0.}, {0., 0.}}};
 
   float ssyb25n[4][6] = {1., 2., 1., -3., 4., 0.,
                     -1., 2., 0., -1., 2., 0.,
                     0., -1., -1., -1., -1., 0.,
                     2., -2., 1., -1., 1., 0.};
 
   double dsyb25n[4][6] = {1., 2., 1., -3., 4., 0.,
                     -1., 2., 0., -1., 2., 0.,
                     0., -1., -1., -1., -1., 0.,
                     2., -2., 1., -1., 1., 0.};
 
   cmplx  csyb25n[4][6] = {{{1.,3.}, {2.,2.}, {1.,1.}, {-3.,0.},{ 4.,1.},
    {0.,0.}},             {{-1.,2.}, {2.,3.}, {0.,6.}, {-1.,5.},{ 2.,4.},
    {0.,0.}},             {{0.,3.}, {-1.,2.}, {-1.,1.},{ 1.,0.},{-1.,1.},
    {0.,0.}},             {{2.,2.}, {-2.,3.}, {1.,4.}, {-1.,5.},{ 1.,8.},
    {0.,0.}}};
 
   dcmplx  zsyb25n[4][6] = {{{1.,3.},{2.,2.},{1.,1.},{-3.,0.},{ 4.,1.},
  {0.,0.}},               {{-1.,2.}, {2.,3.}, {0.,6.}, {-1.,5.},{ 2.,4.},
  {0.,0.}},               {{0.,3.},{-1.,2.}, {-1.,1.},{ 1.,0.},{ -1.,1.},
  {0.,0.}},               {{2.,2.}, {-2.,3.}, {1.,4.}, {-1.,5.},{ 1.,8.},
  {0.,0.}}};
 
   float  ssyc25n[4][5] = {23., -4., 5., -4., 8.,
                           12., -5., 6., 1., -4.,
                           -6., 1., -1., 0., -2.,
                            2., 3., -4., -5., 13.};
 
   double dsyc25n[4][5] = {23., -4., 5., -4., 8.,
                           12., -5., 6., 1., -4.,
                           -6., 1., -1., 0., -2.,
                            2., 3., -4., -5., 13.};
 
   cmplx csyc25n[4][5] = {{{23.,8.},{-4.,5.},{5.,7.},{-4.,4.}, {8.,9.}},
                        {{12.,1.}, {-5.,1.},{ 6.,5.}, {1.,0.}, {-4.,2.}},
                        {{-6.,3.}, {1.,6.},{ -1.,2.}, {0.,0.}, {-2.,4.}},
                        {{2.,7.}, {3.,6.},{ -4.,8.},{ -5.,3.},{13.,6.}}};
 
   dcmplx zsyc25n[4][5] = {{{23.,8.},{-4.,5.},{5.,7.},{-4.,4.},{8.,9.}},
                        {{12.,1.}, {-5.,1.},{ 6.,5.}, {1.,0.}, {-4.,2.}},
                        {{-6.,3.}, {1.,6.},{ -1.,2.}, {0.,0.}, {-2.,4.}},
                       {{2.,7.}, {3.,6.},{ -4.,8.},{ -5.,3.},{ 13.,6.}}};
 
   cmplx  chea26n[5][8] = {{{1.,0.}, {0.,0.}, {0.,0.}, {0.,0.}, {0.,0.},
{0.,0.},{ 0.,0.}, {0.,0.}},{{2.,3.},{0.,0.},{0.,0.},{0.,0.},{0.,0.},
{0.,0.},{ 0.,0.}, {0.,0.}},{{-1.,4.},{1.,1.},{-1.,0.},{0.,0.}, {0.,0.},
{0.,0.},{ 0.,0.}, {0.,0.}},{{-1.,1.},{1.,3.},{2.,3.},{2.,0.}, {0.,0.},
{0.,0.},{ 0.,0.}, {0.,0.}},{{4.,2.},{-1.,0.},{2.,5.},{0.,0.},{-1.,0.},
{0.,0.},{ 0.,0.}, {0., 0.}}};
   cmplx  cheb26n[4][6] = {{{1.,3.}, {2.,2.}, {1.,1.}, {-3.,0.},{ 4.,1.},
{0.,0.}},             {{-1.,2.}, {2.,3.}, {0.,6.}, {-1.,5.},{ 2.,4.},
{0.,0.}},             {{0.,3.}, {-1.,2.}, {-1.,1.},{ 1.,0.},{ -1.,1.},
{0.,0.}},             {{2.,2.}, {-2.,3.}, {1.,4.}, {-1.,5.},{ 1.,8.},
{0.,0.}}};
   cmplx chec26n[4][5] = {{{23.,8.},{-4.,5.},{5.,7.},{-4.,4.},{8.,9.}},
                     {{12.,1.}, {-5.,1.},{ 6.,5.}, {1.,0.}, {-4.,2.}},
                     {{-6.,3.}, {1.,6.},{ -1.,2.}, {0.,0.}, {-2.,4.}},
                     {{2.,7.}, {3.,6.},{ -4.,8.},{ -5.,3.},{ 13.,6.}}};
   dcmplx zhea26n[5][8] = {{{1.,0.},{0.,0.},{0.,0.},{0.,0.},{0.,0.},
{0.,0.},{ 0.,0.}, {0.,0.}},{{2.,3.},{0.,0.},{0.,0.}, {0.,0.}, {0.,0.},
{0.,0.},{ 0.,0.}, {0.,0.}},{{-1.,4.},{1.,1.},{-1.,0.},{0.,0.},{0.,0.},
{0.,0.},{ 0.,0.}, {0.,0.}},{{-1.,1.},{1.,3.},{2.,3.},{2.,0.},{0.,0.},
{0.,0.},{ 0.,0.}, {0.,0.}},{{4.,2.},{-1.,0.}, {2.,5.},{0.,0.},{-1.,0.},
{0.,0.},{ 0.,0.}, {0., 0.}}};
     dcmplx  zheb26n[4][6] = {{{1.,3.},{2.,2.},{1.,1.},{-3.,0.},{ 4.,1.},
{0.,0.}},              {{-1.,2.}, {2.,3.}, {0.,6.}, {-1.,5.},{ 2.,4.},
{0.,0.}},              {{0.,3.}, {-1.,2.}, {-1.,1.},{ 1.,0.},{ -1.,1.},
{0.,0.}},              {{2.,2.}, {-2.,3.}, {1.,4.}, {-1.,5.},{ 1.,8.},
{0.,0.}}};
     dcmplx zhec26n[4][5] = {{{23.,8.},{-4.,5.},{ 5.,7.},{ -4.,4.},
          {8.,9.}}, {{12.,1.}, {-5.,1.},{ 6.,5.}, {1.,0.}, {-4.,2.}},
                     {{-6.,3.}, {1.,6.},{ -1.,2.}, {0.,0.}, {-2.,4.}},
                     {{2.,7.}, {3.,6.},{ -4.,8.},{ -5.,3.},{ 13.,6.}}};
/***************************************************
*  input data for matrix operations subroutines :  *
***************************************************/
     float    sa99[5][8] = {
                    0.e0, 1.e0, 2.e0, 3.e0, 4.e0,5.e0,
                    6.e0, 7.e0, 8.e0, 9.e0,10.e0,
                   11.e0,12.e0,13.e0,14.e0,15.e0,
                   16.e0,17.e0,18.e0,19.e0,20.e0,
                   21.e0,22.e0,23.e0,24.e0,25.e0,
                   26.e0,27.e0,28.e0,29.e0,30.e0,
                   31.e0,32.e0,33.e0,34.e0,35.e0,
                   36.e0,37.e0,38.e0,39.e0 };
     float    sc99[8][9] = {
            0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 0.0,
            0.0, 8.0, 9.0,10.0,11.0,12.0,13.0,14.0, 0.0,
            0.0, 0.0,15.0,16.0,17.0,18.0,19.0,20.0, 0.0,
            0.0, 0.0, 0.0,21.0,22.0,23.0,24.0,25.0, 0.0,
            0.0, 0.0, 0.0, 0.0,26.0,27.0,28.0,29.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0,30.0,31.0,32.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0,33.0,34.0, 0.0,
            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,35.0, 0.0};
     double da99[8][4] = {
                    0.0, 1.0, 2.0,0.0,
                    3.0, 4.0, 5.0,0.0,
                    6.0, 7.0, 8.0,0.0,
                    9.0,10.0,11.0,0.0,
                   12.0,13.0,14.0,0.0,
                   15.0,16.0,17.0,0.0,
                   18.0,19.0,20.0,0.0,
                   21.0,22.0,23.0,0.0 };
     double dc99[8][8] = {
                    0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0,
                    0.0, 8.0, 9.0,10.0,11.0,12.0,13.0,14.0,
                    0.0, 0.0,15.0,16.0,17.0,18.0,19.0,20.0,
                    0.0, 0.0, 0.0,21.0,22.0,23.0,24.0,25.0,
                    0.0, 0.0, 0.0, 0.0,26.0,27.0,28.0,29.0,
                    0.0, 0.0, 0.0, 0.0, 0.0,30.0,31.0,32.0,
                    0.0, 0.0, 0.0, 0.0, 0.0, 0.0,33.0,34.0,
                    0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,35.0 };
     float sab35[5][7] = {2.,2.,2.,0.,2.,0.,0.,0.,3.,1.,
                         3.,4.,0.,0.,0.,0.,1.,0.,-1.,0.,
                         0.,0.,0.,0.,-2.,2.,0.,0.,0.,0.,
                         0.,0.,-1.,0.,0.};
     float sbb35[5][4] = {3.,2.,-2.,0.,4.,1.,-1.,0.,-1.,-1.,-3.,
                         0.,-1.,0.,0.,0.,-1.,3.,2.,0.};
     double dab35[5][7] = {3.,0.,0.,0.,0.,0.,0.,-1.,-2.,
                          0.,0.,0.,0.,0.,2.,4.,-3.,0.,0.,
                          0.,0.,2.,-1.,0.,4.,0.,0.,0.,1.,
                          3.,2.,-2.,1.,0.,0.};
     double dbb35[3][6] = {2.,5.,0.,3.,-1.,0.,3.,5.,1.,
                          1.,2.,0.,1.,4.,2.,-3.,1.,0.};
     cmplx cab35[5][7] = {{{3.,-3.},{0.,0.},{0.,0.},{0.,0.},{0.,0.},{0.,0.},
                                                                  {0.,0.}},
                       {{-1.,-5.},{-2.,0.},{0.,0.},{0.,0.},{0.,0.},{0.,0.},
                                                                  {0.,0.}},
                        {{2.,0.},{4.,3.},{-3.,-5.},{0.,0.},{0.,0.},{0.,0.},
                                                                  {0.,0.}},
                        {{2.,1.},{-1.,0.},{0.,-2.},{4.,4.},{0.,0.},{0.,0.},
                                                                  {0.,0.}},
                        {{1.,4.},{3.,0.},{2.,-5.},{-2.,3},{1.,1.},{0.,0.},
                                                                  {0.,0.}}};
 
      cmplx cbb35[3][6] = {{{2.,4.},{5.,2.},{0.,-3.},{3.,0.},{-1.,2.},{0.,0.}},
                         {{3.,-2.},{5.,1.},{1.,2.},{1.,4.},{2.,-2.},{0.,0.}},
                          {{1.,-3.},{4.,4.},{2.,2.},{-3.,2.},{1.,-1.},{0.,0.}}};
      dcmplx zab35[5][7] = {{{3.,-3.},{0.,0.},{0.,0.},{0.,0.},{0.,0.},{0.,0.},
                                                                  {0.,0.}},
                       {{-1.,-5.},{-2.,0.},{0.,0.},{0.,0.},{0.,0.},{0.,0.},
                                                                  {0.,0.}},
                        {{2.,0.},{4.,3.},{-3.,-5.},{0.,0.},{0.,0.},{0.,0.},
                                                                  {0.,0.}},
                        {{2.,1.},{-1.,0.},{0.,-2.},{4.,4.},{0.,0.},{0.,0.},
                                                                  {0.,0.}},
                        {{1.,4.},{3.,0.},{2.,-5.},{-2.,3},{1.,1.},{0.,0.},
                                                                  {0.,0.}}};
 
    dcmplx zbb35[3][6] = {{{2.,4.},{5.,2.},{0.,-3.},{3.,0.},{-1.,2.},{0.,0.}},
                          {{3.,-2.},{5.,1.},{1.,2.},{1.,4.},{2.,-2.},{0.,0.}},
                          {{1.,-3.},{4.,4.},{2.,2.},{-3.,2.},{1.,-1.},{0.,0.}}};
    double dab36[3][4] = {1.,0.,0.,0.,-3.,4.,0.,0.,1.,-1.,2.,0.};
    double dbb36[3][3] = {1.,2.,0.,-3.,4.,0.,3.,-1.,0.};
    double dcb36[3][5] = {13.,-11.,0.,0.,0.,
                         -18.,11.,0.,0.,0.,
                          10.,-4.,0.,0.,0.};
    float sab36[3][4] = {1.,2.,1.,0.,0.,4.,
                         -1.,0.,0.,0.,-1.,0.};
    float sbb36[6][3] = {1.,2.,1.,-3.,4.,-1.,2.,0.,-1.,2.,0.,-1.,-1,
                         1.,-1.,2.,-2.,1.};
    float scb36[6][5] = {6.,9.,-2.,0.,0.,4.,11.,-6.,0.,0.,1.,5.,3.,
                         0.,0.,1.,5.,3.,0.,0.,0.,3.,-1.,0.,0.,
                         -1.,-5.,3.,0.,0.};
    float sab37[5][9] = {0.,1.,2.,3.,4.,5.,6.,7.,0.,8.,9.,10.,11.,
                         12.,13.,14.,15.,0.,16.,17.,18.,19.,20.,21.,
                         22.,23.,0.,24.,25.,26.,27.,28.,29.,30.,31.,
                         0.,32.,33.,34.,35.,36.,37.,38.,39.,0.};
    float sbb37[5][8] = {39.,38.,37.,36.,35.,34.,33.,32.,31.,30.,
                         29.,28.,27.,26.,25.,24.,23.,22.,21.,20.,
                         19.,18.,17.,16.,15.,14.,13.,12.,11.,10.,
                          9., 8., 7., 6., 5., 4., 3., 2., 1., 0.};
    float scb37[8][9] = { 0.,1.,2.,3.,4.,5.,6.,7.,0.,0.,8.,9.,10.,
                          11.,12.,13.,14.,0.,0.,0.,15.,16.,17.,18.,19.,
                          20.,0.,0.,0.,0.,21.,22.,23.,24.,25.,0.,0.,
                          0.,0.,0.,26.,27.,28.,29.,0.,0.,0.,0.,0.,
                          0.,30.,31.,32.,0.,0.,0.,0.,0.,0.,0.,33.,
                          34.,0.,0.,0.,0.,0.,0.,0.,0.,35.,0. };
    double dab37[5][9] = {0.,1.,2.,3.,4.,5.,6.,7.,0.,8.,9.,10.,11.,
                         12.,13.,14.,15.,0.,16.,17.,18.,19.,20.,21.,
                         22.,23.,0.,24.,25.,26.,27.,28.,29.,30.,31.,
                         0.,32.,33.,34.,35.,36.,37.,38.,39.,0.};
    double dbb37[5][8] = {39.,38.,37.,36.,35.,34.,33.,32.,31.,30.,
                         29.,28.,27.,26.,25.,24.,23.,22.,21.,20.,
                         19.,18.,17.,16.,15.,14.,13.,12.,11.,10.,
                          9., 8., 7., 6., 5., 4., 3., 2., 1., 0.};
    double dcb37[8][9] = { 0.,1.,2.,3.,4.,5.,6.,7.,0.,0.,8.,9.,10.,
                          11.,12.,13.,14.,0.,0.,0.,15.,16.,17.,18.,19.,
                          20.,0.,0.,0.,0.,21.,22.,23.,24.,25.,0.,0.,
                          0.,0.,0.,26.,27.,28.,29.,0.,0.,0.,0.,0.,
                          0.,30.,31.,32.,0.,0.,0.,0.,0.,0.,0.,33.,
                          34.,0.,0.,0.,0.,0.,0.,0.,0.,35.,0. };
 
/*******************************************
* ADDED FOR Z/CGECMO, Z/CGECMI       10/12 *
*******************************************/
 
    dcmplx za60[7][10] = { { { 99., 99. }, { 99., 99. }, { 99., 99. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                             { 99., 99. }, { 99., 99. }, { 99., 99. } },
                           { { 99., 99. }, { 99., 99. }, { 99., 99. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                             { 99., 99. }, { 99., 99. }, { 99., 99. } },
                               { { 99., 99. }, { 1., 1. }, { 2., 2. },
                                   { 3., 3. }, { 4., 4. }, { 5., 5. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                               { { 99., 99. }, { 6., 6. }, { 7., 7. },
                                 { 8., 8. }, { 9., 9. }, { 10., 10. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                           { { 99., 99. }, { 11., 11. }, { 12., 12. },
                             { 13., 13. }, { 14., 14. }, { 15., 15. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                           { { 99., 99. }, { 16., 16. }, { 17., 17. },
                             { 18., 18. }, { 19., 19. }, { 20., 20. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                           { { 99., 99. }, { 99., 99. }, { 99., 99. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                             { 99., 99. }, { 99., 99. }, { 99., 99. } } };
 
    cmplx ca60[7][10] = { { { 99., 99. }, { 99., 99. }, { 99., 99. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                            { 99., 99. }, { 99., 99. }, { 99., 99. } },
                          { { 99., 99. }, { 99., 99. }, { 99., 99. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                            { 99., 99. }, { 99., 99. }, { 99., 99. } },
                              { { 99., 99. }, { 1., 1. }, { 2., 2. },
                                  { 3., 3. }, { 4., 4. }, { 5., 5. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                              { { 99., 99. }, { 6., 6. }, { 7., 7. },
                                { 8., 8. }, { 9., 9. }, { 10., 10. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                          { { 99., 99. }, { 11., 11. }, { 12., 12. },
                            { 13., 13. }, { 14., 14. }, { 15., 15. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                          { { 99., 99. }, { 16., 16. }, { 17., 17. },
                            { 18., 18. }, { 19., 19. }, { 20., 20. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                          { { 99., 99. }, { 99., 99. }, { 99., 99. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                            { 99., 99. }, { 99., 99. }, { 99., 99. } } };
 
    dcmplx za61[7][10] = { { { 99., 99. }, { 99., 99. }, { 99., 99. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                             { 99., 99. }, { 99., 99. }, { 99., 99. } },
                           { { 99., 99. }, { 99., 99. }, { 99., 99. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                             { 99., 99. }, { 99., 99. }, { 99., 99. } },
                               { { 99., 99. }, { 1., 1. }, { 2., 2. },
                                   { 3., 3. }, { 4., 4. }, { 5., 5. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                               { { 99., 99. }, { 6., 6. }, { 7., 7. },
                                 { 8., 8. }, { 9., 9. }, { 10., 10. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                           { { 99., 99. }, { 11., 11. }, { 12., 12. },
                             { 13., 13. }, { 14., 14. }, { 15., 15. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                           { { 99., 99. }, { 16., 16. }, { 17., 17. },
                             { 18., 18. }, { 19., 19. }, { 20., 20. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                           { { 99., 99. }, { 21., 21. }, { 22., 22. },
                             { 23., 23. }, { 24., 24. }, { 25., 25. },
               { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } } };
 
    cmplx ca61[7][10] = { { { 99., 99. }, { 99., 99. }, { 99., 99. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                            { 99., 99. }, { 99., 99. }, { 99., 99. } },
                          { { 99., 99. }, { 99., 99. }, { 99., 99. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. },
                            { 99., 99. }, { 99., 99. }, { 99., 99. } },
                              { { 99., 99. }, { 1., 1. }, { 2., 2. },
                                  { 3., 3. }, { 4., 4. }, { 5., 5. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                              { { 99., 99. }, { 6., 6. }, { 7., 7. },
                                { 8., 8. }, { 9., 9. }, { 10., 10. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                          { { 99., 99. }, { 11., 11. }, { 12., 12. },
                            { 13., 13. }, { 14., 14. }, { 15., 15. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                          { { 99., 99. }, { 16., 16. }, { 17., 17. },
                            { 18., 18. }, { 19., 19. }, { 20., 20. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } },
                          { { 99., 99. }, { 21., 21. }, { 22., 22. },
                            { 23., 23. }, { 24., 24. }, { 25., 25. },
              { 99., 99. }, { 99., 99. }, { 99., 99. }, { 99., 99. } } };
 
/*******************************************************
*  CALL STATEMENT FOR MATRIX OPERATIONS SUBROUTINES :  *
*******************************************************/
 
    cmplx  c41, c42, c4;
    dcmplx c81, c82, c8;
    sgeadd (sa33, 6, "N", sb33, 4, "N", sc33, 5, 4, 3);
    smtchk(4, 3, sc33, sres33, 5, "sgeadd", pass);
    dgeadd (da33, 6, "N", db33, 4, "N", dc33, 5, 4, 3);
    dmtchk(4, 3, dc33, dres33, 5, "dgeadd", pass);
    cgeadd (ca33, 6, "N", cb33, 5, "N", cc33, 5, 4, 3);
    cmtchk(4, 3, cc33, cres33, 5, "cgeadd", pass);
    zgeadd (za33, 6, "N", zb33, 5, "N", zc33, 5, 4, 3);
    zmtchk(4, 3, zc33, zres33, 5, "zgeadd", pass);
    sgesub (sa34, 6, "N", sb34, 4, "N", sc34, 5, 4, 3);
    smtchk(4, 3, sc34, sres34, 5, "sgesub", pass);
    dgesub (da34, 6, "N", db34, 4, "N", dc34, 5, 4, 3);
    dmtchk(4, 3, dc34, dres34, 5, "dgesub", pass);
    cgesub (ca33, 6, "N", cb33, 5, "N", cc34, 5, 4, 3);
    cmtchk(4, 3, cc34, cres34, 5, "cgesub", pass);
    zgesub (za33, 6, "N", zb33, 5, "N", zc34, 5, 4, 3);
    zmtchk(4, 3, zc34, zres34, 5, "zgesub", pass);
    sgemul (sa35, 8, "N", sb35, 6, "N", sc35, 7, 6, 5, 4);
    smtchk(6, 4, sc35, sres35, 7, "sgemul", pass);
    dgemul (da35, 8, "N", db35, 6, "N", dc35, 7, 6, 5, 4);
    dmtchk(6, 4, dc35, dres35, 7, "dgemul", pass);
    cgemul (ca35, 6, "T", cb35, 7, "N", cc35, 3, 2, 3, 3);
    cmtchk(2, 3, cc35, cres35, 3, "cgemul", pass);
    zgemul (za35, 6, "T", zb35, 7, "N", zc35, 3, 2, 3, 3);
    zmtchk(2, 3, zc35, zres35, 3, "zgemul", pass);
    sgemms (sa35, 8, "N", sb35, 6, "N", sc35, 7, 6, 5, 4, saux, 0);
    smtchk(6, 4, sc35, sres35, 7, "sgemms", pass);
    dgemms (da35, 8, "N", db35, 6, "N", dc35, 7, 6, 5, 4, daux, 0);
    dmtchk(6, 4, dc35, dres35, 7, "dgemms", pass);
    cgemms (ca36, 4, "N", cb36, 3, "C", cc36, 4, 3, 2, 3, saux, 0);
    cmtchk(3, 3, cc36, cres36, 4, "cgemms", pass);
    zgemms (za36, 4, "N", zb36, 3, "C", zc36, 4, 3, 2, 3, daux, 0);
    zmtchk(3, 3, zc36, zres36, 4, "zgemms", pass);
    sgemm ("N", "N", 6, 4, 5, 1., sa35, 8, sb35, 6, 2., sc36a, 7);
    smtchk(6, 4, sc36a, sre36a, 7, "sgemm", pass);
    dgemm ("N", "N", 6, 4, 5, 1., da35, 8, db35, 6, 2., dc36a, 7);
    dmtchk(6, 4, dc36a, dre36a, 7, "dgemm", pass);
    CSET(c41, 1., 0.);
    CSET(c42, 2., 0.);
    cgemm ("N", "N", 6, 2, 3, c41, ca36a, 8, cb36a, 4, c42, cc36a, 8);
    cmtchk(6, 2, cc36a, cre36a, 8, "cgemm", pass);
    CSET(c81, 1., 0.);
    CSET(c82, 2., 0.);
    zgemm ("N", "N", 6, 2, 3, c81, za36a, 8, zb36a, 4, c82, zc36a, 8);
    zmtchk(6, 2, zc36a, zre36a, 8, "zgemm", pass);
    sgetmi (&sx97[2][1], 10, 5);
    smtchk(5, 5, &sx97[2][1], &sres97[2][1], 10, "sgetmi", pass);
    dgetmi (&dx97[2][1], 10, 5);
    dmtchk(5, 5, &dx97[2][1], &dres97[2][1], 10, "dgetmi", pass);
    cgetmi (&cx97[2][1], 10, 5);
    cmtchk(5, 5, &cx97[2][1], &cres97[2][1], 10, "cgetmi", pass);
    zgetmi (&zx97[2][1], 10, 5);
    zmtchk(5, 5, &zx97[2][1], &zres97[2][1], 10, "zgetmi", pass);
 
/*******************************************
* ADDED Z/CGECMI                     10/12 *
*******************************************/
 
    zgecmi (&za61[2][1], 10, 5);
    zmtchk(5, 5, &za61[2][1], &zares61[2][1], 10, "zgecmi", pass);
    cgecmi (&ca61[2][1], 10, 5);
    cmtchk(5, 5, &ca61[2][1], &cares61[2][1], 10, "cgecmi", pass);
 
    sgetmo (&sxa98[2][1], 10, 5, 4, &sxb98[1][1], 6);
    smtchk(4, 5, &sxb98[1][1], &sres98[1][1], 6, "sgetmo", pass);
    dgetmo (&dxa98[2][1], 10, 5, 4, &dxb98[1][1], 6);
    dmtchk(4, 5, &dxb98[1][1], &dres98[1][1], 6, "dgetmo", pass);
    cgetmo (&cxa98[2][1], 10, 5, 4, &cxb98[1][1], 6);
    cmtchk(4, 5, &cxb98[1][1], &cres98[1][1], 6, "cgetmo", pass);
    zgetmo (&zxa98[2][1], 10, 5, 4, &zxb98[1][1], 6);
    zmtchk(4, 5, &zxb98[1][1], &zres98[1][1], 6, "zgetmo", pass);
 
/*******************************************
* ADDED Z/CGECMO                     10/12 *
*******************************************/
 
    zgecmo (&za60[2][1], 10, 5, 4, &zb60[1][1], 6);
    zmtchk(4, 5, &zb60[1][1], &zbres60[1][1], 6, "zgecmo", pass);
    cgecmo (&ca60[2][1], 10, 5, 4, &cb60[1][1], 6);
    cmtchk(4, 5, &cb60[1][1], &cbres60[1][1], 6, "cgecmo", pass);
 
    ssyrk ("L", "N", 8, 5, 1.0, sa99, 8, 1.0, sc99, 9);
    smtckd(8, 8, sc99, scr99, 9, "ssyrk ", pass);
    dsyrk ("l", "t", 8, 3, 1.0, da99, 4, 1.0, dc99, 8);
    dmtckd(8, 8, dc99, dcr99, 8, "dsyrk ", pass);
    strmm("R","L","N","N",3,5,1.,sab35,7,sbb35,4);
    smtchk(5,4,sbb35,sreb35,4,"strmm",pass);
    dtrmm("L","U","N","N",5,3,1.,dab35,7,dbb35,6);
    dmtchk(5,3,dbb35,dreb35,6,"dtrmm",pass);
    CSET(c4,1.,0.);
    ctrmm("L","U","N","N",5,3,c4,cab35,7,cbb35,6);
    cmtchk(5,3,cbb35,creb35,6,"ctrmm",pass);
    CSET(c8,1.,0.);
    ztrmm("L","U","N","N",5,3,c8,zab35,7,zbb35,6);
    zmtchk(5,3,zbb35,zreb35,6,"ztrmm",pass);
    ssymm("L","L",3,6,2.,sab36,4,sbb36,3,2.,scb36,5);
    smtchk(3,6,scb36,sreb36,5,"ssymm",pass);
    dsymm("R","U",2,3,2.,dab36,4,dbb36,3,1.,dcb36,5);
    dmtchk(2,3,dcb36,dreb36,5,"dsymm",pass);
    dsyr2k("L","N",8,5,1.,dab37,9,dbb37,8,1,dcb37,9);
    dmtchk(8,5,dcb37,dreb37,9,"dsyr2k",pass);
    ssyr2k("L","N",8,5,1.,sab37,9,sbb37,8,1,scb37,9);
    smtchk(8,5,scb37,sreb37,9,"ssyr2k",pass);
    CSET(c1n, 1.,1.);
    CSET(c2n, 1.,1.);
    csyrk("L","N",3,5,c1n,cska21n,3,c2n,cskx21n,4);
    cmtchk(3,3,cskx21n,cskres21n,4,"csyrk",pass);
    CSET(dc1n, 1.,1.);
    CSET(dc2n, 1.,1.);
    zsyrk("L","N",3,5,dc1n,zska21n,3,dc2n,zskx21n,4);
    zmtchk(3,3,zskx21n,zskres21n,4,"zsyrk",pass);
 
    cherk("L","N",3,5,1.,chka22n,3,1.,chkx22n,4);
    cmtchk(3,3,chkx22n,chkres22n,4,"cherk",pass);
    zherk("L","N",3,5,1.,zhka22n,3,1.,zhkx22n,4);
    zmtchk(3,3,zhkx22n,zhkres22n,4,"zherk",pass);
 
    CSET(c1n, 1.,1.);
    csyr2k("L","N",3,5,c1n,cs2ka23n,3,cs2kb23n,4,c1n,cs2kc23n,4);
    cmtchk(3,3,cs2kc23n,cs2kres23n,4,"csyr2k",pass);
    CSET(dc1n, 1.,1.);
    zsyr2k("L","N",3,5,dc1n,zs2ka23n,3,zs2kb23n,4,dc1n,zs2kc23n,4);
    zmtchk(3,3,zs2kc23n,zs2kres23n,4,"zsyr2k",pass);
 
    cher2k("L","N",3,5,c1n,ch2ka23n,3,ch2kb23n,4,1.,ch2kc23n,4);
    cmtchk(3,3,ch2kc23n,ch2kres23n,4,"cher2k",pass);
    zher2k("L","N",3,5,dc1n,zh2ka23n,3,zh2kb23n,4,1.,zh2kc23n,4);
    zmtchk(3,3,zh2kc23n,zh2kres23n,4,"zher2k",pass);
 
    CSET(c1n, 2., 3.);
    CSET(c2n, 1., 6.);
    csymm("L","U",5,4,c1n,csya25n,8,csyb25n,6,c2n,csyc25n,5);
    cmtchk(5,4,csyc25n,csyres25n,5,"csymm",pass);
    CSET(dc1n, 2., 3.);
    CSET(dc2n, 1., 6.);
    zsymm("L","U",5,4,dc1n,zsya25n,8,zsyb25n,6,dc2n,zsyc25n,5);
    zmtchk(5,4,zsyc25n,zsyres25n,5,"zsymm",pass);
 
    CSET(c1n,2.,3.);
    CSET(c2n,1.,6.);
    chemm("L","U",5,4,c1n,chea26n,8,cheb26n,6,c2n,chec26n,5);
    cmtchk(5,4,chec26n,cheres26n,5,"chemm",pass);
    CSET(dc1n,2.,3.);
    CSET(dc2n,1.,6.);
    zhemm("L","U",5,4,dc1n,zhea26n,8,zheb26n,6,dc2n,zhec26n,5);
    zmtchk(5,4,zhec26n,zheres26n,5,"zhemm",pass);
 
}
 
/*********************************************************************
*                      SUBROUTINE:  LIEQS                            *
*********************************************************************/
void lieqs(pass)
_ESVINT *pass;
{
/**********************************************************
*  VARIABLES OF LINEAR ALGEBRAIC EQUATIONS SUBROUTINES :  *
**********************************************************/
 
/*****************************
*  (DENSE LINEAR ALGEBRAIC)  *
*****************************/
 
    _ESVINT itmp,info;
    cmplx  caux1[1];
    dcmplx zaux2[1];
    double dtmp;
    _ESVINT ipvt37[9], ipvt45[9],ipvt40[5];
    float  saux1[10000];
    double daux1[10000], daux2[10000];
    _ESVINT ci37[4];
    _ESVINT zi37[4];
    double ddet41[2], drcn41[1];
    float  sdet41[2], srcn41[1];
    double ddet43[2], drcn43[1];
    float  sdet43[2], srcn43[1];
    _ESVINT ipvt43[9];
    double ddet44[2], drcn44[1];
    float  sdet44[2], srcn44[1];
    double dr99, dd99[2];
    float  sr99, sd99[2];
    cmplx c4;
    dcmplx c8;
    float saux1n[100];
    double daux1n[100];
    float spcrc30n[1],spcde30n[2];
    double dpcrc30n[1],dpcde30n[2];
    float  spirc31n[1],spide31n[2];
    double dpirc31n[1],dpide31n[2];
    cmplx c1n;
    dcmplx dc1n;
    double daux1d[99], daux2d[63];
    double tau[2],work[2];
    float  stau9[2],swork9[2];
    dcmplx ztau9[2],zwork9[2];
    cmplx  ctau9[2],cwork9[2];
    float  swork10[1];
    dcmplx zwork10[1];
    cmplx  cwork10[1];
 
/******************************
*  (BANDED LINEAR ALGEBRAIC)  *
******************************/
 
    _ESVINT ipvt[100];
    float  sd49[4];
    double dd49[4];
 
/******************************
*  (SPARSE LINEAR ALGEBRAIC)  *
******************************/
 
    double dop105[5];
 
/***************************
*  (LINEAR LEAST SQUARES)  *
***************************/
 
    double ds56[3], ddum[1], dummy[1][1];
    float  ss56[3], sdum[1], summy[1][1];
    double dx57[1][3];
    float  sx57[1][3];
    double dx58[1][3];
    float  sx58[1][3];
    _ESVINT ik58[1];
 
/***************************************************************
*  EXPECTED VALUE FOR LINEAR ALGEBRAIC EQUATIONS SUBROUTINES : *
***************************************************************/
 
    float  sr37[9][9] = { 4., 0., 0., 0., 0., 0.,  0., 2.5e-1, 2.5e-1,
                          1., 5., 0., 0., 0., 0.,  0., 1.5e-1, 1.5e-1,
                          1., 1., 6., 0., 0., 0.,  0., 1.e-1,  1.e-1,
                          1., 1., 1., 7., 0., 0.,  0., 7.14e-2, 7.14e-2,
                          1., 1., 1., 1., 8., 0.,  0., 5.36e-2, -7.14e-2,
                          1., 1., 1., 1., 1., 9.,  0., -6.94e-2, -5.555e-2,
                          0., 1., 1., 1., 1., 1., 10., -3.06e-2, -1.94e-2,
                          0., 0., 1., 1., 1., 1., 11., 1.806e-1, 9.385e-1,
                          0., 0., 0., 1., 1., 1., 12., 3.111e-1, -3.1e-3 };
    double dr37[9][9] = { 4., 0., 0., 0., 0., 0.,  0., 2.5e-1, 2.5e-1,
                          1., 5., 0., 0., 0., 0.,  0., 1.5e-1, 1.5e-1,
                          1., 1., 6., 0., 0., 0.,  0., 1.e-1,  1.e-1,
                          1., 1., 1., 7., 0., 0.,  0., 7.14e-2, 7.14e-2,
                          1., 1., 1., 1., 8., 0.,  0., 5.36e-2, -7.14e-2,
                          1., 1., 1., 1., 1., 9.,  0., -6.94e-2, -5.555e-2,
                          0., 1., 1., 1., 1., 1., 10., -3.06e-2, -1.94e-2,
                          0., 0., 1., 1., 1., 1., 11., 1.806e-1, 9.385e-1,
                          0., 0., 0., 1., 1., 1., 12., 3.111e-1, -3.1e-3 };
    cmplx  car37[4][4] = {
        { { 8., 5. },{ .2022, .1236 },{ .2360, -.0225 }, { .1798, -.1124 } },
        { { 1., 9. },{ 1.9101, 5.0562 },{ -.0654, -.9269 },{ .2462, .1308 } },
        { { 6., 6. },{ 1.5281, 2.0449 },{ -.3462, 6.2692 },{ .4412, -.3655 } },
        { { 8., 1. },{ 1.5056, -.191 },{ -1.6346, 1.3269 },{ .29, 2.3864 } } };
    _ESVINT cir37[4] = { 4, 4, 3, 4 };
    dcmplx zar37[4][4] = {
        { { 8., 5. },{ .2022, .1236 },{ .2360, -.0225 }, { .1798, -.1124 } },
        { { 1., 9. },{ 1.9101, 5.0562 },{ -.0654, -.9269 },{ .2462, .1308 } },
        { { 6., 6. },{ 1.5281, 2.0449 },{ -.3462, 6.2692 },{ .4412, -.3655 } },
        { { 8., 1. },{ 1.5056, -.191 },{ -1.6346, 1.3269 },{ .29, 2.3864 } } };
    _ESVINT zir37[4] = { 4, 4, 3, 4 };
    cmplx  cbr37[4] = { { 9., 0. }, { 5., 1. }, { 1., 6. }, { 3., 4. } };
    dcmplx zbr37[4] = { { 9., 0. }, { 5., 1. }, { 1., 6. }, { 3., 4. } };
    float  sres38[9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    double dres38[9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    float  sres39[54] = { 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                          1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                          1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                          1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                          1., 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    double dres39[54] = { 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                          1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                          1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                          1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                          1., 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    float  sres40[9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    double dres40[9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    float  sres41[9][9] = {
        0.333,  56.833, -55.167, -1., -1., -1., -226.,  56.e1, -325.,
       -0.667, -52.167,  51.833,  1.,  1.,  1.,  206., -52.e1,  305.,
        0.333,  -1.167,   0.833,  0.,  0.,  0.,    5., -10.,      5.,
        0.,     -0.5,     0.5,    0.,  0.,  0.,    3.,  -6.,      3.,
        0.,     -0.5,     0.5,    0.,  0.,  0.,    2.,  -4.,      2.,
        0.,     -3.571e-1, 0.214, 0.143, 0., 0., 1.429, -2.857,   1.429,
        0.042,   6.836,  -6.735, -0.143, 0., -0.125, -27.179, 67.857, -39.554,
       -0.042,  -0.479,   0.521,  0.,  0.,  0.125, 1.75, -5.,     3.125,
        0.,     -0.5,     0.5,    0.,  0.,  0.,    2.,   -5.,     3. };
    float  srrc41[1] = { 5.436e-5 };
    float  srde41[2] = { 3.36, 2. };
    double dres41[9][9] = {
        0.333,  56.833, -55.167, -1., -1., -1., -226.,  56.e1, -325.,
       -0.667, -52.167,  51.833,  1.,  1.,  1.,  206., -52.e1,  305.,
        0.333,  -1.167,   0.833,  0.,  0.,  0.,    5., -10.,      5.,
        0.,     -0.5,     0.5,    0.,  0.,  0.,    3.,  -6.,      3.,
        0.,     -0.5,     0.5,    0.,  0.,  0.,    2.,  -4.,      2.,
        0.,     -3.571e-1, 0.214, 0.143, 0., 0., 1.429, -2.857,   1.429,
        0.042,   6.836,  -6.735, -0.143, 0., -0.125, -27.179, 67.857, -39.554,
       -0.042,  -0.479,   0.521,  0.,  0.,  0.125, 1.75, -5.,     3.125,
        0.,     -0.5,     0.5,    0.,  0.,  0.,    2.,   -5.,     3. };
    double drrc41[1] = { 5.436e-5 };
    double drde41[2] = { 3.36, 2. };
    float  sres43[9][9] = {
        4., 0., 0., 0., 0., 0., 0., 2.5e-1, 2.5e-1,
        1., 5., 0., 0., 0., 0., 0., 1.5e-1, 1.5e-1,
        1., 1., 6., 0., 0., 0., 0., 1.e-1,  1.e-1,
        1., 1., 1., 7., 0., 0., 0., 7.14e-2, 7.14e-2,
        1., 1., 1., 1., 8., 0., 0., 5.36e-2, -7.14e-2,
        1., 1., 1., 1., 1., 9., 0., -6.94e-2, -5.555e-2,
        0., 1., 1., 1., 1., 1., 10., -3.06e-2, -1.94e-2,
        0., 0., 1., 1., 1., 1., 11., 1.806e-1, 9.385e-1,
        0., 0., 0., 1., 1., 1., 12., 3.111e-1, -3.1e-3 };
    double dres43[9][9] = {
        4., 0., 0., 0., 0., 0., 0., 2.5e-1, 2.5e-1,
        1., 5., 0., 0., 0., 0., 0., 1.5e-1, 1.5e-1,
        1., 1., 6., 0., 0., 0., 0., 1.e-1,  1.e-1,
        1., 1., 1., 7., 0., 0., 0., 7.14e-2, 7.14e-2,
        1., 1., 1., 1., 8., 0., 0., 5.36e-2, -7.14e-2,
        1., 1., 1., 1., 1., 9., 0., -6.94e-2, -5.555e-2,
        0., 1., 1., 1., 1., 1., 10., -3.06e-2, -1.94e-2,
        0., 0., 1., 1., 1., 1., 11., 1.806e-1, 9.385e-1,
        0., 0., 0., 1., 1., 1., 12., 3.111e-1, -3.1e-3 };
    float  srrc43[1] = { 6.1057e-5 };
    float  srde43[2] = { 3.36, 2. };
    _ESVINT irpv43[9] = { 3, 4, 5, 6, 7, 8, 9, 8, 9 };
    double drrc43[1] = { 6.1057e-5 };
    double drde43[2] = { 3.36, 2. };
    float  srde44[2] = { 1., 0. };
    float  srrc44[1] = { 5.8667e-3 };
    double drde44[2] = { 1., 0. };
    double drrc44[1] = { 5.8667e-3 };
    _ESVINT ires45[9] = { 2, -65534, -131070, -196606, -262142,
                      -327678, -327678,-327680, -327680 };
    float  sres45[9][23] = {
        0., 0., 0., 0., 0.,2.5e-1, 0., 2.5e-1, 2.5e-1, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 0., 0., 1.,0.2, 1.5e-1, 1.5e-1, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 0., 1., 1.,0.16, 0., 1.0e-1, 1.0e-1, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1.,0.14, 7.14e-2, 7.14e-2, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
        0., 1., 1., 1., 1.,0.125, 0.,-7.14e-2, 5.36e-2, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0.,
        1., 1., 1., 1., 1.,0.11, -5.56e-2, -6.94e-2, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
        1., 1., 1., 1., 1.,0.10, -3.06e-2, -1.94e-2, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
        1., 1., 1., 1., 11., 5.538, 93.85e-2, 0., 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
        1., 1., 1., 12., 31.11e-2, -325., 0., 0., 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    double dres45[9][23] = {
        0., 0., 0., 0., 0.,2.5e-1, 0., 2.5e-1, 2.5e-1, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 0., 0., 1.,0.2, 1.5e-1, 1.5e-1, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 0., 1., 1.,0.16, 0., 1.0e-1, 1.0e-1, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1.,0.14, 7.14e-2, 7.14e-2, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
        0., 1., 1., 1., 1.,0.125, 0.,-7.14e-2, 5.36e-2, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0.,
        1., 1., 1., 1., 1.,0.11, -5.56e-2, -6.94e-2, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
        1., 1., 1., 1., 1.,0.10, -3.06e-2, -1.94e-2, 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
        1., 1., 1., 1., 11., 5.538, 93.85e-2, 0., 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
        1., 1., 1., 12., 31.11e-2, -325., 0., 0., 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    float  sres46[9] = { 1., 1., 1., 1., 1., 1., 1., 0.9999, 1.0001 };
    double dres46[9] = { 1., 1., 1., 1., 1., 1., 1., 0.9999, 1.0001 };
    float  sres47[12][3] = { 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                             1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 0.,
                             1., 0., 0., 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    double dres47[12][3] = { 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                             1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 0.,
                             1., 0., 0., 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    float  sres48[9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    double dres48[9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    float  sres48n[9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    double dres48n[9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    float  srea49[4] = { 0., -0.5, -0.5, -0.5 };
    float  sreb49[4] = { -0.5, -0.5, -0.5, -0.5 };
    float  srec49[4] = { 2., 2., 2., 0. };
    float  sred49[4] = { 0., 0., 0., 0. };
    double drea49[4] = { 0., -0.5, -0.5, -0.5 };
    double dreb49[4] = { -0.5, -0.5, -0.5, -0.5 };
    double drec49[4] = { 2., 2., 2., 0. };
    double dred49[4] = { 0., 0., 0., 0. };
    float  sres50[4] = { 1.0, 1.0, 1.0, 1.0 };
    double dres50[4] = { 1.0, 1.0, 1.0, 1.0 };
    float  srea51[4] = { -1., -1., -1., -1. };
    float  sreb51[4] = { 0., -1., -1., 1. };
    float  srec51[4] = { 1., 1., -1., 0. };
    float  sres52[4] = { 1.0, 1.0, 1.0, 1.0 };
    double drea51[4] = { -1., -1., -1., -1. };
    double dreb51[4] = { 0., -1., -1., 1. };
    double drec51[4] = { 1., 1., -1., 0. };
    double dres52[4] = { 1.0, 1.0, 1.0, 1.0 };
    float sxrestnp[4] = { 1., 1., 1., 1. };
    double dxrestnp[4] = { 1., 1., 1., 1. };
    float  srea53[4] = { -1., -1., -1., -1. };
    float  sreb53[4] = { 0., -1., -1., -1. };
    double drea53[4] = { -1., -1., -1., -1. };
    double dreb53[4] = { 0., -1., -1., -1. };
    double dres55[9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    double d2rp55[1] = { 0. };
    double d3rp55[1] = { 0.351e-15 };
    _ESVINT ires55[1] = { 5 };
    double dre55a[9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    double rpr55a[2] = { 0., 0.46e-16 };
    _ESVINT ire55a[1] = { 5 };
    float  sres56[3] = { 25.462, 1.291, 0. };
    double dres56[3] = { 25.462, 1.291, 0. };
    float  sres57[1][3] = { -0.6, -1.2, 2.0 };
    double dres57[1][3] = { -0.6, -1.2, 2.0 };
    float  sres58[1][3] = { -0.6, -1.2, 2.0 };
    double dres58[1][3] = { -0.6, -1.2, 2.0 };
    _ESVINT ires58[1] = { 2 };
    double duz103[9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    double duz104[9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    double daz105[27] = { .5, 2., .3333, 1., 1., 1., 3., 3., 2.,
                          2., 2., 1., 1., 0., 0., 0., 0., 0.,
                          0., 0., -1.6666, -.5, -1., -1., 4, -3., -4. };
    _ESVINT diz105[27] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                       0, 0, 0, 0, 0, 0, 2, 1, 1, 3, 3, 5, 5 };
    _ESVINT djz105[27] = { 1, 0, 5, 2, 0, 4, 0, 2, 0, 0, 0, 3, 4, 0,
                       0, 0, 0, 0, 0, 0, 4, 2, 4, 4, 1, 3, 1 };
    double doz105[5] = { 1., .333333, 3., 0., 0. };
    double dbz106[5] = { -5.5, 9.5, 3., -2., -1. };
    double drz99 = { .005556 };
    double ddrs99[2] = { 1., 0. };
    double daz99[45] = { 2., -1., 0., 0., 0., 0., 0., 0., 0.,
                         2., -1., 0., 0., 0., 0., 0., 0.,
                         2., -1., 0., 0., 0., 0., 0.,
                         2., -1., 0., 0., 0., 0.,
                         2., -1., 0., 0., 0.,
                         2., -1., 0., 0.,
                         2., -1., 0.,
                         2., -1.,
                         1. };
    float  srz99 = { .005556 };
    float  sdrs99[2] = { 1., 0. };
    float  saz99[45] = { 2., -1., 0., 0., 0., 0., 0., 0., 0.,
                         2., -1., 0., 0., 0., 0., 0., 0.,
                         2., -1., 0., 0., 0., 0., 0.,
                         2., -1., 0., 0., 0., 0.,
                         2., -1., 0., 0., 0.,
                         2., -1., 0., 0.,
                         2., -1., 0.,
                         2., -1.,
                         1. };
    float  sbr102[3][6] = { 2., 5., 0., 3., -1., 0.,
                            3., 5., 1., 1., 2., 0.,
                            1., 4., 2., -3., 1., 0. };
    double dbr102[3][6] = { 2., 5., 0., 3., -1., 0.,
                            3., 5., 1., 1., 2., 0.,
                            1., 4., 2., -3., 1., 0. };
    float sreb34[4] = {5.,4.,3.,2.};
    double dreb34[4] = {5.,4.,3.,2.};
    cmplx creb34[4] = {{5.,5.},{4.,4.},{3.,3.},{2.,2.}};
    dcmplx zreb34[4] = {{5.,5.},{4.,4.},{3.,3.},{2.,2.}};
    cmplx creb38[3][6]={{{2.,4.},{5.,2.},{0.,-3.},{3.,0.},{-1.,2.},
                        {0.,0.}},
                        {{3.,-2.},{5.,1.},{1.,2.},{1.,4.},{2.,-2.},
                        {0.,0.}},
                        {{1.,-3.},{4.,4.},{2.,2.},{-3.,2.},{1.,-1.},
                        {0.,0.}}};
    dcmplx zreb38[3][6]={{{2.,4.},{5.,2.},{0.,-3.},{3.,0.},{-1.,2.},
                        {0.,0.}},
                        {{3.,-2.},{5.,1.},{1.,2.},{1.,4.},{2.,-2.},
                        {0.,0.}},
                        {{1.,-3.},{4.,4.},{2.,2.},{-3.,2.},{1.,-1.},
                        {0.,0.}}};
    float  sgeres27n[2][9] ={ 1.,1.,1.,1.,1.,1.,1.,1.,1.,
                              1.,2.,3.,4.,5.,6.,7.,8.,9.};
    double dgeres27n[2][9] = {1.,1.,1.,1.,1.,1.,1.,1.,1.,
                              1.,2.,3.,4.,5.,6.,7.,8.,9.};
    cmplx  cgeres27n[2][4] = {{{9.,0.}, {5.,1.}, {1.,6.},{ 3.,4.}},
                             {{1.,1.}, {2.,2.}, {3.,3.}, {4.,4.}}};
    dcmplx zgeres27n[2][4] = {{{9.,0.}, {5.,1.}, {1.,6.},{ 3.,4.}},
                             {{1.,1.},{ 2.,2.},{ 3.,3.},{ 4.,4.}}};
    _ESVINT ipvtr37[9] = {3,4,5,6,7,8,9,8,9};
    _ESVINT ipvtr40[5] = {5,5,5,5,5};
    float  spores28n[9][9] = {1.,1.,1.,1.,1.,1.,1.,1.,1.,
                         1.,1.,1.,1.,1.,1.,1.,1.,1.,
                         1.,2.,1.,1.,1.,1.,1.,1.,1.,
                         1.,2.,3.,1.,1.,1.,1.,1.,1.,
                         1.,2.,3.,4.,1.,1.,1.,1.,1.,
                         1.,2.,3.,4.,5.,1.,1.,1.,1.,
                         1.,2.,3.,4.,5.,6.,1.,1.,1.,
                         1.,2.,3.,4.,5.,6.,7.,1.,1.,
                         1.,2.,3.,4.,5.,6.,7.,8.,1.};
    double dpores28n[9][9] = {1.,1.,1.,1.,1.,1.,1.,1.,1.,
                         1.,1.,1.,1.,1.,1.,1.,1.,1.,
                         1.,2.,1.,1.,1.,1.,1.,1.,1.,
                         1.,2.,3.,1.,1.,1.,1.,1.,1.,
                         1.,2.,3.,4.,1.,1.,1.,1.,1.,
                         1.,2.,3.,4.,5.,1.,1.,1.,1.,
                         1.,2.,3.,4.,5.,6.,1.,1.,1.,
                         1.,2.,3.,4.,5.,6.,7.,1.,1.,
                         1.,2.,3.,4.,5.,6.,7.,8.,1.};
    cmplx  cpores28n[3][3] = {{{5.,0.}, {-1.,1.}, {2.,-1.}},
                         {{-5.,-5.}, {7.,0.}, {1.,1.}},
                         {{10.,5.}, {4.,-6.}, {8.,0.}}};
    dcmplx zpores28n[3][3] = {{{5.,0.}, {-1.,1.}, {2.,-1.}},
                         {{-5.,-5.},{ 7.,0.}, {1.,1.}},
                         {{10.,5.}, {4.,-6.}, {8.,0.}}};
    cmplx  cposres29n[2][3]= {{{2.,-1.},{ 1.,1.}, {0.,-2.}},
                         {{2.,0.}, {-1.,2.}, {1.,1.}}};
    dcmplx zposres29n[2][3]= {{{2.,-1.},{ 1.,1.},{ 0.,-2.}},
                         {{2.,0.}, {-1.,2.}, {1.,1.}}};
    cmplx cpoa41n1[3][3] = { {{ 25.0, 0.0},{ -5.0, 5.0},{ 10.0,-5.0}},
                             {{-5.0,-5.0},{51.0, 0.0},{ 4.0, 6.0}},
                             {{10.0, 5.0},{ 4.0,-6.0},{71.0, 0.0}} };
    cmplx  cpores41[3][3] = {{{.05,0.0},{0.0,-0.01},{-0.01,0.0}},
                 {{-5.0,-5.0},{0.02,0.0},{0.0,0.0}},
                 {{10.0,5.0},{4.0,-6.0},{0.02,0.0}}};
    dcmplx zpoa41n1[3][3] = { {{ 25.0, 0.0},{ -5.0, 5.0},{ 10.0,-5.0}},
                             {{-5.0,-5.0},{51.0, 0.0},{ 4.0, 6.0}},
                             {{10.0, 5.0},{ 4.0,-6.0},{71.0, 0.0}} };
    dcmplx  zpores41[3][3] = {{{.05,0.0},{0.0,-0.01},{-0.01,0.0}},
                 {{-5.0,-5.0},{0.02,0.0},{0.0,0.0}},
                 {{10.0,5.0},{4.0,-6.0},{0.02,0.0}}};
    float  spcderes30n[2] = {1.,0.};
    float  spcrcres30n[1] ={ 0.005555};
    double dpcderes30n[2] = {1.,0.};
    double dpcrcres30n[1] = {0.005555};
 
    float  spires31n[9][9] = {2.,0.,0.,0.,0.,0.,0.,0.,0.,
                         -1.,2.,0.,0.,0.,0.,0.,0.,0.,
                         0.,-1.,2.,0.,0.,0.,0.,0.,0.,
                         0.,0.,-1.,2.,0.,0.,0.,0.,0.,
                         0.,0.,0.,-1.,2.,0.,0.,0.,0.,
                         0.,0.,0.,0.,-1.,2.,0.,0.,0.,
                         0.,0.,0.,0.,0.,-1.,2.,0.,0.,
                         0.,0.,0.,0.,0.,0.,-1.,2.,0.,
                         0.,0.,0.,0.,0.,0.,0.,-1.,1.};
    float  spideres31n[2] = {1.,0.};
    float  spircres31n[1] = {0.005555};
    double dpires31n[9][9] ={ 2.,0.,0.,0.,0.,0.,0.,0.,0.,
                         -1.,2.,0.,0.,0.,0.,0.,0.,0.,
                         0.,-1.,2.,0.,0.,0.,0.,0.,0.,
                         0.,0.,-1.,2.,0.,0.,0.,0.,0.,
                         0.,0.,0.,-1.,2.,0.,0.,0.,0.,
                         0.,0.,0.,0.,-1.,2.,0.,0.,0.,
                         0.,0.,0.,0.,0.,-1.,2.,0.,0.,
                         0.,0.,0.,0.,0.,0.,-1.,2.,0.,
                         0.,0.,0.,0.,0.,0.,0.,-1.,1.};
    double dpideres31n[2] = {1.,0.};
    double dpircres31n[1] = {0.005555};
    float stpiares31n[15] = {1., -1.5,0.5, 2.,-1.,0.25,
                       3.75,-1.75,1.,-0.5, 35.,-14.,7.,-3.,-1.};
    double dtpiares31n[15] = {1., -1.5,0.5, 2.,-1.,0.25,
                       3.75,-1.75,1.,-0.5, 35.,-14.,7.,-3.,-1.};
    cmplx ctpiares31n[15] = { {-0.24,-0.06},{-0.56, 0.24},{-0.50, 0.00},
                              { 0.41, 0.09},{ 0.18, 0.21},{-0.15,-0.09},
                              {-0.22, 0.13},{-0.22,-0.06},{ 0.07,-0.11},
                              { 0.12, 0.12},{ 1.32, 2.12},{ 0.21, 1.87},
                              { 0.02,-0.47},{-0.05,-0.15},{ 0.40, 0.20} };
    dcmplx ztpiares31n[15] = {{-0.24,-0.06},{-0.56, 0.24},{-0.50, 0.00},
                              { 0.41, 0.09},{ 0.18, 0.21},{-0.15,-0.09},
                              {-0.22, 0.13},{-0.22,-0.06},{ 0.07,-0.11},
                              { 0.12, 0.12},{ 1.32, 2.12},{ 0.21, 1.87},
                              { 0.02,-0.47},{-0.05,-0.15},{ 0.40, 0.20} };
 
 
    float  strires32n[5][5]={1.,0.,0.,0.,0., -1.5,0.5,0.,0.,0.,
          2.,-1.,0.25,0.,0., 3.75,-1.75,1.,-0.5,0., 35.,-14.,7.,-3.,-1.};
    double dtrires32n[5][5]={1.,0.,0.,0.,0., -1.5,0.5,0.,0.,0.,
          2.,-1.,0.25,0.,0., 3.75,-1.75,1.,-0.5,0., 35.,-14.,7.,-3.,-1.};
    cmplx ctrires32n[5][5]={
     { {-0.24,-0.06},{0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0}},
     { {-0.56, 0.24},{-0.50, 0.00},{0.0,0.0},{0.0,0.0},{0.0,0.0}},
     { { 0.41, 0.09},{ 0.18, 0.21},{-0.15,-0.09},{0.0,0.0},{0.0,0.0}},
     { {-0.22, 0.13},{-0.22,-0.06},{ 0.07,-0.11},{ 0.12, 0.12},{0.0,0.0}},
     { { 1.32, 2.12},{ 0.21, 1.87},{ 0.02,-0.47},{-0.05,-0.15},{ 0.40, 0.20}}
 };
    dcmplx ztrires32n[5][5]={
     { {-0.24,-0.06},{0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0}},
     { {-0.56, 0.24},{-0.50, 0.00},{0.0,0.0},{0.0,0.0},{0.0,0.0}},
     { { 0.41, 0.09},{ 0.18, 0.21},{-0.15,-0.09},{0.0,0.0},{0.0,0.0}},
     { {-0.22, 0.13},{-0.22,-0.06},{ 0.07,-0.11},{ 0.12, 0.12},{0.0,0.0}},
     { { 1.32, 2.12},{ 0.21, 1.87},{ 0.02,-0.47},{-0.05,-0.15},{ 0.40, 0.20}}
 };
 
    float  stpvres35n[4] = {1.,2.,3.,4.};
    double dtpvres35n[4] = {1.,2.,3.,4.};
    cmplx  ctpvres35n[4] = {{1.,1.}, {2.,2.}, {3.,3.}, {4.,4.}};
    dcmplx ztpvres35n[4] = {{1.,1.}, {2.,2.}, {3.,3.}, {4.,4.}};
 
    float  stbvres36n[9] = {1.,1.,0.,1.,0.,2.,0.,1.,3.};
    double dtbvres36n[9] = {1.,1.,0.,1.,0.,2.,0.,1.,3.};
    cmplx  ctbvres36n[7] = {{-12.048,-13.136},{6.304,-1.472},
    {-1.88,1.04},{2.6,-1.8},{-2.16,1.88},{0.8,-1.4},{0.8,0.6}};
    dcmplx  ztbvres36n[7] = {{-12.048,-13.136},{6.304,-1.472},
    {-1.88,1.04},{2.6,-1.8},{-2.16,1.88},{0.8,-1.4},{0.8,0.6}};
 
    cmplx  cgtfcres37n[4] = {{0.,0.},{ -0.142,0.},{ -0.269,0.},{3.,3.}};
    cmplx  cgtfares37n[4] = {{-0.0714,0.0714},{-0.0673,0.0673},
                         {-0.0854,0.0854}, {-0.05,0.05}};
    cmplx  cgtfbres37n[4] = {{4.,4.}, {5.,5.}, {-.6,0.}, {0.,0.}};
    dcmplx zgtfcres37n[4] = {{0.,0.},{ -0.142,0.}, {-0.269,0.}, {3.,3.}};
    dcmplx zgtfares37n[4] = {{-0.0714,0.0714}, {-0.0673,0.0673},
                         {-0.0854,0.0854}, {-0.05,0.05}};
    dcmplx zgtfbres37n[4] = {{4.,4.}, {5.,5.}, {-.6,0.}, {0.,0.}};
 
    cmplx  cgtsxres38n[4] = {{0.,1.}, {1.,2.}, {2.,3.}, {3.,4.}};
    dcmplx zgtsxres38n[4] = {{0.,1.}, {1.,2.}, {2.,3.}, {3.,4.}};
    cmplx  cxrestnp[4] = {{0.,1.},{1.,2.},{2.,3.},{3.,4.}};
    dcmplx  zxrestnp[4] = {{0.,1.},{1.,2.},{2.,3.},{3.,4.}};
 
    double dssxres39n[9] = {1.,1.,1.,1.,1.,1.,1.,1.,1.};
    _ESVINT    dssipres39n[6] = {20,4,0,1,0,9};
    double dssrpres39n[3] = {1.0e-07,0.46e-16,1.};
 
  float  sresa7[9][9] = {
   2.6000,   .3846,   .4615,   .5385,  .6154,  .6923,  .7692, .8462, .9231,
   2.4000,   .2769,  -.3889,  -.3333, -.2778, -.2222, -.1667,-.1111,-.0556,
   2.2000,   .5538,   .4000,   .0000,  .0000,  .0000,  .0000, .0000, .0000,
   2.0000,   .8308,   .8000,   .4000,  .0000,  .0000,  .0000, .0000, .0000,
   1.8000,  1.1077,  1.2000,   .8000,  .4000,  .0000,  .0000, .0000, .0000,
   1.6000,  1.3846,  1.6000,  1.2000,  .8000,  .4000,  .0000, .0000, .0000,
   1.4000,  1.6615,  2.0000,  1.6000, 1.2000,  .8000,  .4000, .0000, .0000,
   1.2000,  1.9385,  2.4000,  2.0000, 1.6000, 1.2000,  .8000, .4000, .0000,
   1.0000,  2.2154,  2.8000,  2.4000, 2.0000, 1.6000, 1.2000, .8000, .4000 };
 
  double dresa7[9][9] = {
   2.6000,   .3846,   .4615,   .5385,  .6154,  .6923,  .7692, .8462, .9231,
   2.4000,   .2769,  -.3889,  -.3333, -.2778, -.2222, -.1667,-.1111,-.0556,
   2.2000,   .5538,   .4000,   .0000,  .0000,  .0000,  .0000, .0000, .0000,
   2.0000,   .8308,   .8000,   .4000,  .0000,  .0000,  .0000, .0000, .0000,
   1.8000,  1.1077,  1.2000,   .8000,  .4000,  .0000,  .0000, .0000, .0000,
   1.6000,  1.3846,  1.6000,  1.2000,  .8000,  .4000,  .0000, .0000, .0000,
   1.4000,  1.6615,  2.0000,  1.6000, 1.2000,  .8000,  .4000, .0000, .0000,
   1.2000,  1.9385,  2.4000,  2.0000, 1.6000, 1.2000,  .8000, .4000, .0000,
   1.0000,  2.2154,  2.8000,  2.4000, 2.0000, 1.6000, 1.2000, .8000, .4000 };
 
  float  sresb7[5][9] = {
   1.0,  2.0,  3.0,  4.0,  5.0,  6.0,  7.0,  8.0,  9.0,
   2.0,  4.0,  6.0,  8.0, 10.0, 12.0, 14.0, 16.0, 18.0,
   3.0,  6.0,  9.0, 12.0, 15.0, 18.0, 21.0, 24.0, 27.0,
   4.0,  8.0, 12.0, 16.0, 20.0, 24.0, 28.0, 32.0, 36.0,
   5.0, 10.0, 15.0, 20.0, 25.0, 30.0, 35.0, 40.0, 45.0 };
 
  double dresb7[5][9] = {
   1.0,  2.0,  3.0,  4.0,  5.0,  6.0,  7.0,  8.0,  9.0,
   2.0,  4.0,  6.0,  8.0, 10.0, 12.0, 14.0, 16.0, 18.0,
   3.0,  6.0,  9.0, 12.0, 15.0, 18.0, 21.0, 24.0, 27.0,
   4.0,  8.0, 12.0, 16.0, 20.0, 24.0, 28.0, 32.0, 36.0,
   5.0, 10.0, 15.0, 20.0, 25.0, 30.0, 35.0, 40.0, 45.0 };
 
    _ESVINT ipvtr7[9] = {9,9,9,9,9,9,9,9,9};
 
  cmplx cresa7[9][9] = {
      {{5.20,1.0},{0.4, 0.1}, {0.5, 0.1},{0.6,0.1},{0.6,0.1},
       {0.7, 0.1},{0.8, 0.0}, {0.9, 0.0}, {0.9, 0.0}},  /* 1st col end */
      {{4.80,1.0}, {0.6,-2.0},{0.0,-0.1},{0.0,-0.1},{0.0,-0.1},
       {0.0,-0.1}, {0.0,0.0},{0.0, 0.0}, {0.0, 0.0}},  /* 2nd  col end */
      {{4.40,1.0}, {1.1,-1.9}, {0.6,-1.9},{-0.1,-0.1},{-0.1,-0.1},
       {0.0,0.0},{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},  /* 3rd  col end */
      {{4.00,1.0}, {1.7,-1.9}, {1.2,-1.8},{0.7,-1.9},{-0.1,0.0},
       {-0.1,0.0},{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},  /* 4th col end */
      {{3.60,1.0}, {2.3,-1.8}, {1.8,-1.7},{1.3,-1.7},{0.7,-1.9},
       {-0.1,0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},  /* 5th col end*/
      {{3.20,1.0}, {2.8,-1.8}, {2.5,-1.6},{2.0,-1.6},{1.5,-1.7},
       {0.8,-1.9},{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},  /* 6th  col end*/
      {{2.80,1.0}, {3.4,-1.7}, {3.1,-1.5},{2.7,-1.5},{2.2,-1.6},
       {1.6,-1.8},{0.8,-1.9}, {0.0, 0.0}, {0.0, 0.0}},  /* 7th  col end*/
      {{2.40,1.0}, {3.9,-1.7}, {3.7,-1.4},{3.4,-1.4},{2.9,-1.5},
       {2.4,-1.6},{1.7,-1.8}, {0.8,-2.0}, {0.0, 0.0}},  /* 8th  col end*/
      {{2.00,1.0}, {4.5,-1.6}, {4.3,-1.3},{4.0,-1.2},{3.7,-1.3},
       {3.2,-1.5},{2.5,-1.8}, {1.7,-1.9}, {0.8,-2.0}}}; /* 9th  col end*/
 
  cmplx cresb7[5][9] = {
      {{1.0, 1.0}, {2.0, 1.0},{3.0, 1.0},{4.0,1.0},{5.0,1.0},
       {6.0, 1.0},{7.0, 1.0}, {8.0, 1.0}, {9.0, 1.0}},  /* 2nd col end */
      {{1.0, 2.0},{2.0, 2.0}, {3.0, 2.0},{4.0,2.0},{5.0,2.0},
       {6.0,2.0},{7.0,2.0}, {8.0, 2.0}, {9.0, 2.0}},  /* 3rd   col end */
      {{1.0, 3.0}, {2.0, 3.0},{3.0, 3.0},{4.0,3.0},{5.0,3.0},
       {6.0, 3.0},{7.0, 3.0}, {8.0, 3.0}, {9.0, 3.0}},  /* 4th col end */
      {{1.0, 4.0}, {2.0, 4.0}, {3.0, 4.0},{4.0,4.0},{5.0,4.0},
       {6.0,4.0},{7.0, 4.0}, {8.0, 4.0}, {9.0, 4.0}},  /* 5th  col end */
      {{1.0, 5.0}, {2.0, 5.0},{3.0, 5.0},{4.0,5.0},{5.0,5.0},
       {6.0,5.0},{7.0, 5.0}, {8.0, 5.0}, {9.0, 5.0}}}; /* 6th  col end */
 
  dcmplx zresa7[9][9] = {
      {{5.20,1.0},{0.4, 0.1}, {0.5, 0.1},{0.6,0.1},{0.6,0.1},
       {0.7, 0.1},{0.8, 0.0}, {0.9, 0.0}, {0.9, 0.0}},  /* 1st col end */
      {{4.80,1.0}, {0.6,-2.0},{0.0,-0.1},{0.0,-0.1},{0.0,-0.1},
       {0.0,-0.1}, {0.0,0.0},{0.0, 0.0}, {0.0, 0.0}},  /* 2nd  col end */
      {{4.40,1.0}, {1.1,-1.9}, {0.6,-1.9},{-0.1,-0.1},{-0.1,-0.1},
       {0.0,0.0},{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},  /* 3rd  col end */
      {{4.00,1.0}, {1.7,-1.9}, {1.2,-1.8},{0.7,-1.9},{-0.1,0.0},
       {-0.1,0.0},{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},  /* 4th col end */
      {{3.60,1.0}, {2.3,-1.8}, {1.8,-1.7},{1.3,-1.7},{0.7,-1.9},
       {-0.1,0.0}, {0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},  /* 5th col end*/
      {{3.20,1.0}, {2.8,-1.8}, {2.5,-1.6},{2.0,-1.6},{1.5,-1.7},
       {0.8,-1.9},{0.0, 0.0}, {0.0, 0.0}, {0.0, 0.0}},  /* 6th  col end*/
      {{2.80,1.0}, {3.4,-1.7}, {3.1,-1.5},{2.7,-1.5},{2.2,-1.6},
       {1.6,-1.8},{0.8,-1.9}, {0.0, 0.0}, {0.0, 0.0}},  /* 7th  col end*/
      {{2.40,1.0}, {3.9,-1.7}, {3.7,-1.4},{3.4,-1.4},{2.9,-1.5},
       {2.4,-1.6},{1.7,-1.8}, {0.8,-2.0}, {0.0, 0.0}},  /* 8th  col end*/
      {{2.00,1.0}, {4.5,-1.6}, {4.3,-1.3},{4.0,-1.2},{3.7,-1.3},
       {3.2,-1.5},{2.5,-1.8}, {1.7,-1.9}, {0.8,-2.0}}}; /* 9th  col end*/
 
  dcmplx zresb7[5][9] = {
      {{1.0, 1.0}, {2.0, 1.0},{3.0, 1.0},{4.0,1.0},{5.0,1.0},
       {6.0, 1.0},{7.0, 1.0}, {8.0, 1.0}, {9.0, 1.0}},  /* 2nd col end */
      {{1.0, 2.0},{2.0, 2.0}, {3.0, 2.0},{4.0,2.0},{5.0,2.0},
       {6.0,2.0},{7.0,2.0}, {8.0, 2.0}, {9.0, 2.0}},  /* 3rd   col end */
      {{1.0, 3.0}, {2.0, 3.0},{3.0, 3.0},{4.0,3.0},{5.0,3.0},
       {6.0, 3.0},{7.0, 3.0}, {8.0, 3.0}, {9.0, 3.0}},  /* 4th col end */
      {{1.0, 4.0}, {2.0, 4.0}, {3.0, 4.0},{4.0,4.0},{5.0,4.0},
       {6.0,4.0},{7.0, 4.0}, {8.0, 4.0}, {9.0, 4.0}},  /* 5th  col end */
      {{1.0, 5.0}, {2.0, 5.0},{3.0, 5.0},{4.0,5.0},{5.0,5.0},
       {6.0,5.0},{7.0, 5.0}, {8.0, 5.0}, {9.0, 5.0}}}; /* 6th  col end */
 
  double dresa8[36]  = {3.0, 5.0, 1.0,-1.0, 1.0, 0.0, 1.0,-1.0,
                        3.0,-1.0, 1.0, 0.0,-1.0,-1.0, 0.0,
                        4.0,-1.0, 0.0, 1.0, 1.0, 1.0,
                        8.0,-1.0,-1.0,-1.0,-1.0,
                        1.0, 3.0,-1.0, 1.0,
                        1.0, 1.0, 0.0,
                        2.0, 1.0,
                       16.0};
 
  double dresb8[3][8] = {
   1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,
   1.0,  2.0,  3.0,  4.0,  5.0,  6.0,  7.0,  8.0,
   8.0,  7.0,  6.0,  5.0,  4.0,  3.0,  2.0,  1.0};
 
   _ESVINT ipvtr8[8] = {-2,-2,3,4,-6,-6,7,8};
 
  double dresa9[2][6] = {
  -4.0,  0.5,  0.5,       0.0,       0.5,       0.5,
   2.0,  2.5,  0.285714, -0.428571,  0.285714,  0.285714};
 
  float sresa9[2][6] = {
  -4.0,  0.5,  0.5,       0.0,       0.5,       0.5,
   2.0,  2.5,  0.285714, -0.428571,  0.285714,  0.285714};
 
  dcmplx zresa9[2][6] = {
 {{  3.660601,  0.000000}, {  0.255874, -0.225302}, {  0.187102,  0.024101},
  { -0.193177,  0.050152}, { -0.109713, -0.110108}, {  0.288000,  0.080724}},
 {{ -1.731956, -0.524504}, { -3.865905,  0.000000}, {  0.135165, -0.000348},
  {  0.057186, -0.451900}, { -0.065903, -0.220144}, {  0.110147, -0.200172}} };
 
  cmplx cresa9[2][6] = {
 {{  3.660601,  0.000000}, {  0.255874, -0.225302}, {  0.187102,  0.024101},
  { -0.193177,  0.050152}, { -0.109713, -0.110108}, {  0.288000,  0.080724}},
 {{ -1.731956, -0.524504}, { -3.865905,  0.000000}, {  0.135165, -0.000348},
  {  0.057186, -0.451900}, { -0.065903, -0.220144}, {  0.110147, -0.200172}} };
 
  double drest9[2] = {1.0, 1.4};
  float  srest9[2] = {1.0, 1.4};
  dcmplx zrest9[2] = { { 1.491723, 0.245861}, { 1.268358, 0.545419} };
  cmplx  crest9[2] = { { 1.491723, 0.245861}, { 1.268358, 0.545419} };
 
  double dresb10[3][6] = {
   0.78,  0.56,  0.042857,  0.185714, 0.042857, 0.042857,
   1.00,  2.00, -1.285714,  0.428571, 0.714285,-1.285714,
   1.025, 0.8,  -0.250000,  1.250000,-0.250000,-0.250000};
 
  float sresb10[3][6] = {
   0.78,  0.56,  0.042857,  0.185714, 0.042857, 0.042857,
   1.00,  2.00, -1.285714,  0.428571, 0.714285,-1.285714,
   1.025, 0.8,  -0.250000,  1.250000,-0.250000,-0.250000};
   dcmplx zresb10[4][3] = {
       {{-0.16, 0.15}, { 0.11, 0.02}, {-0.13,-0.32}},
       {{ 0.37,-0.05}, {-0.08, 0.18}, { 0.21,-0.50}},
       {{ 0.16, 0.12}, { 0.04, 0.06}, { 0.16,-0.31}},
       {{-0.38, 0.65}, {-0.27,-0.28}, {-0.19, 0.33}}
   };
   cmplx cresb10[4][3] = {
       {{-0.16, 0.15}, { 0.11, 0.02}, {-0.13,-0.32}},
       {{ 0.37,-0.05}, {-0.08, 0.18}, { 0.21,-0.50}},
       {{ 0.16, 0.12}, { 0.04, 0.06}, { 0.16,-0.31}},
       {{-0.38, 0.65}, {-0.27,-0.28}, {-0.19, 0.33}}
   };
 
/***********************************************
* ADDED FOR D/S/Z/CPBTRF/S, D/S/Z/CPBSV  10/12 *
***********************************************/
 
   double dares113[9][4] = { 1., 1., 1., 1., 1., 1., 1., 1.,
                             1., 1., 1., 1., 1., 1., 1., 1.,
                             1., 1., 1., 1., 1., 1., 1., 1.,
                             1., 1., 1., 99., 1., 1., 99., 99.,
                             1., 99., 99., 99.};
   double dbres113[3][9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                             1., -1., 1., -1., 1., -1., 1., -1., 1.,
                             1., 0., -1., 1., 0., -1., 1., 0., -1.};
   double dares114[9][4] = { 1., 1., 1., 1., 1., 1., 1., 1.,
                             1., 1., 1., 1., 1., 1., 1., 1.,
                             1., 1., 1., 1., 1., 1., 1., 1.,
                             1., 1., 1., 99., 1., 1., 99., 99.,
                             1., 99., 99., 99.};
   double dbres114[3][9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                             1., -1., 1., -1., 1., -1., 1., -1., 1.,
                             1., 0., -1., 1., 0., -1., 1., 0., -1.};
   float sares113[9][4] = { 1., 1., 1., 1., 1., 1., 1., 1.,
                            1., 1., 1., 1., 1., 1., 1., 1.,
                            1., 1., 1., 1., 1., 1., 1., 1.,
                            1., 1., 1., 99., 1., 1., 99., 99.,
                            1., 99., 99., 99.};
   float sbres113[3][9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                            1., -1., 1., -1., 1., -1., 1., -1., 1.,
                            1., 0., -1., 1., 0., -1., 1., 0., -1.};
   float sares114[9][4] = { 1., 1., 1., 1., 1., 1., 1., 1.,
                            1., 1., 1., 1., 1., 1., 1., 1.,
                            1., 1., 1., 1., 1., 1., 1., 1.,
                            1., 1., 1., 99., 1., 1., 99., 99.,
                            1., 99., 99., 99.};
   float sbres114[3][9] = { 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                            1., -1., 1., -1., 1., -1., 1., -1., 1.,
                            1., 0., -1., 1., 0., -1., 1., 0., -1.};
   dcmplx zares113[6][4] = {
                         { {1., 0.}, {1., 1.}, {1., 1.}, {1., 1.} },
                         { {1., 0.}, {1., 1.}, {1., 1.}, {1., 1.} },
                         { {1., 0.}, {1., 1.}, {1., 1.}, {1., 1.} },
                         { {1., 0.}, {1., 1.}, {1., 1.}, {99., 99.} },
                         { {1., 0.}, {1., 1.}, {99., 99.}, {99., 99.} },
                         { {1., 0.}, {99., 99.}, {99., 99.}, {99., 99.} } };
   dcmplx zbres113[3][6] = {
                         { {7., 33.}, {19., -1.}, {5., -13.},
                           {-11., -5.}, {-3., 9.}, {5., -1.} },
                         { {1., -1.}, {1., -1.}, {1., -1.},
                           {1., -1.}, {1., -1.}, {1., -1.} },
                         { {1., 1.}, {1., -1.}, {2., 1.},
                           {2., -1.}, {1., 2.}, {1., -2.} } };
   dcmplx zares114[6][4] = {
                         { {1., 0.}, {1., 1.}, {1., 1.}, {1., 1.} },
                         { {1., 0.}, {1., 1.}, {1., 1.}, {1., 1.} },
                         { {1., 0.}, {1., 1.}, {1., 1.}, {1., 1.} },
                         { {1., 0.}, {1., 1.}, {1., 1.}, {99., 99.} },
                         { {1., 0.}, {1., 1.}, {99., 99.}, {99., 99.} },
                         { {1., 0.}, {99., 99.}, {99., 99.}, {99., 99.} } };
   dcmplx zbres114[3][6] = {
                         { {7., 33.}, {19., -1.}, {5., -13.},
                           {-11., -5.}, {-3., 9.}, {5., -1.} },
                         { {1., -1.}, {1., -1.}, {1., -1.},
                           {1., -1.}, {1., -1.}, {1., -1.} },
                         { {1., 1.}, {1., -1.}, {2., 1.},
                           {2., -1.}, {1., 2.}, {1., -2.} } };
   cmplx cares113[6][4] = {
                        { {1., 0.}, {1., 1.}, {1., 1.}, {1., 1.} },
                        { {1., 0.}, {1., 1.}, {1., 1.}, {1., 1.} },
                        { {1., 0.}, {1., 1.}, {1., 1.}, {1., 1.} },
                        { {1., 0.}, {1., 1.}, {1., 1.}, {99., 99.} },
                        { {1., 0.}, {1., 1.}, {99., 99.}, {99., 99.} },
                        { {1., 0.}, {99., 99.}, {99., 99.}, {99., 99.} } };
   cmplx cbres113[3][6] = {
                        { {7., 33.}, {19., -1.}, {5., -13.},
                          {-11., -5.}, {-3., 9.}, {5., -1.} },
                        { {1., -1.}, {1., -1.}, {1., -1.},
                          {1., -1.}, {1., -1.}, {1., -1.} },
                        { {1., 1.}, {1., -1.}, {2., 1.},
                          {2., -1.}, {1., 2.}, {1., -2.} } };
   cmplx cares114[6][4] = {
                        { {1., 0.}, {1., 1.}, {1., 1.}, {1., 1.} },
                        { {1., 0.}, {1., 1.}, {1., 1.}, {1., 1.} },
                        { {1., 0.}, {1., 1.}, {1., 1.}, {1., 1.} },
                        { {1., 0.}, {1., 1.}, {1., 1.}, {99., 99.} },
                        { {1., 0.}, {1., 1.}, {99., 99.}, {99., 99.} },
                        { {1., 0.}, {99., 99.}, {99., 99.}, {99., 99.} } };
   cmplx cbres114[3][6] = {
                        { {7., 33.}, {19., -1.}, {5., -13.},
                          {-11., -5.}, {-3., 9.}, {5., -1.} },
                        { {1., -1.}, {1., -1.}, {1., -1.},
                          {1., -1.}, {1., -1.}, {1., -1.} },
                        { {1., 1.}, {1., -1.}, {2., 1.},
                          {2., -1.}, {1., 2.}, {1., -2.} } };
 
/***********************************************
* ADDED FOR D/S/Z/CPTTRF/S, D/S/Z/CPTSV  10/12 *
***********************************************/
 
   double ddres115[4] = { 1., 1., 2., 0.5 };
   double deres115[3] = { 1., 1., 0.5 };
   double dbres115[2][4] = { { 1., 1., 1., 1. },
                             { -1., -1., 0., 1. } };
   float sdres115[4] = { 1., 1., 2., 0.5 };
   float seres115[3] = { 1., 1., 0.5 };
   float sbres115[2][4] = { { 1., 1., 1., 1. },
                            { -1., -1., 0., 1. } };
   double zdres115[4] = { 1., 2., 3., 4. };
   dcmplx zeres115[3] = { { 1., 1. }, { 1., 1. }, { 1., 1. } };
   dcmplx zbres115[3][4] = { { { 1., 0. }, { 1., 0. }, { 1., 0. }, { 1., 0. } },
                             { { 1., 1. }, { 1., 1. }, { 1., 1. }, { 1., 1. } },
                             { { 1., -1. }, { 1., -1. }, { 1., -1. }, { 1., -1. } } };
   float cdres115[4] = { 1., 2., 3., 4. };
   cmplx ceres115[3] = { { 1., 1. }, { 1., 1. }, { 1., 1. } };
   cmplx cbres115[3][4] = { { { 1., 0. }, { 1., 0. }, { 1., 0. }, { 1., 0. } },
                            { { 1., 1. }, { 1., 1. }, { 1., 1. }, { 1., 1. } },
                            { { 1., -1. }, { 1., -1. }, { 1., -1. }, { 1., -1. } } };
 
   double ddres116[4] = { 1., 1., 2., 0.5 };
   double deres116[3] = { 1., 1., 0.5 };
   double dbres116[2][4] = { { 1., 1., 1., 1. },
                             { -1., -1., 0., 1. } };
   float sdres116[4] = { 1., 1., 2., 0.5 };
   float seres116[3] = { 1., 1., 0.5 };
   float sbres116[2][4] = { { 1., 1., 1., 1. },
                             { -1., -1., 0., 1. } };
   double zdres116[4] = { 1., 2., 3., 4. };
   dcmplx zeres116[3] = { { 1., 1. }, { 1., 1. }, { 1., 1. } };
   dcmplx zbres116[3][4] = { { { 1., 0. }, { 1., 0. }, { 1., 0. }, { 1., 0. } },
                             { { 1., 1. }, { 1., 1. }, { 1., 1. }, { 1., 1. } },
                             { { 1., -1. }, { 1., -1. }, { 1., -1. }, { 1., -1. } } };
   float cdres116[4] = { 1., 2., 3., 4. };
   cmplx ceres116[3] = { { 1., 1. }, { 1., 1. }, { 1., 1. } };
   cmplx cbres116[3][4] = { { { 1., 0. }, { 1., 0. }, { 1., 0. }, { 1., 0. } },
                            { { 1., 1. }, { 1., 1. }, { 1., 1. }, { 1., 1. } },
                            { { 1., -1. }, { 1., -1. }, { 1., -1. }, { 1., -1. } } };
 
/***********************************************************
*  INPUT DATA FOR LINEAR ALGEBRAIC EQUATIONS SUBROUTINES : *
***********************************************************/
 
    float  sa37[9][9] = { 1., 1., 4., 0., 0., 0., 0., 0., 0.,
                          1., 1., 1., 5., 0., 0., 0., 0., 0.,
                          1., 1., 1., 1., 6., 0., 0., 0., 0.,
                          1., 1., 1., 1., 1., 7., 0., 0., 0.,
                          0., 1., 1., 1., 1., 1., 8., 0., 0.,
                          0., 0., 1., 1., 1., 1., 1., 9., 0.,
                          0., 0., 0., 1., 1., 1., 1., 1., 10.,
                          0., 0., 0., 0., 1., 1., 1., 1., 11.,
                          0., 0., 0., 0., 0., 1., 1., 1., 12. };
    double da37[9][9] = { 1., 1., 4., 0., 0., 0., 0., 0., 0.,
                          1., 1., 1., 5., 0., 0., 0., 0., 0.,
                          1., 1., 1., 1., 6., 0., 0., 0., 0.,
                          1., 1., 1., 1., 1., 7., 0., 0., 0.,
                          0., 1., 1., 1., 1., 1., 8., 0., 0.,
                          0., 0., 1., 1., 1., 1., 1., 9., 0.,
                          0., 0., 0., 1., 1., 1., 1., 1., 10.,
                          0., 0., 0., 0., 1., 1., 1., 1., 11.,
                          0., 0., 0., 0., 0., 1., 1., 1., 12. };
    cmplx  ca37[4][4] = {
        { { 1., 2. }, { 2., 0. }, { 2., 1. }, { 8., 5. } },
        { { 1., 7. }, { 1., 3. }, { 5., 0. }, { 1., 9. } },
        { { 2., 4. }, { 4., 4. }, { 3., 6. }, { 6., 6. } },
        { { 3., 1. }, { 2., 3. }, { 0., 0. }, { 8., 1. } } };
    dcmplx za37[4][4] = {
        { { 1., 2. }, { 2., 0. }, { 2., 1. }, { 8., 5. } },
        { { 1., 7. }, { 1., 3. }, { 5., 0. }, { 1., 9. } },
        { { 2., 4. }, { 4., 4. }, { 3., 6. }, { 6., 6. } },
        { { 3., 1. }, { 2., 3. }, { 0., 0. }, { 8., 1. } } };
    cmplx  cb37[4] = { { -10., 85. },{ -6., 61. },{ 10., 38. },{ 58., 168. } };
    dcmplx zb37[4] = { { -10., 85. },{ -6., 61. },{ 10., 38. },{ 58., 168. } };
 
    cmplx ca41[9][9] = {
           {{2.0, 1.0},{2.4, 1.0},{2.8, 1.0},{3.2, 1.0},{3.6, 1.0},
           {4.0, 1.0},{4.4, 1.0},{4.8, 1.0},{5.2, 1.0}},
           {{2.4,-1.0},{2.0, 1.0},{2.4, 1.0},{2.8, 1.0},{3.2, 1.0},
           {3.6, 1.0},{4.0, 1.0},{4.4, 1.0},{4.8, 1.0}},
           {{2.8,-1.0},{2.4,-1.0},{2.0, 1.0},{2.4, 1.0},{2.8, 1.0},
           {3.2, 1.0},{3.6, 1.0},{4.0, 1.0},{4.4, 1.0}},
           {{3.2,-1.0},{2.8,-1.0},{2.4,-1.0},{2.0, 1.0},{2.4, 1.0},
           {2.8, 1.0},{3.2, 1.0},{3.6, 1.0},{4.0, 1.0}},
           {{3.6,-1.0},{3.2,-1.0},{2.8,-1.0},{2.4,-1.0},{2.0, 1.0},
           {2.4, 1.0},{2.8, 1.0},{3.2, 1.0},{3.6, 1.0}},
           {{4.0,-1.0},{3.6,-1.0},{3.2,-1.0},{2.8,-1.0},{2.4,-1.0},
           {2.0, 1.0},{2.4, 1.0},{2.8, 1.0},{3.2, 1.0}},
           {{4.4,-1.0},{4.0,-1.0},{3.6,-1.0},{3.2,-1.0},{2.8,-1.0},
           {2.4,-1.0},{2.0, 1.0},{2.4, 1.0},{2.8, 1.0}},
           {{4.8,-1.0},{4.4,-1.0},{4.0,-1.0},{3.6,-1.0},{3.2,-1.0},
           {2.8,-1.0},{2.4,-1.0},{2.0, 1.0},{2.4, 1.0}},
           {{5.2,-1.0},{4.8,-1.0},{4.4,-1.0},{4.0,-1.0},{3.6,-1.0},
           {3.2,-1.0},{2.8,-1.0},{2.4,-1.0},{2.0, 1.0}} };
    cmplx cres41[9][9] = {
          {{-.2, -.4},{ .2,  .4},{ .0,  .0},{ .0,  .0},{ .0,  .0},
          { .0,  .0},{ .0,  .0},{ .0,  .0},{ .0,  .0}},
          {{-.1,  .1},{ .0, -.6},{ .2,  .4},{ .0,  .0},{ .0,  .0},
          { .0,  .0},{ .0,  .0},{ .0,  .0},{ .0,  .0}},
          {{-.1,  .1},{-.1,  .0},{ .0, -.6},{ .2,  .4},{ .0,  .0},
          { .0,  .0},{ .0,  .0},{ .0,  .0},{ .0,  .0}},
          {{ .0,  .1},{-.1,  .0},{-.1,  .0},{ .0, -.6},{ .2,  .4},
          { .0,  .0},{ .0,  .0},{ .0,  .0},{ .0,  .0}},
          {{ .1,  .1},{-.1,  .0},{-.1,  .0},{-.1,  .0},{ .0, -.6},
          { .2,  .4},{ .0,  .0},{ .0,  .0},{ .0,  .0}},
          {{ .1,  .1},{ .0,  .0},{-.1,  .0},{-.1,  .0},{-.1,  .0},
          { .0, -.6},{ .2,  .4},{ .0,  .0},{ .0,  .0}},
          {{ .1,  .0},{ .0,  .0},{ .0,  .0},{-.1,  .0},{-.1,  .0},
          {-.1,  .0},{ .0, -.6},{ .2,  .4},{ .0,  .0}},
          {{ .1,  .0},{ .0,  .0},{ .0,  .0},{ .0,  .0},{-.1,  .0},
          {-.1,  .0},{-.1,  .0},{ .0, -.6},{ .2,  .4}},
          {{ .0, -.3},{ .1,  .0},{ .1,  .0},{ .1,  .1},{ .1,  .1},
          { .0,  .1},{-.1,  .1},{-.1,  .1},{-.2, -.4}} };
 
    dcmplx za41[9][9] = {
           {{2.0, 1.0},{2.4, 1.0},{2.8, 1.0},{3.2, 1.0},{3.6, 1.0},
           {4.0, 1.0},{4.4, 1.0},{4.8, 1.0},{5.2, 1.0}},
           {{2.4,-1.0},{2.0, 1.0},{2.4, 1.0},{2.8, 1.0},{3.2, 1.0},
           {3.6, 1.0},{4.0, 1.0},{4.4, 1.0},{4.8, 1.0}},
           {{2.8,-1.0},{2.4,-1.0},{2.0, 1.0},{2.4, 1.0},{2.8, 1.0},
           {3.2, 1.0},{3.6, 1.0},{4.0, 1.0},{4.4, 1.0}},
           {{3.2,-1.0},{2.8,-1.0},{2.4,-1.0},{2.0, 1.0},{2.4, 1.0},
           {2.8, 1.0},{3.2, 1.0},{3.6, 1.0},{4.0, 1.0}},
           {{3.6,-1.0},{3.2,-1.0},{2.8,-1.0},{2.4,-1.0},{2.0, 1.0},
           {2.4, 1.0},{2.8, 1.0},{3.2, 1.0},{3.6, 1.0}},
           {{4.0,-1.0},{3.6,-1.0},{3.2,-1.0},{2.8,-1.0},{2.4,-1.0},
           {2.0, 1.0},{2.4, 1.0},{2.8, 1.0},{3.2, 1.0}},
           {{4.4,-1.0},{4.0,-1.0},{3.6,-1.0},{3.2,-1.0},{2.8,-1.0},
           {2.4,-1.0},{2.0, 1.0},{2.4, 1.0},{2.8, 1.0}},
           {{4.8,-1.0},{4.4,-1.0},{4.0,-1.0},{3.6,-1.0},{3.2,-1.0},
           {2.8,-1.0},{2.4,-1.0},{2.0, 1.0},{2.4, 1.0}},
           {{5.2,-1.0},{4.8,-1.0},{4.4,-1.0},{4.0,-1.0},{3.6,-1.0},
           {3.2,-1.0},{2.8,-1.0},{2.4,-1.0},{2.0, 1.0}} };
    dcmplx zres41[9][9] = {
          {{-.2, -.4},{ .2,  .4},{ .0,  .0},{ .0,  .0},{ .0,  .0},
          { .0,  .0},{ .0,  .0},{ .0,  .0},{ .0,  .0}},
          {{-.1,  .1},{ .0, -.6},{ .2,  .4},{ .0,  .0},{ .0,  .0},
          { .0,  .0},{ .0,  .0},{ .0,  .0},{ .0,  .0}},
          {{-.1,  .1},{-.1,  .0},{ .0, -.6},{ .2,  .4},{ .0,  .0},
          { .0,  .0},{ .0,  .0},{ .0,  .0},{ .0,  .0}},
          {{ .0,  .1},{-.1,  .0},{-.1,  .0},{ .0, -.6},{ .2,  .4},
          { .0,  .0},{ .0,  .0},{ .0,  .0},{ .0,  .0}},
          {{ .1,  .1},{-.1,  .0},{-.1,  .0},{-.1,  .0},{ .0, -.6},
          { .2,  .4},{ .0,  .0},{ .0,  .0},{ .0,  .0}},
          {{ .1,  .1},{ .0,  .0},{-.1,  .0},{-.1,  .0},{-.1,  .0},
          { .0, -.6},{ .2,  .4},{ .0,  .0},{ .0,  .0}},
          {{ .1,  .0},{ .0,  .0},{ .0,  .0},{-.1,  .0},{-.1,  .0},
          {-.1,  .0},{ .0, -.6},{ .2,  .4},{ .0,  .0}},
          {{ .1,  .0},{ .0,  .0},{ .0,  .0},{ .0,  .0},{-.1,  .0},
          {-.1,  .0},{-.1,  .0},{ .0, -.6},{ .2,  .4}},
          {{ .0, -.3},{ .1,  .0},{ .1,  .0},{ .1,  .1},{ .1,  .1},
          { .0,  .1},{-.1,  .1},{-.1,  .1},{-.2, -.4}} };
    float  sbx38[9] = { 4., 5., 9., 10., 11., 12., 12., 12., 33. };
    double dbx38[9] = { 4., 5., 9., 10., 11., 12., 12., 12., 33. };
    float  sap39[54] = { 1., 1., 1., 1., 1., 1., 1., 1., 1., 2., 2.,
                         2., 2., 2., 2., 2., 2., 3., 3., 3., 3., 3.,
                         3., 3., 4., 4., 4., 4., 4., 4., 5., 5., 5.,
                         5., 5., 6., 6., 6., 6., 7., 7., 7., 8., 8.,
                         9., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    double dap39[54] = { 1., 1., 1., 1., 1., 1., 1., 1., 1., 2., 2.,
                         2., 2., 2., 2., 2., 2., 3., 3., 3., 3., 3.,
                         3., 3., 4., 4., 4., 4., 4., 4., 5., 5., 5.,
                         5., 5., 6., 6., 6., 6., 7., 7., 7., 8., 8.,
                         9., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    float  sbx40[9] = { 9., 17., 24., 30., 35., 39., 42., 44., 45. };
    double dbx40[9] = { 9., 17., 24., 30., 35., 39., 42., 44., 45. };
    float sap93[45] = { 1., 1., 2., 1., 2., 3., 1., 2., 3., 4., 1.,
                         2., 3., 4., 5., 1., 2., 3., 4., 5., 6., 1.,
                         2., 3., 4., 5., 6., 7., 1., 2., 3., 4., 5.,
                         6., 7.,  8., 1., 2., 3., 4., 5., 6., 7., 8., 9. };
    float snrm93, swrk93[27], srcnd93;
    float snrm93res  = 45.0;
    float srcnd93res =  0.0055555;
    _ESVINT   siwrk93[9];
 
    float sbx93[9][2] = { 9., 17., 24., 30., 35., 39., 42., 44., 45.,
                          45., 89., 131., 170., 205., 235., 259., 276., 285.};
    float sap932[45] = { 1., 1., 2., 1., 2., 3., 1., 2., 3., 4., 1.,
                         2., 3., 4., 5., 1., 2., 3., 4., 5., 6., 1.,
                         2., 3., 4., 5., 6., 7., 1., 2., 3., 4., 5.,
                         6., 7.,  8., 1., 2., 3., 4., 5., 6., 7., 8., 9. };
    float sbx932[29][2] = { 9., 17., 24., 30., 35., 39., 42., 44., 45.,
                          45., 89., 131., 170., 205., 235., 259., 276., 285.};
 
    float sapres93[45]= { 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                           1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                           1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                           1., 1., 1., 1., 1., 1., 1., 1., 1.};
    float sapres93inv[45]= {2.,-1.,2.,0.,-1.,2.,0.,0.,-1.,2.,0.,0.,0.,-1.,
                            2.,0.,0.,0.,0.,-1.,2.,0.,0.,0.,0.,0.,-1.,2.,0.,
                            0.,0.,0.,0.,0.,-1.,2.,0.,0.,0.,0.,0.,0.,0.,-1.,1.};
    float sbxres93[9][2] = { 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                              1., 2., 3., 4., 5., 6., 7., 8., 9. };
    double dap93[45] = { 1., 1., 2., 1., 2., 3., 1., 2., 3., 4., 1.,
                         2., 3., 4., 5., 1., 2., 3., 4., 5., 6., 1.,
                         2., 3., 4., 5., 6., 7., 1., 2., 3., 4., 5.,
                         6., 7.,  8., 1., 2., 3., 4., 5., 6., 7., 8., 9. };
    double dnrm93, dwrk93[27], drcnd93;
    double dnrm93res  = 45.0;
    double drcnd93res =  0.0055555;
    _ESVINT    diwrk93[9];
 
    double dbx93[9][2] = { 9., 17., 24., 30., 35., 39., 42., 44., 45.,
                          45., 89., 131., 170., 205., 235., 259., 276., 285.};
    double dap932[45] = { 1., 1., 2., 1., 2., 3., 1., 2., 3., 4., 1.,
                         2., 3., 4., 5., 1., 2., 3., 4., 5., 6., 1.,
                         2., 3., 4., 5., 6., 7., 1., 2., 3., 4., 5.,
                         6., 7.,  8., 1., 2., 3., 4., 5., 6., 7., 8., 9. };
    double dbx932[9][2] = { 9., 17., 24., 30., 35., 39., 42., 44., 45.,
                          45., 89., 131., 170., 205., 235., 259., 276., 285.};
 
    double dapres93[45]= { 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                           1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                           1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                           1., 1., 1., 1., 1., 1., 1., 1., 1.};
    double dapres93inv[45]= {2.,-1.,2.,0.,-1.,2.,0.,0.,-1.,2.,0.,0.,0.,-1.,
                            2.,0.,0.,0.,0.,-1.,2.,0.,0.,0.,0.,0.,-1.,2.,0.,
                            0.,0.,0.,0.,0.,-1.,2.,0.,0.,0.,0.,0.,0.,0.,-1.,1.};
    double dbxres93[9][2] = { 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                              1., 2., 3., 4., 5., 6., 7., 8., 9. };
    dcmplx zap93[6] = {{25.0, 0.0}, {-5.0, 5.0}, {10.0, -5.0},
                       {51.0, 0.0}, { 4.0, 6.0}, {71.0,  0.0} };
    dcmplx zap93i[6];
    dcmplx zapres93i[6] = {{0.05, 0.00}, {0.00, -0.01}, {-0.01, 0.00},
                           {0.02, 0.00}, {0.00,  0.00}, { 0.02, 0.00}};
 
    dcmplx zwrk93[12];
    double rzwrk93[3];
    double znrm93, zrcnd93;
    double znrm93res  = 89.3914424384269353;
    double zrcnd93res =  0.184714364056937669;
    double zrwrk93[3];
 
 
    dcmplx zap932[6] = { {25.0, 0.0}, {-5.0, 5.0}, {10.0, -5.0},
                           {51.0, 0.0}, { 4.0, 6.0}, {71.0,  0.0} };
 
    dcmplx zapres93[6] = { {5.0, 0.0},{-1.0, 1.0},{2.0, -1.0},
                             {7.0, 0.0},{ 1.0, 1.0},{8.0,  0.0} };
 
    dcmplx zbx93[3][2] = { { {60.0,  -55.0}, { 34.0,  58.0} },
                           { {13.0, -152.0}, {70.0, 10.0} },
                           { {-51.0, 110.0}, {75.0, 63.0} } };
 
    dcmplx zbx932[3][2] = {{ {60.0,  -55.0}, { 34.0,  58.0} },
                           { {13.0, -152.0}, {70.0, 10.0} },
                           { {-51.0, 110.0}, {75.0, 63.0} } };
 
    dcmplx zbxres93[3][2] = {{ { 2.0, -1.0}, {1.0, 1.0} },
                             { {0.0, -2.0}, {2.0,  0.0} },
                             { {-1.0,  2.0}, {1.0, 1.0} } };
    cmplx cap93[6] = {{25.0, 0.0}, {-5.0, 5.0}, {10.0, -5.0},
                       {51.0, 0.0}, { 4.0, 6.0}, {71.0,  0.0} };
    cmplx cap93i[6];
    cmplx capres93i[6] = { {0.05, 0.00},{0.00,-0.01},{0.02, 0.00},
                           {0.02, 0.00},{0.00, 0.00},{0.02, 0.00} };
 
    cmplx cwrk93[12];
    float rwrk93[3];
    float cnrm93, crcnd93;
    float cnrm93res  = 89.3914424384269353;
    float crcnd93res =  0.184714364056937669;
    float crwrk93[3];
 
 
    cmplx cap932[6] = { {25.0, 0.0}, {-5.0, 5.0}, {10.0, -5.0},
                           {51.0, 0.0}, { 4.0, 6.0}, {71.0,  0.0} };
 
    cmplx capres93[6] = { {5.0, 0.0},{-1.0, 1.0},{2.0, -1.0},
                             {7.0, 0.0},{ 1.0, 1.0},{8.0,  0.0} };
 
    cmplx cbx93[3][2] = { { {60.0,  -55.0}, { 34.0,  58.0} },
                           { {13.0, -152.0}, {70.0, 10.0} },
                           { {-51.0, 110.0}, {75.0, 63.0} } };
 
    cmplx cbx932[3][2] = {{ {60.0,  -55.0}, { 34.0,  58.0} },
                           { {13.0, -152.0}, {70.0, 10.0} },
                           { {-51.0, 110.0}, {75.0, 63.0} } };
 
    cmplx cbxres93[3][2] = {{ { 2.0, -1.0}, {1.0, 1.0} },
                             { {0.0, -2.0}, {2.0,  0.0} },
                             { {-1.0,  2.0}, {1.0, 1.0} } };
 
    float  sa41[9][9] = { 1., 1., 4., 0., 0., 0., 0., 0., 0.,
                          1., 1., 1., 5., 0., 0., 0., 0., 0.,
                          1., 1., 1., 1., 6., 0., 0., 0., 0.,
                          1., 1., 1., 1., 1., 7., 0., 0., 0.,
                          0., 1., 1., 1., 1., 1., 8., 0., 0.,
                          0., 0., 1., 1., 1., 1., 1., 9., 0.,
                          0., 0., 0., 1., 1., 1., 1., 1., 10.,
                          0., 0., 0., 0., 1., 1., 1., 1., 11.,
                          0., 0., 0., 0., 0., 1., 1., 1., 12. };
    double da41[9][9] = { 1., 1., 4., 0., 0., 0., 0., 0., 0.,
                          1., 1., 1., 5., 0., 0., 0., 0., 0.,
                          1., 1., 1., 1., 6., 0., 0., 0., 0.,
                          1., 1., 1., 1., 1., 7., 0., 0., 0.,
                          0., 1., 1., 1., 1., 1., 8., 0., 0.,
                          0., 0., 1., 1., 1., 1., 1., 9., 0.,
                          0., 0., 0., 1., 1., 1., 1., 1., 10.,
                          0., 0., 0., 0., 1., 1., 1., 1., 11.,
                          0., 0., 0., 0., 0., 1., 1., 1., 12. };
    float  sa43[9][9] = { 1., 1., 4., 0., 0., 0., 0., 0., 0.,
                          1., 1., 1., 5., 0., 0., 0., 0., 0.,
                          1., 1., 1., 1., 6., 0., 0., 0., 0.,
                          1., 1., 1., 1., 1., 7., 0., 0., 0.,
                          0., 1., 1., 1., 1., 1., 8., 0., 0.,
                          0., 0., 1., 1., 1., 1., 1., 9., 0.,
                          0., 0., 0., 1., 1., 1., 1., 1., 10.,
                          0., 0., 0., 0., 1., 1., 1., 1., 11.,
                          0., 0., 0., 0., 0., 1., 1., 1., 12. };
    double da43[9][9] = { 1., 1., 4., 0., 0., 0., 0., 0., 0.,
                          1., 1., 1., 5., 0., 0., 0., 0., 0.,
                          1., 1., 1., 1., 6., 0., 0., 0., 0.,
                          1., 1., 1., 1., 1., 7., 0., 0., 0.,
                          0., 1., 1., 1., 1., 1., 8., 0., 0.,
                          0., 0., 1., 1., 1., 1., 1., 9., 0.,
                          0., 0., 0., 1., 1., 1., 1., 1., 10.,
                          0., 0., 0., 0., 1., 1., 1., 1., 11.,
                          0., 0., 0., 0., 0., 1., 1., 1., 12. };
    float  sap44[54] = { 1., 1., 1., 1., 1., 1., 1., 1., 1., 2., 2.,
                         2., 2., 2., 2., 2., 2., 3., 3., 3., 3., 3.,
                         3., 3., 4., 4., 4., 4., 4., 4., 5., 5., 5.,
                         5., 5., 6., 6., 6., 6., 7., 7., 7., 8., 8.,
                         9., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    double dap44[54] = { 1., 1., 1., 1., 1., 1., 1., 1., 1., 2., 2.,
                         2., 2., 2., 2., 2., 2., 3., 3., 3., 3., 3.,
                         3., 3., 4., 4., 4., 4., 4., 4., 5., 5., 5.,
                         5., 5., 6., 6., 6., 6., 7., 7., 7., 8., 8.,
                         9., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    float  sagb45[9][23] = {
        0., 0., 0., 0., 0., 1., 1., 4., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0.,
        0., 0., 0., 0., 1., 1., 1., 5., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0.,
        0., 0., 0., 1., 1., 1., 1., 6., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1., 1., 1., 7., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1., 1., 1., 8., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1., 1., 1., 9., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1., 1., 1., 10., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1., 1., 11., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1., 12., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0. };
    double dagb45[9][23] = {
        0., 0., 0., 0., 0., 1., 1., 4., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 0., 0., 1., 1., 1., 5., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 0., 1., 1., 1., 1., 6., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1., 1., 1., 7., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1., 1., 1., 8., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1., 1., 1., 9., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1., 1., 1., 10., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1., 1., 11., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0., 0.,
        0., 0., 1., 1., 1., 12., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
 0., 0., 0., 0., 0., 0., 0. };
    float  sbx46[9] = { 4.0, 5.0, 9.0, 1.0e1, 1.1e1, 1.2e1, 1.2e1,
 1.2e1, 3.3e1 };
    double dbx46[9] = { 4.0, 5.0, 9.0, 1.0e1, 1.1e1, 1.2e1, 1.2e1,
 1.2e1, 3.3e1 };
    float  sap47[12][3] = { 1., 1., 1., 2., 2., 1., 3., 2., 1., 3., 2., 1.,
                            3., 2., 1., 3., 2., 1., 3., 2., 1., 3., 2., 0.,
                            3., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    double dap47[12][3] = { 1., 1., 1., 2., 2., 1., 3., 2., 1., 3., 2., 1.,
                            3., 2., 1., 3., 2., 1., 3., 2., 1., 3., 2., 0.,
                            3., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    float  sap47n[12][3] = { 1., 1., 1., 2., 2., 1., 3., 2., 1., 3., 2.,1.,
                            3., 2., 1., 3., 2., 1., 3., 2., 1., 3., 2., 0.,
                            3., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    double dap47n[12][3] = { 1., 1., 1., 2., 2., 1., 3., 2., 1., 3., 2.,1.,
                            3., 2., 1., 3., 2., 1., 3., 2., 1., 3., 2., 0.,
                            3., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    float  sbx48[9] = { 3., 6., 9., 9., 9., 9., 9., 8., 6. };
    double dbx48[9] = { 3., 6., 9., 9., 9., 9., 9., 8., 6. };
    float  sbx48n[9] = { 3., 6., 9., 9., 9., 9., 9., 8., 6. };
    double dbx48n[9] = { 3., 6., 9., 9., 9., 9., 9., 8., 6. };
    float  sa49[4] = { 0.0, 1.0, 1.0, 1.0 };
    float  sb49[4] = { 2.0, 3.0, 3.0, 3.0 };
    float  sc49[4] = { 2.0, 2.0, 2.0, 0.0 };
    float  sx50[4] = { 4.0, 6.0, 6.0, 4.0 };
    double da49[4] = { 0.0, 1.0, 1.0, 1.0 };
    double db49[4] = { 2.0, 3.0, 3.0, 3.0 };
    double dc49[4] = { 2.0, 2.0, 2.0, 0.0 };
    double dx50[4] = { 4.0, 6.0, 6.0, 4.0 };
    float  sa51[4] = { 1.0, 2.0, 3.0, 1.0 };
    float  sb51[4] = { 0.0, 1.0, 1.0, 1.0 };
    float  sc51[4] = { 1.0, 1.0, 1.0, 0.0 };
    float  sx52[4] = { 2.0, 4.0, 5.0, 2.0 };
    float  sctnp[4] = { 0., 1., 2., 3. };
    float  satnp[4] = { 7., 8., 9., 10. };
    float  sbtnp[4] = { 4., 5., 6., 0. };
    float  sxtnp[4] = { 11., 14., 17., 13. };
    double da51[4] = { 1.0, 2.0, 3.0, 1.0 };
    double db51[4] = { 0.0, 1.0, 1.0, 1.0 };
    double dc51[4] = { 1.0, 1.0, 1.0, 0.0 };
    double dx52[4] = { 2.0, 4.0, 5.0, 2.0 };
    double dctnp[4] = { 0., 1., 2., 3. };
    double datnp[4] = { 7., 8., 9., 10. };
    double dbtnp[4] = { 4., 5., 6., 0. };
    double dxtnp[4] = { 11., 14., 17., 13. };
    float  sa53[4] = { 1.0, 2.0, 3.0, 1.0 };
    float  sb53[4] = { 0.0, 1.0, 1.0, 1.0 };
    float  sx54[4] = { 2.0, 4.0, 5.0, 2.0 };
    double da53[4] = { 1.0, 2.0, 3.0, 1.0 };
    double db53[4] = { 0.0, 1.0, 1.0, 1.0 };
    double dx54[4] = { 2.0, 4.0, 5.0, 2.0 };
    double dac55[3][9] = { 2., 2., -1., -1., -1., -1., -1., -1., -1.,
                          -1., -1., 2., 2., 2., 2., 2., 2., 2.,
                           0., 0., 0., -1., -1., -1., -1., -1., 0. };
    double drpa55[3] = { 1.e-7, 0., 0. };
    double db55[9] = { 1., 1., 1., 0., 0., 0., 0., 0., 1. };
    double du55[9] = { 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    _ESVINT ipar55[4] = { 20, 0, 0, 0 };
    _ESVINT ika55[3][9] = { 1, 2, 2, 1, 4, 5, 6, 7, 8,
                        4, 3, 3, 4, 5, 6, 7, 8, 9,
                        1, 1, 1, 5, 6, 7, 8, 9, 1 };
    double dad55a[3][9] = { 2., 2., 2., 2., 2., 2., 2., 2., 2.,
                            0., 0., -1., -1., -1., -1., -1., -1., -1.,
                           -1., -1., -1., -1., -1., -1., -1., 0., 0. };
    double db55a[9] = { 1., 1., 0., 0., 0., 0., 0., 1., 1. };
    double du55a[9] = { 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    double dpr55a[3] = { 1.e-7, 0., 0. };
    _ESVINT ipa55a[4] = { 20, 0, 0, 0 };
    _ESVINT kd55a[3] = { 0, -2, 2 };
    float  sa56[3][4] = { 1., 4., 7., 1.e1, 2., 5., 8., 11., 3., 6., 9., 12. };
    double da56[3][4] = { 1., 4., 7., 1.e1, 2., 5., 8., 11., 3., 6., 9., 12. };
    float  sv57[3][3] = { -0.304, -0.608, -0.733, -0.894, 0.447, 0., 0.328,
 0.656, -0.680 };
    float  sub57[1][3] = { -4.061, 0., -0.714 };
    float  ss57[3] = { 7.342, 0., 0.305 };
    double dv57[3][3] = { -0.304, -0.608, -0.733, -0.894, 0.447, 0., 0.328,
 0.656, -0.680 };
    double dub57[1][3] = { -4.061, 0., -0.714 };
    double ds57[3] = { 7.342, 0., 0.305 };
    float  sa58[3][2] = { 1., 2., 2., 4., 2., 5. };
    float  sb58[1][2] = { 1., 4. };
    double da58[3][2] = { 1., 2., 2., 4., 2., 5. };
    double db58[1][2] = { 1., 4. };
    _ESVINT di103[4] = { 20, 0, 0, 0 };
    double dr103[3] = { 1.e-7, 0., 0. };
    double da103[3][9] = { 2., 2., 1., 1., 1., 1., 1., 1., 1.,
                           0., -1., 2., 2., 2., 2., 2., 2., 2.,
                           0., 0., 0., -1., -1., -1., -1., -1., 0. };
    _ESVINT dk103[3][9] = { 1, 2, 2, 1, 4, 5, 6, 7, 8, 1, 3, 3, 4, 5, 6, 7, 8, 9,
                        1, 1, 1, 5, 6, 7, 8, 9, 1 };
    double db103[9] = { 2., 1., 3., 2., 2., 2., 2., 2., 3. };
    double du103[9] = { 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    _ESVINT di104[4] = { 20, 0, 0, 0 };
    double dr104[3] = { 1.e-7, 0., 0. };
    double da104[3][9] = { 2., 2., 2., 2., 2., 2., 2., 2., 2.,
                          -1., -1., -1., -1., -1., -1., -1., 0., 0.,
                           0., 0., 0., 0., 1., 1., 1., 1., 1. };
    _ESVINT dl104[3] = { 0, 2, -4 };
    double db104[9] = { 1., 1., 1., 1., 2., 2., 2., 3., 3. };
    double du104[9] = { 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    double da105[27] = { 2., 1., 1., 3., 4., 1., 5., 2., 2.,
                         1., 1., 4., 3., 0., 0., 0., 0., 0.,
                         0., 0., 0., 0., 0., 0., 0., 0., 0. };
    _ESVINT di105[27] = { 1, 2, 2, 3, 3, 4, 4, 4, 4, 5, 5, 1, 2, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0, 0, 0 };
    _ESVINT dj105[27] = { 1, 1, 2, 3, 4, 4, 5, 1, 2, 3, 4, 3, 5, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0, 0, 0, 0, 0 };
    _ESVINT dip105[5] = { 1, 3, 1, 1, 0 };
    double drp105[5] = { 1.e-12, .1, 0., 0., 0. };
    double db106[5] = { 1., 1., 1., 1., 1. };
    double da99[45] = { 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                        2., 2., 2., 2., 2., 2., 2., 2.,
                        3., 3., 3., 3., 3., 3., 3.,
                        4., 4., 4., 4., 4., 4.,
                        5., 5., 5., 5., 5.,
                        6., 6., 6., 6.,
                        7., 7., 7.,
                        8., 8.,
                        9. };
    float  sa99[45] = { 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                        2., 2., 2., 2., 2., 2., 2., 2.,
                        3., 3., 3., 3., 3., 3., 3.,
                        4., 4., 4., 4., 4., 4.,
                        5., 5., 5., 5., 5.,
                        6., 6., 6., 6.,
                        7., 7., 7.,
                        8., 8.,
                        9. };
    float  sa102[5][7] = {  3.,  0.,  0.,  0.,  0.,  0.,  0.,
                           -1., -2.,  0.,  0.,  0.,  0.,  0.,
                            2.,  4., -3.,  0.,  0.,  0.,  0.,
                            2.,	-1.,  0.,  4.,  0.,  0.,  0.,
                            1.,	 3.,  2., -2.,  1.,  0.,  0. };
    float  sb102[3][6] = { 6., -16., -2., 14., -1., 0.,
                          10., -1., 1., 0., 2., 0.,
                          -2., 6., -4., -14., 1., 0. };
    double da102[5][7] = {  3.,  0.,  0.,  0.,  0.,  0.,  0.,
                           -1., -2.,  0.,  0.,  0.,  0.,  0.,
                            2.,  4., -3.,  0.,  0.,  0.,  0.,
                            2.,	-1.,  0.,  4.,  0.,  0.,  0.,
                            1.,	 3.,  2., -2.,  1.,  0.,  0. };
    double db102[3][6] = { 6., -16., -2., 14., -1., 0.,
                          10., -1., 1., 0., 2., 0.,
                          -2., 6., -4., -14., 1., 0. };
 
/*     dskfs uses 107,108
*     dgkfs uses 112           */
    _ESVINT ip107[25];
    double d107[25];
 
    double dbr107[36] ={1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,
                  1.0,0.0,0.0,0.0,
                  2.0,2.0,2.0,2.0,2.0,2.0,2.0,2.0,
                  2.0,0.0,0.0,0.0,
                  3.0,3.0,3.0,3.0,3.0,3.0,3.0,3.0,
                  3.0,0.0,0.0,0.0 };
    _ESVINT    ir107[7] = {0,8,0,0,0,0,9};
    _ESVINT    ir108[6]= {1,10,0,1,0,0};
    _ESVINT    idr108[10] = {1,2,4,7,11, 15,20,23,30,34 };
    double dr107[3] = {7.0,1.0,0.0};
    _ESVINT    ipr112[5] = {0,0,0,0,9};
    double dru112[33] ={0.50,0.50,2.00,0.50,2.00,2.00,0.50,2.00,
                  2.00,0.50,2.00,2.00,2.00,0.50,2.00,2.00,
                  0.50,2.00,2.00,2.00,2.00,0.50,2.00,2.00,
                  2.00,0.50,2.00,2.00,2.00,2.00,2.00,2.00,
                  2.00};
    double  dbr112[36] ={1.00,1.00,1.00,1.00,1.00,1.00,1.00,1.00,
                  1.00,0.00,0.00,0.00,
                  2.00,2.00,2.00,2.00,2.00,2.00,2.00,2.00,
                  2.00,0.00,0.00,0.00,
                  3.00,3.00,3.00,3.00,3.00,3.00,3.00,3.00,
                  3.00,0.00,0.00,0.00 };
    double  da107[33]={1.0,2.0,1.0,3.0,2.0,1.0,4.0,3.0,2.0,1.0,
                 4.0,3.0,2.0,1.0,5.0,4.0,3.0,2.0,1.0,3.0,
                 2.0,1.0,7.0,3.0,5.0,4.0,3.0,2.0,1.0,4.0,
                 3.0,2.0,1.0 };
    _ESVINT  id107[10] = {1,2,4,7,11, 15,20,23,30,34};
    _ESVINT  id108[10] = {1,3,6,10,14, 19,22,29,33,34};
    double db107[36] = { 4.0,10.0,15.0,19.0,19.0,23.0,
                   11.0,28.0,10.0, 1.0, 1.0, 1.0,
                    8.0,20.0,30.0,38.0,38.0,46.0,
                   22.0,56.0,20.0, 1.0, 1.0, 1.0,
                   12.0,30.0,45.0,57.0,57.0,69.0,
                   33.0,84.0,30.0, 1.0, 1.0, 1.0 };
 
    double dar107[33] = {1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,
                      1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,
                      1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0};
 
    double  da108[33] = {1.0,1.0,2.0,1.0,2.0,3.0,1.0,2.0,3.0,4.0,
                 1.0,2.0,3.0,4.0,1.0,2.0,3.0,4.0,5.0,1.0,
                 2.0,3.0,1.0,2.0,3.0,4.0,5.0,3.0,7.0,1.0,
                 2.0,3.0,4.0 };
 
    double  db108[36] = { 4.0,10.0,15.0,19.0,19.0,23.0,
                 11.0,28.0,10.0, 1.0, 1.0, 1.0,
                  8.0,20.0,30.0,38.0,38.0,46.0,
                 22.0,56.0,20.0, 1.0, 1.0, 1.0,
                 12.0,30.0,45.0,57.0,57.0,69.0,
                 33.0,84.0,30.0, 1.0, 1.0, 1.0 };
 
    _ESVINT  idu112[10] = {1,2,4,7,10, 14,17,22,26,34};
 
    _ESVINT  idl112[10] = {1,2,4,7,11, 13,18,22,27,36};
 
    double dau112[33] ={2.0,4.0,2.0,6.0,4.0, 2.0, 6.0,4.0, 2.0,4.0,
                  6.0,4.0,2.0,6.0,4.0, 2.0, 8.0,8.0, 4.0,4.0,
                  2.0,8.0,6.0,4.0,2.0,16.0,10.0,8.0,10.0,4.0,
                  6.0,4.0,2.0};
 
    double daur112[33] ={0.5,0.5,2.0,0.5,2.0,2.0,0.5,2.0,2.0,0.5,2.0,2.0,
                       2.0,0.5,2.0,2.0,0.5,2.0,2.0,2.0,2.0,0.5,2.0,2.0,
                       2.0,0.5,2.0,2.0,2.0,2.0,2.0,2.0,2.0};
 
    double dal112[35] ={0.0,0.0,2.0,0.0,4.0, 2.0,0.0,6.0, 4.0,2.0,
                  0.0,2.0,0.0,8.0,6.0, 4.0,2.0,0.0, 6.0,4.0,
                  2.0,0.0,8.0,6.0,4.0, 2.0,0.0,8.0,10.0,6.0,
                  8.0,6.0,6.0,4.0,2.0 };
    double db112[36] ={ 6.0, 16.0, 26.0, 36.0,20.0, 48.0,
                 34.0, 38.0, 66.0,  1.0, 1.0,  1.0,
                 12.0, 32.0, 52.0, 72.0,40.0, 96.0,
                 68.0, 76.0,132.0,  1.0, 1.0,  1.0,
                 18.0, 48.0, 78.0,108.0,60.0,144.0,
                102.0,114.0,198.0,  1.0, 1.0,  1.0 };
    float sab34[4][4] = {1.,0.,0.,0.,2.,1.,0.,0.,
                         3.,2.,1.,0.,2.,5.,3.,1.};
    float sxb34[4] = {26.,20.,9.,2.};
    double dab34[4][4] = {1.,0.,0.,0.,2.,1.,0.,0.,
                         3.,2.,1.,0.,2.,5.,3.,1.};
    double dxb34[4] = {26.,20.,9.,2.};
    cmplx cab34[4][4] = {{{1.,1.},{0.,0.},{0.,0.},{0.,0.}},
                         {{2.,2,},{1.,1.},{0.,0.},{0.,0.}},
                         {{3.,3.},{2.,2.},{1.,1.},{0.,0.}},
                         {{2.,2.},{5.,5.},{3.,3.},{1.,1.}} };
    cmplx cxb34[4] = {{0.,10.},{0.,28.},{0.,52.},{0.,82.}};
    dcmplx zab34[4][4] = {{{1.,1.},{0.,0.},{0.,0.},{0.,0.}},
                          {{2.,2.},{1.,1.},{0.,0.},{0.,0.}},
                          {{3.,3.},{2.,2.},{1.,1.},{0.,0.}},
                          {{2.,2.},{5.,5.},{3.,3.},{1.,1.}} };
    dcmplx zxb34[4] = {{5.,47.},{4.,36.},{3.,15.},{2.,2.}};
    cmplx cab38[5][7] = {{{3.,-3.},{0.,0.},{0.,0.},{0.,0.},{0.,0.},
                         {0.,0.},{0.,0.}},
                         {{-1.,-5.},{-2.,0.},{0.,0.},{0.,0.},{0.,0.},
                         {0.,0.},{0.,0.}},
                         {{2.,0.},{4.,3.},{-3.,-5.},{0.,0.},{0.,0.},
                         {0.,0.},{0.,0.}},
                         {{2.,1.},{-1.,0.},{0.,-2.},{4.,4.},{0.,0.},
                         {0.,0.},{0.,0.}},
                         {{1.,4.},{3.,0.},{2.,-5.},{-2.,3},{1.,1.},
                         {0.,0.},{0.,0.}}};
    cmplx cbb38[3][6] = {{{20.,-26.},{-7.,-10.},{-7.,12.},{8.,5.},{-3.,1.},
                         {0.,0.}},
                         {{13.,-22.},{-7.,-1.},{9.,-27.},{-10.,30.},{4.,0.},
                         {0.,0.}},
                         {{11.,-28.},{0.,1.},{5.,-17.},{-19.,1.},{2.,0.},
                         {0.,0.}}};
    dcmplx zab38[5][7] ={{{3.,-3.},{0.,0.},{0.,0.},{0.,0.},{0.,0.},
                         {0.,0.},{0.,0.}},
                         {{-1.,-5.},{-2.,0.},{0.,0.},{0.,0.},{0.,0.},
                         {0.,0.},{0.,0.}},
                         {{2.,0.},{4.,3.},{-3.,-5.},{0.,0.},{0.,0.},
                         {0.,0.},{0.,0.}},
                         {{2.,1.},{-1.,0.},{0.,-2.},{4.,4.},{0.,0.},
                         {0.,0.},{0.,0.}},
                         {{1.,4.},{3.,0.},{2.,-5.},{-2.,3.},{1.,1.},
                         {0.,0.},{0.,0.}}};
    dcmplx zbb38[3][6] = {{{20.,-26.},{-7.,-10.},{-7.,12.},{8.,5.},{-3.,1.},
                         {0.,0.}},
                         {{13.,-22.},{-7.,-1.},{9.,-27.},{-10.,30.},{4.,0.},
                         {0.,0.}},
                          {{11.,-28.},{0.,1.},{5.,-17.},{-19.,1.},{2.,0.},
                         {0.,0.}}};
    float  sgeb27n[2][9] = {4.,5.,9.,10.,11.,12.,12.,12.,33.,
                      10.,15.,24.,35.,48.,63.,70.,78.,266.};
 
    double dgeb27n[2][9] = {4.,5.,9.,10.,11.,12.,12.,12.,33.,
                      10.,15.,24.,35.,48.,63.,70.,78.,266.};
    cmplx  cgeb27n[2][4] = {{{-10.,85.},{-6.,61.},{10.,38.},{ 58.,168.}},
                  {{-11.,53.}, {-6.,54.}, {2.,40.}, {15., 105.}}};
    dcmplx zgeb27n[2][4] = {{{-10.,85.},{-6.,61.},{10.,38.}, {58.,168.}},
                  {{-11.,53.}, {-6.,54.}, {2.,40.}, {15., 105.}}};
    float  spoa28n[9][9] = {1.,1.,1.,1.,1.,1.,1.,1.,1.,
                       1.,2.,2.,2.,2.,2.,2.,2.,2.,
                       1.,2.,3.,3.,3.,3.,3.,3.,3.,
                       1.,2.,3.,4.,4.,4.,4.,4.,4.,
                       1.,2.,3.,4.,5.,5.,5.,5.,5.,
                       1.,2.,3.,4.,5.,6.,6.,6.,6.,
                       1.,2.,3.,4.,5.,6.,7.,7.,7.,
                       1.,2.,3.,4.,5.,6.,7.,8.,8.,
                       1.,2.,3.,4.,5.,6.,7.,8.,9.};
    float  spoa28n1[9][9] = {1.,1.,1.,1.,1.,1.,1.,1.,1.,
                       1.,2.,2.,2.,2.,2.,2.,2.,2.,
                       1.,2.,3.,3.,3.,3.,3.,3.,3.,
                       1.,2.,3.,4.,4.,4.,4.,4.,4.,
                       1.,2.,3.,4.,5.,5.,5.,5.,5.,
                       1.,2.,3.,4.,5.,6.,6.,6.,6.,
                       1.,2.,3.,4.,5.,6.,7.,7.,7.,
                       1.,2.,3.,4.,5.,6.,7.,8.,8.,
                       1.,2.,3.,4.,5.,6.,7.,8.,9.};
    float snrm211,srcnd211,swrk211[27];
    _ESVINT   siwrk211[9];
    float snrm211res  = 45.0;
    float srcnd211res =  0.005555;
 
    double dpoa28n[9][9] = {1.,1.,1.,1.,1.,1.,1.,1.,1.,
                       1.,2.,2.,2.,2.,2.,2.,2.,2.,
                       1.,2.,3.,3.,3.,3.,3.,3.,3.,
                       1.,2.,3.,4.,4.,4.,4.,4.,4.,
                       1.,2.,3.,4.,5.,5.,5.,5.,5.,
                       1.,2.,3.,4.,5.,6.,6.,6.,6.,
                       1.,2.,3.,4.,5.,6.,7.,7.,7.,
                       1.,2.,3.,4.,5.,6.,7.,8.,8.,
                       1.,2.,3.,4.,5.,6.,7.,8.,9.};
    double dpoa28n1[9][9] = {1.,1.,1.,1.,1.,1.,1.,1.,1.,
                       1.,2.,2.,2.,2.,2.,2.,2.,2.,
                       1.,2.,3.,3.,3.,3.,3.,3.,3.,
                       1.,2.,3.,4.,4.,4.,4.,4.,4.,
                       1.,2.,3.,4.,5.,5.,5.,5.,5.,
                       1.,2.,3.,4.,5.,6.,6.,6.,6.,
                       1.,2.,3.,4.,5.,6.,7.,7.,7.,
                       1.,2.,3.,4.,5.,6.,7.,8.,8.,
                       1.,2.,3.,4.,5.,6.,7.,8.,9.};
    double dnrm211,drcnd211,dwrk211[27];
    _ESVINT    diwrk211[9];
    double dnrm211res  = 45.0;
    double drcnd211res =  0.005555;
 
    cmplx  cpoa28n[3][3] = {{{25.,0.},{ -5.,5.}, {10.,-5.}},
                       {{-5.,-5.},{ 51.,0.}, {4.,6.}},
                       {{10.,5.}, {4.,-6.}, {71., 0.}}};
    cmplx  cpoa28n1[3][3] = {{{25.,0.},{ -5.,5.}, {10.,-5.}},
                       {{-5.,-5.},{ 51.,0.}, {4.,6.}},
                       {{10.,5.}, {4.,-6.}, {71., 0.}}};
    float cnrm211,crcnd211;
    cmplx cwrk211[6];
    float crwrk211[3];
    float cnrm211res  = 89.3914424384269353;
    float crcnd211res =  0.184714364056937669;
 
    dcmplx zpoa28n[3][3] = {{{25.,0.}, {-5.,5.}, {10.,-5.}},
                       {{-5.,-5.}, {51.,0.},{ 4.,6.}},
                       {{10.,5.}, {4.,-6.},{ 71., 0.}}};
    dcmplx zpoa28n1[3][3] = {{{25.,0.}, {-5.,5.}, {10.,-5.}},
                       {{-5.,-5.}, {51.,0.},{ 4.,6.}},
                       {{10.,5.}, {4.,-6.},{ 71., 0.}}};
    double znrm211,zrcnd211;
    dcmplx zwrk211[6];
    double zrwrk211[3];
    double znrm211res  = 89.3914424384269353;
    double zrcnd211res =  0.184714364056937669;
 
 
    float  sposb29n[2][9] = {9.,17.,24.,30.,35.,39.,42.,44.,45.,
                45.,89.,131.,170.,205.,235.,259.,276.,285.};
    float  spob29n1[2][9] = {9.,17.,24.,30.,35.,39.,42.,44.,45.,
                45.,89.,131.,170.,205.,235.,259.,276.,285.};
    double dposb29n[2][9] = {9.,17.,24.,30.,35.,39.,42.,44.,45.,
                45.,89.,131.,170.,205.,235.,259.,276.,285.};
    double dpob29n1[2][9] = {9.,17.,24.,30.,35.,39.,42.,44.,45.,
                45.,89.,131.,170.,205.,235.,259.,276.,285.};
    cmplx cposb29n[2][3] = {{{60.,-55.},{ 34.,58.}, {13.,-152.}},
                       {{70.,10.}, {-51.,110.}, {75.,63.}}};
    cmplx cpob29n1[2][3] = {{{60.,-55.},{ 34.,58.}, {13.,-152.}},
                       {{70.,10.}, {-51.,110.}, {75.,63.}}};
    dcmplx zposb29n[2][3] ={{{60.,-55.},{ 34.,58.}, {13.,-152.}},
                       {{70.,10.},{ -51.,110.}, {75.,63.}}};
    dcmplx zpob29n1[2][3] ={{{60.,-55.},{ 34.,58.}, {13.,-152.}},
                       {{70.,10.},{ -51.,110.}, {75.,63.}}};
 
    float  spfa30n[9][9] = {1.,1.,1.,1.,1.,1.,1.,1.,1.,
                       1.,2.,2.,2.,2.,2.,2.,2.,2.,
                       1.,2.,3.,3.,3.,3.,3.,3.,3.,
                       1.,2.,3.,4.,4.,4.,4.,4.,4.,
                       1.,2.,3.,4.,5.,5.,5.,5.,5.,
                       1.,2.,3.,4.,5.,6.,6.,6.,6.,
                       1.,2.,3.,4.,5.,6.,7.,7.,7.,
                       1.,2.,3.,4.,5.,6.,7.,8.,8.,
                       1.,2.,3.,4.,5.,6.,7.,8.,9.};
    double dpfa30n[9][9] = {1.,1.,1.,1.,1.,1.,1.,1.,1.,
                       1.,2.,2.,2.,2.,2.,2.,2.,2.,
                       1.,2.,3.,3.,3.,3.,3.,3.,3.,
                       1.,2.,3.,4.,4.,4.,4.,4.,4.,
                       1.,2.,3.,4.,5.,5.,5.,5.,5.,
                       1.,2.,3.,4.,5.,6.,6.,6.,6.,
                       1.,2.,3.,4.,5.,6.,7.,7.,7.,
                       1.,2.,3.,4.,5.,6.,7.,8.,8.,
                       1.,2.,3.,4.,5.,6.,7.,8.,9.};
 
    float  spia31n[9][9] = {1.,0.,0.,0.,0.,0.,0.,0.,0.,
                       1.,2.,0.,0.,0.,0.,0.,0.,0.,
                       1.,2.,3.,0.,0.,0.,0.,0.,0.,
                       1.,2.,3.,4.,0.,0.,0.,0.,0.,
                       1.,2.,3.,4.,5.,0.,0.,0.,0.,
                       1.,2.,3.,4.,5.,6.,0.,0.,0.,
                       1.,2.,3.,4.,5.,6.,7.,0.,0.,
                       1.,2.,3.,4.,5.,6.,7.,8.,0.,
                       1.,2.,3.,4.,5.,6.,7.,8.,9.};
    float  spia31n1[9][9] = {1.,0.,0.,0.,0.,0.,0.,0.,0.,
                       1.,2.,0.,0.,0.,0.,0.,0.,0.,
                       1.,2.,3.,0.,0.,0.,0.,0.,0.,
                       1.,2.,3.,4.,0.,0.,0.,0.,0.,
                       1.,2.,3.,4.,5.,0.,0.,0.,0.,
                       1.,2.,3.,4.,5.,6.,0.,0.,0.,
                       1.,2.,3.,4.,5.,6.,7.,0.,0.,
                       1.,2.,3.,4.,5.,6.,7.,8.,0.,
                       1.,2.,3.,4.,5.,6.,7.,8.,9.};
    double dpia31n[9][9] = {1.,0.,0.,0.,0.,0.,0.,0.,0.,
                       1.,2.,0.,0.,0.,0.,0.,0.,0.,
                       1.,2.,3.,0.,0.,0.,0.,0.,0.,
                       1.,2.,3.,4.,0.,0.,0.,0.,0.,
                       1.,2.,3.,4.,5.,0.,0.,0.,0.,
                       1.,2.,3.,4.,5.,6.,0.,0.,0.,
                       1.,2.,3.,4.,5.,6.,7.,0.,0.,
                       1.,2.,3.,4.,5.,6.,7.,8.,0.,
                       1.,2.,3.,4.,5.,6.,7.,8.,9.};
    double dpia31n1[9][9] = {1.,0.,0.,0.,0.,0.,0.,0.,0.,
                       1.,2.,0.,0.,0.,0.,0.,0.,0.,
                       1.,2.,3.,0.,0.,0.,0.,0.,0.,
                       1.,2.,3.,4.,0.,0.,0.,0.,0.,
                       1.,2.,3.,4.,5.,0.,0.,0.,0.,
                       1.,2.,3.,4.,5.,6.,0.,0.,0.,
                       1.,2.,3.,4.,5.,6.,7.,0.,0.,
                       1.,2.,3.,4.,5.,6.,7.,8.,0.,
                       1.,2.,3.,4.,5.,6.,7.,8.,9.};
    float stpia31n[15] = {1., 3.,2., 4.,8.,4., 5.,9.,8.,-2.,
                     6.,1.,4.,6.,-1};
    float stpia31n1[15] = {1., 3.,2., 4.,8.,4., 5.,9.,8.,-2.,
                     6.,1.,4.,6.,-1};
    double dtpia31n[15] = {1., 3.,2., 4.,8.,4., 5.,9.,8.,-2.,
                      6.,1.,4.,6.,-1};
    double dtpia31n1[15] = {1., 3.,2., 4.,8.,4., 5.,9.,8.,-2.,
                      6.,1.,4.,6.,-1};
    cmplx  ctpia31n1[15] = { {-4.00, 1.00},{ 4.00,-3.00},{-2.00, 0.00},
                             {-1.00, 3.00},{-3.00,-1.00},{-5.00, 3.00},
                             { 0.00, 0.00},{-2.00,-1.00},{-3.00,-3.00},
                             { 4.00,-4.00},{-1.00, 0.00},{ 4.00, 3.00},
                             {-5.00,-5.00},{ 2.00, 0.00},{ 2.00,-1.00} };
    dcmplx  ztpia31n1[15] = {{-4.00, 1.00},{ 4.00,-3.00},{-2.00, 0.00},
                             {-1.00, 3.00},{-3.00,-1.00},{-5.00, 3.00},
                             { 0.00, 0.00},{-2.00,-1.00},{-3.00,-3.00},
                             { 4.00,-4.00},{-1.00, 0.00},{ 4.00, 3.00},
                             {-5.00,-5.00},{ 2.00, 0.00},{ 2.00,-1.00} };
    float  stria32n[5][5] = {1.,0.,0.,0.,0., 3.,2.,0.,0.,0.,
           4.,8.,4.,0.,0., 5.,9.,8.,-2.,0., 6.,1.,4.,6.,-1.};
    float  stria32n1[5][5] = {1.,0.,0.,0.,0., 3.,2.,0.,0.,0.,
           4.,8.,4.,0.,0., 5.,9.,8.,-2.,0., 6.,1.,4.,6.,-1.};
    double dtria32n[5][5] = {1.,0.,0.,0.,0., 3.,2.,0.,0.,0.,
           4.,8.,4.,0.,0., 5.,9.,8.,-2.,0., 6.,1.,4.,6.,-1.};
    double dtria32n1[5][5] = {1.,0.,0.,0.,0., 3.,2.,0.,0.,0.,
           4.,8.,4.,0.,0., 5.,9.,8.,-2.,0., 6.,1.,4.,6.,-1.};
    cmplx ctria32n[5][5] = {
      {{-4.00, 1.00},{0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0}},
      {{ 4.00,-3.00},{-2.00, 0.00},{0.0,0.0},{0.0,0.0},{0.0,0.0}},
      {{-1.00, 3.00},{-3.00,-1.00},{-5.00, 3.00},{0.0,0.0},{0.0,0.0}},
      {{ 0.00, 0.00},{-2.00,-1.00},{-3.00,-3.00},{ 4.00,-4.00},{0.0,0.0}},
      {{-1.00, 0.00},{ 4.00, 3.00},{-5.00,-5.00},{ 2.00, 0.00},{ 2.00,-1.00}}
  };
    dcmplx ztria32n[5][5] = {
      {{-4.00, 1.00},{0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0}},
      {{ 4.00,-3.00},{-2.00, 0.00},{0.0,0.0},{0.0,0.0},{0.0,0.0}},
      {{-1.00, 3.00},{-3.00,-1.00},{-5.00, 3.00},{0.0,0.0},{0.0,0.0}},
      {{ 0.00, 0.00},{-2.00,-1.00},{-3.00,-3.00},{ 4.00,-4.00},{0.0,0.0}},
      {{-1.00, 0.00},{ 4.00, 3.00},{-5.00,-5.00},{ 2.00, 0.00},{ 2.00,-1.00}}
  };
 
    float  stpva35n[10] = {1.,1.,2.,3.,1.,3.,4.,1.,3.,1.};
    float stpvx35n[4]= {1.,3.,11.,24.};
    double dtpva35n[10] = {1.,1.,2.,3.,1.,3.,4.,1.,3.,1.};
    double dtpvx35n[4]= {1.,3.,11.,24.};
    cmplx  ctpva35n[10] = {{1.,0.}, {1.,1.}, {1.,1.}, {3.,3.},
           {1.,0.},{3.,3.}, {4.,4.},       {1.,0.}, {3.,3.},{1.,0.}};
    cmplx  ctpvx35n[4] = {{1.,1.}, {2.,4.}, {3.,17.}, {4.,44.}};
    dcmplx  ztpva35n[10] = {{1.,0.}, {1.,1.}, {1.,1.}, {3.,3.},
            {1.,0.},{3.,3.}, {4.,4.},       {1.,0.}, {3.,3.},{1.,0.}};
    dcmplx ztpvx35n[4] = {{1.,1.}, {2.,4.}, {3.,17.}, {4.,44.}};
 
    float  stbva36n[9][3] = {1.,1.,1., 4.,2.,3., 4.,1.,1., 4.,2.,2.,
           3.,1.,1., 3.,2.,2., 3.,1.,0., 2.,2.,0., 1.,0.,0.};
    float  stbvx36n[9] = {2.,7.,1.,8.,2.,8.,1.,8.,3.};
    double dtbva36n[9][3] = {1.,1.,1., 4.,2.,3., 4.,1.,1., 4.,2.,2.,
           3.,1.,1., 3.,2.,2., 3.,1.,0., 2.,2.,0., 1.,0.,0.};
    double dtbvx36n[9] = {2.,7.,1.,8.,2.,8.,1.,8.,3.};
    cmplx  ctbva36n[7][4]={{{1.,0.}, {1.,2.},{1.,3.},{1.,4.}},{{2.,1.},
    {2.,2.},{2.,3.},{2.,4.}},{{3.,1.},{3.,3.},{3.,3.},{3.,3.}},{{4.,1.},
    {4.,2.},{4.,3.},{4.,4.}},{{2.,1.},{5.,1.},{5.,2.},{5.,3.}},{{3.,1.},
    {6.,1.},{6.,3.},{6.,4.}},{{2.,1.},{5.,1.},{5.,2.},{5.,3.}}};
    cmplx  ctbvx36n[7]={{2.,2.},{7.,1.},{1.,1.},{8.,1.},
    {2.,0.},{8.,1.},{1.,2.}};
    dcmplx  ztbva36n[7][4]={{{1.,0.},{1.,2.},{1.,3.},{1.,4.}},{{2.,1.},
    {2.,2.},{2.,3.},{2.,4.}},{{3.,1.},{3.,3.},{3.,3.},{3.,3.}},{{4.,1.},
    {4.,2.},{4.,3.},{4.,4.}},{{2.,1.},{5.,1.},{5.,2.},{5.,3.}},{{3.,1.},
    {6.,1.},{6.,3.},{6.,4.}},{{2.,1.},{5.,1.},{5.,2.},{5.,3.}}};
    dcmplx  ztbvx36n[7]={{2.,2.},{7.,1.},{1.,1.},{8.,1.},
    {2.,0.},{8.,1.},{1.,2.}};
 
    cmplx  cgtfc37n[4]={{0.,0.}, {1.,1.}, {2.,2.}, {3.,3.}};
    cmplx  cgtfa37n[4] ={{ 7.,7.}, {8.,8.}, {9.,9.}, {10.,10.}};
    cmplx  cgtfb37n[4] = {{4.,4.}, {5.,5.}, {6.,6.}, {0.,0.}};
    cmplx cctnp[4] = {{0.,0.},{1.,1.},{2.,2.},{3.,3.}};
    cmplx catnp[4] = {{7.,7.},{8.,8.},{9.,9.},{10.,10}};
    cmplx cbtnp[4] = {{4.,4.},{5.,5.},{6.,6.},{0.,0.}};
    dcmplx zgtfc37n[4]={{0.,0.}, {1.,1.}, {2.,2.}, {3.,3.}};
    dcmplx zgtfa37n[4] = {{7.,7.}, {8.,8.}, {9.,9.}, {10.,10.}};
    dcmplx zgtfb37n[4] = {{4.,4.},{ 5.,5.}, {6.,6.}, {0.,0.}};
    dcmplx  zctnp[4] = {{0.,0.},{1.,1.},{2.,2.},{3.,3.}};
    dcmplx  zatnp[4] = {{7.,7.},{8.,8.},{9.,9.},{10.,10.}};
    dcmplx  zbtnp[4] = {{4.,4.},{5.,5.},{6.,6.},{0.,0.}};
 
    cmplx  cgtsx38n[4] = {{-11.,19.},{-14.,50.}, {-17.,93.},{ -13.,85.}};
    dcmplx zgtsx38n[4] ={{-11.,19.}, {-14.,50.}, {-17.,93.},{ -13.,85.}};
    cmplx cxtnp[4] = {{-11.,19.},{-14.,50.},{-17.,93.},{-13.,85.}};
    dcmplx  zxtnp[4] = {{-11.,19.},{-14.,50.},{-17.,93.},{-13.,85.}};
 
 
    double  dssar39n[22] = {2.,2.,-1.,1.,2.,1.,2.,-1.,1.,2.,-1.,
                            1.,2.,-1.,1.,2.,-1.,1.,2.,-1.,1.,2.};
    _ESVINT     dssja39n[22] = {1,2,3,2,3,1,4,5,4,5,6,5,6,7,6,7,8,7,8,9,8,9};
    _ESVINT     dssia39n[10] = {1,2,4,6,9,12,15,18,21,23};
    double  dssb39n[9] = {2.,1.,3.,2.,2.,2.,2.,2.,3.};
    double  dssx39n[9] = {0.,0.,0.,0.,0.,0.,0.,0.,0.};
    _ESVINT     dssip39n[6] = {20,4,0,1,0,0};
    double  dssrp39n[3] = {1.0e-07,0.,1.};
 
  float  sta7[9][9] =  { 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2, 2.4, 2.6,
                        1.2, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2, 2.4,
                        1.4, 1.2, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2,
                        1.6, 1.4, 1.2, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0,
                        1.8, 1.6, 1.4, 1.2, 1.0, 1.2, 1.4, 1.6, 1.8,
                        2.0, 1.8, 1.6, 1.4, 1.2, 1.0, 1.2, 1.4, 1.6,
                        2.2, 2.0, 1.8, 1.6, 1.4, 1.2, 1.0, 1.2, 1.4,
                        2.4, 2.2, 2.0, 1.8, 1.6, 1.4, 1.2, 1.0, 1.2,
                        2.6, 2.4, 2.2, 2.0, 1.8, 1.6, 1.4, 1.2, 1.0 };
 
  float sa411[9][9] = { 1.0, 1.0, 4.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                        1.0, 1.0, 1.0, 5.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                        1.0, 1.0, 1.0, 1.0, 6.0, 0.0, 0.0, 0.0, 0.0,
                        1.0, 1.0, 1.0, 1.0, 1.0, 7.0, 0.0, 0.0, 0.0,
                        0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 8.0, 0.0, 0.0,
                        0.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 9.0, 0.0,
                        0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0,10.0,
                        0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0,11.0,
                        0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0,12.0 };
  float  snrm411, srcnd411;
  float  swrk411[36];
  float  snrm411res  = 15.0;
  float  srcnd411res = 0.00005436259853;
  _ESVINT    siwrk411[9];
 
  double dta7[9][9] =  { 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2, 2.4, 2.6,
                        1.2, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2, 2.4,
                        1.4, 1.2, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2,
                        1.6, 1.4, 1.2, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0,
                        1.8, 1.6, 1.4, 1.2, 1.0, 1.2, 1.4, 1.6, 1.8,
                        2.0, 1.8, 1.6, 1.4, 1.2, 1.0, 1.2, 1.4, 1.6,
                        2.2, 2.0, 1.8, 1.6, 1.4, 1.2, 1.0, 1.2, 1.4,
                        2.4, 2.2, 2.0, 1.8, 1.6, 1.4, 1.2, 1.0, 1.2,
                        2.6, 2.4, 2.2, 2.0, 1.8, 1.6, 1.4, 1.2, 1.0 };
 
  double da411[9][9] = { 1.0, 1.0, 4.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                        1.0, 1.0, 1.0, 5.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                        1.0, 1.0, 1.0, 1.0, 6.0, 0.0, 0.0, 0.0, 0.0,
                        1.0, 1.0, 1.0, 1.0, 1.0, 7.0, 0.0, 0.0, 0.0,
                        0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 8.0, 0.0, 0.0,
                        0.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0, 9.0, 0.0,
                        0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0, 1.0,10.0,
                        0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0,11.0,
                        0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0,12.0 };
  double dnrm411, drcnd411;
  double dwrk411[36];
  double dnrm411res  = 15.0;
  double drcnd411res = 0.00005436259853;
  _ESVINT    diwrk411[9];
 
  float  stb7[5][9] = {
   93.0,  84.4,  76.6,  70.0,  65.0,  62.0,  61.4,  63.6,  69.0,
  186.0, 168.8, 153.2, 140.0, 130.0, 124.0, 122.8, 127.2, 138.0,
  279.0, 253.2, 229.8, 210.0, 195.0, 186.0, 184.2, 190.8, 207.0,
  372.0, 337.6, 306.4, 280.0, 260.0, 248.0, 245.6, 254.4, 276.0,
  465.0, 422.0, 383.0, 350.0, 325.0, 310.0, 307.0, 318.0, 345.0 };
 
  double dtb7[5][9] = {
   93.0,  84.4,  76.6,  70.0,  65.0,  62.0,  61.4,  63.6,  69.0,
  186.0, 168.8, 153.2, 140.0, 130.0, 124.0, 122.8, 127.2, 138.0,
  279.0, 253.2, 229.8, 210.0, 195.0, 186.0, 184.2, 190.8, 207.0,
  372.0, 337.6, 306.4, 280.0, 260.0, 248.0, 245.6, 254.4, 276.0,
  465.0, 422.0, 383.0, 350.0, 325.0, 310.0, 307.0, 318.0, 345.0 };
 
  cmplx cta7[9][9] = {
      {{2.0,1.0},{2.4, 1.0},{2.8, 1.0},{3.2,1.0},{3.6,1.0},
       {4.0,1.0},{4.4, 1.0},{4.8, 1.0}, {5.2, 1.0}},  /* 1st  col end */
      {{2.4,-1.0},{2.0, 1.0}, {2.4, 1.0},{2.8,1.0},{3.2,1.0},
       {3.6,1.0},{4.0, 1.0},{4.4, 1.0}, {4.8, 1.0}},  /* 2nd  col end */
      {{2.8,-1.0},{2.4,-1.0},{2.0, 1.0},{2.4,1.0},{2.8,1.0},
       {3.2,1.0},{3.6, 1.0}, {4.0, 1.0}, {4.4, 1.0}},  /* 3rd col end */
      {{3.2,-1.0},{2.8,-1.0},{2.4,-1.0},{2.0,1.0},{2.4,1.0},
       {2.8, 1.0},{3.2, 1.0}, {3.6, 1.0}, {4.0, 1.0}}, /* 4th col end */
      {{3.6,-1.0}, {3.2,-1.0}, {2.8,-1.0},{2.4,-1.0},{2.0,1.0},
       {2.4, 1.0},{2.8, 1.0}, {3.2, 1.0}, {3.6, 1.0}}, /* 5th col end */
      {{4.0,-1.0}, {3.6,-1.0}, {3.2,-1.0},{2.8,-1.0},{2.4,-1.0},
       {2.0, 1.0},{2.4, 1.0}, {2.8, 1.0}, {3.2, 1.0}}, /* 6th col end */
      {{4.4,-1.0}, {4.0,-1.0}, {3.6,-1.0},{3.2,-1.0},{2.8,-1.0},
       {2.4, -1.0},{2.0, 1.0}, {2.4, 1.0}, {2.8, 1.0}}, /* 7th col end*/
      {{4.8,-1.0}, {4.4,-1.0}, {4.0,-1.0},{3.6,-1.0},{3.2,-1.0},
       {2.8, -1.0},{2.4,-1.0}, {2.0, 1.0}, {2.4, 1.0}}, /* 8th col end*/
      {{5.2,-1.0}, {4.8,-1.0}, {4.4,-1.0},{4.0,-1.0},{3.6,-1.0},
       {3.2, -1.0},{2.8,-1.0}, {2.4,-1.0}, {2.0, 1.0}}};/* 9th col end*/
 
  cmplx ca411[4][4] = {
      {{1.0000,2.0000},{2.0000,0.0000},{2.0000,1.0000},{8.0000,5.0000}},
      {{1.0000,7.0000},{1.0000,3.0000},{5.0000,0.0000},{1.0000,9.0000}},
      {{2.0000,4.0000},{4.0000,4.0000},{3.0000,6.0000},{6.0000,6.0000}},
      {{3.0000,1.0000},{2.0000,3.0000},{0.0000,0.0000},{8.0000,1.0000}} };
 
  cmplx cwrk411[8];
  float cnrm411, crcnd411, crwrk411[8];
  float cnrm411res  = 25.3225;
  float crcnd411res =  0.03659055;
 
  cmplx ctb7[5][9] = {
      {{193.0,-10.60}, {173.8,-9.40}, {156.2, -5.40},{141.0,1.4},
       {129.0,11.00},{121.0, 23.40},{117.8,38.6},{120.2,56.6},
       {129.0,77.4}},{{200.0,21.80},{178.8,20.20}, {159.2, 22.20 },
       {142.0, 27.80}, {128.0,37.00}, {118.0,49.80}, {112.8,66.2},
       {113.2,86.20},{ 120.0,109.80}},{{207.0,54.2},{183.8,49.8},
       {162.2,49.8},{143.0, 54.20}, {127.0,63.00}, {115.0, 76.20},
       {107.8, 93.80}, {106.2,115.8}, {111.0,142.20}},{{214.0,86.6},
       {188.8,79.40}, {165.2, 77.40},{144.0,80.6},{126.0,89.0},
       {112.0,102.60},{102.8,121.40}, { 99.2,145.4}, {102.0,174.60}},
      {{221.0,119.00}, {193.8,109.0}, {168.2,105.00},{145.0,107.0},
       {125.0,115.0}, {109.0,129.00},{97.8,149.0},{92.2,175.0},
       {93.0,207.00}}};
 
  dcmplx zta7[9][9] = {
      {{2.0,1.0},{2.4, 1.0},{2.8, 1.0},{3.2,1.0},{3.6,1.0},
       {4.0,1.0},{4.4, 1.0},{4.8, 1.0}, {5.2, 1.0}},  /* 1st  col end */
      {{2.4,-1.0},{2.0, 1.0}, {2.4, 1.0},{2.8,1.0},{3.2,1.0},
       {3.6,1.0},{4.0, 1.0},{4.4, 1.0}, {4.8, 1.0}},  /* 2nd  col end */
      {{2.8,-1.0},{2.4,-1.0},{2.0, 1.0},{2.4,1.0},{2.8,1.0},
       {3.2,1.0},{3.6, 1.0}, {4.0, 1.0}, {4.4, 1.0}},  /* 3rd col end */
      {{3.2,-1.0},{2.8,-1.0},{2.4,-1.0},{2.0,1.0},{2.4,1.0},
       {2.8, 1.0},{3.2, 1.0}, {3.6, 1.0}, {4.0, 1.0}}, /* 4th col end */
      {{3.6,-1.0}, {3.2,-1.0}, {2.8,-1.0},{2.4,-1.0},{2.0,1.0},
       {2.4, 1.0},{2.8, 1.0}, {3.2, 1.0}, {3.6, 1.0}}, /* 5th col end */
      {{4.0,-1.0}, {3.6,-1.0}, {3.2,-1.0},{2.8,-1.0},{2.4,-1.0},
       {2.0, 1.0},{2.4, 1.0}, {2.8, 1.0}, {3.2, 1.0}}, /* 6th col end */
      {{4.4,-1.0}, {4.0,-1.0}, {3.6,-1.0},{3.2,-1.0},{2.8,-1.0},
       {2.4, -1.0},{2.0, 1.0}, {2.4, 1.0}, {2.8, 1.0}}, /* 7th col end*/
      {{4.8,-1.0}, {4.4,-1.0}, {4.0,-1.0},{3.6,-1.0},{3.2,-1.0},
       {2.8, -1.0},{2.4,-1.0}, {2.0, 1.0}, {2.4, 1.0}}, /* 8th col end*/
      {{5.2,-1.0}, {4.8,-1.0}, {4.4,-1.0},{4.0,-1.0},{3.6,-1.0},
       {3.2, -1.0},{2.8,-1.0}, {2.4,-1.0}, {2.0, 1.0}}};/* 9th col end*/
 
  dcmplx za411[4][4] = {
      {{1.0000,2.0000},{2.0000,0.0000},{2.0000,1.0000},{8.0000,5.0000}},
      {{1.0000,7.0000},{1.0000,3.0000},{5.0000,0.0000},{1.0000,9.0000}},
      {{2.0000,4.0000},{4.0000,4.0000},{3.0000,6.0000},{6.0000,6.0000}},
      {{3.0000,1.0000},{2.0000,3.0000},{0.0000,0.0000},{8.0000,1.0000}} };
 
  dcmplx zwrk411[8];
  double znrm411, zrcnd411, zrwrk411[8];
  double znrm411res  = 25.3225;
  double zrcnd411res =  0.03659055;
 
  dcmplx ztb7[5][9] = {
      {{193.0,-10.60}, {173.8,-9.40}, {156.2, -5.40},{141.0,1.4},
       {129.0,11.00},{121.0, 23.40},{117.8,38.6},{120.2,56.6},
       {129.0,77.4}},{{200.0,21.80},{178.8,20.20}, {159.2, 22.20 },
       {142.0, 27.80}, {128.0,37.00}, {118.0,49.80}, {112.8,66.2},
       {113.2,86.20},{ 120.0,109.80}},{{207.0,54.2},{183.8,49.8},
       {162.2,49.8},{143.0, 54.20}, {127.0,63.00}, {115.0, 76.20},
       {107.8, 93.80}, {106.2,115.8}, {111.0,142.20}},{{214.0,86.6},
       {188.8,79.40}, {165.2, 77.40},{144.0,80.6},{126.0,89.0},
       {112.0,102.60},{102.8,121.40}, { 99.2,145.4}, {102.0,174.60}},
      {{221.0,119.00}, {193.8,109.0}, {168.2,105.00},{145.0,107.0},
       {125.0,115.0}, {109.0,129.00},{97.8,149.0},{92.2,175.0},
       {93.0,207.00}}};
 
  double dta8[36]   =  {3.0, 5.0,-2.0, 2.0, 3.0,-5.0,-2.0,-3.0,
                        3.0, 2.0,-2.0, 5.0,-3.0, 2.0,-5.0,
                        0.0, 0.0,-2.0, 2.0, 0.0, 6.0,
                        8.0,-6.0,-10.,-8.0,-14.,
                       12.0, 6.0, 8.0, 6.0,
                       16.0, 8.0,20.0,
                        6.0,18.0,
                       34.0};
 
  double dtb8[3][8] = {
    1.0,   7.0,   6.0,- 30.0,  32.0,  34.0,  32.0,  62.0,
 - 38.0,- 10.0,  52.0,-228.0, 183.0, 297.0, 244.0, 497.0,
   47.0,  73.0,   2.0,- 42.0, 105.0,   9.0,  44.0,  61.0};
 
  double dta81[36]   =  {3.0, 5.0,-2.0, 2.0, 3.0,-5.0,-2.0,-3.0,
                        3.0, 2.0,-2.0, 5.0,-3.0, 2.0,-5.0,
                        0.0, 0.0,-2.0, 2.0, 0.0, 6.0,
                        8.0,-6.0,-10.,-8.0,-14.,
                       12.0, 6.0, 8.0, 6.0,
                       16.0, 8.0,20.0,
                        6.0,18.0,
                       34.0};
 
  double dtb81[3][8] = {
    1.0,   7.0,   6.0,- 30.0,  32.0,  34.0,  32.0,  62.0,
 - 38.0,- 10.0,  52.0,-228.0, 183.0, 297.0, 244.0, 497.0,
   47.0,  73.0,   2.0,- 42.0, 105.0,   9.0,  44.0,  61.0};
 
  double dta9[2][6] = {
    0.0,   2.0,   2.0,   0.0,   2.0,   2.0,
    2.0,  -1.0,  -1.0,   1.5,  -1.0,  -1.0};
 
  float sta9[2][6] = {
    0.0,   2.0,   2.0,   0.0,   2.0,   2.0,
    2.0,  -1.0,  -1.0,   1.5,  -1.0,  -1.0};
 
  dcmplx zta9[2][6] = {
   {{-1.80, -0.90}, {-1.60, 1.00}, {-1.00, -0.30}, { 1.10, -0.10}, {  0.50,  0.70}, {-1.50, -0.70}},
   {{ 1.10, -0.80}, { 1.70, 1.40}, { 1.20,  0.30}, { 0.70, -1.90}, { -0.20, -1.50}, { 1.80, -0.60}} };
 
  cmplx cta9[2][6] = {
   {{-1.80, -0.90}, {-1.60, 1.00}, {-1.00, -0.30}, { 1.10, -0.10}, {  0.50,  0.70}, {-1.50, -0.70}},
   {{ 1.10, -0.80}, { 1.70, 1.40}, { 1.20,  0.30}, { 0.70, -1.90}, { -0.20, -1.50}, { 1.80, -0.60}} };
 
  double dta10[2][6] = {
    0.0,   2.0,   2.0,   0.0,   2.0,   2.0,
    2.0,  -1.0,  -1.0,   1.5,  -1.0,  -1.0};
 
  double dtb10[3][6] = {
    1.0,   1.0,   1.0,   1.0,   1.0,   1.0,
    4.0,   1.0,  -1.0,   3.0,   1.0,  -1.0,
    1.0,   2.0,   1.0,   2.0,   1.0,   1.0};
 
  float sta10[2][6] = {
    0.0,   2.0,   2.0,   0.0,   2.0,   2.0,
    2.0,  -1.0,  -1.0,   1.5,  -1.0,  -1.0};
 
  float stb10[3][6] = {
    1.0,   1.0,   1.0,   1.0,   1.0,   1.0,
    4.0,   1.0,  -1.0,   3.0,   1.0,  -1.0,
    1.0,   2.0,   1.0,   2.0,   1.0,   1.0};
 
  dcmplx zta10[3][4] = {
      {{ 1.00, 0.00}, { 1.00,-1.00}, { 1.00,-2.00}, {-2.00, 1.00}},
      {{ 2.00, 2.00}, {-2.00, 3.00}, {-3.00,-1.00}, {-3.00, 0.00}},
      {{-3.00, 1.00}, { 4.00,-3.00}, {-4.00,-2.00}, { 4.00,-1.00}}
  };
 
  dcmplx ztb10[4][3] = {
      {{ 1.00, 0.00}, {-1.00, 1.00}, { 2.00, 1.00}},
      {{ 0.00, 0.00}, { 0.00, 1.00}, { 1.00,-1.00}},
      {{ 1.00, 2.00}, { 0.00, 0.00}, { 1.00, 1.00}},
      {{ 0.00, 0.00}, {-1.00,-1.00}, { 0.00, 0.00}}
  };
 
  cmplx cta10[3][4] = {
      {{ 1.00, 0.00}, { 1.00,-1.00}, { 1.00,-2.00}, {-2.00, 1.00}},
      {{ 2.00, 2.00}, {-2.00, 3.00}, {-3.00,-1.00}, {-3.00, 0.00}},
      {{-3.00, 1.00}, { 4.00,-3.00}, {-4.00,-2.00}, { 4.00,-1.00}}
  };
 
  cmplx ctb10[4][3] = {
      {{ 1.00, 0.00}, {-1.00, 1.00}, { 2.00, 1.00}},
      {{ 0.00, 0.00}, { 0.00, 1.00}, { 1.00,-1.00}},
      {{ 1.00, 2.00}, { 0.00, 0.00}, { 1.00, 1.00}},
      {{ 0.00, 0.00}, {-1.00,-1.00}, { 0.00, 0.00}}
  };
 
  float  spoa40n1[9][9] = {1.,1.,1.,1.,1.,1.,1.,1.,1.,
                       1.,2.,2.,2.,2.,2.,2.,2.,2.,
                       1.,2.,3.,3.,3.,3.,3.,3.,3.,
                       1.,2.,3.,4.,4.,4.,4.,4.,4.,
                       1.,2.,3.,4.,5.,5.,5.,5.,5.,
                       1.,2.,3.,4.,5.,6.,6.,6.,6.,
                       1.,2.,3.,4.,5.,6.,7.,7.,7.,
                       1.,2.,3.,4.,5.,6.,7.,8.,8.,
                       1.,2.,3.,4.,5.,6.,7.,8.,9.};
  double  dpoa40n1[9][9] = {1.,1.,1.,1.,1.,1.,1.,1.,1.,
                       1.,2.,2.,2.,2.,2.,2.,2.,2.,
                       1.,2.,3.,3.,3.,3.,3.,3.,3.,
                       1.,2.,3.,4.,4.,4.,4.,4.,4.,
                       1.,2.,3.,4.,5.,5.,5.,5.,5.,
                       1.,2.,3.,4.,5.,6.,6.,6.,6.,
                       1.,2.,3.,4.,5.,6.,7.,7.,7.,
                       1.,2.,3.,4.,5.,6.,7.,8.,8.,
                       1.,2.,3.,4.,5.,6.,7.,8.,9.};
  cmplx  cpoa40n1[3][3] = {{{25.,0.}, {-5.,5.}, {10.,-5.}},
                       {{-5.,-5.}, {51.,0.},{ 4.,6.}},
                       {{10.,5.}, {4.,-6.},{ 71., 0.}}};
  dcmplx zpoa40n1[3][3] = {{{25.,0.}, {-5.,5.}, {10.,-5.}},
                       {{-5.,-5.}, {51.,0.},{ 4.,6.}},
                       {{10.,5.}, {4.,-6.},{ 71., 0.}}};
  float  spob40n1[2][9] = {9.,17.,24.,30.,35.,39.,42.,44.,45.,
                45.,89.,131.,170.,205.,235.,259.,276.,285.};
  double dpob40n1[2][9] = {9.,17.,24.,30.,35.,39.,42.,44.,45.,
                45.,89.,131.,170.,205.,235.,259.,276.,285.};
  cmplx  cpob40n1[2][3] ={{{60.,-55.},{ 34.,58.}, {13.,-152.}},
                       {{70.,10.},{ -51.,110.}, {75.,63.}}};
  dcmplx zpob40n1[2][3] ={{{60.,-55.},{ 34.,58.}, {13.,-152.}},
                       {{70.,10.},{ -51.,110.}, {75.,63.}}};
 
 
  float  sta40[9][9] =  { 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2, 2.4, 2.6,
                        1.2, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2, 2.4,
                        1.4, 1.2, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2,
                        1.6, 1.4, 1.2, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0,
                        1.8, 1.6, 1.4, 1.2, 1.0, 1.2, 1.4, 1.6, 1.8,
                        2.0, 1.8, 1.6, 1.4, 1.2, 1.0, 1.2, 1.4, 1.6,
                        2.2, 2.0, 1.8, 1.6, 1.4, 1.2, 1.0, 1.2, 1.4,
                        2.4, 2.2, 2.0, 1.8, 1.6, 1.4, 1.2, 1.0, 1.2,
                        2.6, 2.4, 2.2, 2.0, 1.8, 1.6, 1.4, 1.2, 1.0 };
  double dta40[9][9] =  { 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2, 2.4, 2.6,
                        1.2, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2, 2.4,
                        1.4, 1.2, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2,
                        1.6, 1.4, 1.2, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0,
                        1.8, 1.6, 1.4, 1.2, 1.0, 1.2, 1.4, 1.6, 1.8,
                        2.0, 1.8, 1.6, 1.4, 1.2, 1.0, 1.2, 1.4, 1.6,
                        2.2, 2.0, 1.8, 1.6, 1.4, 1.2, 1.0, 1.2, 1.4,
                        2.4, 2.2, 2.0, 1.8, 1.6, 1.4, 1.2, 1.0, 1.2,
                        2.6, 2.4, 2.2, 2.0, 1.8, 1.6, 1.4, 1.2, 1.0 };
  cmplx  cta40[5][5] = {
{{  .500,  .866},{-1.092, -.949},{  .496,-1.555},{ 1.655, -.640},{ 1.757,  .709}},
{{-1.092, -.949},{  .500,  .866},{-1.092, -.949},{  .496,-1.555},{ 1.655, -.640}},
{{  .496,-1.555},{-1.092, -.949},{  .500,  .866},{-1.092, -.949},{  .496,-1.555}},
{{ 1.655, -.640},{  .496,-1.555},{-1.092, -.949},{  .500,  .866},{-1.092, -.949}},
{{ 1.757,  .709},{ 1.655, -.640},{  .496,-1.555},{-1.092, -.949},{  .500,  .866}}
  };
 
  cmplx  ctares40[5][5] = {
{{1.757,.709},{.416,.325},{-.722,-.249},{-.065,-.859},{.684,-.641}},
{{1.655, -.640},{-1.989,-1.221},{-.860,.118},{.062,-.255},{.095,-.030}},
{{.496,-1.555},{-.216,-1.070},{-.659,-2.844},{-.568,.626},{-.146,-.169}},
{{-1.092,-.949},{1.801,.110},{1.505,-2.631},{-1.279,-3.934},{-.342,.484}},
{{.500,.866},{ 1.830,.187},{ 3.397,.054},{ 1.586,-2.710},{-2.451,-2.301}}
 
};
 
  dcmplx zta40[9][9] = {
      {{2.0,1.0},{2.4, 1.0},{2.8, 1.0},{3.2,1.0},{3.6,1.0},
       {4.0,1.0},{4.4, 1.0},{4.8, 1.0}, {5.2, 1.0}},  /* 1st  col end */
      {{2.4,-1.0},{2.0, 1.0}, {2.4, 1.0},{2.8,1.0},{3.2,1.0},
       {3.6,1.0},{4.0, 1.0},{4.4, 1.0}, {4.8, 1.0}},  /* 2nd  col end */
      {{2.8,-1.0},{2.4,-1.0},{2.0, 1.0},{2.4,1.0},{2.8,1.0},
       {3.2,1.0},{3.6, 1.0}, {4.0, 1.0}, {4.4, 1.0}},  /* 3rd col end */
      {{3.2,-1.0},{2.8,-1.0},{2.4,-1.0},{2.0,1.0},{2.4,1.0},
       {2.8, 1.0},{3.2, 1.0}, {3.6, 1.0}, {4.0, 1.0}}, /* 4th col end */
      {{3.6,-1.0}, {3.2,-1.0}, {2.8,-1.0},{2.4,-1.0},{2.0,1.0},
       {2.4, 1.0},{2.8, 1.0}, {3.2, 1.0}, {3.6, 1.0}}, /* 5th col end */
      {{4.0,-1.0}, {3.6,-1.0}, {3.2,-1.0},{2.8,-1.0},{2.4,-1.0},
       {2.0, 1.0},{2.4, 1.0}, {2.8, 1.0}, {3.2, 1.0}}, /* 6th col end */
      {{4.4,-1.0}, {4.0,-1.0}, {3.6,-1.0},{3.2,-1.0},{2.8,-1.0},
       {2.4, -1.0},{2.0, 1.0}, {2.4, 1.0}, {2.8, 1.0}}, /* 7th col end*/
      {{4.8,-1.0}, {4.4,-1.0}, {4.0,-1.0},{3.6,-1.0},{3.2,-1.0},
       {2.8, -1.0},{2.4,-1.0}, {2.0, 1.0}, {2.4, 1.0}}, /* 8th col end*/
      {{5.2,-1.0}, {4.8,-1.0}, {4.4,-1.0},{4.0,-1.0},{3.6,-1.0},
       {3.2, -1.0},{2.8,-1.0}, {2.4,-1.0}, {2.0, 1.0}}};/* 9th col end*/
 
  float  stb40[5][9] = {
   93.0,  84.4,  76.6,  70.0,  65.0,  62.0,  61.4,  63.6,  69.0,
  186.0, 168.8, 153.2, 140.0, 130.0, 124.0, 122.8, 127.2, 138.0,
  279.0, 253.2, 229.8, 210.0, 195.0, 186.0, 184.2, 190.8, 207.0,
  372.0, 337.6, 306.4, 280.0, 260.0, 248.0, 245.6, 254.4, 276.0,
  465.0, 422.0, 383.0, 350.0, 325.0, 310.0, 307.0, 318.0, 345.0 };
  double  dtb40[5][9] = {
   93.0,  84.4,  76.6,  70.0,  65.0,  62.0,  61.4,  63.6,  69.0,
  186.0, 168.8, 153.2, 140.0, 130.0, 124.0, 122.8, 127.2, 138.0,
  279.0, 253.2, 229.8, 210.0, 195.0, 186.0, 184.2, 190.8, 207.0,
  372.0, 337.6, 306.4, 280.0, 260.0, 248.0, 245.6, 254.4, 276.0,
  465.0, 422.0, 383.0, 350.0, 325.0, 310.0, 307.0, 318.0, 345.0 };
  cmplx ctb40[5][2] = {
{{16.776, -1.397},{10.118,-11.021}},{{ 2.065,-13.123},{ -.863, -7.415}},
{{ 6.255, -1.389},{18.345,  1.919}},{{13.346,-10.555},{ 6.208,-13.816}},
{{ 2.365, -6.949},{ 7.824,  1.926}}
};
  cmplx ctbres40[5][2] = {
{{1.,1.},{2.,1.}},{{3.,1.},{4.,1.}},{{5.,1.},{1.,2.}},
{{2.,2.},{3.,2.}},{{4.,2.},{5.,2.}}
};
  dcmplx ztb40[5][9] = {
      {{193.0,-10.60}, {173.8,-9.40}, {156.2, -5.40},{141.0,1.4},
       {129.0,11.00},{121.0, 23.40},{117.8,38.6},{120.2,56.6},
       {129.0,77.4}},{{200.0,21.80},{178.8,20.20}, {159.2, 22.20 },
       {142.0, 27.80}, {128.0,37.00}, {118.0,49.80}, {112.8,66.2},
       {113.2,86.20},{ 120.0,109.80}},{{207.0,54.2},{183.8,49.8},
       {162.2,49.8},{143.0, 54.20}, {127.0,63.00}, {115.0, 76.20},
       {107.8, 93.80}, {106.2,115.8}, {111.0,142.20}},{{214.0,86.6},
       {188.8,79.40}, {165.2, 77.40},{144.0,80.6},{126.0,89.0},
       {112.0,102.60},{102.8,121.40}, { 99.2,145.4}, {102.0,174.60}},
      {{221.0,119.00}, {193.8,109.0}, {168.2,105.00},{145.0,107.0},
       {125.0,115.0}, {109.0,129.00},{97.8,149.0},{92.2,175.0},
       {93.0,207.00}}};
 
/***********************************************
* ADDED FOR D/S/Z/CPBTRF/S, D/S/Z/CPBSV  10/12 *
***********************************************/
 
   double da113[9][4] = { 1., 1., 1., 1., 2., 2., 2., 1.,
                          3., 3., 2., 1., 4., 3., 2., 1.,
                          4., 3., 2., 1., 4., 3., 2., 1.,
                          4., 3., 2., 99., 4., 3., 99., 99.,
                          4., 99., 99., 99.};
   double db113[3][9] = { 4., 8., 12., 16., 16., 16., 15., 13., 10.,
                          0., 0., 0., 0., 0., 0., 1., 1., 2.,
                          1., 1., 0., 1., 0., -1., 0., -2., -3.};
   double da114[9][4] = { 1., 1., 1., 1., 2., 2., 2., 1.,
                          3., 3., 2., 1., 4., 3., 2., 1.,
                          4., 3., 2., 1., 4., 3., 2., 1.,
                          4., 3., 2., 99., 4., 3., 99., 99.,
                          4., 99., 99., 99.};
   double db114[3][9] = { 4., 8., 12., 16., 16., 16., 15., 13., 10.,
                          0., 0., 0., 0., 0., 0., 1., 1., 2.,
                          1., 1., 0., 1., 0., -1., 0., -2., -3.};
   float sa113[9][4] = { 1., 1., 1., 1., 2., 2., 2., 1.,
                         3., 3., 2., 1., 4., 3., 2., 1.,
                         4., 3., 2., 1., 4., 3., 2., 1.,
                         4., 3., 2., 99., 4., 3., 99., 99.,
                         4., 99., 99., 99.};
   float sb113[3][9] = { 4., 8., 12., 16., 16., 16., 15., 13., 10.,
                         0., 0., 0., 0., 0., 0., 1., 1., 2.,
                         1., 1., 0., 1., 0., -1., 0., -2., -3.};
   float sa114[9][4] = { 1., 1., 1., 1., 2., 2., 2., 1.,
                         3., 3., 2., 1., 4., 3., 2., 1.,
                         4., 3., 2., 1., 4., 3., 2., 1.,
                         4., 3., 2., 99., 4., 3., 99., 99.,
                         4., 99., 99., 99.};
   float sb114[3][9] = { 4., 8., 12., 16., 16., 16., 15., 13., 10.,
                         0., 0., 0., 0., 0., 0., 1., 1., 2.,
                         1., 1., 0., 1., 0., -1., 0., -2., -3.};
   dcmplx za113[6][4] = {
                         { {1., 99.}, {1., 1.}, {1., 1.}, {1., 1.} },
                         { {3., 99.}, {3., 1.}, {3., 1.}, {1., 1.} },
                         { {5., 99.}, {5., 1.}, {3., 1.}, {1., 1.} },
                         { {7., 99.}, {5., 1.}, {3., 1.}, {99., 99.} },
                         { {7., 99.}, {5., 1.}, {99., 99.}, {99., 99.} },
                         { {7., 99.}, {99., 99.}, {99., 99.}, {99., 99.} } };
   dcmplx zb113[3][6] = {
                         { {1., 1.}, {1., 1.}, {1., 1.},
                           {1., 1.}, {1., 1.}, {1., 1.} },
                         { {1., -7.}, {9., -13.}, {17., -19.},
                           {25., -23.}, {23., -19.}, {19., -13.} },
                         { {5., -5.}, {18., -4.}, {27., 0.},
                           {35., 2.}, {28., 5.}, {18., -1.} } };
   dcmplx za114[6][4] = {
                         { {1., 99.}, {1., 1.}, {1., 1.}, {1., 1.} },
                         { {3., 99.}, {3., 1.}, {3., 1.}, {1., 1.} },
                         { {5., 99.}, {5., 1.}, {3., 1.}, {1., 1.} },
                         { {7., 99.}, {5., 1.}, {3., 1.}, {99., 99.} },
                         { {7., 99.}, {5., 1.}, {99., 99.}, {99., 99.} },
                         { {7., 99.}, {99., 99.}, {99., 99.}, {99., 99.} } };
   dcmplx zb114[3][6] = {
                         { {1., 1.}, {1., 1.}, {1., 1.},
                           {1., 1.}, {1., 1.}, {1., 1.} },
                         { {1., -7.}, {9., -13.}, {17., -19.},
                           {25., -23.}, {23., -19.}, {19., -13.} },
                         { {5., -5.}, {18., -4.}, {27., 0.},
                           {35., 2.}, {28., 5.}, {18., -1.} } };
   cmplx ca113[6][4] = {
                        { {1., 99.}, {1., 1.}, {1., 1.}, {1., 1.} },
                        { {3., 99.}, {3., 1.}, {3., 1.}, {1., 1.} },
                        { {5., 99.}, {5., 1.}, {3., 1.}, {1., 1.} },
                        { {7., 99.}, {5., 1.}, {3., 1.}, {99., 99.} },
                        { {7., 99.}, {5., 1.}, {99., 99.}, {99., 99.} },
                        { {7., 99.}, {99., 99.}, {99., 99.}, {99., 99.} } };
   cmplx cb113[3][6] = {
                        { {1., 1.}, {1., 1.}, {1., 1.},
                          {1., 1.}, {1., 1.}, {1., 1.} },
                        { {1., -7.}, {9., -13.}, {17., -19.},
                          {25., -23.}, {23., -19.}, {19., -13.} },
                        { {5., -5.}, {18., -4.}, {27., 0.},
                          {35., 2.}, {28., 5.}, {18., -1.} } };
   cmplx ca114[6][4] = {
                        { {1., 99.}, {1., 1.}, {1., 1.}, {1., 1.} },
                        { {3., 99.}, {3., 1.}, {3., 1.}, {1., 1.} },
                        { {5., 99.}, {5., 1.}, {3., 1.}, {1., 1.} },
                        { {7., 99.}, {5., 1.}, {3., 1.}, {99., 99.} },
                        { {7., 99.}, {5., 1.}, {99., 99.}, {99., 99.} },
                        { {7., 99.}, {99., 99.}, {99., 99.}, {99., 99.} } };
   cmplx cb114[3][6] = {
                        { {1., 1.}, {1., 1.}, {1., 1.},
                          {1., 1.}, {1., 1.}, {1., 1.} },
                        { {1., -7.}, {9., -13.}, {17., -19.},
                          {25., -23.}, {23., -19.}, {19., -13.} },
                        { {5., -5.}, {18., -4.}, {27., 0.},
                          {35., 2.}, {28., 5.}, {18., -1.} } };
 
/***********************************************
* ADDED FOR D/S/Z/CPTTRF/S, D/S/Z/CPTSV  10/12 *
***********************************************/
 
   double dd115[4] = { 1., 2., 3., 1. };
   double de115[3] = { 1., 1., 1. };
   double db115[2][4] = { { 2., 4., 5., 2. },
                          { -2., -3., 0., 1. } };
   float sd115[4] = { 1., 2., 3., 1. };
   float se115[3] = { 1., 1., 1. };
   float sb115[2][4] = { { 2., 4., 5., 2. },
                         { -2., -3., 0., 1. } };
   double zd115[4] = { 1., 4., 7., 10. };
   dcmplx ze115[3] = { { 1., 1. }, { 2., 2. }, { 3., 3. } };
   dcmplx zb115[3][4] = { { { 2., -1. }, { 7., -1. }, { 12., -1. }, { 13., 3. } },
                          { { 3., 1. }, { 8., 6. }, { 13., 11. }, { 10., 16. } },
                          { { 1., -3. }, { 6., -8. }, { 11., -13. }, { 16., -10. } } };
   float cd115[4] = { 1., 4., 7., 10. };
   cmplx ce115[3] = { { 1., 1. }, { 2., 2. }, { 3., 3. } };
   cmplx cb115[3][4] = { { { 2., -1. }, { 7., -1. }, { 12., -1. }, { 13., 3. } },
                         { { 3., 1. }, { 8., 6. }, { 13., 11. }, { 10., 16. } },
                         { { 1., -3. }, { 6., -8. }, { 11., -13. }, { 16., -10. } } };
 
   double dd116[4] = { 1., 2., 3., 1. };
   double de116[3] = { 1., 1., 1. };
   double db116[2][4] = { { 2., 4., 5., 2. },
                          { -2., -3., 0., 1. } };
   float sd116[4] = { 1., 2., 3., 1. };
   float se116[3] = { 1., 1., 1. };
   float sb116[2][4] = { { 2., 4., 5., 2. },
                         { -2., -3., 0., 1. } };
   double zd116[4] = { 1., 4., 7., 10. };
   dcmplx ze116[3] = { { 1., 1. }, { 2., 2. }, { 3., 3. } };
   dcmplx zb116[3][4] = { { { 2., -1. }, { 7., -1. }, { 12., -1. }, { 13., 3. } },
                          { { 3., 1. }, { 8., 6. }, { 13., 11. }, { 10., 16. } },
                          { { 1., -3. }, { 6., -8. }, { 11., -13. }, { 16., -10. } } };
   float cd116[4] = { 1., 4., 7., 10. };
   cmplx ce116[3] = { { 1., 1. }, { 2., 2. }, { 3., 3. } };
   cmplx cb116[3][4] = { { { 2., -1. }, { 7., -1. }, { 12., -1. }, { 13., 3. } },
                         { { 3., 1. }, { 8., 6. }, { 13., 11. }, { 10., 16. } },
                         { { 1., -3. }, { 6., -8. }, { 11., -13. }, { 16., -10. } } };
 
/*************************************************************
* CALL STATEMENT FOR LINEAR ALGEBRAIC EQUATION SUBROUTINES : *
*************************************************************/
 
/***************************
*  DENSE LINEAR ALGEBRAIC  *
***************************/
/* make sure the _gef rtns are call before the _gesm rtns */
    sgef (sa37, 9, 9, ipvt37);
    smtchk(9, 9, sa37, sr37, 9, "sgef", pass);
    intchk(9, 1, ipvt37, ipvtr37, "sgef", pass);
    sgesm("N",sa37,9,9,ipvt37,sgeb27n,9,2);
    smtchk(9,2,sgeb27n,sgeres27n,9,"sgesm",pass);
    dgef (da37, 9, 9, ipvt37);
    dmtchk(9, 9, da37, dr37, 9, "dgef", pass);
    intchk(9, 1, ipvt37, ipvtr37, "dgef", pass);
    dgesm("N",da37,9,9,ipvt37,dgeb27n,9,2);
    dmtchk(9,2,dgeb27n,dgeres27n,9,"dgesm",pass);
    cgef (ca37, 4, 4, ci37);
    cmtchk(4, 4, ca37, car37, 4, "cgef", pass);
    intchk(4, 1, ci37, cir37, "cgef", pass);
    cgesm("N",ca37,4,4,ci37,cgeb27n,4,2);
    cmtchk(4,2,cgeb27n,cgeres27n,4,"cgesm",pass);
    zgef (za37, 4, 4, zi37);
    zmtchk(4, 4, za37, zar37, 4, "zgef", pass);
    intchk(4, 1, zi37, zir37, "zgef", pass);
    zgesm("N",za37,4,4,zi37,zgeb27n,4,2);
    zmtchk(4,2,zgeb27n,zgeres27n,4,"zgesm",pass);
    sges (sa37, 9, 9, ipvt37, sbx38, 0);
    svecck(9, 1, sbx38, sres38, "sges", pass);
    dges (da37, 9, 9, ipvt37, dbx38, 0);
    dvecck(9, 1, dbx38, dres38, "dges", pass);
    cges (ca37, 4, 4, ci37, cb37, 0);
    cvecck(4, 1, cb37, cbr37, "cges", pass);
    zges (za37, 4, 4, zi37, zb37, 0);
    zvecck(4, 1, zb37, zbr37, "zges", pass);
 
    sgetri (9, sa37, 9, ipvt37, saux1, 0, &info);
    if (info != 0)
     {
      printf( " INFO from SGETRI = %4d\n", info);
      *pass = *pass + 1;
     }
    else
    smtchk(9, 9, sa37, sres41, 9, "sgetri", pass);
 
    dgetri (9, da37, 9, ipvt37, daux1, 0, &info);
    if (info != 0)
     {
      printf( " INFO from DGETRI = %4d\n", info);
      *pass = *pass + 1;
     }
    else
    dmtchk(9, 9, da37, dres41, 9, "dgetri", pass);
 
    cgetrf(9,9,ca41,9,ipvt37,&info);
    cgetri (9, ca41, 9, ipvt37, caux1, 0, &info);
    if (info != 0)
     {
      printf( " INFO from CGETRI = %4d\n", info);
      *pass = *pass + 1;
     }
   else
    cmtchk(9, 9, ca41, cres41, 9, "cgetri", pass);
 
    zgetrf(9,9,za41,9,ipvt37,&info);
    zgetri(9,  za41,9,ipvt37, zaux2, 0, &info);
    if (info != 0)
     {
      printf( " INFO from ZGETRI = %4d\n", info);
      *pass = *pass + 1;
     }
  else
    zmtchk(9, 9, za41, zres41, 9, "zgetri", pass);
 
    sppf (sap39, 9, 0);
    svecck(54, 1, sap39, sres39, "sppf", pass);
    dppf (dap39, 9, 0);
    dvecck(54, 1, dap39, dres39, "dppf", pass);
    spps (sap39, 9, sbx40, 0);
    svecck(9, 1, sbx40, sres40, "spps", pass);
    dpps (dap39, 9, dbx40, 0);
    dvecck(9, 1, dbx40, dres40, "dpps", pass);
 
    snrm93 = slansp("1", "U", 9, sap93, swrk93);
    svecck(1, 1, &snrm93, &snrm93res, "slansp", pass);
 
    info = 0;
    spptrf("U", 9, sap93, &info);
    if (info != 0) {
        printf( " INFO from SPPTRF = %4d\n", info);
        *pass = *pass + 1;
    } else {
        smtchk(45,1,sap93,sapres93,9,"spptrf",pass);
 
        sppcon("U",9,sap93,snrm93,&srcnd93,swrk93,siwrk93,&info);
        svecck(1,1,&srcnd93,&srcnd93res,"sppcon",pass);
 
        spptrs("U", 9, 2, sap93, sbx93, 9, &info);
        if (info != 0) {
            printf( " INFO from SPPTRS = %4d\n", info);
            *pass = *pass + 1;
        }
        smtchk(9,2,sbx93,sbxres93,9,"spptrs",pass);
        spptri("U", 9, sap93, &info);
        if (info != 0) {
            printf( " INFO from SPPTRI = %4d\n", info);
            *pass = *pass + 1;
        } else {
            smtchk(45,1,sap93,sapres93inv,9,"spptri",pass);
        }
 
    }
 
    sppsv("U", 9, 2, sap932, sbx932, 9, &info);
    if (info != 0) {
        printf( " INFO from SPPSV = %4d\n", info);
        *pass = *pass + 1;
    } else {
        smtchk(45,1,sap932,sapres93,9,"sppsv",pass);
        smtchk(9,2,sbx932,sbxres93,9,"sppsv",pass);
    }
 
 
    dnrm93 = dlansp("1", "U", 9, dap93, dwrk93);
    dvecck(1, 1, &dnrm93, &dnrm93res, "dlansp", pass);
 
    info = 0;
    dpptrf("U", 9, dap93, &info);
    if (info != 0) {
        printf( " INFO from DPPTRF = %4d\n", info);
        *pass = *pass + 1;
    } else {
        dmtchk(45,1,dap93,dapres93,9,"dpptrf",pass);
 
        dppcon("U",9,dap93,dnrm93,&drcnd93,dwrk93,diwrk93,&info);
        dvecck(1,1,&drcnd93,&drcnd93res,"dppcon",pass);
 
        dpptrs("U", 9, 2, dap93, dbx93, 9, &info);
        if (info != 0) {
            printf( " INFO from DPPTRS = %4d\n", info);
            *pass = *pass + 1;
        }
        dmtchk(9,2,dbx93,dbxres93,9,"dpptrs",pass);
        dpptri("U", 9, dap93, &info);
        if (info != 0) {
            printf( " INFO from DPPTRI = %4d\n", info);
            *pass = *pass + 1;
        } else {
            smtchk(45,1,dap93,dapres93inv,9,"dpptri",pass);
        }
 
    }
 
    dppsv("U", 9, 2, dap932, dbx932, 9, &info);
    if (info != 0) {
        printf( " INFO from DPPSV = %4d\n", info);
        *pass = *pass + 1;
    } else {
        dmtchk(45,1,dap932,dapres93,9,"dppsv",pass);
        dmtchk(9,2,dbx932,dbxres93,9,"dppsv",pass);
    }
 
    cnrm93 = clanhp("1", "L", 3, cap93, rwrk93);
    svecck(1, 1, &cnrm93, &cnrm93res, "clanhp", pass);
 
    cpptrf("L", 3, cap93, &info);
    if (info != 0) {
        printf( " INFO from CPPTRF = %4d\n", info);
        *pass = *pass + 1;
    } else {
        cmtchk(6,1,cap93,capres93,6,"cpptrf",pass);
 
        cppcon("L",3,cap93,cnrm93,&crcnd93,cwrk93,crwrk93,&info);
        svecck(1,1,&crcnd93,&crcnd93res,"cppcon",pass);
 
        ccopy(6,cap93,1,cap93i,1);
        cpptri("L",3,cap93i,&info);
        if (info != 0) {
            printf( " INFO from CPPTRI = %4d\n", info);
            *pass = *pass + 1;
        }
        cmtchk(6,1,cap93i,capres93i,6,"cpptri",pass);
 
        cpptrs("L", 3, 2, cap93, cbx93, 3, &info);
        if (info != 0) {
            printf( " INFO from CPPTRS = %4d\n", info);
            *pass = *pass + 1;
        }
        cmtchk(3,2,cbx93,cbxres93,3,"cpptrs",pass);
    }
 
    cppsv("L", 3, 2, cap932, cbx932, 3, &info);
    if (info != 0) {
        printf( " INFO from CPPSV = %4d\n", info);
        *pass = *pass + 1;
    } else {
        cmtchk(6,1,cap932,capres93,6,"cppsv",pass);
        cmtchk(3,2,cbx932,cbxres93,3,"cppsv",pass);
    }
 
    znrm93 = zlanhp("1", "L", 3, zap93, rzwrk93);
    dvecck(1, 1, &znrm93, &znrm93res, "zlanhp", pass);
 
    zpptrf("L", 3, zap93, &info);
    if (info != 0) {
        printf( " INFO from ZPPTRF = %4d\n", info);
        *pass = *pass + 1;
    } else {
        zmtchk(6,1,zap93,zapres93,6,"zpptrf",pass);
 
        zppcon("L",3,zap93,znrm93,&zrcnd93,zwrk93,zrwrk93,&info);
        dvecck(1,1,&zrcnd93,&zrcnd93res,"zppcon",pass);
 
        zcopy(6,zap93,1,zap93i,1);
        zpptri("L",3,zap93i,&info);
        if (info != 0) {
            printf( " INFO from ZPPTRI = %4d\n", info);
            *pass = *pass + 1;
        }
        zmtchk(6,1,zap93i,zapres93i,6,"zpptri",pass);
 
        zpptrs("L", 3, 2, zap93, zbx93, 3, &info);
        if (info != 0) {
            printf( " INFO from ZPPTRS = %4d\n", info);
            *pass = *pass + 1;
        }
        zmtchk(3,2,zbx93,zbxres93,3,"zpptrs",pass);
    }
 
    zppsv("L", 3, 2, zap932, zbx932, 3, &info);
    if (info != 0) {
        printf( " INFO from ZPPSV = %4d\n", info);
        *pass = *pass + 1;
    } else {
        zmtchk(6,1,zap932,zapres93,6,"zppsv",pass);
        zmtchk(3,2,zbx932,zbxres93,3,"zppsv",pass);
    }
 
    sgeicd (sa41, 9, 9, 3, &srcn41[0], sdet41, saux1, 0);
    smtchk(9, 9, sa41, sres41, 9, "sgeicd", pass);
    svecck(2, 1, sdet41, srde41, "sgeicd", pass);
    svecck(1, 1, srcn41, srrc41, "sgeicd", pass);
    dgeicd (da41, 9, 9, 3, &drcn41[0], ddet41, daux1, 0);
    dmtchk(9, 9, da41, dres41, 9, "dgeicd", pass);
    dvecck(2, 1, ddet41, drde41, "dgeicd", pass);
    dvecck(1, 1, drcn41, drrc41, "dgeicd", pass);
    sgefcd (sa43, 9, 9, ipvt43, 3, &srcn43[0], sdet43, saux1, 0);
    smtchk(9, 9, sa43, sres43, 9, "sgefcd", pass);
    intchk(9, 1, ipvt43, irpv43, "sgefcd", pass);
    svecck(2, 1, sdet43, srde43, "sgefcd", pass);
    svecck(1, 1, srcn43, srrc43, "sgefcd", pass);
    dgefcd (da43, 9, 9, ipvt43, 3, &drcn43[0], ddet43, daux1, 0);
    dmtchk(9, 9, da43, dres43, 9, "dgefcd", pass);
    intchk(9, 1, ipvt43, irpv43, "dgefcd", pass);
    dvecck(2, 1, ddet43, drde43, "dgefcd", pass);
    dvecck(1, 1, drcn43, drrc43, "dgefcd", pass);
    sppfcd (sap44, 9, 3, &srcn44[0], sdet44, saux1, 0);
    svecck(54, 1, sap44, sres39, "sppfcd", pass);
    svecck(2, 1, sdet44, srde44, "sppfcd", pass);
    svecck(1, 1, srcn44, srrc44, "sppfcd", pass);
    dppfcd (dap44, 9, 3, &drcn44[0], ddet44, daux1, 0);
    dvecck(54, 1, dap44, dres39, "dppfcd", pass);
    dvecck(2, 1, ddet44, drde44, "dppfcd", pass);
    dvecck(1, 1, drcn44, drrc44, "dppfcd", pass);
    dppicd (da99, 9, 3, &dr99, dd99, daux1, 0);
    dvecck(1, 1, &dr99, &drz99, "dppicd", pass);
    dvecck(2, 1, dd99, ddrs99, "dppicd", pass);
    dvecck(45, 1, da99, daz99, "dppicd", pass);
    sppicd (sa99, 9, 3, &sr99, sd99, saux1, 0);
    svecck(1, 1, &sr99, &srz99, "sppicd", pass);
    svecck(2, 1, sd99, sdrs99, "sppicd", pass);
    svecck(45, 1, sa99, saz99, "sppicd", pass);
    strsm ("L", "U", "N", "N", 5, 3, 1., sa102, 7, sb102, 6);
    smtchk(5, 3, sb102, sbr102, 6, "strsm", pass);
    dtrsm ("L", "U", "N", "N", 5, 3, 1., da102, 7, db102, 6);
    dmtchk(5, 3, db102, dbr102, 6, "dtrsm", pass);
    CSET(c4,1.,0.);
    ctrsm("L","U","N","N",5,3,c4,cab38,7,cbb38,6);
    cmtchk(5,3,cbb38,creb38,6,"ctrsm",pass);
    CSET(c8,1.,0.);
    ztrsm("L","U","N","N",5,3,c8,zab38,7,zbb38,6);
    zmtchk(5,3,zbb38,zreb38,6,"ztrsm",pass);
    strsv("U","N","U",4,sab34,4,sxb34,1);
    svecck(4,1,sxb34,sreb34,"strsv",pass);
    dtrsv("U","N","U",4,dab34,4,dxb34,1);
    dvecck(4,1,dxb34,dreb34,"dtrsv",pass);
    ctrsv("U","T","N",4,cab34,4,cxb34,1);
    cvecck(4,1,cxb34,creb34,"ctrsv",pass);
    ztrsv("U","N","U",4,zab34,4,zxb34,1);
    zvecck(4,1,zxb34,zreb34,"ztrsv",pass);
    spof("L",spoa28n,9,9);
    smtchk(9,9,spoa28n,spores28n,9,"spof",pass);
 
    snrm211 = slansy("1","L",9,spoa28n1,9,swrk211);
    svecck(1,1,&snrm211,&snrm211res,"slansy",pass);
 
    spotrf("L",9,spoa28n1,9,&info);
 
    if (info != 0) {
        printf( " INFO from SPOTRF = %4d\n", info);
        *pass = *pass + 1;
    } else {
        smtchk(9,9,spoa28n1,spores28n,9,"spotrf",pass);
 
        spocon("L",9,spoa28n1,9,snrm211,&srcnd211,swrk211,siwrk211,&info);
        svecck(1,1,&srcnd211,&srcnd211res,"spocon",pass);
 
        spotrs("L",9,2,spoa28n1,9,spob29n1,9,&info);
        if (info != 0) {
            printf( " INFO from SPOTRS = %4d\n", info);
            *pass = *pass + 1;
        }
        smtchk(9,2,spob29n1,sgeres27n,9,"spotrs",pass);
    }
 
    sposv("L",9,2,spoa40n1,9,spob40n1,9,&info);
    if (info != 0) {
        printf( " INFO from SPOSV = %4d\n", info);
        *pass = *pass + 1;
    } else {
        smtchk(9,9,spoa40n1,spores28n,9,"sposv",pass);
        smtchk(9,2,spob40n1,sgeres27n,9,"sposv",pass);
    }
 
    dpof("L",dpoa28n,9,9);
    dmtchk(9,9,dpoa28n,dpores28n,9,"dpof",pass);
 
    dnrm211 = dlansy("1","L",9,dpoa28n1,9,dwrk211);
    dvecck(1,1,&dnrm211,&dnrm211res,"dlansy",pass);
 
    dpotrf("L",9,dpoa28n1,9,&info);
    if (info != 0) {
        printf( " INFO from DPOTRF = %4d\n", info);
        *pass = *pass + 1;
    } else {
 
        dmtchk(9,9,dpoa28n1,dpores28n,9,"dpotrf",pass);
 
        dpocon("L",9,dpoa28n1,9,dnrm211,&drcnd211,dwrk211,diwrk211,&info);
        dvecck(1,1,&drcnd211,&drcnd211res,"dpocon",pass);
 
        dpotrs("L",9,2,dpoa28n1,9,dpob29n1,9,&info);
        if (info != 0) {
            printf( " INFO from DPOTRS = %4d\n", info);
            *pass = *pass + 1;
        }
        dmtchk(9,2,dpob29n1,dgeres27n,9,"dpotrs",pass);
    }
 
    dposv("L",9,2,dpoa40n1,9,dpob40n1,9,&info);
    if (info != 0) {
        printf( " INFO from DPOSV = %4d\n", info);
        *pass = *pass + 1;
    } else {
        dmtchk(9,9,dpoa40n1,dpores28n,9,"dposv",pass);
        dmtchk(9,2,dpob40n1,dgeres27n,9,"dposv",pass);
    }
 
    cpof("L",cpoa28n,3,3);
    cmtchk(3,3,cpoa28n,cpores28n,3,"cpof",pass);
 
    cnrm211 = clanhe("1","L",3,cpoa28n1,3,crwrk211);
    svecck(1,1,&cnrm211,&cnrm211res,"clanhe",pass);
 
    cpotrf("L",3,cpoa28n1,3,&info);
    if (info != 0) {
        printf( " INFO from CPOTRF = %4d\n", info);
        *pass = *pass + 1;
    } else {
 
        cmtchk(3,3,cpoa28n1,cpores28n,3,"cpotrf",pass);
 
        cpocon("L",3,cpoa28n1,3,cnrm211,&crcnd211,cwrk211,crwrk211,&info);
        svecck(1,1,&crcnd211,&crcnd211res,"cpocon",pass);
 
        cpotrs("L",3,2,cpoa28n1,3,cpob29n1,3,&info);
        if (info != 0) {
            printf( " INFO from CPOTRS = %4d\n", info);
            *pass = *pass + 1;
        }
        cmtchk(3,2,cpob29n1,cposres29n,3,"cpotrs",pass);
     }
 
    cposv("L",3,2,cpoa40n1,3,cpob40n1,3,&info);
    if (info != 0) {
        printf( " INFO from CPOSV = %4d\n", info);
        *pass = *pass + 1;
    } else {
        cmtchk(3,3,cpoa40n1,cpores28n, 3,"cposv",pass);
        cmtchk(3,2,cpob40n1,cposres29n,3,"cposv",pass);
    }
 
    cpotrf("L",3,cpoa41n1,3,&info);
    cpotri("L",3,cpoa41n1,3,&info);
    if(info != 0) {
      printf( " INFO from CPOTRI = %4d\n", info);
      *pass = *pass + 1;
    } else
        cmtchk(3,3,cpoa41n1,cpores41,3,"cpotri",pass);
 
    zpof("L",zpoa28n,3,3);
    zmtchk(3,3,zpoa28n,zpores28n,3,"zpof",pass);
 
    znrm211 = zlanhe("1","L",3,zpoa28n1,3,zrwrk211);
    dvecck(1,1,&znrm211,&znrm211res,"zlanhe",pass);
 
    zpotrf("L",3,zpoa28n1,3,&info);
    if (info != 0) {
        printf( " INFO from ZPOTRF = %4d\n", info);
        *pass = *pass + 1;
    } else {
        zmtchk(3,3,zpoa28n1,zpores28n,3,"zpotrf",pass);
 
        zpocon("L",3,zpoa28n1,3,znrm211,&zrcnd211,zwrk211,zrwrk211,&info);
        dvecck(1,1,&zrcnd211,&zrcnd211res,"zpocon",pass);
 
        zpotrs("L",3,2,zpoa28n1,3,zpob29n1,3,&info);
        if (info != 0) {
            printf( " INFO from ZPOTRS = %4d\n", info);
            *pass = *pass + 1;
        }
        zmtchk(3,2,zpob29n1,zposres29n,3,"zpotrs",pass);
    }
 
    zposv("L",3,2,zpoa40n1,3,zpob40n1,3,&info);
    if (info != 0) {
        printf( " INFO from ZPOSV = %4d\n", info);
        *pass = *pass + 1;
    } else {
        zmtchk(3,3,zpoa40n1,zpores28n ,3,"zposv",pass);
        zmtchk(3,2,zpob40n1,zposres29n,3,"zposv",pass);
    }
 
    zpotrf("L",3,zpoa41n1,3,&info);
    zpotri("L",3,zpoa41n1,3,&info);
    if(info != 0) {
      printf( " INFO from ZPOTRI = %4d\n", info);
      *pass = *pass + 1;
    } else
        zmtchk(3,3,zpoa41n1,zpores41,3,"ZPOTRI",pass);
    /* _posm use the output of _pof */
    sposm("L",spoa28n,9,9,sposb29n,9,2);
    smtchk(9,2,sposb29n,sgeres27n,9,"sposm",pass);
    dposm("L",dpoa28n,9,9,dposb29n,9,2);
    dmtchk(9,2,dposb29n,dgeres27n,9,"dposm",pass);
    cposm("L",cpoa28n,3,3,cposb29n,3,2);
    cmtchk(3,2,cposb29n,cposres29n,3,"cposm",pass);
    zposm("L",zpoa28n,3,3,zposb29n,3,2);
    zmtchk(3,2,zposb29n,zposres29n,3,"zposm",pass);
 
    /* input and results for pofcd are smiliar to pof */
    spofcd("L",spfa30n,9,9,3,spcrc30n,spcde30n,saux1n,0);
    smtchk(9,9,spfa30n,spores28n,9,"spofcd",pass);
    svecck(2,1,spcde30n,spcderes30n,"spofcd",pass);
    svecck(1,1,spcrc30n,spcrcres30n,"spofcd",pass);
    dpofcd("L",dpfa30n,9,9,3,dpcrc30n,dpcde30n,daux1n,0);
    dmtchk(9,9,dpfa30n,dpores28n,9,"dpofcd",pass);
    dvecck(2,1,dpcde30n,dpcderes30n,"dpofcd",pass);
    dvecck(1,1,dpcrc30n,dpcrcres30n,"dpofcd",pass);
 
    spoicd("U",spia31n,9,9,3,spirc31n,spide31n,saux1n,0);
    smtchk(9,9,spia31n,spires31n,9,"spoicd",pass);
    svecck(2,1,spide31n,spideres31n,"spoicd",pass);
    svecck(1,1,spirc31n,spircres31n,"spoicd",pass);
    dpoicd("U",dpia31n,9,9,3,dpirc31n,dpide31n,daux1n,0);
    dmtchk(9,9,dpia31n,dpires31n,9,"dpoicd",pass);
    dvecck(2,1,dpide31n,dpideres31n,"dpoicd",pass);
    dvecck(1,1,dpirc31n,dpircres31n,"dpoicd",pass);
    stpi("U","N",stpia31n,5);
    svecck(15,1,stpia31n,stpiares31n,"stpi",pass);
    dtpi("U","N",dtpia31n,5);
    dvecck(15,1,dtpia31n,dtpiares31n,"stpi",pass);
    spof("U",spia31n1,9,9);
    spotri("U",9,spia31n1,9,&info);
    if (info != 0)
     {
      printf( " INFO from SPOTRI = %4d\n", info);
      *pass = *pass + 1;
     }
  else
    smtchk(9,9,spia31n1,spires31n,9,"spotri",pass);
    dpof("U",dpia31n1,9,9);
    dpotri("U",9,dpia31n1,9,&info);
    if (info != 0)
     {
      printf( " INFO from DPOTRI = %4d\n", info);
      *pass = *pass + 1;
     }
  else
    dmtchk(9,9,dpia31n1,dpires31n,9,"dpotri",pass);
 
    stri("U","N",stria32n,5,5);
    smtchk(5,5,stria32n,strires32n,5,"stri",pass);
    strtri("U","N",5,stria32n1,5,&info);
    if (info != 0)
     {
      printf( " INFO from STRTRI = %4d\n", info);
      *pass = *pass + 1;
     }
  else
    smtchk(5,5,stria32n1,strires32n,5,"strtri",pass);
    dtri("U","N",dtria32n,5,5);
    dmtchk(5,5,dtria32n,dtrires32n,5,"dtri",pass);
    dtrtri("U","N",5,dtria32n1,5,&info);
    if (info != 0)
     {
      printf( " INFO from DTRTRI = %4d\n", info);
      *pass = *pass + 1;
     }
  else
    dmtchk(5,5,dtria32n1,dtrires32n,5,"dtrtri",pass);
 
    ctrtri("U","N",5,ctria32n,5,&info);
    if (info != 0)
     {
      printf( " INFO from CTRTRI = %4d\n", info);
      *pass = *pass + 1;
     }
  else
    cmtchk(5,5,ctria32n,ctrires32n,5,"ctrtri",pass);
    ztrtri("U","N",5,ztria32n,5,&info);
    if (info != 0)
     {
      printf( " INFO from ZTRTRI = %4d\n", info);
      *pass = *pass + 1;
     }
  else
    zmtchk(5,5,ztria32n,ztrires32n,5,"ztrtri",pass);
    stptri("U","N",5,stpia31n1,&info);
    if (info != 0)
     {
      printf( " INFO from STPTRI = %4d\n", info);
      *pass = *pass + 1;
     }
  else
    svecck(15,1,stpia31n1,stpiares31n,"stptri",pass);
    dtptri("U","N",5,dtpia31n1,&info);
    if (info != 0)
     {
      printf( " INFO from DTPTRI = %4d\n", info);
      *pass = *pass + 1;
     }
  else
    dvecck(15,1,dtpia31n1,dtpiares31n,"dtptri",pass);
    ctptri("U","N",5,ctpia31n1,&info);
    if (info != 0)
     {
      printf( " INFO from CTPTRI = %4d\n", info);
      *pass = *pass + 1;
     }
  else
    cvecck(15,1,ctpia31n1,ctpiares31n,"ctptri",pass);
    ztptri("U","N",5,ztpia31n1,&info);
    if (info != 0)
     {
      printf( " INFO from ZTPTRI = %4d\n", info);
      *pass = *pass + 1;
     }
  else
    zvecck(15,1,ztpia31n1,ztpiares31n,"ztptri",pass);
 
    stpsv("L","N","U",4,stpva35n,stpvx35n,1);
    svecck(4,1,stpvx35n,stpvres35n,"stpsv",pass);
    dtpsv("L","N","U",4,dtpva35n,dtpvx35n,1);
    dvecck(4,1,dtpvx35n,dtpvres35n,"dtpsv",pass);
    ctpsv("L","N","U",4,ctpva35n,ctpvx35n,1);
    cvecck(4,1,ctpvx35n,ctpvres35n,"ctpsv",pass);
    ztpsv("L","N","U",4,ztpva35n,ztpvx35n,1);
    zvecck(4,1,ztpvx35n,ztpvres35n,"ztpsv",pass);
 
    snrm411 = slange("1",9,9,&sa411[0][0],9,swrk411);
    svecck(1,1,&snrm411,&snrm411res,"slange",pass);
 
    sgetrf(9,9,sa411,9,ipvt45,&info);
 
    sgecon("1",9,&sa411[0][0],9,snrm411,&srcnd411,swrk411,siwrk411,&info);
    svecck(1,1,&srcnd411,&srcnd411res,"sgecon",pass);
 
    sgetrf (9, 9, sta7, 9,  ipvt37, &info);
    if (info != 0) {
        printf( " INFO from SGETRF = %4d\n", info);
        *pass = *pass + 1;
    } else {
        smtchk(9, 9, sta7, sresa7, 9, "sgetrf", pass);
        intchk(9, 1, ipvt37, ipvtr7, "sgetrf", pass);
        sgetrs("N",9,5,sta7,9,ipvt37,stb7,9,&info);
        if (info != 0) {
            printf( " INFO from SGETRS = %4d\n", info);
            *pass = *pass + 1;
        }
        smtchk(9,5,stb7,sresb7,9,"sgetrs",pass);
    }
 
    sgesv(9,5,sta40,9,ipvt37,stb40,9,&info);
    if (info != 0) {
        printf( " INFO from SGESV = %4d\n", info);
        *pass = *pass + 1;
    } else {
        smtchk(9,9,sta40,sresa7,9,"sgesv",pass);
        intchk(9,1,ipvt37,ipvtr7, "sgesv",pass);
        smtchk(9,5,stb40,sresb7,9,"sgesv",pass);
    }
 
    dnrm411 = dlange("1",9,9,&da411[0][0],9,dwrk411);
    dvecck(1,1,&dnrm411,&dnrm411res,"dlange",pass);
 
    dgetrf(9,9,da411,9,ipvt45,&info);
 
    dgecon("1",9,&da411[0][0],9,dnrm411,&drcnd411,dwrk411,diwrk411,&info);
    dvecck(1,1,&drcnd411,&drcnd411res,"dgecon",pass);
 
    dgetrf (9, 9, dta7, 9,  ipvt37, &info);
    if (info != 0) {
        printf( " INFO from DGETRF = %4d\n", info);
        *pass = *pass + 1;
    } else {
        dmtchk(9, 9, dta7, dresa7, 9, "dgetrf", pass);
        intchk(9, 1, ipvt37, ipvtr7, "dgetrf", pass);
        dgetrs("N",9,5,dta7,9,ipvt37,dtb7,9,&info);
        if (info != 0) {
            printf( " INFO from DGETRS = %4d\n", info);
            *pass = *pass + 1;
        }
        dmtchk(9,5,dtb7,dresb7,9,"dgetrs",pass);
    }
 
    dgesv(9,5,dta40,9,ipvt37,dtb40,9,&info);
    if (info != 0) {
        printf( " INFO from DGESV = %4d\n", info);
        *pass = *pass + 1;
    } else {
        dmtchk(9,9,dta40,dresa7,9,"dgesv",pass);
        intchk(9,1,ipvt37,ipvtr7, "dgesv",pass);
        dmtchk(9,5,dtb40,dresb7,9,"dgesv",pass);
    }
 
    cnrm411 = clange("1",4,4,&ca411[0][0],4,crwrk411);
    svecck(1,1,&cnrm411,&cnrm411res,"clange",pass);
 
    cgetrf(4,4,ca411,4,ipvt45,&info);
 
    cgecon("1",4,&ca411[0][0],4,cnrm411,&crcnd411,cwrk411,crwrk411,&info);
    svecck(1,1,&crcnd411,&crcnd411res,"cgecon",pass);
 
    cgetrf (9, 9, cta7, 9,  ipvt37, &info);
    if (info != 0) {
        printf( " INFO from CGETRF = %4d\n", info);
        *pass = *pass + 1;
    } else {
        cmtchk(9, 9, cta7, cresa7, 9, "cgetrf", pass);
        intchk(9, 1, ipvt37, ipvtr7, "cgetrf", pass);
        cgetrs("N",9,5,cta7,9,ipvt37,ctb7,9,&info);
        if (info != 0) {
            printf( " INFO from CGETRS = %4d\n", info);
            *pass = *pass + 1;
        }
        cmtchk(9,5,ctb7,cresb7,9,"cgetrs",pass);
    }
 
    cgesv(5,2,cta40,5,ipvt40,ctb40,5,&info);
    if (info != 0) {
        printf( " INFO from CGESV = %4d\n", info);
        *pass = *pass + 1;
    } else {
        cmtchk(5,5,cta40,ctares40,5,"cgesv",pass);
        intchk(5,1,ipvt40,ipvtr40,  "cgesv",pass);
        cmtchk(5,2,ctb40,ctbres40,5,"cgesv",pass);
    }
 
    znrm411 = zlange("1",4,4,&za411[0][0],4,zrwrk411);
    dvecck(1,1,&znrm411,&znrm411res,"zlange",pass);
 
    zgetrf(4,4,za411,4,ipvt45,&info);
 
    zgecon("1",4,&za411[0][0],4,znrm411,&zrcnd411,zwrk411,zrwrk411,&info);
    dvecck(1,1,&zrcnd411,&zrcnd411res,"zgecon",pass);
 
    zgetrf (9, 9, zta7, 9,  ipvt37, &info);
    if (info != 0) {
        printf( " INFO from ZGETRF = %4d\n", info);
        *pass = *pass + 1;
    } else {
        zmtchk(9, 9, zta7, zresa7, 9, "zgetrf", pass);
        intchk(9, 1, ipvt37, ipvtr7, "zgetrf", pass);
        zgetrs("N",9,5,zta7,9,ipvt37,ztb7,9,&info);
        if (info != 0) {
            printf( " INFO from ZGETRS = %4d\n", info);
            *pass = *pass + 1;
        }
        zmtchk(9,5,ztb7,zresb7,9,"zgetrs",pass);
    }
 
   zgesv(9,5,zta40,9,ipvt37,ztb40,9,&info);
   if (info != 0) {
       printf( " INFO from ZGESV = %4d\n", info);
       *pass = *pass + 1;
   } else {
       zmtchk(9,9,zta40,zresa7,9,"zgesv",pass);
       intchk(9,1,ipvt37,ipvtr7, "zgesv",pass);
       zmtchk(9,5,ztb40,zresb7,9,"zgesv",pass);
   }
 
    info = 36;
    dbstrf ("L", 8, dta8, ipvt37, &info);
    if (info != 0)
     {
      printf( " INFO from DBSTRF = %4d\n", info);
      *pass = *pass + 1;
     }
  else
     {
 
    dbstrs("L",8,3,dta8,ipvt37,dtb8,8,&info);
    if (info != 0)
     {
      printf( " INFO from DBSTRS = %4d\n", info);
      *pass = *pass + 1;
     }
    dmtchk(8,3,dtb8,dresb8,8,"dbstrs",pass);
    dvecck(36, 1, dta8, dresa8,"dbstrf",pass);
    intchk(8, 1, ipvt37, ipvtr8, "dbstrf", pass);
   }
 
    info = 36;
    dbssv ("L",8,3,dta81,ipvt37,dtb81,8,&info);
    if (info != 0)
     {
      printf( " INFO from DBSSV  = %4d\n", info);
      *pass = *pass + 1;
     }
    dmtchk(8,3,dtb81,dresb8,8,"dbstrs",pass);
    dvecck(36, 1, dta81, dresa8,"dbssv ",pass);
    intchk(8, 1, ipvt37, ipvtr8, "dbssv ", pass);
 
/****************************
*  BANDED LINEAR ALGEBRAIC  *
****************************/
 
    sgbf (sagb45, 23, 9, 2, 3, ipvt45);
    smtchk(23, 9, sagb45, sres45, 23, "sgbf", pass);
/*  intchk(9, 1, ipvt45, ires45, "sgbf", pass);  */
    sgbs (sagb45, 23, 9, 2, 3, ipvt45, sbx46);
    svecck(9, 1, sbx46, sres46, "sgbs", pass);
    dgbf (dagb45, 23, 9, 2, 3, ipvt45);
    dmtchk(23, 9, dagb45, dres45, 23, "dgbf", pass);
/*  intchk(9, 1, ipvt45, ires45, "dgbf", pass);  */
    dgbs (dagb45, 23, 9, 2, 3, ipvt45, dbx46);
    dvecck(9, 1, dbx46, dres46, "dgbs", pass);
    spbf (sap47, 3, 9, 2);
    smtchk(3, 9, sap47, sres47, 3, "spbf", pass);
    dpbf (dap47, 3, 9, 2);
    dmtchk(3, 9, dap47, dres47, 3, "dpbf", pass);
    spbs (sap47, 3, 9, 2, sbx48);
    svecck(9, 1, sbx48, sres48, "spbs", pass);
    dpbs (dap47, 3, 9, 2, dbx48);
    dvecck(9, 1, dbx48, dres48, "dpbs", pass);
    spbchf(sap47n, 3, 9, 2);
    spbchs(sap47n, 3, 9, 2, sbx48n);
    svecck(9,1,sbx48n,sres48n,"spbchs",pass);
    dpbchf(dap47n, 3, 9, 2);
    dpbchs(dap47n, 3, 9, 2, dbx48n);
    dvecck(9,1,dbx48n,dres48n,"dpbchs",pass);
/* V2.2 */
    sgtf (4, sa49, sb49, sc49, sd49, ipvt);
    svecck(4, 1, sa49, srea49, "sgtf", pass);
    svecck(4, 1, sb49, sreb49, "sgtf", pass);
    svecck(4, 1, sc49, srec49, "sgtf", pass);
/*  commented out 3/92 - should not be checking per routine owner */
/*  svecck(3, 1, sd49, sred49, "sgtf", pass); */
    sgts (4, sa49, sb49, sc49, sd49, ipvt, sx50);
    svecck(4, 1, sx50, sres50, "sgts", pass);
    dgtf (4, da49, db49, dc49, dd49, ipvt);
    dvecck(4, 1, da49, drea49, "dgtf", pass);
    dvecck(4, 1, db49, dreb49, "dgtf", pass);
    dvecck(4, 1, dc49, drec49, "dgtf", pass);
/*  commented out 3/92 - should not be checking per routine owner */
/*  dvecck(3, 1, dd49, dred49, "dgtf", pass);   */
    dgts (4, da49, db49, dc49, dd49, ipvt, dx50);
    dvecck(4, 1, dx50, dres50, "dgts", pass);
    sgtnpf (4, sb51, sa51, sc51, 0);
    svecck(3, 1, &sb51[0], sreb51, "sgtnpf", pass);
    svecck(4, 1, sa51, srea51, "sgtnpf", pass);
    svecck(3, 1, sc51, srec51, "sgtnpf", pass);
    dgtnpf (4, db51, da51, dc51, 0);
    dvecck(3, 1, &db51[0], dreb51, "dgtnpf", pass);
    dvecck(4, 1, da51, drea51, "dgtnpf", pass);
    dvecck(3, 1, dc51, drec51, "dgtnpf", pass);
    sgtnps (4, sb51, sa51, sc51, sx52);
    svecck(4, 1, sx52, sres52, "sgtnps", pass);
    dgtnps (4, db51, da51, dc51, dx52);
    dvecck(4, 1, dx52, dres52, "dgtnps", pass);
    sptf (4, sb53, sa53, 0);
    svecck(3, 1, &sb53[0], sreb53, "sptf", pass);
    svecck(4, 1, sa53, srea53, "sptf", pass);
    dptf (4, db53, da53, 0);
    dvecck(3, 1, &db53[0], dreb53, "dptf", pass);
    dvecck(4, 1, da53, drea53, "dptf", pass);
    spts (4, sb53, sa53, sx54);
    svecck(4, 1, sx54, sres52, "spts", pass);
    dpts (4, db53, da53, dx54);
    dvecck(4, 1, dx54, dres52, "dpts", pass);
 
    stbsv("L","T","N",9,2,stbva36n,3,stbvx36n,1);
    svecck(9,1,stbvx36n,stbvres36n,"stbsv",pass);
    dtbsv("L","T","N",9,2,dtbva36n,3,dtbvx36n,1);
    dvecck(9,1,dtbvx36n,dtbvres36n,"dtbsv",pass);
    ctbsv("L","T","N",7,3,ctbva36n,4,ctbvx36n,1);
    cvecck(7,1,ctbvx36n,ctbvres36n,"ctbsv",pass);
    ztbsv("L","T","N",7,3,ztbva36n,4,ztbvx36n,1);
    zvecck(7,1,ztbvx36n,ztbvres36n,"ztbsv",pass);
 
    cgtnpf(4,cgtfc37n,cgtfa37n,cgtfb37n,0);
    cvecck(4,1,cgtfc37n,cgtfcres37n,"cgtnpf",pass);
    cvecck(4,1,cgtfa37n,cgtfares37n,"cgtnpf",pass);
    cvecck(4,1,cgtfb37n,cgtfbres37n,"cgtnpf",pass);
    zgtnpf(4,zgtfc37n,zgtfa37n,zgtfb37n,0);
    zvecck(4,1,zgtfc37n,zgtfcres37n,"zgtnpf",pass);
    zvecck(4,1,zgtfa37n,zgtfares37n,"zgtnpf",pass);
    zvecck(4,1,zgtfb37n,zgtfbres37n,"zgtnpf",pass);
 
    cgtnps(4,cgtfc37n,cgtfa37n,cgtfb37n,cgtsx38n);
    cvecck(4,1,cgtsx38n,cgtsxres38n,"cgtnps",pass);
    zgtnps(4,zgtfc37n,zgtfa37n,zgtfb37n,zgtsx38n);
    zvecck(4,1,zgtsx38n,zgtsxres38n,"zgtnps",pass);
 
    sgtnp (4,sctnp,satnp,sbtnp,sxtnp);
    svecck(4,1,sxtnp,sxrestnp,"sgtnp",pass);
    dgtnp (4,dctnp,datnp,dbtnp,dxtnp);
    dvecck(4,1,dxtnp,dxrestnp,"dgtnp",pass);
    cgtnp (4,cctnp,catnp,cbtnp,cxtnp);
    cvecck(4,1,cxtnp,cxrestnp,"cgtnp",pass);
    zgtnp (4,zctnp,zatnp,zbtnp,zxtnp);
    zvecck(4,1,zxtnp,zxrestnp,"zgtnp",pass);
 
/*******************************************
* ADDED D/S/Z/CPBTRF/S, D/S/Z/CPBSV  10/12 *
*******************************************/
 
    dpbtrf("L",9,3,da113,4,&info);
    if (info != 0)
     {
      printf( " INFO from DPBTRF = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     dmtchk(4,9,da113,dares113,4,"dpbtrf",pass);
 
    dpbtrs("L",9,3,3,da113,4,db113,9,&info);
    if (info != 0)
     {
      printf( " INFO from DPBTRS = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     dmtchk(9,3,db113,dbres113,9,"dpbtrs",pass);
 
    dpbsv("L",9,3,3,da114,4,db114,9,&info);
    if (info != 0)
     {
      printf( " INFO from DPBSV  = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      dmtchk(4,9,da114,dares114,4,"dpbsv ",pass);
      dmtchk(9,3,db114,dbres114,9,"dpbsv ",pass);
     }
 
    spbtrf("L",9,3,sa113,4,&info);
    if (info != 0)
     {
      printf( " INFO from SPBTRF = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     smtchk(4,9,sa113,sares113,4,"spbtrf",pass);
 
    spbtrs("L",9,3,3,sa113,4,sb113,9,&info);
    if (info != 0)
     {
      printf( " INFO from SPBTRS = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     smtchk(9,3,sb113,sbres113,9,"spbtrs",pass);
 
    spbsv("L",9,3,3,sa114,4,sb114,9,&info);
    if (info != 0)
     {
      printf( " INFO from SPBSV  = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      smtchk(4,9,sa114,sares114,4,"spbsv ",pass);
      smtchk(9,3,sb114,sbres114,9,"spbsv ",pass);
     } 
 
    zpbtrf("L",6,3,za113,4,&info);
    if (info != 0)
     {
      printf( " INFO from ZPBTRF = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     zmtchk(4,6,za113,zares113,4,"zpbtrf",pass);
 
    zpbtrs("L",6,3,3,za113,4,zb113,6,&info);
    if (info != 0)
     {
      printf( " INFO from ZPBTRS = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     zmtchk(6,3,zb113,zbres113,6,"zpbtrs",pass);
 
    zpbsv("L",6,3,3,za114,4,zb114,6,&info);
    if (info != 0)
     {
      printf( " INFO from ZPBSV  = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      zmtchk(4,6,za114,zares114,4,"zpbsv ",pass);
      zmtchk(6,3,zb114,zbres114,6,"zpbsv ",pass);
     } 
 
    cpbtrf("L",6,3,ca113,4,&info);
    if (info != 0)
     {
      printf( " INFO from CPBTRF = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     cmtchk(4,6,ca113,cares113,4,"cpbtrf",pass);
 
    cpbtrs("L",6,3,3,ca113,4,cb113,6,&info);
    if (info != 0)
     {
      printf( " INFO from CPBTRS = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     cmtchk(6,3,cb113,cbres113,6,"cpbtrs",pass);
 
    cpbsv("L",6,3,3,ca114,4,cb114,6,&info);
    if (info != 0)
     {
      printf( " INFO from CPBSV  = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      cmtchk(4,6,ca114,cares114,4,"cpbsv ",pass);
      cmtchk(6,3,cb114,cbres114,6,"cpbsv ",pass);
     } 
 
/*******************************************
* ADDED D/S/Z/CPTTRF/S, D/S/Z/CPTSV  10/12 *
*******************************************/
 
    dpttrf(4,dd115,de115,&info);
    if (info != 0)
     {
      printf( " INFO from DPTTRF = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      dvecck(4,1,dd115,ddres115,"dpttrf",pass);
      dvecck(3,1,de115,deres115,"dpttrf",pass);
     }
 
    dpttrs(4,2,dd115,de115,db115,4,&info);
    if (info != 0)
     {
      printf( " INFO from DPTTRS = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      dmtchk(4,2,db115,dbres115,4,"dpttrs",pass);
     }
 
    dptsv(4,2,dd116,de116,db116,4,&info);
    if (info != 0)
     {
      printf( " INFO from DPTSV  = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      dvecck(4,1,dd116,ddres116,"dptsv ",pass);
      dvecck(3,1,de116,deres116,"dptsv ",pass);
      dmtchk(4,2,db116,dbres116,4,"dptsv ",pass);
     }
 
    spttrf(4,sd115,se115,&info);
    if (info != 0)
     {
      printf( " INFO from SPTTRF = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      svecck(4,1,sd115,sdres115,"spttrf",pass);
      svecck(3,1,se115,seres115,"spttrf",pass);
     }
 
    spttrs(4,2,sd115,se115,sb115,4,&info);
    if (info != 0)
     {
      printf( " INFO from SPTTRS = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      smtchk(4,2,sb115,sbres115,4,"spttrs",pass);
     }
 
    sptsv(4,2,sd116,se116,sb116,4,&info);
    if (info != 0)
     {
      printf( " INFO from SPTSV  = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      svecck(4,1,sd116,sdres116,"sptsv ",pass);
      svecck(3,1,se116,seres116,"sptsv ",pass);
      smtchk(4,2,sb116,sbres116,4,"sptsv ",pass);
     }
 
    zpttrf(4,zd115,ze115,&info);
    if (info != 0)
     {
      printf( " INFO from ZPTTRF = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      dvecck(4,1,zd115,zdres115,"zpttrf",pass);
      zvecck(3,1,ze115,zeres115,"zpttrf",pass);
     }
 
    zpttrs("L",4,3,zd115,ze115,zb115,4,&info);
    if (info != 0)
     {
      printf( " INFO from ZPTTRS = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      zmtchk(4,3,zb115,zbres115,4,"zpttrs",pass);
     }
 
    zptsv(4,3,zd116,ze116,zb116,4,&info);
    if (info != 0)
     {
      printf( " INFO from ZPTSV  = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      dvecck(4,1,zd116,zdres116,"zptsv ",pass);
      zvecck(3,1,ze116,zeres116,"zptsv ",pass);
      zmtchk(4,3,zb116,zbres116,4,"zptsv ",pass);
     }
 
    cpttrf(4,cd115,ce115,&info);
    if (info != 0)
     {
      printf( " INFO from CPTTRF = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      svecck(4,1,cd115,cdres115,"cpttrf",pass);
      cvecck(3,1,ce115,ceres115,"cpttrf",pass);
     }
 
    cpttrs("L",4,3,cd115,ce115,cb115,4,&info);
    if (info != 0)
     {
      printf( " INFO from CPTTRS = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      cmtchk(4,3,cb115,cbres115,4,"cpttrs",pass);
     }
 
    cptsv(4,3,cd116,ce116,cb116,4,&info);
    if (info != 0)
     {
      printf( " INFO from CPTSV  = %4d\n", info);
      *pass = *pass + 1;
     }
    else
     {
      svecck(4,1,cd116,cdres116,"cptsv ",pass);
      cvecck(3,1,ce116,ceres116,"cptsv ",pass);
      cmtchk(4,3,cb116,cbres116,4,"cptsv ",pass);
     }
 
/****************************
*  SPARSE LINEAR ALGEBRAIC  *
****************************/
 
    dgsf (0, 5, 13, da105, di105, dj105, 27, dip105, drp105, dop105,
        daux1, 200);
    dvecck(27, 1, da105, daz105, "dgsf", pass);
    intchk(27, 1, di105, diz105, "dgsf", pass);
    intchk(27, 1, dj105, djz105, "dgsf", pass);
    dvecck(3, 1, dop105, doz105, "dgsf", pass);
    dgss (0, 5, da105, di105, dj105, 27, db106, daux1, 200);
    dvecck(5, 1, db106, dbz106, "dgss", pass);
/*  intchk(27, 1, di105, diz106, "dgss", pass);	*/
    dsmcg (9, 3, dac55, ika55, 9, db55, du55, ipar55, drpa55,
        daux1, 0, daux2, 0);
    dvecck(9, 1, du55, dres55, "dsmcg", pass);
    intchk(1, 1, &ipar55[3], ires55, "dsmcg", pass);
    dvecck(1, 1, &drpa55[1], d2rp55, "dsmcg", pass);
    dvecck(1, 1, &drpa55[2], d3rp55, "dsmcg", pass);
/*  dsdcg(0, 9, 3, dad55a, 9, kd55a, db55a, du55a, ipa55a, dpr55a,
        daux1, 283, daux2, 0);  */
    dsdcg (0, 9, 3, dad55a, 9, kd55a, db55a, du55a, ipa55a, dpr55a,
        daux1, 0, daux2, 0);
    dvecck(9, 1, du55a, dre55a, "dsdcg", pass);
    intchk(1, 1, &ipa55a[3], ire55a, "dsdcg", pass);
    dvecck(2, 1, &dpr55a[1], rpr55a, "dsdcg", pass);
    dsmgcg (9, 3, da103, dk103, 9, db103, du103, di103, dr103,
        daux1, 0, daux2, 0);
    dvecck(9, 1, du103, duz103, "dsmgcg", pass);
    itmp = 9;
    intchk(1, 1, &di103[3], &itmp, "dsmgcg", pass);
    dtmp = .150e-19;
    dvecck(1, 1, &dr103[2], &dtmp, "dsmgcg", pass);
    dsdgcg (9, 3, da104, 9, dl104, db104, du104, di104, dr104,
        daux1, 0, daux2, 0);
    dvecck(9, 1, du104, duz104, "dsdgcg", pass);
    itmp = 8;
    intchk(1, 1, &di104[3], &itmp, "dsdgcg", pass);
    dtmp = .308e-17;
    dvecck(1, 1, &dr104[2], &dtmp, "dsdgcg", pass);
 
    dsris("G","I",9,dssar39n,dssja39n,dssia39n,dssb39n,dssx39n,
                         dssip39n,dssrp39n,daux1d,200,daux2d,0);
    dvecck(9,1,dssx39n,dssxres39n,"dsris",pass);
    intchk(6,1,dssip39n,dssipres39n,"dsris",pass);
    dvecck(3,1,dssrp39n,dssrpres39n,"dsris",pass);
 
/*  Skyline dskfs and dgkfs  routines  */
 
    ip107[0]=0;
    dskfs (9, da107, 33, id107, ip107, d107, daux1, 0, db107, 12, 3);
    dvecck(33, 1, da107, dar107, "dskfs ", pass);
    dvecck(9, 1, db107, dbr107, "dskfs ", pass);
    intchk(1, 1, &ip107[0], &ir107[0], "dskfs ", pass);
    intchk(1, 1, &ip107[15], &ir107[1], "dskfs ", pass);
    intchk(5, 1, &ip107[20], &ir107[2], "dskfs ", pass);
    dvecck(1, 1, &d107[15], &dr107[0], "dskfs ", pass);
    ip107[0] =1;
    ip107[1] =10;
    ip107[2] =0;
    ip107[3] =1;
    ip107[4] =0;
    ip107[9]=0;
    dskfs (9, da108, 33, id108, ip107, d107, daux1, 0, db108, 12, 3);
    dvecck(33, 1, da108, dar107, "dskfs ", pass);
    dvecck(9, 1, db108, dbr107, "dskfs ", pass);
    intchk(5, 1, &ip107[0], &ir108[0], "dskfs ", pass);
    intchk(5, 1, &ip107[20], &ir107[2], "dskfs ", pass);
    intchk(1, 1, &ip107[15], &ir107[1], "dskfs ", pass);
    intchk(10, 1, id108, idr108, "dskfs ", pass);
    dvecck(3, 1, &d107[15], &dr107[0], "dskfs ", pass);
    ip107[0]=0;
    dgkfs (9,dau112,33,idu112,dal112,35,idl112,ip107,d107,daux1,0,
           db112, 12, 3);
    dvecck(33,1,dau112,daur112,"dgkfs ",pass);
    dvecck(9,1,db112,dbr112,"dgkfs ",pass);
    intchk(5,1,&ip107[20],&ipr112[0],"dgkfs ",pass);
    intchk(1,1,&ip107[15],&ipr112[4],"dgkfs ",pass);
    dvecck(33,1,&dau112[0],&dru112[0],"dgkfs ",pass);
 
/*************************
*  LINEAR LEAST SQUARES  *
*************************/
 
    sgesvf (0, sa56, 4, summy, 1, 0, ss56, 4, 3, saux1, 0);
    svecck(3, 1, ss56, sres56, "sgesvf", pass);
    dgesvf (0, da56, 4, dummy, 1, 0, ds56, 4, 3, daux1, 0);
    dvecck(3, 1, ds56, dres56, "dgesvf", pass);
    sgesvs (sv57, 3, sub57, 3, 1, ss57, sx57, 3, 2, 3, 0.3993e-14);
    smtchk(3, 1, sx57, sres57, 3, "sgesvs", pass);
    dgesvs (dv57, 3, dub57, 3, 1, ds57, dx57, 3, 2, 3, 0.3993e-14);
    dmtchk(3, 1, dx57, dres57, 3, "dgesvs", pass);
    sgells (0, sa58, 2, sb58, 2, sx58, 3, &sdum[0], 0., 2, 3, 1, &ik58[0],
        saux1, 0);
    smtchk(3, 1, sx58, sres58, 3, "sgells", pass);
    intchk(1, 1, ik58, ires58, "sgells", pass);
    dgells (0, da58, 2, db58, 2, dx58, 3, &ddum[0],0., 2, 3, 1, &ik58[0],
        daux1, 0);
    dmtchk(3, 1, dx58, dres58, 3, "dgells", pass);
    intchk(1, 1, ik58, ires58, "dgells", pass);
 
   dgeqrf (6, 2, dta9, 6, tau, work, 0, &info);
   if (info != 0) {
       printf( " INFO from DGEQRF = %4d\n", info);
       *pass = *pass + 1;
   } else {
       dmtchk(6, 2, dta9, dresa9, 6, "dgeqrf", pass);
       dvecck(2, 1, tau, drest9,"dgeqrf",pass);
   }
 
   sgeqrf (6, 2, sta9, 6, stau9, swork9, 0, &info);
   if (info != 0) {
       printf( " INFO from SGEQRF = %4d\n", info);
       *pass = *pass + 1;
   } else {
       smtchk(6, 2, sta9, sresa9, 6, "sgeqrf", pass);
       svecck(2, 1, stau9, srest9,"sgeqrf",pass);
   }
 
   zgeqrf (6, 2, zta9, 6, ztau9, zwork9, 0, &info);
   if (info != 0) {
       printf( " INFO from ZGEQRF = %4d\n", info);
       *pass = *pass + 1;
   } else {
       zmtchk(6, 2, zta9, zresa9, 6, "zgeqrf", pass);
       zvecck(2, 1, ztau9, zrest9,"zgeqrf",pass);
   }
 
   cgeqrf (6, 2, cta9, 6, ctau9, cwork9, 0, &info);
   if (info != 0) {
       printf( " INFO from CGEQRF = %4d\n", info);
       *pass = *pass + 1;
   } else {
       cmtchk(6, 2, cta9, cresa9, 6, "cgeqrf", pass);
       cvecck(2, 1, ctau9, crest9,"cgeqrf",pass);
   }
 
   sgels ("N",6, 2, 3, sta10, 6, stb10, 6, swork10, 0, &info);
   if (info != 0) {
       printf( " INFO from SGELS  = %4d\n", info);
       *pass = *pass + 1;
   } else {
       smtchk(6, 3, stb10, sresb10, 6, "sgels", pass);
   }
 
   dgels ("N",6, 2, 3, dta10, 6, dtb10, 6, work, 0, &info);
   if (info != 0) {
       printf( " INFO from DGELS  = %4d\n", info);
       *pass = *pass + 1;
   } else {
       dmtchk(6, 3, dtb10, dresb10, 6, "dgels", pass);
   }
 
   cgels ("N",3, 4, 3, cta10, 3, ctb10, 4, cwork10, 0, &info);
 
   if (info != 0) {
       printf( " INFO from CGELS  = %4d\n", info);
       *pass = *pass + 1;
   } else {
       cmtchk(4, 3, ctb10, cresb10, 4, "cgels", pass);
   }
 
   zgels ("N",3, 4, 3, zta10, 3, ztb10, 4, zwork10, 0, &info);
 
   if (info != 0) {
       printf( " INFO from ZGELS  = %4d\n", info);
       *pass = *pass + 1;
   } else {
       zmtchk(4, 3, ztb10, zresb10, 4, "zgels", pass);
   }
 
}
/**********************************************************************
*                    SUBROUTINE:  EIGNS                               *
**********************************************************************/
 
void eigns(pass)
_ESVINT *pass;
{
/***************************************************
*  VARIABLES OF EIGENSYSTEM ANALYSIS SUBROUTINES : *
***************************************************/
 
    _ESVINT    info;
 
/****************************************************
* ADDED FOR D/SS(Y/P)EVD, Z/CH(E/P)EVD        10/12 *
****************************************************/
 
    double dw111[4], zw111[3], daz111[4][4];
    float  sw111[4], cw111[3], saz111[4][4];
    dcmplx zaz111[4][4];
    cmplx  caz111[4][4];
 
/****************************************************
* ADDED FOR D/S/Z/CGGEV                       10/12 *
****************************************************/
 
    double dalphr112[5], dalphi112[5], dbeta112[5], dwork112[50];
    double dvl112[5][5], dvr112[5][5];
    float salphr112[5], salphi112[5], sbeta112[5], swork112[50];
    float svl112[5][5], svr112[5][5];
    dcmplx zalph112[3], zbeta112[3], zwork112[20];
    double zrwork112[24];
    dcmplx zvl112[3][3], zvr112[3][3];
    cmplx calph112[3], cbeta112[3], cwork112[20];
    float crwork112[24];
    cmplx cvl112[3][3], cvr112[3][3];
 
/****************************************************
* ADDED FOR D/SS(Y/P)GVX, Z/CH(E/P)GVX        10/12 *
****************************************************/
 
    double dw110[4],daz110[4][4];
    float sw110[4],saz110[4][4];
    double zw110[4];
    float cw110[4];
    dcmplx zaz110[4][4];
    cmplx caz110[4][4];
    _ESVINT difail110x[4], sifail110x[4];
    _ESVINT zifail110x[4], cifail110x[4];
 
/*********************************************************
*  EXPECTED VALUE FOR EIGENSYSTEM ANALYSIS SUBROUTINES : *
*********************************************************/
 
/****************************************************
* ADDED FOR D/SS(Y/P)EVD, Z/CH(E/P)EVD        10/12 *
****************************************************/
 
    double dwq111[4] = { 1., 2., 5., 10. };
    double dares111[4][4] = { { 0.707107, -0.707107, 0., 0. },
                              { 0., 0., 0.707107, -0.707107 },
                              { 0.316228, 0.316228, -0.632456, -0.632456 },
                              { 0.632456, 0.632456, 0.316228, 0.316228 } };
    float swq111[4] = { 1., 2., 5., 10. };
    float sares111[4][4] = { { 0.707107, -0.707107, 0., 0. },
                             { 0., 0., 0.707107, -0.707107 },
                             { 0.316228, 0.316228, -0.632456, -0.632456 },
                             { 0.632456, 0.632456, 0.316228, 0.316228 } };
    double zwq111[3] = { 1., 3., 3. };
    dcmplx zares111[3][3] = { { { 0., 0.707107 }, { -0.707107, 0. }, { 0., 0. } },
                              { { 0., 0.707107 }, { 0.707107, 0. }, {0., 0. } },
                              { { 0., 0. }, { 0., 0. }, {1., 0. } } };
    float cwq111[3] = { 1., 3., 3. };
    cmplx cares111[3][3] = { { { 0., 0.707107 }, { -0.707107, 0. }, { 0., 0. } },
                             { { 0., 0.707107 }, { 0.707107, 0. }, {0., 0. } },
                             { { 0., 0. }, { 0., 0. }, {1., 0. } } };
 
/****************************************************
* ADDED FOR D/S/Z/CGGEV                       10/12 *
****************************************************/
 
    double dalphrres112[5] = { 7.95005, -0.277338, 2.149669,
                               6.720718, 10.987556 };
    double dalphires112[5] = { 0., 0., 0., 0., 0. };
    double dbetares112[5] = { 0.374183, 1.480299, 1.636872,
                              1.213574, 0.908837 };
    double dvlres112[5][5] = { { -0.003801, -0.018291, -0.074427, -0.279354, -1. },
                               { 1., -0.546838, -0.174281, -0.051285, -0.026864 },
                               { -0.778812, 0.133707, 1., -0.696402, 0.282389 },
                               { -0.032145, -0.028432, 0.076908, 0.285338, -1. },
                               { -0.005114, -0.0129, -0.009062, 0.123783, 1. } };
    double dvrres112[5][5] = { { -1., -0.565497, -0.180429, -0.034182, -0.003039 },
                               { 0.483408, -1., 0.661372, -0.180646, 0.017732 },
                               { -0.540696, -0.684441, 1., -0.363671, 0.041865 },
                               { -1., -0.722065, 0.089003, 0.223599, -0.050111 },
                               { -1., -0.610415, -0.116987, 0.038979, 0.018653 } };
 
    float salphrres112[5] = { 7.95005, -0.277338, 2.149669,
                              6.720718, 10.987556 };
    float salphires112[5] = { 0., 0., 0., 0., 0. };
    float sbetares112[5] = { 0.374183, 1.480299, 1.636872,
                             1.213574, 0.908837 };
    float svlres112[5][5] = { { 0.003801, 0.018291, 0.074427, 0.279354, 1. },
                              { 1., -0.546838, -0.174281, -0.051285, -0.026864 },
                              { 0.778812, -0.133707, -1., 0.696402, -0.282389 },
                              { -0.032145, -0.028432, 0.076908, 0.285338, -1. },
                              { -0.005114, -0.0129, -0.009062, 0.123783, 1. } };
    float svrres112[5][5] = { { 1., 0.565497, 0.180429, 0.034182, 0.003039 },
                              { 0.483408, -1., 0.661372, -0.180646, 0.017732 },
                              { 0.540696, 0.684441, -1., 0.363671, -0.041865 },
                              { -1., -0.722065, 0.089003, 0.223599, -0.050111 },
                              { -1., -0.610415, -0.116987, 0.038979, 0.018653 } };
 
    dcmplx zalphres112[3] = { { 15.863783, 41.115283 }, { -12.917205, 19.973815 },
                              { 3.215518, -4.912439 } };
    dcmplx zbetares112[3] = { { 1.668461, 0. }, { 2.024212, 0. },
                              { 2.664836, 0. } };
    dcmplx zvlres112[3][3] = { { { 0.0634, -0.8686 }, { -0.3652, -0.3826 }, { 0.3605, -0.6395 } },
                               { { 0.8988, -0.1012 }, { 0.0108, -0.3479 }, { -0.2029, 0.6348 } },
                               { { -0.6456, -0.3544 }, { 0.0001, -0.0852 }, { 0.4537, 0.4128 } } };
    dcmplx zvrres112[3][3] = { { { 0.3799, -0.1986 }, { 0.0132, -0.9868 }, { -0.0976, -0.2383 } },
                               { { 0.2712, -0.0943 }, { -0.5085, -0.4915 }, { -0.0633, 0.1388 } },
                               { { -0.1470, 0.1422 }, { 0.3239, -0.6761 }, { -0.0395, 0.1663 } } };
 
    cmplx calphres112[3] = { { 15.863783, 41.115283 }, { -12.917205, 19.973815 },
                             { 3.215518, -4.912439 } };
    cmplx cbetares112[3] = { { 1.668461, 0. }, { 2.024212, 0. },
                             { 2.664836, 0. } };
    cmplx cvlres112[3][3] = { { { 0.0634, -0.8686 }, { -0.3652, -0.3826 }, { 0.3605, -0.6395 } },
                              { { 0.8988, -0.1012 }, { 0.0108, -0.3479 }, { -0.2029, 0.6348 } },
                              { { -0.6456, -0.3544 }, { 0.0001, -0.0852 }, { 0.4537, 0.4128 } } };
    cmplx cvrres112[3][3] = { { { 0.3799, -0.1986 }, { 0.0132, -0.9868 }, { -0.0976, -0.2383 } },
                              { { 0.2712, -0.0943 }, { -0.5085, -0.4915 }, { -0.0633, 0.1388 } },
                              { { -0.1470, 0.1422 }, { 0.3239, -0.6761 }, { -0.0395, 0.1663 } } };
 
/****************************************************
* ADDED FOR D/SS(Y/P)GVX, Z/CH(E/P)GVX        10/12 *
****************************************************/
 
    double dwq110[2] = { -1., 5.};
    double dbres110[4][4] = { 1., 0., 0., 0., 0., 1., 0., 0.,
                           0., 0., 1., 0., 0., 0., 0., 1. };
    double dbpres110[10] = { 1., 0., 1., 0., 0., 1., 0., 0., 0., 1. };
    double dzres110[4][2] = { .5, -.5, -.5, .5, .544042, .451683,
                            -.451683, -.544042 };
 
    float swq110[2] = { -1., 5.};
    float sbres110[4][4] = { 1., 0., 0., 0., 0., 1., 0., 0.,
                           0., 0., 1., 0., 0., 0., 0., 1. };
    float sbpres110[10] = { 1., 0., 1., 0., 0., 1., 0., 0., 0., 1. };
    float szres110[4][2] = { .5, -.5, -.5, .5, -.645537, .288584,
                            -.288584, .645537 };
 
    double zwq110[2] = { -1., 5.};
    dcmplx zbres110[4][4] = { { { 1., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. } },
                              { { 0., 0. }, { 1., 0. }, { 0., 0. }, { 0., 0. } },
                              { { 0., 0. }, { 0., 0. }, { 1., 0. }, { 0., 0. } },
                              { { 0., 0. }, { 0., 0. }, { 0., 0. }, { 1., 0. } } };
    dcmplx zbpres110[10] = { { 1., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
                            { 1., 0. }, { 0., 0. }, { 0., 0. },
                            { 1., 0. }, { 0., 0. }, { 1., 0. } };
    dcmplx zzres110[2][4] = { { { 0.5, 0. }, { -0.5, 0. }, { -0.5, 0. }, { 0.5, 0. } },
                              { { -0.154815, 0. }, { -0.689950, 0. },
                                { 0.689950, 0. }, { 0.154815, 0. } } };
 
    float cwq110[2] = { -1., 5.};
    cmplx cbres110[4][4] = { { { 1., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. } },
                              { { 0., 0. }, { 1., 0. }, { 0., 0. }, { 0., 0. } },
                              { { 0., 0. }, { 0., 0. }, { 1., 0. }, { 0., 0. } },
                              { { 0., 0. }, { 0., 0. }, { 0., 0. }, { 1., 0. } } };
    cmplx cbpres110[10] = { { 1., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
                           { 1., 0. }, { 0., 0. }, { 0., 0. },
                           { 1., 0. }, { 0., 0. }, { 1., 0. } };
    cmplx czres110[2][4] = { { { 0.5, 0. }, { -0.5, 0. }, { -0.5, 0. }, { 0.5, 0. } },
                              { { 0.308260, 0. }, { -0.636376, 0. },
                                { 0.636376, 0. }, { -0.308260, 0. } } };
 
/*****************************************************
*  INPUT DATA FOR EIGENSYSTEM ANALYSIS SUBROUTINES : *
*****************************************************/
 
    double dap60y[4][4] = { 5., 4., 1., 1.,
                            4., 5., 1., 1.,
                            1., 1., 4., 2.,
                            1., 1., 2., 4. };
 
    double dapres60y[4][4] = { 5., -4.242641, 0.121320, 0.121320,
                               4., 6., 1.414214, 0.414214,
                               1., 1., 5., 0.,
                               1., 1., 2., 2.  };
 
    double dw60y[4], dz60y[1], dwork60y[32];
    double dwres60y[4] = { 1., 2., 5., 10. };
    _ESVINT dm60y, difail60y[1], diwork60y[20];
 
    float sap60y[4][4] = { 5., 4., 1., 1.,
                           4., 5., 1., 1.,
                           1., 1., 4., 2.,
                           1., 1., 2., 4. };
 
    float sapres60y[4][4] = { 5., -4.242641, 0.121320, 0.121320,
                               4., 6., 1.414214, 0.414214,
                               1., 1., 5., 0.,
                               1., 1., 2., 2.  };
 
    float sw60y[4], sz60y[1], swork60y[32];
    float swres60y[4] = { 1., 2., 5., 10. };
    _ESVINT sm60y, sifail60y[1], siwork60y[20];
 
 
 
    dcmplx za60y[3][3] = {
        { { 2.0, 0.0}, { 0.0,-1.0}, { 0.0, 0.0} },
        { { 0.0, 1.0}, { 2.0, 0.0}, { 0.0, 0.0} },
        { { 0.0, 0.0}, { 0.0, 0.0}, { 3.0, 0.0} } };
 
    dcmplx zares60y[3][3] = {
        { { 2.0, 0.0}, {-1.0, 0.0}, { 0.0, 0.0} },
        { { 0.0, 1.0}, { 2.0, 0.0}, { 0.0, 0.0} },
        { { 0.0, 0.0}, { 0.0, 0.0}, { 3.0, 0.0} } };
 
    dcmplx zz60y[1], zwork60y[1];
    double zw60y[3], zrwork60y[21];
    double zwres60y[3] = { 1., 3., 3. };
    _ESVINT zm60y, zifail60y[1], ziwork60y[15];
 
 
    cmplx ca60y[3][3] = {
        { { 2.0, 0.0}, { 0.0,-1.0}, { 0.0, 0.0} },
        { { 0.0, 1.0}, { 2.0, 0.0}, { 0.0, 0.0} },
        { { 0.0, 0.0}, { 0.0, 0.0}, { 3.0, 0.0} } };
 
    cmplx cares60y[3][3] = {
        { { 2.0, 0.0}, {-1.0, 0.0}, { 0.0, 0.0} },
        { { 0.0, 1.0}, { 2.0, 0.0}, { 0.0, 0.0} },
        { { 0.0, 0.0}, { 0.0, 0.0}, { 3.0, 0.0} } };
 
    cmplx cz60y[1], cwork60y[1];
    float cw60y[3], crwork60y[21];
    float cwres60y[3] = { 1., 3., 3. };
    _ESVINT cm60y, cifail60y[1], ciwork60y[15];
 
 
 
 
    double dap60x[10] = { 5., 4., 1., 1., 5., 1., 1., 4., 2., 4. };
    float  sap60x[10] = { 5., 4., 1., 1., 5., 1., 1., 4., 2., 4. };
    dcmplx zap60x[6] = {{2.,0.},{0.,-1.},{0.,0.},{2.,0.},{0.,0.},{3.,0.}};
    cmplx  cap60x[6] = {{2.,0.},{0.,-1.},{0.,0.},{2.,0.},{0.,0.},{3.,0.}};
    dcmplx zaz60x[1], zwork60x[32];
    cmplx  caz60x[1], cwork60x[32];
    double dapres60x[10] = { 5., -4.242641, 0.121320, 0.121320,
                             6., 1.414214, 0.414214, 5., 0., 2.  };
    float  sapres60x[10] = { 5., -4.242641, 0.121320, 0.121320,
                             6., 1.414214, 0.414214, 5., 0., 2.  };
    dcmplx zapres60x[6] = { {2.,0.},{-1.,0.},{0.,0.},{2.,0.},{0.,0.},{3.,0.} };
    cmplx  capres60x[6] = { {2.,0.},{-1.,0.},{0.,0.},{2.,0.},{0.,0.},{3.,0.} };
    double daw60x[4], daz60x[1], dwork60x[32];
    float  saw60x[4], saz60x[1], swork60x[32];
    double zaw60x[3],zrwork60x[28];
    float  caw60x[3],crwork60x[28];
    double dawres60x[4] = { 1., 2., 5., 10. };
    float  sawres60x[4] = { 1., 2., 5., 10. };
    double zawres60x[3] = { 1., 3., 3.  };
    float  cawres60x[3] = { 1., 3., 3.  };
    _ESVINT dam60x, difail60x[1], diwork60x[20];
    _ESVINT sam60x, sifail60x[1], siwork60x[20];
    _ESVINT zam60x, zifail60x[1], ziwork60x[20];
    _ESVINT cam60x, cifail60x[1], ciwork60x[20];
 
    double da60g[4][4] = { -2., -3., -2., -1.,
                            2., 3., 0., 0.,
                            2., 2., 4., 0.,
                            2., 2., 2., 5.  };
    double dwr60g[4],dwi60g[4],dscale60g[4],dwork60g[1];
    double dwrres60g[4] = { 1., 2., 3., 4.  };
    double dwires60g[4] = { 0., 0., 0., 0.  };
    double dscaleres60g[4] = { 1., 1., 1., 1.  };
 
    double dabnrm60g,dum160g,dum260g,dum360g,dum460g;
    double dabnrmres60g = {11.0};
    _ESVINT dilo60g,dihi60g,idum60g;
 
    float sa60g[4][4] = { -2., -3., -2., -1.,
                           2., 3., 0., 0.,
                           2., 2., 4., 0.,
                           2., 2., 2., 5.  };
    float swr60g[4],swi60g[4],sscale60g[4],swork60g[1];
    float swrres60g[4] = { 1., 2., 3., 4.  };
    float swires60g[4] = { 0., 0., 0., 0.  };
    float sscaleres60g[4] = { 1., 1., 1., 1.  };
 
    float sabnrm60g,sdum160g,sdum260g,sdum360g,sdum460g;
    float sabnrmres60g = {11.0};
    _ESVINT   silo60g,sihi60g,sidum60g;
 
     dcmplx za60g[3][3] = { { { 1.0, 2.0}, {43.0,44.0}, { 5.0, 6.0} },
                            { { 3.0, 4.0}, {13.0,14.0}, { 7.0, 8.0} },
                            { {21.0,22.0}, {15.0,16.0}, {25.0,26.0} } };
 
     dcmplx zw60g[3] = { {6.7009, -7.8760}, {-7.4775, 6.8803}, {39.7767, 42.9957} };
     dcmplx zdum160g[1], zdum260g[1], zwork60g[1];
     double zdum360g[1], zdum460g[1], zscale60g[3], zrdum60g[1], zabnrm60g;
     _ESVINT zilo60g, zihi60g;
     dcmplx zwres60g[3] = { {6.7009,-7.8760 }, {-7.4775,6.8803}, {39.7767,42.9957} };
 
     double zabnrmres60g = { 88.4149 };
 
     cmplx ca60g[3][3] = { { { 1.0, 2.0}, {43.0,44.0}, { 5.0, 6.0} },
                           { { 3.0, 4.0}, {13.0,14.0}, { 7.0, 8.0} },
                           { {21.0,22.0}, {15.0,16.0}, {25.0,26.0} } };
 
     cmplx cw60g[3] = { {6.7009, -7.8760}, {-7.4775, 6.8803}, {39.7767, 42.9957} };
     cmplx cdum160g[1], cdum260g[1], cwork60g[1];
     float cdum360g[1], cdum460g[1], cscale60g[3], crdum60g[1], cabnrm60g;
     _ESVINT cilo60g, cihi60g;
     cmplx cwres60g[3] = { {6.7009,-7.8760 }, {-7.4775,6.8803}, {39.7767,42.9957} };
 
     float cabnrmres60g = { 88.4149 };
 
/****************************************************
* ADDED FOR D/SS(Y/P)EVD, Z/CH(E/P)EVD        10/12 *
****************************************************/
 
    double da111[4][4] = { { 5., 4., 1., 1. },
                           { 4., 5., 1., 1. },
                           { 1., 1., 4., 2. },
                           { 1., 1., 2., 4. } };
    double dap111[10] = { 5., 4., 5., 1., 1.,
                          4., 1., 1., 2., 4. };
    float sa111[4][4] = { { 5., 4., 1., 1. },
                          { 4., 5., 1., 1. },
                          { 1., 1., 4., 2. },
                          { 1., 1., 2., 4. } };
    float sap111[10] = { 5., 4., 5., 1., 1.,
                         4., 1., 1., 2., 4. };
    dcmplx za111[3][3] = { { { 2., 0. }, { 0., -1. }, { 0., 0. } },
                           { { 0., 1. }, { 2., 0. }, { 0., 0. } },
                           { { 0., 0. }, { 0., 0. }, { 3., 0. } } };
    dcmplx zap111[6] = { { 2., 0. }, { 0., 1. }, { 2., 0. },
                         { 0., 0. }, { 0., 0. }, { 3., 0. } };
    cmplx ca111[3][3] = { { { 2., 0. }, { 0., -1. }, { 0., 0. } },
                          { { 0., 1. }, { 2., 0. }, { 0., 0. } },
                          { { 0., 0. }, { 0., 0. }, { 3., 0. } } };
    cmplx cap111[6] = { { 2., 0. }, { 0., 1. }, { 2., 0. },
                        { 0., 0. }, { 0., 0. }, { 3., 0. } };
 
/****************************************************
* ADDED FOR D/S/Z/CGGEV                       10/12 *
****************************************************/
 
    double da112[5][5] = { { 2., 4., 0., 0., 0. },
                           { 3., 4., 3., 0., 0. },
                           { 4., 5., 6., 2., 0. },
                           { 5., 6., 7., 8., 1. },
                           { 6., 7., 8., 9., 10. } };
    double db112[5][5] = { { 1., 0., 0., 0., 0. },
                           { -1., 1., 0., 0., 0. },
                           { -1., -1., 1., 0., 0. },
                           { -1., -1., -1., 1., 0. },
                           { -1., -1., -1., -1., 1. } };
    float sa112[5][5] = { { 2., 4., 0., 0., 0. },
                          { 3., 4., 3., 0., 0. },
                          { 4., 5., 6., 2., 0. },
                          { 5., 6., 7., 8., 1. },
                          { 6., 7., 8., 9., 10. } };
    float sb112[5][5] = { { 1., 0., 0., 0., 0. },
                          { -1., 1., 0., 0., 0. },
                          { -1., -1., 1., 0., 0. },
                          { -1., -1., -1., 1., 0. },
                          { -1., -1., -1., -1., 1. } };
    dcmplx za112[3][3] = { { { 1., 2. }, { 43., 44. }, { 5., 6. } },
                           { { 3., 4. }, { 13., 14. }, { 7., 8. } },
                           { { 21., 22. }, { 15., 16. }, { 25., 26. } } };
    dcmplx zb112[3][3] = { { { 2., 0. }, { 0., 1. }, { 0., 0. } },
                           { { 0., -1. }, { 2., 0. }, { 0., 0. } },
                           { { 0., 0. }, { 0., 0. }, { 3., 0. } } };
    cmplx ca112[3][3] = { { { 1., 2. }, { 43., 44. }, { 5., 6. } },
                          { { 3., 4. }, { 13., 14. }, { 7., 8. } },
                          { { 21., 22. }, { 15., 16. }, { 25., 26. } } };
    cmplx cb112[3][3] = { { { 2., 0. }, { 0., 1. }, { 0., 0. } },
                          { { 0., -1. }, { 2., 0. }, { 0., 0. } },
                          { { 0., 0. }, { 0., 0. }, { 3., 0. } } };
 
/****************************************************
* ADDED FOR D/SS(Y/P)GVX, Z/CH(E/P)GVX        10/12 *
****************************************************/
 
    double da110[4][4] = { 6., 4., 4., 1., 4., 6., 1., 4.,
                           4., 1., 6., 4., 1., 4., 4., 6. };
    double db110[4][4] = { 1., 0., 0., 0., 0., 1., 0., 0.,
                           0., 0., 1., 0., 0., 0., 0., 1. };
    double dap110[10] = { 6., 4., 6., 4., 1., 6., 1., 4., 4., 6. };
    double dbp110[10] = { 1., 0., 1., 0., 0., 1., 0., 0., 0., 1. };
 
    float sa110[4][4] = { 6., 4., 4., 1., 4., 6., 1., 4.,
                          4., 1., 6., 4., 1., 4., 4., 6. };
    float sb110[4][4] = { 1., 0., 0., 0., 0., 1., 0., 0.,
                          0., 0., 1., 0., 0., 0., 0., 1. };
    float sap110[10] = { 6., 4., 6., 4., 1., 6., 1., 4., 4., 6. };
    float sbp110[10] = { 1., 0., 1., 0., 0., 1., 0., 0., 0., 1. };
 
    dcmplx za110[4][4] = { { { 6., 0. }, { 4., 0. }, { 4., 0. }, { 1., 0. } },
                           { { 4., 0. }, { 6., 0. }, { 1., 0. }, { 4., 0. } },
                           { { 4., 0. }, { 1., 0. }, { 6., 0. }, { 4., 0. } },
                           { { 1., 0. }, { 4., 0. }, { 4., 0. }, { 6., 0. } } };
    dcmplx zb110[4][4] = { { { 1., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. } },
                           { { 0., 0. }, { 1., 0. }, { 0., 0. }, { 0., 0. } },
                           { { 0., 0. }, { 0., 0. }, { 1., 0. }, { 0., 0. } },
                           { { 0., 0. }, { 0., 0. }, { 0., 0. }, { 1., 0. } } };
    dcmplx zap110[10] = { { 6., 0. }, { 4., 0. }, { 4., 0. }, { 1., 0. },
                          { 6., 0. }, { 1., 0. }, { 4., 0. },
                          { 6., 0. }, { 4., 0. }, { 6., 0. } };
    dcmplx zbp110[10] = { { 1., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
                          { 1., 0. }, { 0., 0. }, { 0., 0. },
                          { 1., 0. }, { 0., 0. }, { 1., 0. } };
 
    cmplx ca110[4][4] = { { { 6., 0. }, { 4., 0. }, { 4., 0. }, { 1., 0. } },
                          { { 4., 0. }, { 6., 0. }, { 1., 0. }, { 4., 0. } },
                          { { 4., 0. }, { 1., 0. }, { 6., 0. }, { 4., 0. } },
                          { { 1., 0. }, { 4., 0. }, { 4., 0. }, { 6., 0. } } };
    cmplx cb110[4][4] = { { { 1., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. } },
                          { { 0., 0. }, { 1., 0. }, { 0., 0. }, { 0., 0. } },
                          { { 0., 0. }, { 0., 0. }, { 1., 0. }, { 0., 0. } },
                          { { 0., 0. }, { 0., 0. }, { 0., 0. }, { 1., 0. } } };
    cmplx cap110[10] = { { 6., 0. }, { 4., 0. }, { 4., 0. }, { 1., 0. },
                         { 6., 0. }, { 1., 0. }, { 4., 0. },
                         { 6., 0. }, { 4., 0. }, { 6., 0. } };
    cmplx cbp110[10] = { { 1., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
                         { 1., 0. }, { 0., 0. }, { 0., 0. },
                         { 1., 0. }, { 0., 0. }, { 1., 0. } };
 
/*********************************************************
*  CALL STATEMENT FOR EIGENSYSTEM ANALYSIS SUBROUTINES : *
*********************************************************/
 
     sgeevx ("N","N","N","N",4,sa60g,4,swr60g,swi60g,&sdum160g,1,
           &sdum260g,1,&silo60g,&sihi60g, sscale60g,&sabnrm60g,
           &sdum360g,&sdum460g,swork60g,0,&sidum60g,&info);
 
     if (info != 0 || silo60g != 1 || sihi60g != 4) {
         printf( " INFO from SGEEVX  = %4d\n", info);
         *pass = *pass + 1;
     } else {
         svecck(1,1,&sabnrm60g,&sabnrmres60g,"sgeevx",pass);
         svecck(4,1,swr60g,swrres60g,"sgeevx",pass);
         svecck(4,1,swi60g,swires60g,"sgeevx",pass);
         svecck(4,1,sscale60g,sscaleres60g,"sgeevx",pass);
     }
 
     dgeevx ("N","N","N","N",4,da60g,4,dwr60g,dwi60g,&dum160g,1,
           &dum260g,1,&dilo60g,&dihi60g, dscale60g,&dabnrm60g,
           &dum360g,&dum460g,dwork60g,0,&idum60g,&info);
 
     if (info != 0 || dilo60g != 1 || dihi60g != 4) {
         printf( " INFO from DGEEVX  = %4d\n", info);
         *pass = *pass + 1;
     } else {
         dvecck(1,1,&dabnrm60g,&dabnrmres60g,"dgeevx",pass);
         dvecck(4,1,dwr60g,dwrres60g,"dgeevx",pass);
         dvecck(4,1,dwi60g,dwires60g,"dgeevx",pass);
         dvecck(4,1,dscale60g,dscaleres60g,"dgeevx",pass);
     }
 
     cgeevx ("N","N","N","N",3,ca60g,3,cw60g,cdum160g,1,cdum260g,
              1,&cilo60g,&cihi60g,cscale60g,&cabnrm60g,cdum360g,
              cdum460g,cwork60g,0,crdum60g,&info);
 
     if (info != 0 || cilo60g != 1 || cihi60g != 3 ) {
         printf( " INFO from CGEEVX  = %4d\n", info);
         *pass = *pass + 1;
     } else {
         cvecck(3,1,cw60g,cwres60g,"cgeevx",pass);
         svecck(1,1,&cabnrm60g,&cabnrmres60g,"cgeevx",pass);
     }
 
     zgeevx ("N","N","N","N",3,za60g,3,zw60g,zdum160g,1,zdum260g,
              1,&zilo60g,&zihi60g,zscale60g,&zabnrm60g,zdum360g,
              zdum460g,zwork60g,0,zrdum60g,&info);
 
     if (info != 0 || zilo60g != 1 || zihi60g != 3 ) {
         printf( " INFO from ZGEEVX  = %4d\n", info);
         *pass = *pass + 1;
     } else {
         zvecck(3,1,zw60g,zwres60g,"zgeevx",pass);
         dvecck(1,1,&zabnrm60g,&zabnrmres60g,"zgeevx",pass);
     }
 
     ssyevx("N","A","L",4,sap60y,4,0.0,0.0,0,0,-1.0,
                 &sm60y,sw60y,sz60y,4,swork60y,0,siwork60y,sifail60y,
                 &info);
     if (info != 0 || sm60y != 4) {
         printf( " INFO from SSYEVX  = %4d\n", info);
         *pass = *pass + 1;
     } else {
         svecck(4,1,sw60y,swres60y,"ssyevx",pass);
         smtchk(4,4,&sap60y[0][0],&sapres60y[0][0],4,"ssyevx",pass);
     }
 
     dsyevx("N","A","L",4,dap60y,4,0.0,0.0,0,0,-1.0,
                 &dm60y,dw60y,dz60y,4,dwork60y,0,diwork60y,difail60y,
                 &info);
     if (info != 0 || dm60y != 4) {
         printf( " INFO from DSYEVX  = %4d M: %4d\n", info,dm60y);
         *pass = *pass + 1;
     } else {
         dvecck(4,1,dw60y,dwres60y,"dsyevx",pass);
         dmtchk(4,4,&dap60y[0][0],&dapres60y[0][0],4,"dsyevx",pass);
     }
 
     cheevx ("N","A","L",3,ca60y,3,0.00,0.00,0,0,
                     -1.0,&cm60y,cw60y,cz60y,3,cwork60y,0,crwork60y,
                     ciwork60y,cifail60y, &info);
     if (info != 0 || cm60y != 3) {
         printf( " INFO from CHEEVX  = %4d\n", info);
         *pass = *pass + 1;
     } else {
         svecck(3,1,cw60y,cwres60y,"cheevx",pass);
         cmtchk(3,3,&ca60y[0][0],&cares60y[0][0],3,"cheevx",pass);
     }
 
     zheevx ("N","A","L",3,za60y,3,0.00,0.00,0,0,
                     -1.0,&zm60y,zw60y,zz60y,3,zwork60y,0,zrwork60y,
                     ziwork60y,zifail60y, &info);
     if (info != 0 || zm60y != 3) {
         printf( " INFO from ZHEEVX  = %4d\n", info);
         *pass = *pass + 1;
     } else {
         dvecck(3,1,zw60y,zwres60y,"zheevx",pass);
         zmtchk(3,3,&za60y[0][0],&zares60y[0][0],3,"zheevx",pass);
     }
 
    sspevx ( "N","A","L",4,sap60x, 0.00, 0.00, 0, 0,-1.00,
             &sam60x,
             saw60x,saz60x,4,swork60x,siwork60x,sifail60x,&info);
 
    if (info != 0 || sam60x != 4 ) {
        printf( " INFO from SSPEVX  = %4d M: %4d\n", info, sam60x);
        *pass = *pass + 1;
    } else {
        svecck(4, 1, saw60x, sawres60x, "sspevx", pass);
        svecck(6, 1, sap60x, sapres60x, "sspevx", pass);
    }
 
    dspevx ( "N","A","L",4,dap60x, 0.00, 0.00, 0, 0,-1.00,
             &dam60x,
             daw60x,daz60x,4,dwork60x,diwork60x,difail60x,&info);
 
    if (info != 0 || dam60x != 4 ) {
        printf( " INFO from DSPEVX  = %4d M: %4d\n", info, dam60x);
        *pass = *pass + 1;
    } else {
        dvecck(4, 1, daw60x, dawres60x, "dspevx", pass);
        dvecck(6, 1, dap60x, dapres60x, "dspevx", pass);
    }
 
    chpevx ( "N","A","L",3,cap60x, 0.00, 0.00, 0, 0,-1.00,
             &cam60x,
             caw60x,caz60x,3,cwork60x,crwork60x,ciwork60x,cifail60x,&info);
 
    if (info != 0 || cam60x != 3 ) {
        printf( " INFO from CHPEVX  = %4d M: %4d\n", info, cam60x);
        *pass = *pass + 1;
    } else {
        svecck(3, 1, caw60x, cawres60x, "chpevx", pass);
        cvecck(6, 1, cap60x, capres60x, "chpevx", pass);
    }
 
    zhpevx ( "N","A","L",3,zap60x, 0.00, 0.00, 0, 0,-1.00,
             &zam60x,
             zaw60x,zaz60x,3,zwork60x,zrwork60x,ziwork60x,zifail60x,&info);
 
    if (info != 0 || zam60x != 3 ) {
        printf( " INFO from ZHPEVX  = %4d M: %4d\n", info, zam60x);
        *pass = *pass + 1;
    } else {
        dvecck(3, 1, zaw60x, zawres60x, "zhpevx", pass);
        zvecck(6, 1, zap60x, zapres60x, "zhpevx", pass);
    }
 
/****************************************************
* ADDED D/SS(Y/P)EVD, Z/CH(E/P)EVD            10/12 *
****************************************************/
 
    dsyevd ("V","U",4,da111,4,dw111,dwork60x,0,diwork60x,0,&info);
    if (info != 0) {
         printf( " INFO from DSYEVD  = %4d\n", info);
         *pass = *pass + 1;
    } else {
         dvecck(4,1,dw111,dwq111,"dsyevd",pass);
         dmtchk(4,4,da111,dares111,4,"dsyevd",pass);
    }
 
    ssyevd ("V","U",4,sa111,4,sw111,swork60x,0,siwork60x,0,&info);
    if (info != 0) {
         printf( " INFO from SSYEVD  = %4d\n", info);
         *pass = *pass + 1;
    } else {
         svecck(4,1,sw111,swq111,"ssyevd",pass);
         smtchk(4,4,sa111,sares111,4,"ssyevd",pass);
    }
 
    zheevd ("V","U",3,za111,3,zw111,zwork60x,0,zrwork60x,0,ziwork60x,0,&info);
    if (info != 0) {
         printf( " INFO from ZHEEVD  = %4d\n", info);
         *pass = *pass + 1;
    } else {
         dvecck(3,1,zw111,zwq111,"zheevd",pass);
         zmtchk(3,3,za111,zares111,3,"zheevd",pass);
    }
 
    cheevd ("V","U",3,ca111,3,cw111,cwork60x,0,crwork60x,0,ciwork60x,0,&info);
    if (info != 0) {
         printf( " INFO from CHEEVD  = %4d\n", info);
         *pass = *pass + 1;
    } else {
         svecck(3,1,cw111,cwq111,"cheevd",pass);
         cmtchk(3,3,ca111,cares111,3,"cheevd",pass);
    }
 
    dspevd ("V","U",4,dap111,dw111,daz111,4,dwork60x,0,diwork60x,0,&info);
    if (info != 0) {
         printf( " INFO from DSPEVD  = %4d\n", info);
         *pass = *pass + 1;
    } else {
         dvecck(4,1,dw111,dwq111,"dspevd",pass);
         dmtchk(4,4,daz111,dares111,4,"dspevd",pass);
    }
 
    sspevd ("V","U",4,sap111,sw111,saz111,4,swork60x,0,siwork60x,0,&info);
    if (info != 0) {
         printf( " INFO from SSPEVD  = %4d\n", info);
         *pass = *pass + 1;
    } else {
         svecck(4,1,sw111,swq111,"sspevd",pass);
         smtchk(4,4,saz111,sares111,4,"sspevd",pass);
    }
 
    zhpevd ("V","U",3,zap111,zw111,zaz111,3,zwork60x,0,zrwork60x,0,ziwork60x,0,&info);
    if (info != 0) {
         printf( " INFO from ZHPEVD  = %4d\n", info);
         *pass = *pass + 1;
    } else {
         dvecck(3,1,zw111,zwq111,"zhpevd",pass);
         zmtchk(3,3,zaz111,zares111,3,"zhpevd",pass);
    }
 
    chpevd ("V","U",3,cap111,cw111,caz111,3,cwork60x,0,crwork60x,0,ciwork60x,0,&info);
    if (info != 0) {
         printf( " INFO from CHPEVD  = %4d\n", info);
         *pass = *pass + 1;
    } else {
         svecck(3,1,cw111,cwq111,"chpevd",pass);
         cmtchk(3,3,caz111,cares111,3,"chpevd",pass);
    }
 
/****************************************************
* ADDED D/S/Z/CGGEV                           10/12 *
****************************************************/
 
    dggev ("V","V",5,da112,5,db112,5,dalphr112,dalphi112,
           dbeta112,dvl112,5,dvr112,5,dwork112,50,&info);
    if (info != 0) {
         printf( " INFO from DGGEV  = %4d\n", info);
         *pass = *pass + 1;
    } else {
         dvecck(5,1,dalphr112,dalphrres112,"dggev ",pass);
         dvecck(5,1,dalphi112,dalphires112,"dggev ",pass);
         dvecck(5,1,dbeta112,dbetares112,"dggev ",pass);
         dmtchk(5,5,dvl112,dvlres112,5,"dggev ",pass);
         dmtchk(5,5,dvr112,dvrres112,5,"dggev ",pass);
    }
 
    sggev ("V","V",5,sa112,5,sb112,5,salphr112,salphi112,
           sbeta112,svl112,5,svr112,5,swork112,50,&info);
    if (info != 0) {
         printf( " INFO from SGGEV  = %4d\n", info);
         *pass = *pass + 1;
    } else {
         svecck(5,1,salphr112,salphrres112,"sggev ",pass);
         svecck(5,1,salphi112,salphires112,"sggev ",pass);
         svecck(5,1,sbeta112,sbetares112,"sggev ",pass);
         smtchk(5,5,svl112,svlres112,5,"sggev ",pass);
         smtchk(5,5,svr112,svrres112,5,"sggev ",pass);
    }
 
    zggev ("V","V",3,za112,3,zb112,3,zalph112,
           zbeta112,zvl112,3,zvr112,3,zwork112,30,zrwork112,&info);
    if (info != 0) {
         printf( " INFO from ZGGEV  = %4d\n", info);
         *pass = *pass + 1;
    } else { 
         zvecck(3,1,zalph112,zalphres112,"zggev ",pass);
         zvecck(3,1,zbeta112,zbetares112,"zggev ",pass);
         zmtchk(3,3,zvl112,zvlres112,3,"zggev ",pass);
         zmtchk(3,3,zvr112,zvrres112,3,"zggev ",pass);
    }
 
    cggev ("V","V",3,ca112,3,cb112,3,calph112,
           cbeta112,cvl112,3,cvr112,3,cwork112,30,crwork112,&info);
    if (info != 0) {
         printf( " INFO from ZGGEV  = %4d\n", info);
         *pass = *pass + 1;
    } else { 
         cvecck(3,1,calph112,calphres112,"cggev ",pass);
         cvecck(3,1,cbeta112,cbetares112,"cggev ",pass);
         cmtchk(3,3,cvl112,cvlres112,3,"cggev ",pass);
         cmtchk(3,3,cvr112,cvrres112,3,"cggev ",pass);
    }
 
    dsygvx (2, "V","I","U",4,da110,4,db110,4,0.00,0.00,1,2,-1.00,
             &dam60x,dw110,daz110,4,dwork60x,0,diwork60x,difail110x,&info);
    if (info != 0 || dam60x != 2) {
         printf( " INFO from DSYGVX  = %4d M: %4d\n", info,dam60x);
         *pass = *pass + 1;
    } else {
         dvecck(2, 1, dw110, dwq110, "dsygvx", pass);
         dmtchk(4,4,&db110[0][0],&dbres110[0][0],4,"dsygvx",pass);
         dmtchk(4,2,&daz110[0][0],&dzres110[0][0],4,"dsygvx",pass);
    }
 
/****************************************************
* ADDED DSPGVX, SS(Y/P)GVX, Z/CH(E/P)GVX      10/12 *
****************************************************/
 
    ssygvx (2,"V","I","U",4,sa110,4,sb110,4,0.0,0.0,1,2,-1.0,
            &sam60x,sw110,saz110,4,swork60x,0,siwork60x,sifail110x,&info);
    if (info != 0 || sam60x != 2) {
         printf( " INFO from SSYGVX  = %4d M: %4d\n", info,sam60x);
         *pass = *pass + 1;
    } else {
         svecck(2, 1, sw110, swq110, "ssygvx", pass);
         smtchk(4,4,sb110,sbres110,4,"ssygvx",pass);
         smtchk(4,2,saz110,szres110,4,"ssygvx",pass);
    }
 
    zhegvx (2,"V","I","L",4,za110,4,zb110,4,0.0,0.0,1,2,-1.0,
            &zam60x,zw110,zaz110,4,zwork60x,0,zrwork60x,ziwork60x,zifail110x,&info);
    if (info != 0 || zam60x != 2) {
         printf( " INFO from ZHEGVX  = %4d M: %4d\n", info,zam60x);
         *pass = *pass + 1;
    } else {
         dvecck(2, 1, zw110, zwq110, "zhegvx", pass);
         zmtchk(4,4,zb110,zbres110,4,"zhegvx",pass);
         zmtchk(4,2,zaz110,zzres110,4,"zhegvx",pass);
    }
 
    chegvx (2,"V","I","L",4,ca110,4,cb110,4,0.0,0.0,1,2,-1.0,
            &cam60x,cw110,caz110,4,cwork60x,0,crwork60x,ciwork60x,cifail110x,&info);
    if (info != 0 || cam60x != 2) {
         printf( " INFO from CHEGVX  = %4d M: %4d\n", info,cam60x);
         *pass = *pass + 1;
    } else {
         svecck(2, 1, cw110, cwq110, "chegvx", pass);
         cmtchk(4,4,cb110,cbres110,4,"chegvx",pass);
         cmtchk(4,2,caz110,czres110,4,"chegvx",pass);
    }
 
    dspgvx (2, "V","I","U",4,dap110,dbp110,0.0,0.0,1,2,-1.0,
             &dam60x,dw110,daz110,4,dwork60x,diwork60x,difail110x,&info);
    if (info != 0 || dam60x != 2) {
         printf( " INFO from DSPGVX  = %4d M: %4d\n", info,dam60x);
         *pass = *pass + 1;
    } else {
         dvecck(2, 1, dw110, dwq110, "dspgvx", pass);
         dvecck(10,1,dbp110,dbpres110,"dspgvx",pass);
         dmtchk(4,2,daz110,dzres110,4,"dspgvx",pass);
    }
 
    sspgvx (2, "V","I","U",4,sap110,sbp110,0.0,0.0,1,2,-1.0,
             &sam60x,sw110,saz110,4,swork60x,siwork60x,sifail110x,&info);
    if (info != 0 || sam60x != 2) {
         printf( " INFO from SSPGVX  = %4d M: %4d\n", info,sam60x);
         *pass = *pass + 1;
    } else {
         svecck(2, 1, sw110, swq110, "sspgvx", pass);
         svecck(10,1,sbp110,sbpres110,"sspgvx",pass);
         smtchk(4,2,saz110,szres110,4,"sspgvx",pass);
    }
 
    zhpgvx (2, "V","I","L",4,zap110,zbp110,0.0,0.0,1,2,-1.0,
             &zam60x,zw110,zaz110,4,zwork60x,zrwork60x,ziwork60x,zifail110x,&info);
    if (info != 0 || zam60x != 2) {
         printf( " INFO from ZHPGVX  = %4d M: %4d\n", info,zam60x);
         *pass = *pass + 1;
    } else {
         dvecck(2, 1, zw110, zwq110, "zhpgvx", pass);
         zvecck(10,1,zbp110,zbpres110,"zhpgvx",pass);
         zmtchk(4,2,zaz110,zzres110,4,"zhpgvx",pass);
    }
 
    chpgvx (2, "V","I","L",4,cap110,cbp110,0.0,0.0,1,2,-1.0,
             &cam60x,cw110,caz110,4,cwork60x,crwork60x,ciwork60x,cifail110x,&info);
    if (info != 0 || cam60x != 2) {
         printf( " INFO from CHPGVX  = %4d M: %4d\n", info,cam60x);
         *pass = *pass + 1;
    } else {
         svecck(2, 1, cw110, cwq110, "chpgvx", pass);
         cvecck(10,1,cbp110,cbpres110,"chpgvx",pass);
         cmtchk(4,2,caz110,czres110,4,"chpgvx",pass);
    }
 
}
 
/**********************************************************************
*                      SUBROUTINE:  SIGNS                             *
**********************************************************************/
 
void signs(pass)
_ESVINT *pass;
{
/************************************************
*  VARIABLES OF SIGNAL PROCESSING SUBROUTINES : *
************************************************/
 
/**********************
* (FOURIER TRANSFORM) *
**********************/
 
    _ESVINT i;
    float *fp;
    cmplx  *cp;
    cmplx  sy61[32];
    dcmplx dy61[8];
    cmplx  sy62[36];
/*   double daux1[42000], daux2[37500], daux3[50];  */
    float  sy63[12];
    float  sy64[72];
    cmplx  sy65[8][6];
    cmplx  sy66[8][7];
    float  sy67[8][14];
    cmplx  sy111[7 * 12 * 12];
    float  sy112[12 * 12][14];
    double daux1[50000],daux2[50000],daux3[10000];
    dcmplx zaux1[12];
    double *dp;
    dcmplx *dcp;
    dcmplx  dft2y40n[8][6];
    dcmplx  drft2y40n[8][7];
    double  dcft2y40n[8][14];
 
    dcmplx  dft3y41n[6*6*6];
    dcmplx  drft3y41n[7*12*12];
    double  dcft3y41n[12*12][14];
 
/******************************************
* (CONVOLUTION AND CORRELATION VARIABLES) *
******************************************/
 
    float  sy68[45];
    float  sy69[11];
    float  sy70[17];
    float  sy71[6];
    double dy71[6];
    float  sy72[45];
    float  sy74[17];
    float  sy75[6];
    double dy75[6];
    float  sy76[21], sy77[21];
 
/******************************
* (GENERAL SIGNAL PROCESSING) *
*     (3838 ALGORITHMS)       *
******************************/
 
    _ESVINT idum=1;
    float  sy78[3], sdum[1];
    double dy78[3], ddum[1];
    _ESVINT iy79[2];
    float  sy80[8];
    double dy80[8];
    float  sy81[4];
    double dy81[4];
    float  sy82[4];
    double dy82[4];
    cmplx  cy82[4];
    dcmplx zy82[4];
 
/*****************************************************
*  EXPECTED VALUE FOR SIGNAL PROCESSING SUBROUTINES : *
*****************************************************/
 
    cmplx  sres61[32] = {
        { 8., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
        { 0., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
        { 0., 0. }, { 8., 0. }, { 0., 0. }, { 0., 0. },
        { 0., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
        { 0., 0. }, { 0., 0. }, { 8., 0. }, { 0., 0. },
        { 0., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
        { 0., 0. }, { 0., 0. }, { 0., 0. }, { 8., 0. },
        { 0., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. } };
    dcmplx dres61[8] = {
        { 1., 0. }, { 1., 0. }, { 1., 0. }, { 1., 0. },
        { 1., 0. }, { 1., 0. }, { 1., 0. }, { 1., 0. } };
    cmplx  sres62[36] = {
        { 1.0, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 },
        { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 },
        { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.5, 0.0 }, { 0.0, 0.0 },
        { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 },
        { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 },
        { 0.5, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 },
        { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 },
        { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.5, 0.0 }, { 0.0, 0.0 },
        { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 }, { 0.0, 0.0 } };
    float  sres63[12] = { 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    float  sres64[72] = { 32., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
                           0., 0., 0., 0., 0., 0., 0., 16., 0., 0., 0., 0.,
                           0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
                           0., 0., 16., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
                           0., 0., 0., 0., 0., 0., 0., 0., 0., 16., 0., 0.,
                           0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
 
    float  scosfyres64[72] = {
             16.0,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,
             0.,8.00,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,
             0.,0.,8.00,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,
             0.,0.,0.,8.00,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.};
 
    double dcosfyres64[72] = {
             16.0,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,
             0.,8.00,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,
             0.,0.,8.00,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,
             0.,0.,0.,8.00,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.};
 
    float  ssinfyres64[54] = {
             0.,8.00,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,
             0.,0.,8.00,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,
             0.,0.,0.,8.00,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.};
 
    double  dsinfyres64[54] = {
             0.,8.00,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,
             0.,0.,8.00,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,
             0.,0.,0.,8.00,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.};
    cmplx  sres65[8][6];  /* initialized in program */
    cmplx  sres66[8][7];  /* initialized in program */
    float  sres67[8][14];  /* initialized in program */
    float  sres68[45] = {  1.,  4., 10., 20., 30., 40., 48., 52., 50.,
                          40., 29., 18.,  8.,  0.,  0.,  2.,  5., 10.,
                          18., 20., 30., 40., 48., 52., 50., 38., 25.,
                          12.,  0.,  0.,  3.,  8., 14., 22., 18., 20.,
                          30., 40., 48., 52., 47., 32., 16.,  0.,  0. };
    float  sres69[11] = { 1.1e1, 3.4e1,  7.0e1,  1.2e2, 1.3e2, 1.4e2,
                          1.5e2, 1.6e2, 1.51e2, 1.22e2, 7.2e1 };
    float  sres70[17] = { 1.1e1, 3.4e1, 7.e1, 12.e1, 18.5e1, 26.6e1,
                          36.4e1, 48.e1, 51.6e1, 55.2e1, 56.7e1, 56.e1,
                          53.e1, 47.6e1, 39.7e1, 29.2e1, 16.e1 };
    float  sres71[6] = { 11., 70., 130., 150., 151., 72. };
    double dres71[6] = { 11., 70., 130., 150., 151., 72. };
    float  sres72[45] = {  4., 11., 20., 30., 40., 50., 52., 48., 40.,
                          30., 16.,  7.,  2.,  0.,  0.,  8., 10., 15.,
                          22., 30., 40., 50., 52., 48., 40., 22., 10.,
                           3.,  0.,  0., 12., 17., 16., 18., 22., 30.,
                          40., 50., 52., 48., 28., 13.,  4.,  0.,  0. };
    float  sres73[11] = { 4.4e1, 8.1e1, 1.1e2, 1.3e2, 1.4e2, 1.5e2,
                          1.6e2, 1.7e2, 1.04e2, 5.3e1, 1.8e1 };
    float  sres74[17] = {  88., 173., 254., 330., 400., 463.,
                          518., 564., 600., 636., 504., 385.,
                          280., 190., 116.,  59.,  20. };
    float  sres75[6] = { 44., 110., 140., 160., 104., 18. };
    double dres75[6] = { 44., 110., 140., 160., 104., 18. };
    float  sres76[21] = { 59., 54., 43., 29., 16.,  7.,  2.,
                          59., 50., 39., 27., 18., 11.,  6.,
                          59., 44., 30., 24., 21., 20., 12. };
    float  sres78[3] = { 4., 4., 4. };
    double dres78[3] = { 4., 4., 4. };
    _ESVINT res79[2] = { 9, 5 };
    float  sres80[8] = { 4., 10., 31., 94., 190., 193., 196., 394. };
    double dres80[8] = { 4., 10., 31., 94., 190., 193., 196., 394. };
    float  sres81[4] = { 9., 11., 13., 15. };
    double dres81[4] = { 9., 11., 13., 15. };
    float  sres82[4] = { 5.1111, 5.5555, 1.22222e1, 1.04444e1 };
    double dres82[4] = { 5.1111, 5.5555, 1.22222e1, 1.04444e1 };
    cmplx  cres82[4] = { {1.0,0.0},{3.0,0.0},{5.0,0.0},{7.0,0.0} };
    dcmplx zres82[4] = { {1.0,0.0},{3.0,0.0},{5.0,0.0},{7.0,0.0} };
 
    double daux1chk[1] = {0.0};
    dcmplx zaux1chk[1] = { {0.0,0.0} };
    cmplx  sq110[6 * 6 * 6];  /* initialized in program */
    cmplx  sq111[7 * 12 * 12];  /* initialized in program */
    float  sq112[144][14];  /* initialized in program */
    dcmplx dft2res40n[8][6];/* initialized in program*/
    dcmplx drft2res40n[8][7];
    double dcft2res40n[8][14];
 
    dcmplx dft3res41n[6*6*6];
    dcmplx drft3res41n[7*12*12];
    double dcft3res41n[144][14];
 
/**************************************************
*  INPUT DATA FOR SIGNAL PROCESSING SUBROUTINES : *
**************************************************/
 
    cmplx  sx61[32] = {
        { 1., 0. }, { 1., 0. }, { 1., 0. }, { 1., 0. },
        { 1., 0. }, { 1., 0. }, { 1., 0. }, { 1., 0. },
        { 1., 0. }, { 0.7071, 0.7071 }, { 0., 1. }, { -0.7071, 0.7071 },
        { -1., 0. }, { -0.7071, -0.7071 }, { 0., -1. }, { 0.7071, -0.7071 },
        { 1., 0. }, { 0., 1. }, { -1., 0. }, { 0., -1. },
        { 1., 0. }, { 0., 1. }, { -1., 0. }, { 0., -1. },
        { 1., 0. }, { -0.7071, 0.7071 }, { 0., -1. }, { 0.7071, 0.7071 },
        { -1., 0. }, { 0.7071, -0.7071 }, { 0., 1. }, { -0.7071, -0.7071 } };
    dcmplx dx61[8] = {
        { 1., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
        { 0., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. } };
 
     dcmplx dx98[32] = { { 1.,     0.    }, { 1.,     0.    }, { 1.,     0.    },
                         { 1.,     0.    }, { 1.,     0.    }, { 1.,     0.    },
                         { 1.,     0.    }, { 1.,     0.    }, { 1.,     0.    },
                         { 0.7071, 0.7071}, { 0.,     1.    }, {-0.7071, 0.7071},
                         {-1.,     0.    }, {-0.7071,-0.7071}, { 0.,    -1.    },
                         { 0.7071,-0.7071}, { 1.,     0.    }, { 0.,     1.    },
                         {-1.,     0.    }, { 0.,    -1.    }, { 1.,     0.    },
                         { 0.,     1.    }, {-1.,     0.    }, { 0.,    -1.    },
                         { 1.,     0.    }, {-0.7071, 0.7071}, { 0.,    -1.    },
                         { 0.7071, 0.7071}, {-1.,      0.   }, { 0.7071,-0.7071},
                         { 0.,     1.    }, {-0.7071,-0.7071} };
 
     dcmplx dyres98[32] = { {8., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.}, {0., 0.},
                          {8., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.}, {0., 0.},
                          {8., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.}, {0., 0.},
                          {8., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.} };
 
     dcmplx dy98[32];
     _ESVINT dincx98[1]  = {1};
     _ESVINT dincy98[1]  = {1};
     _ESVINT dn98[1]     = {8};
     _ESVINT dn99[1]     = {16};
     _ESVINT dn100[1]     = {12};
     _ESVINT disign98[1] = {1};
     double daux198[30000];
     double daux298[1];
 
     cmplx sx98[32] = { { 1.,     0.    }, { 1.,     0.    }, { 1.,     0.    },
                         { 1.,     0.    }, { 1.,     0.    }, { 1.,     0.    },
                         { 1.,     0.    }, { 1.,     0.    }, { 1.,     0.    },
                         { 0.7071, 0.7071}, { 0.,     1.    }, {-0.7071, 0.7071},
                         {-1.,     0.    }, {-0.7071,-0.7071}, { 0.,    -1.    },
                         { 0.7071,-0.7071}, { 1.,     0.    }, { 0.,     1.    },
                         {-1.,     0.    }, { 0.,    -1.    }, { 1.,     0.    },
                         { 0.,     1.    }, {-1.,     0.    }, { 0.,    -1.    },
                         { 1.,     0.    }, {-0.7071, 0.7071}, { 0.,    -1.    },
                         { 0.7071, 0.7071}, {-1.,      0.   }, { 0.7071,-0.7071},
                         { 0.,     1.    }, {-0.7071,-0.7071} };
 
     cmplx syres98[32] = { {8., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.}, {0., 0.},
                          {8., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.}, {0., 0.},
                          {8., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.}, {0., 0.},
                          {8., 0.}, {0., 0.}, {0., 0.},
                          {0., 0.}, {0., 0.} };
 
     cmplx sy98[32];
     _ESVINT sincx98[1]  = {1};
     _ESVINT sincy98[1]  = {1};
     _ESVINT sn98[1]     = {8};
     _ESVINT sn99[1]     = {16};
     _ESVINT sn100[1]     = {12};
     _ESVINT sisign98[1] = {1};
     double saux198[30000];
     double saux298[1];
 
    float  sx62[64] = {
        1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
        1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
        1.0, 0.9239, 0.7071, 0.3827, 0.0, -0.3827, -0.7071, -0.9239,
        -0.1e1,-0.9239, -0.7071, -0.3827, 0.0, 0.3827, 0.7071, 0.9239,
        0.1e1, 0.7071, 0., -0.7071, -0.1e1, -0.7071, 0., 0.7071,
        0.1e1, 0.7071, 0., -0.7071, -0.1e1, -0.7071, 0., 0.7071,
        0.1e1, 0.3827, -0.7071, -0.9239, 0.0, 0.9239, 0.7071, -0.3827,
        -0.1e1, -0.3827, 0.7071, 0.9239, 0.0, -0.9239, -0.7071, 0.3827 };
    cmplx  sx63[7] = { { 1., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
                         { 0., 0. }, { 0., 0. }, { 0., 0. } };
    float  sx64[72] = {
        1., 1., 1., 1., 1., 1., 1., 1.,
        1., 1., 1., 1., 1., 1., 1., 1.,
        1., 0., 1., 0.9808, 0.9239, 0.8315, 0.7071, 0.5556,
        0.3827, 0.1951, 0., -0.1951, -0.3827, -0.5556, -0.7071, -0.8315,
       -0.9239, -0.9808, -1., 0., 1., 0.9239, 0.7071, 0.3827,
        0., -0.3827, -0.7071, -0.9239, -1., -0.9239, -0.7071, -0.3827,
        0., 0.3827, 0.7071, 0.9239, 1., 0., 1., 0.8315,
        0.3827, -0.1951, -0.7071, -0.9808, -0.9239, -0.5556, 0., 0.5556,
        0.9239, 0.9808, 0.7071, 0.1951, -0.3827, -0.8315, -1., 0. };
    float scosfx64[72] = {
          1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,0.,
        1.,0.9808,0.9239,0.8315,0.7071,0.5556,0.3827,0.1951,0.0,
        -0.1951,-0.3827,-0.5556,-0.7071,-0.8315,-0.9239,-0.9808,-1.,0.,
        1.,0.9239,0.7071,0.3827,0.,-0.3827,-0.7071,-0.9239,-1.0,
        -0.9239,-0.7071,-0.3827,0.0,0.3827,0.7071,0.9239,1.0,0.,
        1.,0.8315,0.3827,-0.1951,-0.7071,-0.9808,-0.9239,-0.5556,0.,
        0.5556,0.9239,0.9808,0.7071,0.1951,-0.3827,-0.8315,-1.,0.};
 
    double dcosfx64[72] = {
        1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,1.,0.,
        1.,0.9808,0.9239,0.8315,0.7071,0.5556,0.3827,0.1951,0.0,
        -0.1951,-0.3827,-0.5556,-0.7071,-0.8315,-0.9239,-0.9808,-1.,0.,
        1.,0.9239,0.7071,0.3827,0.,-0.3827,-0.7071,-0.9239,-1.0,
        -0.9239,-0.7071,-0.3827,0.0,0.3827,0.7071,0.9239,1.0,0.,
        1.,0.8315,0.3827,-0.1951,-0.7071,-0.9808,-0.9239,-0.5556,0.,
        0.5556,0.9239,0.9808,0.7071,0.1951,-0.3827,-0.8315,-1.,0.};
 
    float scosfy64[72];
    double dcosfy64[72];
 
    float  ssinfx64[54] = {
        0.,0.1951,0.3827,0.5556,0.7071,0.8315,0.9239,0.9808,1.0,
        0.9808,0.9239,0.8315,0.7071,0.5556,0.3827,0.1951,0.,0.,
        0.,0.3827,0.7071,0.9239,1.0,0.9239,0.7071,0.3827,0.,-0.3827,
        -0.7071,-0.9239,-1.0,-0.9239,-0.7071,-0.3827,0.,0.,
        0.,0.5556,0.9239,0.9808,0.7071,0.1951,-0.3827,-0.8315,-1.0,
        -0.8315,-0.3827,0.1951,0.7071,0.9808,0.9239,0.5556,0.,0.,};
 
    double dsinfx64[54] = {
        0.,0.1951,0.3827,0.5556,0.7071,0.8315,0.9239,0.9808,1.0,
        0.9808,0.9239,0.8315,0.7071,0.5556,0.3827,0.1951,0.,0.,
        0.,0.3827,0.7071,0.9239,1.0,0.9239,0.7071,0.3827,0.,-0.3827,
        -0.7071,-0.9239,-1.0,-0.9239,-0.7071,-0.3827,0.,0.,
        0.,0.5556,0.9239,0.9808,0.7071,0.1951,-0.3827,-0.8315,-1.0,
        -0.8315,-0.3827,0.1951,0.7071,0.9808,0.9239,0.5556,0.,0.,};
 
    float ssinfy64[54];
    double dsinfy64[54];
    cmplx  sx65[8][6];  /* initialized in program */
    float  sx66[8][12];  /* initialized in program */
    cmplx  sx67[8][7];  /* initialized in program */
    float  sh68[4] = { 1., 2., 3., 4. };
    float  sx68[30] = { 1., 2., 3., 4., 5., 6., 5., 4., 3., 2.,
                        2., 1., 2., 3., 4., 5., 6., 5., 4., 3.,
                        3., 2., 1., 2., 3., 4., 5., 6., 5., 4. };
    float  sh69[4] = { 1., 2., 3., 4. };
    float  sx69[8] = { 1.1e1, 1.2e1, 1.3e1, 1.4e1, 1.5e1, 1.6e1, 1.7e1, 1.8e1};
    float  sh70[8] = { 1., 2., 3., 4., 5., 6., 7., 8. };
    float  sx70[1][10] = { 11., 12., 13., 14., 15., 16., 17., 18., 19., 20. };
    float  sh71[4] = { 1., 2., 3., 4. };
    float  sx71[8] = { 11., 12., 13., 14., 15., 16., 17., 18. };
    double dh71[4] = { 1., 2., 3., 4. };
    double dx71[8] = { 11., 12., 13., 14., 15., 16., 17., 18. };
    float  sh72[4] = { 1., 2., 3., 4. };
    float  sx72[30] = { 1., 2., 3., 4., 5., 6., 5., 4., 3., 2.,
                        2., 1., 2., 3., 4., 5., 6., 5., 4., 3.,
                        3., 2., 1., 2., 3., 4., 5., 6., 5., 4. };
    float  sh74[8] = { 1., 2., 3., 4., 5., 6., 7., 8. };
    float  sx74[2][10] = { 11., 12., 13., 14., 15., 16., 17.,
                           18., 19., 20., 12., 13., 14., 15.,
                           16., 17., 18., 19., 20., 11. };
    float  sh75[4] = { 1., 2., 3., 4. };
    double dh75[4] = { 1., 2., 3., 4. };
    float  sx75[8] = { 11., 12., 13., 14., 15., 16., 17., 18. };
    double dx75[8] = { 11., 12., 13., 14., 15., 16., 17., 18. };
    float  sx76[21] = { 1., 2., 3., 4., 4., 3., 2.,
                        2., 1., 2., 3., 4., 4., 3.,
                        3., 2., 1., 2., 3., 4., 4. };
    float  su78[1] = { 4. };
    double du78[1] = { 4. };
    float  sx79[12] = { 2., -1., -3., 3., 0., 8., -2., 0., -5., -3., 2., -9. };
    double dx79[12] = { 2., -1., -3., 3., 0., 8., -2., 0., -5., -3., 2., -9. };
    float  su80[8] = { 1., 2., 3., 3., 2., 1., 1., 2. };
    double du80[8] = { 1., 2., 3., 3., 2., 1., 1., 2. };
    float  sx80[8] = { 3., 2., 1., 1., 2., 3., 3., 2. };
    double dx80[8] = { 3., 2., 1., 1., 2., 3., 3., 2. };
    float  sx81[8] = { 1., 2., 3., 4., 5., 6., 7., 8. };
    double dx81[8] = { 1., 2., 3., 4., 5., 6., 7., 8. };
    float  st81[4] = { 1.5, 2.5, 3.5, 4.5 };
    double dt81[4] = { 1.5, 2.5, 3.5, 4.5 };
    float  sx82[4] = { 1.0e1, -8.0, 7.0, -5.0 };
    double dx82[4] = { 1.0e1, -8.0, 7.0, -5.0 };
    cmplx  cx82[4] = { {10.0,0.0},{2.0,-3.0},{-3.0,1.0},{1.0,1.0} };
    dcmplx zx82[4] = { {10.0,0.0},{2.0,-3.0},{-3.0,1.0},{1.0,1.0} };
    float  su82[4] = { 4.0e1, -1.0e1, 3.0e1, 2.0e1 };
    double du82[4] = { 4.0e1, -1.0e1, 3.0e1, 2.0e1 };
    cmplx  cu82[4] = { {8.0,3.0},{21.0,-5.0},{67.0,-13.0},{72.0,11.0} };
    dcmplx zu82[4] = { {8.0,3.0},{21.0,-5.0},{67.0,-13.0},{72.0,11.0} };
    cmplx  sx110[6 * 6 * 6];  /* initialized in program */
    float sx111[12 * 12 * 12];  /* initialized in program */
    cmplx  sx112[7 * 12 * 12];  /* initialized in program */
 
/*  drcft and dcrft  routines  */
 
/* double daux1[42000],daux2[36000];  */
    double dy84[12];
    dcmplx dy83[64];
 
    double dres84[12] ={1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0};
    dcmplx dres83[36] = {{1.0,0.0}, {0.0,0.0},{0.0,0.0},{0.0,0.0},
                      {0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0},
                   {0.0,0.0},{0.5,0.0},
                      {0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0},
                      {0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0},{0.5,0.0},
                      {0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0},
                      {0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0},{0.5,0.0},
                      {0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0}};
    dcmplx dx84[7] ={{1.0,0.0},{0.0,0.0},{0.0,0.0},{0.0,0.0},
                       {0.0,0.0},{0.0,0.0},{0.0,0.0}};
    double dx83[64] ={1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,
                      1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0,
                           0.92390,  0.70710,  0.38270,  0.00,
                          -0.38270, -0.70710, -0.92390, -0.1e1,
                          -0.92390, -0.70710, -0.38270,  0.0,
                           0.38270,  0.70710,  0.92390,  0.1e1,
                           0.70710,  0.0, -0.70710, -0.1e1,
                          -0.70710,  0.0,  0.70710,  0.1e1,
                           0.70710,  0.0, -0.70710, -0.1e1,
                          -0.70710,  0.0,  0.70710,  0.1e1,
                           0.38270, -0.70710, -0.92390,  0.00,
                           0.92390,  0.70710, -0.38270, -0.1e1,
                          -0.38270,  0.70710,  0.92390,  0.00,
                          -0.92390, -0.70710,  0.38270 };
    dcmplx  dft2x40n[8][6];
    double  drft2x40n[8][12];
    dcmplx  dcft2x40n[8][7];
 
    dcmplx  dft3x41n[6*6*6];
    double  drft3x41n[12*12*12];
    dcmplx  dcft3x41n[7*12*12];
 
    float sscale;
    double dscale;
 
/******************************************************
*  CALL STATEMENT FOR SIGNAL PROCESSING SUBROUTINES : *
******************************************************/
 
/**********************
* (FOURIER TRANSFORM) *
**********************/
 
    scftd(1,1,sx98,sincx98,8,sy98,sincy98,8,sn98,4,sisign98,1.0,saux198,30000,saux298,0);
    scftd(0,1,sx98,sincx98,8,sy98,sincy98,8,sn98,4,sisign98,1.0,saux198,30000,saux298,0);
    cvecck(32,1,sy98,syres98,"scftd",pass);
 
    dcftd(1,1,dx98,dincx98,8,dy98,dincy98,8,dn98,4,disign98,1.0,daux198,30000,daux298,0);
    dcftd(0,1,dx98,dincx98,8,dy98,dincy98,8,dn98,4,disign98,1.0,daux198,30000,daux298,0);
    zvecck(32,1,dy98,dyres98,"dcftd",pass);
 
    scft (1, sx61, 1, 8, sy61, 1, 8, 8, 4, 1, 1., daux1, 1693, daux2, 0);
    scft (0, sx61, 1, 8, sy61, 1, 8, 8, 4, 1, 1., daux1, 1693, daux2, 0);
    cvecck(32, 1, sy61, sres61, "scft", pass);
    dcft (1, dx61, 1, 0, dy61, 1, 0, 8, 1, 1, 1., daux1, 26, daux2, 0);
    dcft (0, dx61, 1, 0, dy61, 1, 0, 8, 1, 1, 1., daux1, 26, daux2, 0);
    zvecck(8, 1, dy61, dres61, "dcft", pass);
 
    srcft (1, sx62, 16, sy62, 9, 16, 4, 1, 1. / 16, daux1, 41928,
        daux2, 0, daux3, 0);
    srcft (0, sx62, 16, sy62, 9, 16, 4, 1, 1. / 16, daux1, 41928,
        daux2, 0, daux3, 0);
 
    cvecck(36, 1, sy62, sres62, "srcft", pass);
 
    srcftd (1, 1, sx62, sincx98, 16, sy62, sincy98, 9, sn99, 4, sisign98,
        1. / 16, daux1, 1000, daux2, 0);
    srcftd (0, 1, sx62, sincx98, 16, sy62, sincy98, 9, sn99, 4, sisign98,
        1. / 16, daux1, 1000, daux2, 0);
 
    cvecck(36, 1, sy62, sres62, "srcftd", pass);
 
    scrft (1, sx63, 0, sy63, 0, 12, 1, 1, 1., daux1, 20000, daux2, 0,
           daux3, 0);
    scrft (0, sx63, 0, sy63, 0, 12, 1, 1, 1., daux1, 20000, daux2, 0,
           daux3, 0);
    svecck(12, 1, sy63, sres63, "scrft", pass);
 
    scrftd (1, 1, sx63, sincx98, 7, sy63, sincy98, 1, sn100, 1, sisign98, 1.,
          daux1, 100, daux2, 0);
    scrftd (0, 1, sx63, sincx98, 7, sy63, sincy98, 1, sn100, 1, sisign98, 1.,
          daux1, 100, daux2, 0);
    svecck(12, 1, sy63, sres63, "scrftd", pass);
 
    scosft (1, sx64, 1, 18, sy64, 1, 18, 16, 4, 1., daux1, 414, daux2, 8960);
    scosft (0, sx64, 1, 18, sy64, 1, 18, 16, 4, 1., daux1, 414, daux2, 8960);
    svecck(17, 1, &sy64[0],  &sres64[0],  "scosft", pass);
    svecck(17, 1, &sy64[18], &sres64[18], "scosft", pass);
    svecck(17, 1, &sy64[36], &sres64[36], "scosft", pass);
    svecck(17, 1, &sy64[54], &sres64[54], "scosft", pass);
    sscale = 1.0;
  scosf(1,scosfx64,1,18,scosfy64,1,18,32,4,sscale,daux1,414,daux2,0);
  scosf(0,scosfx64,1,18,scosfy64,1,18,32,4,sscale,daux1,414,daux2,0);
    svecck(17, 1, &scosfy64[0],  &scosfyres64[0],  "scosf", pass);
    svecck(17, 1, &scosfy64[18], &scosfyres64[18], "scosf", pass);
    svecck(17, 1, &scosfy64[36], &scosfyres64[36], "scosf", pass);
    svecck(17, 1, &scosfy64[54], &scosfyres64[54], "scosf", pass);
 
    dscale = 1.0;
  dcosf(1,dcosfx64,1,18,dcosfy64,1,18,32,4,dscale,daux1,414,daux2,0);
  dcosf(0,dcosfx64,1,18,dcosfy64,1,18,32,4,dscale,daux1,414,daux2,0);
    dvecck(17, 1, &dcosfy64[0],  &dcosfyres64[0],  "dcosf", pass);
    dvecck(17, 1, &dcosfy64[18], &dcosfyres64[18], "dcosf", pass);
    dvecck(17, 1, &dcosfy64[36], &dcosfyres64[36], "dcosf", pass);
    dvecck(17, 1, &dcosfy64[54], &dcosfyres64[54], "dcosf", pass);
 
  ssinf(1,ssinfx64,1,18,ssinfy64,1,18,32,3,sscale,daux1,414,daux2,0);
  ssinf(0,ssinfx64,1,18,ssinfy64,1,18,32,3,sscale,daux1,414,daux2,0);
    svecck(16, 1, &ssinfy64[0],  &ssinfyres64[0],  "ssinf", pass);
    svecck(16, 1, &ssinfy64[18], &ssinfyres64[18], "ssinf", pass);
    svecck(16, 1, &ssinfy64[36], &ssinfyres64[36], "ssinf", pass);
 
  dsinf(1,dsinfx64,1,18,dsinfy64,1,18,32,3,dscale,daux1,414,daux2,0);
  dsinf(0,dsinfx64,1,18,dsinfy64,1,18,32,3,dscale,daux1,414,daux2,0);
    dvecck(16, 1, &dsinfy64[0],  &dsinfyres64[0],  "dsinf", pass);
    dvecck(16, 1, &dsinfy64[18], &dsinfyres64[18], "dsinf", pass);
    dvecck(16, 1, &dsinfy64[36], &dsinfyres64[36], "dsinf", pass);
 
    cp = &sx65[0][0];
    for(i = 0; i < 48; ++i, ++cp) CSET(*cp, 1., 0.);
    CSET(sres65[0][0], 48., 0.);
    cp = &sres65[0][1];
    for(i = 0; i < 47; ++i, ++cp) CSET(*cp, 0., 0.);
    scft2 (1, sx65, 1, 6, sy65, 1, 6, 6, 8, 1, 1., daux1, 20000, daux2, 0);
    scft2 (0, sx65, 1, 6, sy65, 1, 6, 6, 8, 1, 1., daux1, 20000, daux2, 0);
    cmtchk(6, 8, sy65, sres65, 6, "scft2", pass);
 
    sx66[0][0] = 1.;
    fp = &sx66[0][1];
    for(i = 0; i < 95; ++i, ++fp) *fp = 0.;
    cp = &sres66[0][0];
    for(i = 0; i < 56; ++i, ++cp) CSET(*cp, 1.0, 0.);
    srcft2 (1, sx66, 12, sy66, 7, 12, 8, 1, 1., daux1, 1000,
        daux2, 0, daux3, 0);
    srcft2 (0, sx66, 12, sy66, 7, 12, 8, 1, 1., daux1, 1000,
        daux2, 0, daux3, 0);
    cmtchk(7, 8, sy66, sres66, 7, "srcft2", pass);
 
    cp = &sx67[0][0];
    for(i = 0; i < 56; ++i, ++cp) CSET(*cp, 1., 0.);
    sres67[0][0] = 1.;
    fp = &sres67[0][1];
    for(i = 0; i < 111; ++i, ++fp) *fp = 0.;
    scrft2 (1, sx67, 7, sy67, 14, 12, 8, -1, 1. / 96., daux1, 1000,
        daux2, 0, daux3, 0);
    scrft2 (0, sx67, 7, sy67, 14, 12, 8, -1, 1. / 96., daux1, 1000,
        daux2, 0, daux3, 0);
    smtchk(12, 8, sy67, sres67, 14, "scrft2", pass);
 
    CSET(sx110[0], 1., 2.);
    cp = &sx110[1];
    for(i = 0; i < 215; ++i, ++cp) CSET(*cp, 0., 0.);
    cp = &sq110[0];
    for(i = 0; i < 216; ++i, ++cp) CSET(*cp, 1., 2.);
    scft3 (sx110, 6, 36, sx110, 6, 36, 6, 6, 6, 1, 1.0, daux1, 0);
    cvecck(216, 1, sx110, sq110, "scft3", pass);
 
    sx111[0] = 1.;
    fp = &sx111[1];
    for(i = 0; i < 1727; ++i, ++fp)  *fp = 0.;
    cp = &sq111[0];
    for(i = 0; i < 1008; ++i, ++cp) CSET(*cp, 1., 0.);
    srcft3 (sx111, 12, 144, sy111, 7, 84, 12, 12, 12, 1, 1.0, daux1, 0);
    cvecck(1008, 1, sy111, sq111, "srcft3", pass);
 
    CSET(sx112[0], 1., 0.);
    cp = &sx112[1];
    for(i = 0; i < 1007; ++i, ++cp) CSET(*cp, 0., 0.);
    fp = &sq112[0][0];
    for(i = 0; i < 2016; ++i, ++fp) *fp = 1.;
    scrft3 (sx112, 7, 84, sy112, 14, 168, 12, 12, 12, 1, 1.0, daux1, 0);
    smtchk(12, 144, sy112, sq112, 14, "scrft3", pass);
/* drcft and dcrft  */
 
    drcft (1, dx83, 16, dy83, 9, 16, 4, 1, 1.0/16.,
           daux1, 41928, daux2, 0);
    drcft (0, dx83, 16, dy83, 9, 16, 4, 1, 1.0/16.,
           daux1, 41928, daux2, 0);
 
    zvecck(36, 1, dy83, dres83, "drcft ", pass);
 
    drcftd (1, 1, dx83, dincx98, 16, dy83, dincy98, 9, dn99, 4, disign98,
           1.0/16., daux1, 1000, daux2, 1000);
    drcftd (0, 1, dx83, dincx98, 16, dy83, dincy98, 9, dn99, 4, disign98,
           1.0/16., daux1, 1000, daux2, 1000);
 
    zvecck(36, 1, dy83, dres83, "drcftd", pass);
 
    dcrft (1, dx84, 0, dy84, 0, 12, 1, 1, 1.e0, daux1, 20000, daux2, 0);
    dcrft (0, dx84, 0, dy84, 0, 12, 1, 1, 1.e0, daux1, 20000, daux2, 0);
    dvecck(12, 1, dy84, dres84, "dcrft ", pass);
 
    dcrftd (1, 1, dx84, dincx98, 7, dy84, dincy98, 12, dn100, 1,
           disign98, 1.e0, daux1, 100, daux2, 100);
    dcrftd (0, 1, dx84, dincx98, 7, dy84, dincy98, 12, dn100, 1,
           disign98, 1.e0, daux1, 100, daux2, 100);
    dvecck(12, 1, dy84, dres84, "dcrftd", pass);
 
    dcp = &dft2x40n[0][0];
    for (i=0; i <48; ++i, ++dcp)  CSET(*dcp, 1.,0.);
    CSET(dft2res40n[0][0], 48., 0.);
    dcp = &dft2res40n[0][1];
    for (i=0; i<47; ++i,++dcp)  CSET(*dcp,0.,0.);
    dcft2(1,dft2x40n,1,6,dft2y40n,1,6,6,8,1,1.,daux1,20000,daux2,0);
    dcft2(0,dft2x40n,1,6,dft2y40n,1,6,6,8,1,1.,daux1,20000,daux2,0);
    zmtchk(6,8,dft2y40n,dft2res40n,6,"dcft2",pass);
 
    drft2x40n[0][0] = 1.;
    dp = &drft2x40n[0][1];
    for (i=0; i <95; ++i, ++dp) *dp = 0.;
    dcp = &drft2res40n[0][0];
    for (i=0; i<56; ++i,++dcp)  CSET(*dcp,1.,0.);
    drcft2(1,drft2x40n,12,drft2y40n,7,12,8,1,1.,daux1,1000,daux2,0);
    drcft2(0,drft2x40n,12,drft2y40n,7,12,8,1,1.,daux1,1000,daux2,0);
    zmtchk(7,8,drft2y40n,drft2res40n,7,"drcft2",pass);
 
    dcp = &dcft2x40n[0][0];
    for (i=0; i <56; ++i, ++dcp)  CSET(*dcp, 1.,0.);
    dcft2res40n[0][0]= 1.;
    dp = &dcft2res40n[0][1] ;
    for (i=0; i<111; ++i,++dp)  *dp = 0.;
    dcrft2(1,dcft2x40n,7,dcft2y40n,14,12,8,-1,1. / 96.,daux1,1000,
      daux2,0);
    dcrft2(0,dcft2x40n,7,dcft2y40n,14,12,8,-1,1. / 96.,daux1,1000,
      daux2,0);
 
    dmtchk(12,8,dcft2y40n,dcft2res40n,14,"dcrft2",pass);
 
    CSET(dft3x41n[0], 1.,2.);
    dcp = &dft3x41n[1];
    for ( i=0; i < 215; ++i, ++dcp) CSET(*dcp, 0.,0.);
    dcp = &dft3res41n[0];
    for ( i=0; i < 216; ++i, ++dcp) CSET(*dcp, 1.,2.);
    dcft3(dft3x41n,6,36,dft3y41n,6,36,6,6,6,1,1.,daux1,0);
    zvecck(216,1,dft3y41n,dft3res41n,"dcft3",pass);
 
    drft3x41n[0] = 1.;
    dp = &drft3x41n[1];
    for (i=0; i <1727; ++i, ++dp) *dp = 0.;
    dcp = &drft3res41n[0];
    for (i=0; i<1008; ++i,++dcp)  CSET(*dcp,1.,0.);
    drcft3(drft3x41n,12,144,drft3y41n,7,84,12,12,12,1,1.,daux1,0);
    zvecck(1008,1,drft3y41n,drft3res41n,"drcft3",pass);
 
    CSET(dcft3x41n[0],1.,0.);
    dcp = &dcft3x41n[1] ;
    for (i=0; i <1007; ++i, ++dcp)  CSET(*dcp, 0.,0.);
    dp = &dcft3res41n[0][0] ;
    for (i=0; i<2016; ++i,++dp)  *dp = 1.;
    dcrft3(dcft3x41n,7,84,dcft3y41n,14,168,12,12,12,1,1.,daux1,0);
    dmtchk(12,144,dcft3y41n,dcft3res41n,14,"dcrft3",pass);
 
/********************************
* (CONVOLUTION AND CORRELATION) *
********************************/
 
/*  THE SIZE OF ARRAYS SH68, SX68, SY68 HAVE BEEN MODIFIED SO THAT  */
/*  LESS STORAGE WAS WASTED  */
    scon (1, sh68, 1, sx68, 1, 10, sy68, 1, 15, 4, 10, 3, 0, 15,
          daux1, 30788, daux2, 0);
    scon (0, sh68, 1, sx68, 1, 10, sy68, 1, 15, 4, 10, 3, 0, 15,
          daux1, 30788, daux2, 0);
    svecck(45, 1, sy68, sres68, "scon", pass);
    scond (sh69, 1, sx69, 1, sy69, 1, 4, 8, 0, 11);
    svecck(11, 1, sy69, sres69, "scond", pass);
    sconf (1, sh70, 1, sx70, 1, 1, sy70, 1, 1, 8, 10, 1, 0, 17, daux1, 2028,
          daux2, 0);
    sconf (0, sh70, 1, sx70, 1, 1, sy70, 1, 1, 8, 10, 1, 0, 17, daux1, 2028,
          daux2, 0);
    svecck(17, 1, sy70, sres70, "sconf", pass);
    sdcon (sh71, 1, sx71, 1, sy71, 1, 4, 8, 0, 6, 2);
    svecck(6, 1, sy71, sres71, "sdcon", pass);
    ddcon (dh71, 1, dx71, 1, dy71, 1, 4, 8, 0, 6, 2);
    dvecck(6, 1, dy71, dres71, "ddcon", pass);
 
/*  THE SIZE OF ARRAYS SH72, SX72, SY72 HAVE BEEN MODIFIED SO THAT  */
/*  LESS STORAGE WAS WASTED  */
    scor (1, sh72, 1, sx72, 1, 10, sy72, 1, 15, 4, 10, 3, -3, 15,
          daux1, 30788, daux2, 0);
    scor (0, sh72, 1, sx72, 1, 10, sy72, 1, 15, 4, 10, 3, -3, 15,
          daux1, 30788, daux2, 0);
    svecck(45, 1, sy72, sres72, "scor", pass);
    scord (sh69, 1, sx69, 1, sy69, 1, 4, 8, -3, 11);
    svecck(11, 1, sy69, sres73, "scord", pass);
    scorf (1, sh74, 1, sx74, 1, 1, sy74, 1, 1, 8, 10, 1, -7, 17, daux1, 2028,
          daux2, 0);
    scorf (0, sh74, 1, sx74, 1, 1, sy74, 1, 1, 8, 10, 1, -7, 17, daux1, 2028,
          daux2, 0);
    svecck(17, 1, sy74, sres74, "scorf", pass);
    sdcor (sh75, 1, sx75, 1, sy75, 1, 4, 8, -3, 6, 2);
    svecck(6, 1, sy75, sres75, "sdcor", pass);
    ddcor (dh75, 1, dx75, 1, dy75, 1, 4, 8, -3, 6, 2);
    dvecck(6, 1, dy75, dres75, "ddcor", pass);
 
/*  THE SIZE OF ARRAYS SX76, SY76 HAVE BEEN MODIFIED SO THAT  */
/*  LESS STORAGE WAS WASTED  */
    sacor (1, sx76, 1, 7, sy76, 1, 7, 7, 3, 7, daux1, 2959, daux2, 0);
    sacor (0, sx76, 1, 7, sy76, 1, 7, 7, 3, 7, daux1, 2959, daux2, 0);
    svecck(21, 1, sy76, sres76, "sacor", pass);
 
    sacorf (1, sx76, 1, 7, sy77, 1, 7, 7, 3, 7, daux1, 2959, daux2, 0);
    sacorf (0, sx76, 1, 7, sy77, 1, 7, 7, 3, 7, daux1, 2959, daux2, 0);
    svecck(21, 1, sy77, sres76, "sacorf", pass);
 
/******************************************
* (GENERAL SIGNAL PROCESSING SUBROUTINES) *
*          (3838 ALGORITHMS)              *
******************************************/
 
    spoly (su78, idum, 0, sdum, idum, sy78, 1, 3);
    svecck(3, 1, sy78, sres78, "spoly", pass);
    dpoly (du78, idum, 0, ddum, idum, dy78, 1, 3);
    dvecck(3, 1, dy78, dres78, "dpoly", pass);
    sizc (sx79, 1, 12, 5, iy79);
    intchk(2, 1, iy79, res79, "sizc", pass);
    dizc (dx79, 1, 12, 5, iy79);
    intchk(2, 1, iy79, res79, "dizc", pass);
    strec (1., su80, 1, sx80, 1, sy80, 1, 8,0);
    svecck(8, 1, sy80, sres80, "strec", pass);
    dtrec (1., du80, 1, dx80, 1, dy80, 1, 8,0);
    dvecck(8, 1, dy80, dres80, "dtrec", pass);
    sqint (2., 1., 1., sx81, 1, 8, st81, 1, sy81, 1, 4);
    svecck(4, 1, sy81, sres81, "sqint", pass);
    dqint (2., 1., 1., dx81, 1, 8, dt81, 1, dy81, 1, 4);
    dvecck(4, 1, dy81, dres81, "dqint", pass);
    swlev (sx82, 1, su82, 1, sy82, 1, 4, daux1, 12);
    svecck(4, 1, sy82, sres82, "swlev", pass);
    if ( daux1[0] != daux1chk[0] )
     {
        ++*pass;
        printf("FAILED: swlev input is not pos or neg definite\n");
        printf("daux1[0]=  %g\n", daux1[0]);
     }
    dwlev (dx82, 1, du82, 1, dy82, 1, 4, daux1, 12);
    dvecck(4, 1, dy82, dres82, "dwlev", pass);
    if ( daux1[0] != daux1chk[0] )
     {
        ++*pass;
        printf("FAILED: dwlev input is not pos or neg definite\n");
        printf("daux1[0]=  %g\n", daux1[0]);
     }
    cwlev (cx82, 1, cu82, 1, cy82, 1, 4, zaux1, 12);
    cvecck(4, 1, cy82, cres82, "cwlev", pass);
    if ( RE(zaux1[0]) != RE(zaux1chk[0]) )
     {
        ++*pass;
        printf("FAILED: cwlev input is not pos or neg definite\n");
        printf("zaux1[0]=  %g\n", RE(zaux1[0]));
     }
    zwlev (zx82, 1, zu82, 1, zy82, 1, 4, zaux1, 12);
    zvecck(4, 1, zy82, zres82, "zwlev", pass);
    if ( RE(zaux1[0]) != RE(zaux1chk[0]) )
     {
        ++*pass;
        printf("FAILED: zwlev input is not pos or neg definite\n");
        printf("zaux1[0]=  %g\n", RE(zaux1[0]));
     }
}
/**********************************************************************
*                      SUBROUTINE:  SORTS                             *
**********************************************************************/
 
void sorts(pass)
_ESVINT *pass;
{
/****************************************************
*  VARIABLES OF SORTING AND SEARCHING SUBROUTINES : *
****************************************************/
 
    _ESVINT indx84[5];
    _ESVINT indx85[5], rc85[5];
    _ESVINT indx86[3];
/* 2.1 NEW SORTING
I/S/DSORTS(43N) */
 
/* hold answer */
    float  naux[5];
    double daux[5];
    _ESVINT    soti43n[5];
    _ESVINT    doti43n[5];
    _ESVINT    ioti43n[5];
    _ESVINT    iaux[5];
 
 
/*********************************************************
* EXPECTED VALUE FOR SORTING AND SEARCHING SUBROUTINES : *
*********************************************************/
 
    _ESVINT ires83[9] = { -2, 0, -1, 0, 2, 0, 4, 0, 5 };
    float  sres83[9] = { -2., 0., -1., 0., 2., 0., 4., 0., 5. };
    double dres83[9] = { -2., 0., -1., 0., 2., 0., 4., 0., 5. };
    _ESVINT inre84[5] = { 5, 2, 4, 1, 3 };
    _ESVINT ires84[9] = { -2, 0, -1, 0, 1, 0, 2, 0, 5 };
    float  sres84[9] = { -2., 0., -1., 0., 1., 0., 2., 0., 5. };
    double dres84[9] = { -2., 0., -1., 0., 1., 0., 2., 0., 5. };
    _ESVINT inre85[5] = { 1, 11, 3, 2, 8 };
    _ESVINT rcre85[5] = { 1, 1, 0, 0, 1 };
    _ESVINT inre86[3] = { 1, 3, 0 };
    float  sotxres43n[9] ={ -2.,0.,-1.,0.,-1.,0.,2.,0.,5.};
    _ESVINT    sotires43n[5]= {5,2,4,1,3};
    double dotxres43n[9] ={ -2.,0.,-1.,0.,-1.,0.,2.,0.,5.};
    _ESVINT    dotires43n[5]= {5,2,4,1,3};
    _ESVINT    iotxres43n[9] ={ -2,0,-1,0,-1,0,2,0,5};
    _ESVINT    iotires43n[5]= {5,2,4,1,3};
 
 
/****************************************************
* INPUT DATA FOR SORTING AND SEACHING SUBROUTINES : *
****************************************************/
 
    _ESVINT ix83[9] = { 2, 0, -1, 0, 5, 0, 4, 0, -2 };
    float  sx83[9] = { 2., 0., -1., 0., 5., 0., 4., 0., -2. };
    double dx83[9] = { 2., 0., -1., 0., 5., 0., 4., 0., -2. };
    _ESVINT ix84[9] = { 2, 0, -1, 0, 5, 0, 1, 0, -2 };
    float  sx84[9] = { 2., 0., -1., 0., 5., 0., 1., 0., -2. };
    double dx84[9] = { 2., 0., -1., 0., 5., 0., 1., 0., -2. };
    _ESVINT ix85[9] = { -3, 0, 125, 0, 30, 0, 20, 0, 70 };
    _ESVINT iy85[10] = { 10, 20, 30, 30, 40, 50, 60, 80, 90, 100 };
    float  sx85[9] = { -3., 0., 125., 0., 30., 0., 20., 0., 70. };
    float  sy85[10] = { 10, 20, 30, 30, 40, 50, 60, 80, 90, 100 };
    double dx85[9] = { -3., 0., 125., 0., 30., 0., 20., 0., 70. };
    double dy85[10] = { 10, 20, 30, 30, 40, 50, 60, 80, 90, 100 };
    _ESVINT ix86[3] = { 0, 12, 3 };
    _ESVINT iy86[15] = { 0, 0, 8, 0, 12, 0, 0, 0, 1, 0, 4, 0, 0, 0, 2 };
    float  sx86[3] = { 0., 12., 3. };
    float  sy86[15] = { 0.,0.,8.,0.,12.,0.,0.,0.,1.,0.,4.,0.,0.,0.,2.};
    double dx86[3] = { 0., 12., 3. };
    double dy86[15] = { 0.,0.,8.,0.,12.,0.,0.,0.,1.,0.,4.,0.,0.,0.,2.};
    float  sotx43n[9]={2.,0.,-1.,0.,5.,0.,-1.,0.,-2.};
    double dotx43n[9]={2.,0.,-1.,0.,5.,0.,-1.,0.,-2.};
    _ESVINT    iotx43n[9]={2,0,-1,0,5,0,-1,0,-2};
 
 
 
/*********************************************************
* CALL STATEMENT FOR SORTING AND SEARCHING SUBROUTINES : *
*********************************************************/
 
    isort (ix83, 2, 5);
    intchk(5, 2, ix83, ires83, "isort", pass);
    ssort (sx83, 2, 5);
    svecck(5, 2, sx83, sres83, "ssort", pass);
    dsort (dx83, 2, 5);
    dvecck(5, 2, dx83, dres83, "dsort", pass);
    isortx (ix84, 2, 5, indx84);
    intchk(5, 2, ix84, ires84, "isortx", pass);
    intchk(5, 1, indx84, inre84, "isortx", pass);
    ssortx (sx84, 2, 5, indx84);
    svecck(5, 2, sx84, sres84, "ssortx", pass);
    intchk(5, 1, indx84, inre84, "ssortx", pass);
    dsortx (dx84, 2, 5, indx84);
    dvecck(5, 2, dx84, dres84, "dsortx", pass);
    intchk(5, 1, indx84, inre84, "dsortx", pass);
    ibsrch (ix85, 2, 5, iy85, 1, 10, indx85, rc85, 1);
    intchk(5, 1, indx85, inre85, "ibsrch", pass);
    intchk(5, 1, rc85, rcre85, "ibsrch", pass);
    sbsrch (sx85, 2, 5, sy85, 1, 10, indx85, rc85, 1);
    intchk(5, 1, indx85, inre85, "sbsrch", pass);
    intchk(5, 1, rc85, rcre85, "sbsrch", pass);
    dbsrch (dx85, 2, 5, dy85, 1, 10, indx85, rc85, 1);
    intchk(5, 1, indx85, inre85, "dbsrch", pass);
    intchk(5, 1, rc85, rcre85, "dbsrch", pass);
    issrch (ix86, 1, 3, iy86, 2, 8, 1, indx86);
    intchk(3, 1, indx86, inre86, "issrch", pass);
    sssrch (sx86, 1, 3, sy86, 2, 8, 1, indx86);
    intchk(3, 1, indx86, inre86, "sssrch", pass);
    dssrch (dx86, 1, 3, dy86, 2, 8, 1, indx86);
    intchk(3, 1, indx86, inre86, "dssrch", pass);
    ssorts(sotx43n,2,5,soti43n,naux,0);
    intchk(5,1,soti43n,sotires43n,"ssorts",pass);
    svecck(9,1,sotx43n,sotxres43n,"ssorts",pass);
    dsorts(dotx43n,2,5,doti43n,daux,0);
    intchk(5,1,doti43n,dotires43n,"dsorts",pass);
    dvecck(9,1,dotx43n,dotxres43n,"dsorts",pass);
    isorts(iotx43n,2,5,ioti43n,iaux,0);
    intchk(5,1,ioti43n,iotires43n,"isorts",pass);
    intchk(9,1,iotx43n,iotxres43n,"isorts",pass);
}
 
/**********************************************************************
*                       SUBROUTINE:  INPOS                            *
**********************************************************************/
 
void inpos(pass)
_ESVINT *pass;
{
/*******************************************
* VARIABLES OF INTERPOLATION SUBROUTINES : *
*******************************************/
 
      double daux1[100];
      float  saux1[100];
      double dc87[3], dy87[2];
      float  sc87[3], sy87[2];
      double dy88[5];
      float  sy88[5];
      double dc89[6][4], dy89[4];
      float  sc89[6][4], sy89[4];
      float  ss115[4][3];
      double ds115[4][3];
 
/*************************************************
* EXPECTED VALUE FOR INTERPOLATION SUBROUTINES : *
*************************************************/
 
    float  scre87[3] = { 1., 1., 1. };
    float  syre87[2] = { 0.04, 0.04 };
    double dcre87[3] = { 1., 1., 1. };
    double dyre87[2] = { 0.04, 0.04 };
    _ESVINT nire87[1] = { 3 };
    float  sres88[5] = { -1.50, 1.25, 3.20, 3.40, 2.00 };
    double dres88[5] = { -1.50, 1.25, 3.20, 3.40, 2.00 };
    float  scre89[4][6] = { 0.000,  1.000,  2.000,  1.100, 0.000, -1.000,
                           -0.868, -1.264, -0.076,  1.267, 1.010,  0.995,
                            0.000,  0.396, -1.585,  0.243, 0.014,  0.000,
                           -0.132, -0.132,  0.660, -0.609, 0.076,  0.005 };
    float  syre89[4] = { -2.792, 1.649, 1.100, -2.000 };
    _ESVINT nire89[1] = { 4 };
    double dcre89[4][6] = { 0.000,  1.000,  2.000,  1.100, 0.000, -1.000,
                           -0.868, -1.264, -0.076,  1.267, 1.010,  0.995,
                            0.000,  0.396, -1.585,  0.243, 0.014,  0.000,
                           -0.132, -0.132,  0.660, -0.609, 0.076,  0.005 };
    double dyre89[4] = { -2.792, 1.649, 1.100, -2.000 };
    float  ssq115[3][4] = { .001, .00345, .016, .043,
                            .017, .019, .031, .059,
                            .095, .097, .110, .137 };
    double dsq115[3][4] = { .001, .00345, .016, .043,
                            .017, .019, .031, .059,
                            .095, .097, .110, .137 };
 
/*********************************************
* INPUT DATA FOR INTERPOLATION SUBROUTINES : *
*********************************************/
 
    float  ss87[3] = { -0.5, 0., 1. };
    float  sz87[3] = { 0.25, 0., 1. };
    float  st87[2] = { -0.2, 0.2 };
    double ds87[3] = { -0.5, 0., 1. };
    double dz87[3] = { 0.25, 0., 1. };
    double dt87[2] = { -0.2, 0.2 };
    _ESVINT snin87[1] = { 0 };
    _ESVINT dnin87[1] = { 0 };
    float  ss88[10] = { 0., 0.4, 1., 1.5, 2.1, 2.6, 3., 3.4, 3.9, 4.3 };
    float  sz88[10] = { 1., 2., 3., 4., 5., 5., 4., 3., 2., 1. };
    float  st88[5] = { -1., 0.1, 1.1, 1.2, 3.9 };
    double ds88[10] = { 0., 0.4, 1., 1.5, 2.1, 2.6, 3., 3.4, 3.9, 4.3 };
    double dz88[10] = { 1., 2., 3., 4., 5., 5., 4., 3., 2., 1. };
    double dt88[5] = { -1., 0.1, 1.1, 1.2, 3.9 };
    float  ss89[6] = { 1., 2., 3., 4., 5., 6. };
    float  sz89[6] = { 0.0, 1.0, 2.0, 1.1, 0.0, -1.0 };
    float  st89[4] = { -1.0, 2.5, 4.0, 7.0 };
    _ESVINT snit89[1] = { 0 };
    double ds89[6] = { 1., 2., 3., 4., 5., 6. };
    double dz89[6] = { 0.0, 1.0, 2.0, 1.1, 0.0, -1.0 };
    double dt89[4] = { -1.0, 2.5, 4.0, 7.0 };
    _ESVINT dnit89[1] = { 0 };
    float  sx115[6] = { .0, .2, .3, .4, .5, .7 };
    float  sy115[5] = { .0, .2, .3, .4, .6 };
    float  st115[4] = { .1, .15, .25, .35 };
    float  su115[3] = { .05, .25, .45 };
    float  sz115[5][6] = { .000, .008, .027, .064, .125, .343,
                           .008, .016, .035, .072, .133, .351,
                           .027, .035, .054, .091, .152, .370,
                           .064, .072, .091, .128, .189, .407,
                           .216, .224, .243, .280, .341, .559 };
    double dx115[6] = { .0, .2, .3, .4, .5, .7 };
    double dy115[5] = { .0, .2, .3, .4, .6 };
    double dt115[4] = { .1, .15, .25, .35 };
    double du115[3] = { .05, .25, .45 };
    double dz115[5][6] = { .000, .008, .027, .064, .125, .343,
                           .008, .016, .035, .072, .133, .351,
                           .027, .035, .054, .091, .152, .370,
                           .064, .072, .091, .128, .189, .407,
                           .216, .224, .243, .280, .341, .559 };
 
/*************************************************
* CALL STATEMENT FOR INTERPOLATION SUBROUTINES : *
*************************************************/
 
    spint (ss87, sz87, 3, sc87, &snin87[0], st87, sy87, 2);
    svecck(3, 1, sc87, scre87, "spint", pass);
    intchk(1, 1, snin87, nire87, "spint", pass);
    svecck(2, 1, sy87, syre87, "spint", pass);
    dpint (ds87, dz87, 3, dc87, &dnin87[0], dt87, dy87, 2);
    dvecck(3, 1, dc87, dcre87, "dpint", pass);
    intchk(1, 1, dnin87, nire87, "dpint", pass);
    dvecck(2, 1, dy87, dyre87, "dpint", pass);
    stpint (ss88, sz88, 10, 2, st88, sy88, 5, saux1, 0);
    svecck(5, 1, sy88, sres88, "stpint", pass);
    dtpint (ds88, dz88, 10, 2, dt88, dy88, 5, daux1, 0);
    dvecck(5, 1, dy88, dres88, "dtpint", pass);
    scsint (ss89, sz89, sc89, 6, &snit89[0], st89, sy89, 4);
    smtchk(6, 4, sc89, scre89, 6, "scsint", pass);
    intchk(1, 1, snit89, nire89, "scsint", pass);
    svecck(4, 1, sy89, syre89, "scsint", pass);
    dcsint (ds89, dz89, dc89, 6, &dnit89[0], dt89, dy89, 4);
    dmtchk(6, 4, dc89, dcre89, 6, "dcsint", pass);
    intchk(1, 1, dnit89, nire89, "dcsint", pass);
    dvecck(4, 1, dy89, dyre89, "dcsint", pass);
    scsin2 (sx115, sy115, sz115, 6, 5, 6, st115, su115, 4, 3, ss115, 4,
        saux1, 0);
    sverck(4 * 3, 1, ss115, ssq115, "scsin2", pass);
    dcsin2 (dx115, dy115, dz115, 6, 5, 6, dt115, du115, 4, 3, ds115, 4,
        daux1, 0);
    dverck(4 * 3, 1, ds115, dsq115, "dcsin2", pass);
}
 
/**********************************************************************
*                    SUBROUTINE "RANDS"                               *
**********************************************************************/
 
void rands(pass)
_ESVINT *pass;
{
/******************************************************
* VARIABLES OF RANDOM NUMBER GENERATION SUBROUTINES : *
******************************************************/
 
    float  sx95[11];
    float  sx117[10];
    double dx117[10];
    float  saux1[100];
    double daux1[100];
    float  naux[10000];
    float  sorx42n[10];
    double daux[10000];
    double dorx42n[10];
 
/************************************************************
* EXPECTED VALUE FOR RANDOM NUMBER GENERATION SUBROUTINES : *
************************************************************/
 
    float  sres95[11] = { 759150100., 0.631032288, 0.760320187, 0.701523244,
                         0.501486838, 0.48958528, 0.460234404, 0.160360754,
                         0.183256238, 0.989906192, 0.353506804 };
    double dres95[11] = { 759150100., 0.631032288, 0.760320187, 0.701523244,
                         0.501486838, 0.48958528, 0.460234404, 0.160360754,
                         0.183256238, 0.989906192, 0.353506804 };
    double ssz117 = { 48669425. };
    double dsz117 = { 48669425. };
    float  sxz117[10] = {
          .660648465, 1.312504770, 1.906440730, .014065176, -.800936639,
        -3.058144570, -.397425830, -.370634794, -.064151347, -.275886714 };
    double dxz117[10] = {
          .6606495655963802, 1.3125037758861062,  1.9064381379483735,
          .0140658628770492, -.8009353314494662, -3.0581441239248535,
         -.3974260845722097, -.3706349643478602, -.0641514443372938,
         -.2758870636332466 };
    float  sorres42n[10] ={ 0.6440869, 0.5105118, 0.4878680, 0.3209075,
       0.6624528, 0.2499878, 0.0056630, 0.7329214, 0.7486336, 0.8050517};
    _ESVINT iseedres42n[1]={0};
    double dorres42n[10] = {0.64408693438956721, 0.51051182536460887,
           0.48786801310787158, 0.32090755617007065, 0.66245283144861679,
           0.24998782843358093, 0.00566308101257378, 0.73292147005172932,
           0.7486335974102237, 0.80505169697755326};
 
/********************************************************
* INPUT DATA FOR RANDOM NUMBER GENERATION SUBROUTINES : *
********************************************************/
 
    double seed95 = { 80629. };
    double dx95[11] = { 80629., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    double ss117[1] = { 80629. };
    double ds117[1] = { 80629. };
    _ESVINT  iseed42n[1]={137};
    _ESVINT  iseedd42n[1]={137};
 
 
/************************************************************
* CALL STATEMENT FOR RANDOM NUMBER GENERATION SUBROUTINES : *
************************************************************/
 
    surand (&seed95, 10, &sx95[1]);
    sx95[0] = seed95;
    svecck(11, 1, sx95, sres95, "surand", pass);
    durand (&dx95[0], 10, &dx95[1]);
    dvecck(11, 1, dx95, dres95, "durand", pass);
    snrand (ss117, 10, sx117, saux1, 0);
    dvecck(1, 1, ss117, &ssz117, "snrand", pass);
    svecck(10, 1, sx117, sxz117, "snrand", pass);
    dnrand (ds117, 10, dx117, daux1, 0);
    dvecck(1, 1, ds117, &dsz117, "dnrand", pass);
    dvecck(10, 1, dx117, dxz117, "dnrand", pass);
    surxor(iseed42n,10,sorx42n,naux);
    intchk(1,1,iseed42n,iseedres42n,"surxor",pass);
    svecck(10,1,sorx42n,sorres42n,"surxor",pass);
    durxor(iseedd42n,10,dorx42n,daux);
    intchk(1,1,iseedd42n,iseedres42n,"durxor",pass);
    dvecck(10,1,dorx42n,dorres42n,"durxor",pass);
}
 
/*************************************************
*  NUMERICAL QUADRATURE SUBROUTINES              *
**************************************************/
 
void quads(pass)
_ESVINT *pass;
{
/**************************************************
* VARIABLES OF NUMERICAL QUADRATURE SUBROUTINES : *
**************************************************/
 
    void ssub91(),dsub91(),ssub92(),dsub92(),ssub93(),dsub93();
    void ssub94(),dsub94(),ssb116(),dsb116();
    double dxy90[1], dest90[1];
    double dt[1];
    float  sxy90[1], sest90[1];
    float  st[1];
    float  sx116, sz116[10][5];
    double dx116, dz116[10][5];
 
/********************************************************
* EXPECTED VALUE FOR NUMERICAL QUADRATURE SUBROUTINES : *
********************************************************/
 
    float  sxyr90[1] = { 15.137 };
    float  sere90[1] = { -0.003 };
    double dxyr90[1] = { 15.137 };
    double dere90[1] = { -0.003 };
    float  sres91[1] = { 9.056 };
    double dres91[1] = { 9.056 };
    float  sres92[1] = { 5.891 };
    double dres92[1] = { 5.891 };
    float  sres93[1] = { 0.393 };
    double dres93[1] = { 0.393 };
    float  sres94[1] = { 31.646 };
    double dres94[1] = { 31.646 };
    float  sq116 = { -6.1108 };
    double dq116 = { -6.1108 };
 
/****************************************************
* INPUT DATA FOR NUMERICAL QUADRATURE SUBROUTINES : *
****************************************************/
 
    float  sx90[10] = { 0.0, 0.4, 1.0, 1.5, 2.1, 2.6, 3.0, 3.4, 3.9, 4.3 };
    float  sy90[10] = { 1.0, 2.0, 3.0, 4.0, 5.0, 4.5, 4.0, 3.0, 3.5, 3.3 };
    double dx90[10] = { 0.0, 0.4, 1.0, 1.5, 2.1, 2.6, 3.0, 3.4, 3.9, 4.3 };
    double dy90[10] = { 1.0, 2.0, 3.0, 4.0, 5.0, 4.5, 4.0, 3.0, 3.5, 3.3 };
 
/********************************************************
* CALL STATEMENT FOR NUMERICAL QUADRATURE SUBROUTINES : *
********************************************************/
 
    sptnq (sx90, sy90, 10, &sxy90[0], &sest90[0]);
    svecck(1, 1, sxy90, sxyr90, "sptnq", pass);
    svecck(1, 1, sest90, sere90, "sptnq", pass);
    dptnq (dx90, dy90, 10, &dxy90[0], &dest90[0]);
    dvecck(1, 1, dxy90, dxyr90, "dptnq", pass);
    dvecck(1, 1, dest90, dere90, "dptnq", pass);
    st[0] = sglnq (ssub91, 0., 2., 10);
    svecck(1, 1, st, sres91, "sglnq", pass);
    dt[0] = dglnq (dsub91, 0., 2., 10);
    dvecck(1, 1, dt, dres91, "dglnq", pass);
    st[0] = sglgq (ssub92, -2., 1.5, 20);
    svecck(1, 1, st, sres92, "sglgq", pass);
    dt[0] = dglgq (dsub92, -2., 1.5, 20);
    dvecck(1, 1, dt, dres92, "dglgq", pass);
    st[0] = sgraq (ssub93, -2., 0., 10);
    svecck(1, 1, st, sres93, "sgraq", pass);
    dt[0] = dgraq (dsub93, -2., 0., 10);
    dvecck(1, 1, dt, dres93, "dgraq", pass);
    st[0] = sghmq (ssub94, -5., 2., 4);
    svecck(1, 1, st, sres94, "sghmq", pass);
    dt[0] = dghmq (dsub94, -5., 2., 4);
    dvecck(1, 1, dt, dres94, "dghmq", pass);
    sx116 = sglnq2 (ssb116, .0, 2., 10, -2., -1., 5, sz116, 10);
    svecck(1, 1, &sx116, &sq116, "sglnq2", pass);
    dx116 = dglnq2 (dsb116, .0, 2., 10, -2., -1., 5, dz116, 10);
    dvecck(1, 1, &dx116, &dq116, "dglnq2", pass);
}
 
/*********************************************************************
*       USER-SUPPLIED SUBOUTINES WHICH EVALUATES THE INTEGRAND       *
*       FUNCTION FOR NUMERICAL QUADRATURE SUBROUTINES                *
*********************************************************************/
 
 
void ssub91(t, y, m)
float *t, *y;
_ESVINT *m;
{
    _ESVINT i;
 
    for(i = 0; i < *m; ++i) y[i] = (t[i] * t[i]) + exp(t[i]);
}
 
 
void dsub91(t, y, m)
double *t, *y;
_ESVINT *m;
{
    _ESVINT i;
 
    for(i = 0; i < *m; ++i) y[i] = (t[i] * t[i]) + exp(t[i]);
}
 
 
void ssub92(t, y, m)
float *t, *y;
_ESVINT *m;
{
    _ESVINT i;
 
    for(i = 0; i < *m; ++i) y[i] = sin(3. * t[i]) * exp(-1.5 * t[i]);
}
 
 
void dsub92(t, y, m)
double *t, *y;
_ESVINT *m;
{
    _ESVINT i;
 
    for(i = 0; i < *m; ++i) y[i] = sin(3. * t[i]) * exp(-1.5 * t[i]);
}
 
 
void ssub93(t, y, m)
float *t, *y;
_ESVINT *m;
{
    _ESVINT i;
    float temp;
 
    for(i = 0; i < *m; ++i) {
        temp = 1. / t[i];
        y[i] = exp(temp) * temp * temp;
    }
}
 
 
void dsub93(t, y, m)
double *t, *y;
_ESVINT *m;
{
    _ESVINT i;
    double temp;
 
    for(i = 0; i < *m; ++i) {
        temp = 1. / t[i];
        y[i] = exp(temp) * temp * temp;
    }
}
 
 
void ssub94(t, y, m)
float *t, *y;
_ESVINT *m;
{
    _ESVINT i;
    float temp;
 
    for(i = 0; i < *m; ++i) {
        temp = t[i] + 5.;
        y[i] = t[i] * t[i] * exp(-2. * temp * temp);
    }
}
 
 
void dsub94(t, y, m)
double *t, *y;
_ESVINT *m;
{
    _ESVINT i;
    double temp;
 
    for(i = 0; i < *m; ++i) {
        temp = t[i] + 5.;
        y[i] = t[i] * t[i] * exp(-2. * temp * temp);
    }
}
 
 
void ssb116(s, n1, t, n2, z, ldz)
float *s, *t, *z;
_ESVINT *n1, *n2, *ldz;
{
    _ESVINT i, j;
 
    for(j = 0; j < *n2; ++j, z += *ldz) {
        for(i = 0; i < *n1; ++i) z[i] = exp(s[i]) * sin(t[j]);
    }
}
 
 
void dsb116(s, n1, t, n2, z, ldz)
double *s, *t, *z;
_ESVINT *n1, *n2, *ldz;
{
    _ESVINT i, j;
 
    for(j = 0; j < *n2; ++j, z += *ldz) {
        for(i = 0; i < *n1; ++i) z[i] = exp(s[i]) * sin(t[j]);
    }
}
/**********************************************************************
*                          SUBROUTINE "UTILS"                         *
**********************************************************************/
 
void utils(pass)
_ESVINT *pass;
{
/*************************************
* VARIABLES OF UTILITY SUBROUTINES : *
*************************************/
 
    double dac96[5][4];
    _ESVINT ka96[5][4];
    _ESVINT s118, min[1];
    double dsaux[9];
    double dgaux[18];
 
/*******************************************
* EXPECTED VALUE FOR UTILITY SUBROUTINES : *
*******************************************/
 
    double dres96[5][4] = { 11., 22., 33., 44., 14., 24., 34., 0.,
        0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    _ESVINT ires96[5][4] = { 1, 2, 3, 4, 4, 4, 4, 4, 0, 0,
                         0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    _ESVINT irnz96[1] = { 2 };
    _ESVINT sq118[3] = { 65,64,66 };
    double sktares44n[33] = {11.,12.,22.,13.,23.,33.,14.,24.,34.,44.,
                         25.,35.,45.,55.,26.,36.,46.,56.,66.,57.,
                         67.,77.,28.,38.,48.,58.,68.,78.,88.,69.,
                         79.,89.,99.};
    _ESVINT   sktdres44n[10] ={1,3,6,10,14,19,22,29,33,34};
 
    double gktaures45n[33]={11.,12.,22.,13.,23.,33.,24.,34.,44.,25.,
                        35.,45.,55.,46.,56.,66.,37.,47.,57.,67.,
                        77.,58.,68.,78.,88.,29.,39.,49.,59.,69.,
                        79.,89.,99.};
    double gktalres45n[35]={0.,21.,0.,31.,32.,0.,41.,42.,43.,0.,
                        54.,0.,62.,63.,64.,65.,0.,74.,75.,76.,
                        0.,84.,85.,86.,87.,0.,91.,92.,93.,94.,
                        95.,96.,97.,98.,0.};
    _ESVINT  gktiures45n[10]={1,3,6,9,13,16,21,25,33,34};
    _ESVINT  gktilres45n[10]={1,3,6,10,12,17,21,26,35,36};
 
/*              (2/94 Added iessl call for PTF)  */
 
    _ESVINT ires4i[1]  = { 5020000 };
 
/***************************************
* INPUT DATA FOR UTILITY SUBROUTINES : *
***************************************/
 
    double dar96[7] = { 11., 14., 22., 24., 33., 34., 44. };
    _ESVINT ja96[7] = { 1, 4, 2, 4, 3, 4, 4 };
    _ESVINT ia96[5] = { 1, 3, 5, 7, 8 };
    _ESVINT inz96[1] = { 5 };
    double skta44n[33]={11.,22.,12.,33.,23.,13.,44.,34.,24.,14.,
                    55.,45.,35.,25.,66.,56.,46.,36.,26.,77.,
                    67.,57.,88.,78.,68.,58.,48.,38.,28.,99.,
                    89.,79.,69.};
    _ESVINT    sktt44n[3] = {0,1,-1};
    _ESVINT    sktd44n[10] = {1,2,4,7,11,15,20,23,30,34};
 
    double gktau45n[33]=    {11.,22.,12.,33.,23.,13.,44.,34.,24.,55.,
                        45.,35.,25.,66.,56.,46.,77.,67.,57.,47.,
                        37.,88.,78.,68.,58.,99.,89.,79.,69.,59.,
                        49.,39.,29.};
    double gktal45n[35]= {0.,0.,21.,0.,32.,31.,0.,43.,42.,41.,
                      0.,54.,0.,65.,64.,63.,62.,0.,76.,75.,
                      74.,0.,87.,86.,85.,84.,0.,98.,97.,96.,
                      95.,94.,93.,92.,91.};
    _ESVINT  gktiu45n[10]= {1,2,4,7,10,14,17,22,26,34};
    _ESVINT  gktil45n[10]= {1,2,4,7,11,13,18,22,27,36};
    _ESVINT  gktit45n[3]= {0,1,1};
 
 
 
/*******************************************
* CALL STATEMENT FOR UTILITY SUBROUTINES : *
*******************************************/
 
    dsrsm (0, dar96, ja96, ia96, 4, &inz96[0], dac96, ka96, 4);
    dmtchk(4, 2, dac96, dres96, 4, "dsrsm", pass);
    intchk(1, 1, inz96, irnz96, "dsrsm", pass);
    imtchk(4, 2, ka96, ires96, 4, "dsrsm", pass);
    dsktrn(9,skta44n,33,sktd44n,sktt44n,dsaux,0);
    intchk(10,1,sktd44n,sktdres44n,"dsktrn",pass);
    dvecck(33,1,skta44n,sktares44n,"dsktrn",pass);
    dgktrn(9,gktau45n,33,gktiu45n,gktal45n,35,gktil45n,gktit45n,dgaux,0);
    intchk(10,1,gktiu45n,gktiures45n,"dgktrn",pass);
    intchk(10,1,gktil45n,gktilres45n,"dgktrn",pass);
    dvecck(33,1,gktau45n,gktaures45n,"dgktrn",pass);
    dvecck(35,1,gktal45n,gktalres45n,"dgktrn",pass);
 
    stride (128, 64, &s118, "C", 0);
    /* sq118[0] is if VSS = 128 */
    /* sq118[1] is for VSS = 256 (on 3090S) */
    if((s118 == sq118[0]) || (s118 == sq118[1]) || (s118 == sq118[2])) {
      /*  printf("stride PASSED\n");   */
    } else {
        ++pass;
        printf("%d  %d  %d %d\n", s118, sq118[0], sq118[1], sq118[2]);
        printf("stride FAILED\n");
    }
    /* no way to check results: */
    ivsset (256);
    ievops(1);
    min[0] = iessl();
    ieschk(1, 1, min, ires4i, "iessl", pass);
}
 
/**********************************************************************
*                    SUBROUTINE "PARS"                                *
**********************************************************************/
 
void pars(pass)
_ESVINT *pass;
{
/******************************************************
*        VARIABLES OF PARALLEL SUBROUTINES :          *
******************************************************/
 
    _ESVINT i;
    cmplx  *cp;
    double aux1[1000];
    double dc35[4][7];
    _ESVINT ipvt37[9];
    double daux1[37500], daux2[37500];
    cmplx  sy61[32];
 
/************************************************************
*        EXPECTED VALUE FOR PARALLEL SUBROUTINES :          *
************************************************************/
 
    double dres35[4][7] = { 23., -4., 3.,  -3., -5., 15., 0.,
                            12., -5., 0.,   5., -7.,  6., 0.,
                            -6.,  1., 1.,  -2.,  4., -5., 0.,
                             2.,  3., 4., -10.,  4.,  6., 0. };
    double dr37[9][9] = {
        4., 0., 0., 0., 0., 0.,  0.,  2.5e-1,    2.5e-1,
        1., 5., 0., 0., 0., 0.,  0.,  1.5e-1,    1.5e-1,
        1., 1., 6., 0., 0., 0.,  0.,  1.e-1,     1.e-1,
        1., 1., 1., 7., 0., 0.,  0.,  7.14e-2,   7.14e-2,
        1., 1., 1., 1., 8., 0.,  0.,  5.36e-2,  -7.14e-2,
        1., 1., 1., 1., 1., 9.,  0., -6.94e-2,  -5.555e-2,
        0., 1., 1., 1., 1., 1., 10., -3.06e-2,  -1.94e-2,
        0., 0., 1., 1., 1., 1., 11.,  1.806e-1,  9.385e-1,
        0., 0., 0., 1., 1., 1., 12.,  3.111e-1, -3.1e-3 };
    double dres39[54] = { 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                          1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                          1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                          1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
                          1., 1., 1., 1., 1., 1., 1., 1., 1., 1. };
    cmplx  sres61[32] = {
        { 8., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
        { 0., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
        { 0., 0. }, { 8., 0. }, { 0., 0. }, { 0., 0. },
        { 0., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
        { 0., 0. }, { 0., 0. }, { 8., 0. }, { 0., 0. },
        { 0., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. },
        { 0., 0. }, { 0., 0. }, { 0., 0. }, { 8., 0. },
        { 0., 0. }, { 0., 0. }, { 0., 0. }, { 0., 0. } };
    cmplx  sres65[8][6];  /* initialized in program */
    cmplx  sq110[6 * 6 * 6];  /* initialized in program */
 
/********************************************************
*      INPUT DATA FOR PARALLEL  SUBROUTINES :           *
********************************************************/
 
    double da35[5][8] = { 1., 2., 1., -3., 4., -1., 0., 0.,
                          2., 0., -1., 2., 0., -1., 0., 0.,
                         -1., 1., -1., 2., -2., 1., 0., 0.,
                         -1., 1., 1., 2., 1., -3., 0., 0.,
                          4., -1., 2., 0., -1., 2., 0., 0. };
    double db35[4][6] = { 1., 2., 1., -3., 4., 1.,
                         -1., 2., 0., -1., 2., 1.,
                          0., -1., -1., 1., -1., 1.,
                          2., -2., 1., -1., 1., 1. };
    double da37[9][9] = { 1., 1., 4., 0., 0., 0., 0., 0., 0.,
                          1., 1., 1., 5., 0., 0., 0., 0., 0.,
                          1., 1., 1., 1., 6., 0., 0., 0., 0.,
                          1., 1., 1., 1., 1., 7., 0., 0., 0.,
                          0., 1., 1., 1., 1., 1., 8., 0., 0.,
                          0., 0., 1., 1., 1., 1., 1., 9., 0.,
                          0., 0., 0., 1., 1., 1., 1., 1., 10.,
                          0., 0., 0., 0., 1., 1., 1., 1., 11.,
                          0., 0., 0., 0., 0., 1., 1., 1., 12. };
    double dap39[54] = { 1., 1., 1., 1., 1., 1., 1., 1., 1., 2., 2.,
                         2., 2., 2., 2., 2., 2., 3., 3., 3., 3., 3.,
                         3., 3., 4., 4., 4., 4., 4., 4., 5., 5., 5.,
                         5., 5., 6., 6., 6., 6., 7., 7., 7., 8., 8.,
                         9., 0., 0., 0., 0., 0., 0., 0., 0., 0. };
    cmplx  sx61[32] = {
        { 1., 0. }, { 1., 0. }, { 1., 0. }, { 1., 0. },
        { 1., 0. }, { 1., 0. }, { 1., 0. }, { 1., 0. },
        { 1., 0. }, { 0.7071, 0.7071 }, { 0., 1. }, { -0.7071, 0.7071 },
        { -1., 0. }, { -0.7071, -0.7071 }, { 0., -1. }, { 0.7071, -0.7071 },
        { 1., 0. }, { 0., 1. }, { -1., 0. }, { 0., -1. },
        { 1., 0. }, { 0., 1. }, { -1., 0. }, { 0., -1. },
        { 1., 0. }, { -0.7071, 0.7071 }, { 0., -1. }, { 0.7071, 0.7071 },
        { -1., 0. }, { 0.7071, -0.7071 }, { 0., 1. }, { -0.7071, -0.7071 } };
    cmplx  sx65[8][6];  /* initialized in program */
    cmplx  sx110[6 * 6 * 6];  /* initialized in program */
 
    double d108[25];
 
/************************************************************
* expected value for parallel subroutines : *
*************************************************************
c     dskfsp,dgkfsp                 */
     double dar107[33] = {
        -1.00,-1.00, 1.00,-1.00, 1.0,1.0,-1.0,1.0,1.0, 1.0,
        -1.00, 1.00, 1.00, 1.00,-1.0,1.0, 1.0,1.0,1.0,-1.0,
         1.00, 1.00,-1.00, 1.00, 1.0,1.0, 1.0,1.0,1.0,-1.0,
         1.00, 1.00, 1.00 };
     double dr107[4] = {1.0e-15, 7.0, -1.0, 0.0};
     double dbr107[36] = {
           -1.00,-1.00,-1.00,-1.00,-1.00,-1.00,-1.00,-1.00,
           -1.00, 0.00, 0.00, 0.00,
           -2.00,-2.00,-2.00,-2.00,-2.00,-2.00,-2.00,-2.00,
           -2.00, 0.00, 0.00, 0.00,
           -3.00,-3.00,-3.00,-3.00,-3.00,-3.00,-3.00,-3.00,
           -3.00, 0.00, 0.00, 0.00 };
     double dr108[3] = {3.0, 6.4, 1.0 };
     double dru108[33] = {0.50,0.50,2.0,0.5,2.00,2.00,0.50,2.00,
                  2.00,0.50,2.00,2.00,2.0,0.50,2.00,2.00,
                  8.0,8.00,4.00,4.00,2.00,8.0,6.00,4.00,
                  2.00,16.0,10.00, 8.00,10.00,4.00,6.00,4.00,
                  2.00};
     double drl108[35] = {
          0.0,0.0,1.0,0.0,1.0, 1.0,0.0,1.0, 1.0,1.0,
          0.0,1.0,0.0,1.0,1.0, 1.0,1.0,0.0, 6.0,4.0,
          2.0,0.0,8.0,6.0,4.0, 2.0,0.0,8.0,10.0,6.0,
          8.0,6.0,6.0,4.0,2.0};
 
/********************************************************
* input data for parallel  subroutines : *
*********************************************************
c     dskfsp,dgkfsp                         */
    double  da107[33] = {
        -1.0,-2.0,-1.0,-3.0,-2.0,-1.0,-4.0,-3.0,-2.0,-1.0,
        -4.0,-3.0,-2.0,-1.0,-5.0,-4.0,-3.0,-2.0,-1.0,-3.0,
        -2.0,-1.0,-7.0,-3.0,-5.0,-4.0,-3.0,-2.0,-1.0,-4.0,
        -3.0,-2.0,-1.0 };
    _ESVINT  id107[10] = {1,2,4,7,11, 15,20,23,30,34};
    _ESVINT ip107[25] = {1,10, 0, 0, 0, 0, 0, 0, 0, 1,
                  0,-1,-1,-1,-1, 0, 0, 0, 0, 0,
                   0, 0, 0, 0, 0};
    _ESVINT ipr107[25] = {1,10, 0, 0, 0, 0, 0, 0, 0, 1,
                  0,-1,-1,-1,-1, 8, 0, 0, 0, 0,
                   9, 0, 0, 0, 0};
    double d107[25] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 1.0e-15,
                  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                  0, 0, 0, 0, 0};
    double db107[36] = { 4.0,10.0,15.0,19.0,19.0,23.0,
                 11.0,28.0,10.0, 1.0, 1.0, 1.0,
                  8.0,20.0,30.0,38.0,38.0,46.0,
                 22.0,56.0,20.0, 1.0, 1.0, 1.0,
                 12.0,30.0,45.0,57.0,57.0,69.0,
                 33.0,84.0,30.0, 1.0, 1.0, 1.0};
    _ESVINT  ip108[25] = {1,11, 0, 0, 0, 0, 0, 0, 0, 0,
                  0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                  0, 0, 0, 0, 0};
    _ESVINT  ipr108[25] = {1,11, 0, 0, 0, 0, 0, 0, 0, 0,
                  0, 0, 0, 0, 0, 3, 0, 0, 0, 0,
                  0, 0, 0, 0, 6};
    _ESVINT idu108[7] = { 1,2,4,7,10, 14,17};
    _ESVINT idl108[7] = {1,2,4,7,11, 13,18};
    double dau108[33] ={2.0,4.0,2.0,6.0,4.0, 2.0, 6.0,4.0, 2.0,4.0,
                 6.0,4.0,2.0,6.0,4.0, 2.0, 8.0,8.0, 4.0,4.0,
                 2.0,8.0,6.0,4.0,2.0,16.0,10.0,8.0,10.0,4.0,
                 6.0,4.0,2.0};
    double dal108[35] = {0.0,0.0,2.0,0.0,4.0, 2.0,0.0,6.0, 4.0,2.0,
                  0.0,2.0,0.0,8.0,6.0, 4.0,2.0,0.0, 6.0,4.0,
                  2.0,0.0,8.0,6.0,4.0, 2.0,0.0,8.0,10.0,6.0,
                  8.0,6.0,6.0,4.0,2.0};
 
/************************************************************
*       CALL STATEMENT FOR PARALLEL  SUBROUTINES :          *
************************************************************/
    dgemlp (da35, 8, "n", db35, 6, "n", dc35, 7, 6, 5, 4);
    dmtchk(6, 4, dc35, dres35, 7, "dgemlp", pass);
 
    dgefp (da37, 9, 9, ipvt37, daux1,0);
    dmtchk(9, 9, da37, dr37, 9, "dgefp", pass);
 
    dppfp (dap39, 9, aux1,0);
    dvecck(45, 1, dap39, dres39, "dppfp", pass);
 
    scftp (1, sx61, 1, 8, sy61, 1, 8, 8, 4, 1, 1., daux1, 1693, daux2, 0);
    scftp (0, sx61, 1, 8, sy61, 1, 8, 8, 4, 1, 1., daux1, 1693, daux2, 0);
    cvecck(32, 1, sy61, sres61, "scftp", pass);
 
    cp = &sx65[0][0];
    for(i = 0; i < 48; ++i, ++cp) CSET(*cp, 1., 0.);
    CSET(sres65[0][0], 48., 0.);
    cp = &sres65[0][1];
    for(i = 0; i < 47; ++i, ++cp) CSET(*cp, 0., 0.);
/*
    scft2p (1, sx65, 1, 6, sx65, 1, 6, 6, 8, 1, 1., daux1, 20000,
        daux2, 0);
    scft2p (0, sx65, 1, 6, sx65, 1, 6, 6, 8, 1, 1., daux1, 20000,
        daux2, 0);
*/
    scft2p (1, sx65, 1, 6, sx65, 1, 6, 6, 8, 1, 1., daux1, 20000,
        daux2, 0);
    scft2p (0, sx65, 1, 6, sx65, 1, 6, 6, 8, 1, 1., daux1, 20000,
        daux2, 0);
    cmtchk(6, 8, sx65, sres65, 6, "scft2p", pass);
 
    CSET(sx110[0], 1., 2.);
    cp = &sx110[1];
    for(i = 0; i < 215; ++i, ++cp) CSET(*cp, 0., 0.);
    cp = &sq110[0];
    for(i = 0; i < 216; ++i, ++cp) CSET(*cp, 1., 2.);
    /*scft3p (sx110, 6, 36, sx110, 6, 36, 6, 6, 6, 1, 1.0, daux1, 483);*/
    scft3p (sx110, 6, 36, sx110, 6, 36, 6, 6, 6, 1, 1.0, daux1, 0);
    cvecck(216, 1, sx110, sq110, "scft3p", pass);
 
    dskfsp (9, da107, 33, id107, ip107, d107, daux1, 0, db107, 12, 3);
    dvecck(33, 1, da107, dar107, "dskfsp ", pass);
    dvecck(9, 1, db107, dbr107, "dskfsp ", pass);
    intchk(25, 1, ip107, ipr107, "dskfsp ", pass);
    dvecck(1, 1, &d107[9], &dr107[0], "dskfsp ", pass);
    dvecck(3, 1, &d107[15], &dr107[1], "dskfsp ", pass);
    dgkfsp (6, dau108, 33, idu108, dal108, 35, idl108, ip108, d108,
            daux1, 0, db107, 12, 3);
    dvecck(33, 1, dau108, dru108, "dgkfsp", pass);
    intchk(25, 1, ip108, ipr108, "dgkfsp ", pass);
    dvecck(33, 1, dal108, drl108, "dgkfsp", pass);
    dvecck( 3, 1,  &d108[15], &dr108[0],"dgkfsp",pass);
}
 
/***************************************************
*                CHECK SUBROUTINES                 *
****************************************************
*****************************
* INTEGER CHECK SUBROUTINES *
****************************/
void intchk(n, incx, imax, ire, cname, pass)
_ESVINT n, incx, *imax, *ire, *pass;
char *cname;
{
    _ESVINT i;
 
    for(i = 0; i < n * incx; i += incx) {
        if(imax[i] != ire[i]) {
            printf("%s  FAILED\n", cname);
            printf("RESULTS  %d  %d  %d\n", i, ire[i], imax[i]);
            ++*pass;
            return;
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
 
void ieschk(n, incx, imax, ire, cname, pass)
_ESVINT n, incx, *imax, *ire, *pass;
char *cname;
{
    _ESVINT i;
 
     for(i = 0; i < n * incx; i += incx) {
      if(imax[i] < ire[i]) {
            printf("%s  FAILED\n", cname);
            printf("RESULTS  %d  %d  %d\n", i, ire[i], imax[i]);
            ++*pass;
            return;
      }
     }
 /*   printf("%s  PASSED\n", cname);   */
}
 
/****************************
*  VECTOR CHECK SUBROUTINES *
*****************************
*   SHORT PRECISION     *
************************/
void svecck(n, incx, rans, rst, cname, pass)
_ESVINT n, incx, *pass;
float *rans, *rst;
char *cname;
{
    _ESVINT i;
    float tols;
 
    tols = 10.e-2;
    for(i = 0; i < n * incx; i += incx) {
        if(fabs(rst[i] - rans[i]) > tols) {
            printf("%s  FAILED\n", cname);
            printf("RESULTS  %d  %g  %g\n", i, rst[i], rans[i]);
            ++*pass;
            return;
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
 
/****************************************
*   SHORT PRECISION WITH RELATIVE ERROR *
****************************************/
void sverck(n, incx, rans, rst, cname, pass)
_ESVINT n, incx, *pass;
float *rans, *rst;
char *cname;
{
    _ESVINT i;
    float tols;
 
    tols = 10.e-2;
    for(i = 0; i < n * incx; i += incx) {
        if(fabs(rst[i] - rans[i]) > (tols * fabs(rst[i]))) {
            printf("%s  FAILED\n", cname);
            printf("RESULTS  %d  %g  %g\n", i, rst[i], rans[i]);
            ++*pass;
            return;
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
 
/**************************
* SHORT PRECISION COMPLEX *
**************************/
void cvecck(n, incx, rans, rst, cname, pass)
_ESVINT n, incx, *pass;
cmplx *rans, *rst;
char *cname;
{
    _ESVINT i;
    float tols, csabs();
    cmplx c;
 
    tols = 10.e-2;
    for(i = 0; i < n * incx; i += incx) {
        RE(c) = RE(rst[i]) - RE(rans[i]);
        IM(c) = IM(rst[i]) - IM(rans[i]);
        if(csabs(c) > tols) {
            printf("%s  FAILED\n", cname);
            printf("RESULTS  %d  (%g  %g)  (%g  %g)\n", i, RE(rst[i]),
                IM(rst[i]), RE(rans[i]), IM(rans[i]));
            ++*pass;
            return;
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
 
/***************************
*     LONG  PRECISION      *
***************************/
void dvecck(n, incx, rans, rst, cname, pass)
_ESVINT n, incx, *pass;
double *rans, *rst;
char *cname;
{
    _ESVINT i;
    float tols;
 
    tols = 10.e-3;
    for(i = 0; i < n * incx; i += incx) {
        if(fabs(rst[i] - rans[i]) > tols) {
            printf("%s  FAILED\n", cname);
            printf("RESULTS  %d  %g  %g\n", i, rst[i], rans[i]);
            ++*pass;
            return;
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
 
/******************************************
*     LONG  PRECISION WITH RELATIVE ERROR *
******************************************/
void dverck(n, incx, rans, rst, cname, pass)
_ESVINT n, incx, *pass;
double *rans, *rst;
char *cname;
{
    _ESVINT i;
    double tols;
 
    tols = 10.e-2;
    for(i = 0; i < n * incx; i += incx) {
        if(fabs(rst[i] - rans[i]) > (tols * fabs(rst[i]))) {
            printf("%s  FAILED\n", cname);
            printf("RESULTS  %d  %g  %g\n", i, rst[i], rans[i]);
            ++*pass;
            return;
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
 
/***************************
*  LONG PRECISION COMPLEX  *
***************************/
void zvecck(n, incx, rans, rst, cname, pass)
_ESVINT n, incx, *pass;
dcmplx *rans, *rst;
char *cname;
{
    _ESVINT i;
    double tols, cdabs();
    dcmplx c;
 
    tols = 10.e-3;
    for(i = 0; i < n * incx; i += incx) {
        RE(c) = RE(rst[i]) - RE(rans[i]);
        IM(c) = IM(rst[i]) - IM(rans[i]);
        if(cdabs(c) > tols) {
            printf("%s  FAILED\n", cname);
            printf("RESULTS  %d  (%g  %g)  (%g  %g)\n", i, RE(rst[i]),
                IM(rst[i]), RE(rans[i]), IM(rans[i]));
            ++*pass;
            return;
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
 
/******************************
*   MATRIX CHECK SUBROUTINE   *
*******************************
*    INTEGER             *
*************************/
void imtchk(m, n, rans, rst, lda, cname, pass)
_ESVINT m, n, *rans, *rst, lda, *pass;
char *cname;
{
    _ESVINT i, j, *a, *b;
 
    a = rans;
    b = rst;
    for(j = 0; j < n; ++j, a += lda, b += lda) {
        for(i = 0; i < m; ++i) {
            if(a[i] != b[i]) {
                printf("%s  FAILED\n", cname);
                printf("RESULTS  %d  %d  %d  %d\n", i, j, b[i], a[i]);
                ++*pass;
                return;
            }
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
 
/*************************
*    SHORT PRECISION     *
*************************/
void smtchk(m, n, rans, rst, lda, cname, pass)
_ESVINT m, n, lda, *pass;
float *rans, *rst;
char *cname;
{
    _ESVINT i, j;
    float tols, *a, *b;
 
    tols = 10.e-2;
    a = rans;
    b = rst;
    for(j = 0; j < n; ++j, a += lda, b += lda) {
        for(i = 0; i < m; ++i) {
            if(fabs(a[i] - b[i]) > tols) {
                printf("%s  FAILED\n", cname);
                printf("RESULTS  %d  %d  %g  %g\n", i, j, b[i], a[i]);
                ++*pass;
                return;
            }
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
 
/*****************************
*    LONG PRECISION          *
*****************************/
void dmtchk(m, n, rans, rst, lda, cname, pass)
_ESVINT m, n, lda, *pass;
double *rans, *rst;
char *cname;
{
    _ESVINT i, j;
    double tols, *a, *b;
 
    tols = 10.e-3;
    a = rans;
    b = rst;
    for(j = 0; j < n; ++j, a += lda, b += lda) {
        for(i = 0; i < m; ++i) {
            if(fabs(a[i] - b[i]) > tols) {
                printf("%s  FAILED\n", cname);
                printf("RESULTS  %d  %d  %g  %g\n", i, j, b[i], a[i]);
                ++*pass;
                return;
            }
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
 
/****************************
*  SHORT PRECISION COMPLEX  *
****************************/
void cmtchk(m, n, rans, rst, lda, cname, pass)
_ESVINT m, n, lda, *pass;
cmplx *rans, *rst;
char *cname;
{
    _ESVINT i, j;
    float tols, csabs();
    cmplx *a, *b, c;
 
    tols = 10.e-2;
    a = rans;
    b = rst;
    for(j = 0; j < n; ++j, a += lda, b += lda) {
        for(i = 0; i < m; ++i) {
            RE(c) = RE(a[i]) - RE(b[i]);
            IM(c) = IM(a[i]) - IM(b[i]);
            if(csabs(c) > tols) {
                printf("%s  FAILED\n", cname);
                printf("RESULTS  %d  %d  (%g  %g)  (%g  %g)\n", i, j,
                    RE(b[i]), IM(b[i]), RE(a[i]), IM(a[i]));
                ++*pass;
                return;
            }
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
 
/***************************
*  LONG PRECISION COMPLEX  *
***************************/
void zmtchk(m, n, rans, rst, lda, cname, pass)
_ESVINT m, n, lda, *pass;
dcmplx *rans, *rst;
char *cname;
{
    _ESVINT i, j;
    double tols, cdabs();
    dcmplx *a, *b, c;
 
    tols = 10.e-2;
    a = rans;
    b = rst;
    for(j = 0; j < n; ++j, a += lda, b += lda) {
        for(i = 0; i < m; ++i) {
            RE(c) = RE(a[i]) - RE(b[i]);
            IM(c) = IM(a[i]) - IM(b[i]);
            if(cdabs(c) > tols) {
                printf("%s  FAILED\n", cname);
                printf("RESULTS  %d  %d  (%g  %g)  (%g  %g)\n", i, j,
                    RE(b[i]), IM(b[i]), RE(a[i]), IM(a[i]));
                ++*pass;
                return;
            }
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
 
float csabs(c)
cmplx c;
{
    float r;
 
    r = sqrt((RE(c) * RE(c)) + (IM(c) * IM(c)));
    return(r);
}
 
double cdabs(c)
dcmplx c;
{
    double r;
 
    r = sqrt((RE(c) * RE(c)) + (IM(c) * IM(c)));
    return(r);
}
 
 
void smtckd(m, n, rans, rst, lda, cname, pass)
_ESVINT m, n, lda, *pass;
float *rans, *rst;
char *cname;
{
    _ESVINT i, j;
    float tols, *a, *b;
 
    tols = 10.e-2;
    a = rans;
    b = rst;
    for(j = 0; j < n; ++j, a += lda, b += lda) {
        for(i = j; i < m; ++i) {
            if(fabs(a[i] - b[i]) > tols) {
                printf("%s  FAILED\n", cname);
                printf("RESULTS  %d  %d  %g  %g\n", i, j, b[i], a[i]);
                ++*pass;
                return;
            }
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
 
 
void dmtckd(m, n, rans, rst, lda, cname, pass)
_ESVINT m, n, lda, *pass;
double *rans, *rst;
char *cname;
{
    _ESVINT i, j;
    double tols, *a, *b;
 
    tols = 10.e-3;
    a = rans;
    b = rst;
    for(j = 0; j < n; ++j, a += lda, b += lda) {
        for(i = j; i < m; ++i) {
            if(fabs(a[i] - b[i]) > tols) {
                printf("%s  FAILED\n", cname);
                printf("RESULTS  %d  %d  %g  %g\n", i, j, b[i], a[i]);
                ++*pass;
                return;
            }
        }
    }
/*    printf("%s  PASSED\n", cname);
*/
}
